<?php
/**
* WIKINDX: Bibliographic Management system. Copyright (C)

* This program is free software; you can redistribute it and/or modify it under the terms
* of the GNU General Public License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along with this program;
* if not, write to the
* Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* The WIKINDX Team 2012
* sirfragalot@users.sourceforge.net.
*/
/**
* Interface for AJAX elements in various pages.
*
* Load the class through FACTORY:
*	$this->ajax = FACTORY_AJAX::getInstance();
*
* @version 1.0
* @author Mark Grimshaw
* @package wikindx4\core\ajax
*/
class AJAX
{
/**
* AJAX
*/
	public function __construct()
	{
	}
/**
* Load the ajax javascript and the user javascript(s) into the HTML page output.
*
* @param mixed $scripts Either an array of .js scripts to load or a single string.
*/
	public function loadJavascript($scripts)
	{
		$html = FACTORY_HTML::getInstance();
		$pString = $html->jsExternal("core/ajax/json2.js");
		$pString .= $html->jsExternal("core/ajax/ajax.js");
		if(is_array($scripts))
		{
			foreach($scripts AS $script)
				$pString .= $html->jsExternal($script);
		}
		else if($scripts)
			$pString .= $html->jsExternal($scripts);
		GLOBALS::buildOutputString($pString);
	}
/**
* Create and load the HTML form element insert for the AJAX action and function.
*
* @param string $action The javascript action such as 'onclick' etc.
* @param array $jsonArray The parameter to be passed to gateway() as an array ready to be converted to JSON
* @param boolean $return If TRUE, generated javascript has a return from the gateway() function. Default is FALSE.
*/
	public function jActionForm($action, $jsonArray, $return = FALSE)
	{
		$form = FACTORY_FORM::getInstance();
		$json = $this->encode_jArray($jsonArray, TRUE);
		if($return)
			$form->insertJscriptAction = "$action=\"return gateway('$json') ? true : false;\"";
		else
			$form->insertJscriptAction = "$action=\"gateway('$json')\"";
	}
/**
* Create and load the IMG element insert for the AJAX action and function.  Works with core/display/LOADICONS.php.
*
* @param string $icon The icon to make an image of ('add', 'delete', 'view' etc.)
* @param string $action The javascript action such as 'onclick' etc.
* @param array $jsonArray The parameter to be passed to gateway() as an array ready to be converted to JSON
* @param boolean $return If TRUE, generated javascript has a return from the gateway() function. Default is FALSE.
* @return string The image tag
*/
	public function jActionIcon($icon, $action, $jsonArray, $return = FALSE)
	{
		if($return)
			$return = 'return';
		$html = FACTORY_HTML::getInstance();
		$icons = FACTORY_LOADICONS::getInstance();
		$json = $this->encode_jArray($jsonArray, TRUE);
		$icons->jscriptActions[$icon] = "$action=\"$return gateway('$json')\"";
		$icons->resetSetup();
		$icons->setupIcons($icon);
		return $html->span($icons->{$icon}, 'cursorPointer');
	}
/**
* Convert $jsonArray to JSON string and format any array elements referencing javascript functions.
*
* @param array $jsonArray The unformatted JSON array
* @param boolean $quote No encoding of quotation marks (") if set to FALSE. Default is FALSE.
* @return string The JSON string
*/
	public function encode_jArray($jsonArray, $quote = FALSE)
	{
//print_r($jsonArray); print '<P>';
		if($quote)
		{
//			if(function_exists('json_last_error')) // added PHP v5.3
//				$json = json_encode($jsonArray, JSON_HEX_QUOT | JSON_HEX_APOS);
//			else
//			{
				array_walk_recursive($jsonArray, array($this, 'jsonMap'));
				$json = json_encode($jsonArray);
//			}
			if(function_exists('json_last_error')) // added PHP v5.3
				$this->json_error('encode');
			return htmlspecialchars($json, ENT_QUOTES);
		}
		$return = json_encode($jsonArray);
		if(function_exists('json_last_error')) // added PHP v5.3
			$this->json_error('encode');
		return $return;
	}
/**
* Encode JSON string to UTF8.
*
* @param string $value
*/
	private function jsonMap(&$value)
	{
		$value = utf8_encode(addslashes($value));
	}
/**
* Convert JSON-formatted $jsonString to an array or object.
*
* @param string $jsonString The JSON string
* @param boolean $stripslashes In WIKINDX, querystrings are slashed. Default is TRUE.
* @param boolean $array If TRUE (default), return associative array, otherwise return object.
* @return mixed Array or object
*/
	public function decode_jString($jsonString, $stripslashes = TRUE, $array = TRUE)
	{
		if($stripslashes)
			$return = json_decode(stripslashes(utf8_decode($jsonString)), $array);
		else
			$return = json_decode(utf8_decode($jsonString), $array);
		if(function_exists('json_last_error')) // added PHP v5.3
			$this->json_error('decode');
		return $return;
	}
/**
* echo JSON error or return if none.
*
* @param string $type
*/
	private function json_error($type)
	{
		switch(json_last_error())
		{
			case JSON_ERROR_DEPTH:
				echo 'JSON - Maximum stack depth exceeded: ' . $type;
			break;
			case JSON_ERROR_CTRL_CHAR:
				echo 'JSON - Unexpected control character found: ' . $type;
			break;
			case JSON_ERROR_SYNTAX:
				echo 'JSON - Syntax error, malformed JSON: ' . $type;
			break;
			case JSON_ERROR_NONE:
				break;
			break;
		}
	}
}
?>