<?php
/**
* WIKINDX: Bibliographic Management system. Copyright (C)

* This program is free software; you can redistribute it and/or modify it under the terms
* of the GNU General Public License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along with this program;
* if not, write to the
* Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* The WIKINDX Team 2012
* sirfragalot@users.sourceforge.net.
*/
/**
*
* Format a bibliographic resource for output.
*
*	@package wikindx4\core\bibcitation
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*/
class BIBFORMAT
{
/** Preview. */
private $preview = FALSE;
/** Path */
private $dir = '';
/** Bibtex */
private $bibtex = FALSE;
/** Path */
private $bibtexParsePath;
/** UTF8 */
private $utf8;
/** Previous creator */
private $previousCreator = '';
/** Parse style */
private $parsestyle;
/**
* Switch editor and author positions in the style definition for a book in which there are only editors
*/
private $editorSwitch = FALSE;
/** Boolean */
private $dateMonthNoDay = FALSE;
/** array */
private $creators = array('creator1', 'creator2', 'creator3', 'creator4', 'creator5');
/**
* Some styles require different templates and formatting of creator names for a citation in a footnote as opposed to a full bibliography.
*
* Setting this to TRUE (set 
* externally in CITEFORMAT) loads a different set of templates and settings for footnotes.  The default FALSE is for full bibliography.
*/
public $footnotePages = FALSE;
/** boolean */
private $footnoteType = FALSE;
/** boolean */
public $footnoteStyle = FALSE;
/** boolean */
private $partialDone = FALSE;
/** boolean */
public $citationFootnote = FALSE;
/**
* Output medium: 'html', 'rtf', or 'plain'
*/
public $output = 'html';
/** boolean */
public $wikindx = FALSE;
/** string */
public $type;
/**
* Highlight preg pattern and CSS class for HTML display
*/
public $patterns = FALSE;
/** boolean */
public $patternHighlight = FALSE;
/** array */
public $styleMap;

/** 
* BIBFORMAT
*
* @param string $output 'html', plain', 'rtf'. Default is 'html'
*/
	public function __construct($output = 'html')
	{
		$this->output = $output;
		if(!$this->preview) // Not javascript preview
		{
			if(!$this->dir)
				$this->dir = dirname(__FILE__) . "/";
			else
			{
				if($this->dir[strlen($this->dir)-1] != "/")
					$this->dir .= "/";
			}
			$this->bibtexParsePath  = $this->dir . "bibtexParse";
		}
		else // preview
			$this->dir = '';
		if($this->bibtex)
		{
			include_once($this->dir . "STYLEMAPBIBTEX.php");
			$this->styleMap = new STYLEMAPBIBTEX();
		}
		else
		{
			include_once($this->dir . "STYLEMAP.php");
			$this->styleMap = new STYLEMAP();
		}
		include_once($this->dir . "PARSESTYLE.php");
		$this->parsestyle = new PARSESTYLE();
		$this->utf8 = FACTORY_UTF8::getInstance();
	}
/**
* Initialise bibformat system
*/
	public function initialise()
	{
		include_once($this->dir . "EXPORTFILTER.php");
		$this->export = new EXPORTFILTER($this, $this->output);
	}
/**
* Transform the raw data from the XML file into usable arrays and load some data arrays
*
* @author	Mark Grimshaw
* @version	1
*
* @param	array $common		Array of global formatting data
* @param	array $types		Array of style definitions for each resource type
* @param	array $footnote		Array of style definitions for footnote creators
*/
	public function getStyle($common, $types, $footnote)
	{
		$this->style = $common;
		$this->fallback = $types['fallback'];
		unset($types['fallback']);
		$this->backup = $types;
		foreach($types as $name => $value)
		{
			if(array_key_exists($name, $footnote) && $footnote[$name])
			{
				$footnoteType = 'footnote_' . $name;
				foreach($footnote[$name] as $rName => $rValue)
				{
					if(($rName == 'independent') && is_array($rValue))
					{
						$array = array();
						foreach($rValue as $iName => $iValue)
						{
							$split = explode('_', $iName);
							$array[$split[1]] = $iValue;
						}
						$this->{$footnoteType}['independent'] = $array;
					}
					else
						$this->{$footnoteType}[$rName] = $rValue;
				}
			}
			$this->$name = $value;
//print "$name: "; print_r($this->$name); print '<P>';
			$typeTemplateSet = $name . 'TemplateSet';
			if(!empty($this->$name) && (sizeof($this->$name) > 3))
				$this->$typeTemplateSet = TRUE;
			else
				$this->$typeTemplateSet = FALSE;
		}
// Set fallbacks
		foreach($types as $name => $value)
		{
			$typeTemplateSet = $name . 'TemplateSet';
//print "$name: "; print_r($value); print "<P>";
			if(array_key_exists($name, $this->fallback))
			{
				$footnoteType = 'footnote_' . $name;
				if(array_key_exists($name, $footnote) && !$footnote[$name])
					$this->{$footnoteType} = $footnote[$this->fallback[$name]];
			}
			if(array_key_exists('fallbackstyle', $value) && $this->$typeTemplateSet)
				$this->fallback[$name] = $value['fallbackstyle'];
		}
/**
* Load localizations etc.
*/
		$this->loadArrays();
	}
/**
* Restore each $this->type array from $this->backup
*
* @author	Mark Grimshaw
* @version	1
*/
	private function restoreTypes()
	{
		foreach($this->backup as $type => $array)
			$this->$type = $array;
	}
/**
* Perform pre-processing on the raw SQL array
*
* @author	Mark Grimshaw
* @version	1
*
* @param	string $type	The resource type
* @param	array $row	Associate array of raw SQL data
* @return	array	Processed row of raw SQL data
*/
	public function preProcess($type, $row)
	{
/**
* Ensure that $this->item is empty for each resource!!!!!!!!!!
*/
		$this->item = $this->footnoteTypeArray = array();
// Map this system's resource type to OSBib's resource type
		$this->type = array_search($type, $this->styleMap->types);
		$typeTemplateSet = $type . 'TemplateSet';
		if($this->bibtex && array_key_exists('author', $row))
		{
			$row['creator1'] = $row['author'];
			unset($row['author']);
		}
		if($this->bibtex && array_key_exists('editor', $row))
		{
			$row['creator2'] = $row['editor'];
			unset($row['editor']);
		}
/**
* Set any author/editor re-ordering for book and book_article type.
*/
		if(!$this->preview && (($type == 'book') || ($type == 'book_article')) && 
			$row['creator2'] && !$row['creator1'] && $this->style['editorSwitch'] &&
			array_key_exists('author', $this->$type))
		{
			$row['creator1'] = $row['creator2'];
			$row['creator2'] = FALSE;
			$editorArray = $this->parsestyle->parseStringToArray($type, $this->style['editorSwitchIfYes'], 
				$this->styleMap);
			if(!empty($editorArray) && array_key_exists('editor', $editorArray))
			{
				$this->{$type}['author'] = $editorArray['editor'];
 				unset($this->{$type}['editor']);
				$this->editorSwitch = TRUE;
			}
		}
		if($this->style['dateMonthNoDay'] && array_key_exists('date', $this->styleMap->$type) && 
			array_key_exists('dateMonthNoDayString', $this->style) && $this->style['dateMonthNoDayString'])
		{
			$this->dateArray = $this->parsestyle->parseStringToArray($type, $this->style['dateMonthNoDayString'], 
				$this->styleMap, TRUE);
			$this->dateMonthNoDay = TRUE;
		}
/**
* If $row comes in in BibTeX format, process and add items to $this->item
*/
		if($this->bibtex)
			list($type, $row) = $this->preProcessBibtex($row, $this->type);
/**
* Set to catch-all generic style.  For all keys except named database fields, creator1 and year1, 
* we only print if the value in $this->styleMap matches the value in 
* $this->styleMap->generic for each key.
*/
		if($this->citationFootnote) // using footnote template
		{
			$footnoteType = 'footnote_' . $type;
			if(isset($this->$footnoteType)) // footnote template for this resource exists
			{
				$this->footnoteType = $footnoteType;
				$this->footnoteTypeArray[$type] = $footnoteType;
			}
			else 
			{
				$footnoteType = 'footnote_' . $this->fallback[$type];
				if(isset($this->$footnoteType)) // fallback footnote template exists
				{
					$this->footnoteType = $footnoteType;
					$this->footnoteTypeArray[$type] = $footnoteType;
				}
				else if(!$this->$typeTemplateSet) // use fallback bibliography template
				{
					$fallback = $this->fallback[$type];
					$this->footnoteTypeArray[$type] = $fallback;
					$type = $fallback;
				}
				// else, we're using the bibliography template for this resource type
				else
				{
					$this->footnoteTypeArray[$type] = $type;
}
			}
		}
		else
		{
			if(!$this->$typeTemplateSet)
			{
				$fallback = $this->fallback[$type];
				$type = $fallback;
			}
		}
		$this->type = $type;
/**
* Add BibTeX entry to $this->item
*/
		if($this->bibtex)
		{
			foreach($row as $field => $value)
			{
				if(array_key_exists($field, $this->styleMap->$type) && 
					!array_key_exists($this->styleMap->{$type}[$field], $this->item))
					$this->addItem($row[$field], $field);
			}
		}
		return $row;
	}
/**
* Preprocess BibTeX-type entries
* @author Mark Grimshaw
* @version 1
*
* @param array $row assoc. array of elements for one bibtex entry
* @param string $type resource type
* @return string|array resource type|resource assoc. array of elements for one bibtex entry
*/
	private function preProcessBibtex(&$row, $type)
	{
//05/05/2005 G.GARDEY: change bibtexParse name.
/**
* This set of includes is for the OSBib public release and should be uncommented for that and
* the WIKINDX-specific includes below commented out!
*/
/*
		include_once($this->bibtexParsePath . "/PARSECREATORS.php");
		$parseCreator = new PARSECREATORS();
		include_once($this->bibtexParsePath . "/PARSEMONTH.php");
		$parseDate = new PARSEMONTH();
		include_once($this->bibtexParsePath . "/PARSEPAGE.php");
		$parsePages = new PARSEPAGE();
*/
// WIKINDX naming of above files
		include_once($this->bibtexParsePath . "/BIBTEXCREATORPARSE.php");
		$parseCreator = new BIBTEXCREATORPARSE();
		include_once($this->bibtexParsePath . "/BIBTEXMONTHPARSE.php");
		$parseDate = new BIBTEXMONTHPARSE();
		include_once($this->bibtexParsePath . "/BIBTEXPAGEPARSE.php");
		$parsePages = new BIBTEXPAGEPARSE();
/**
* Bibtex-specific types not defined in STYLEMAPBIBTEX
*/
		if(!$this->type)
		{
			if($type == 'mastersthesis')
			{
				$type = 'thesis';
				$row['type'] = "Master's Dissertation";
			}
			if($type == 'phdthesis')
			{
				$type = 'thesis';
				$row['type'] = "PhD Thesis";
			}
			else if($type == 'booklet')
				$type = 'miscellaneous';
			else if($type == 'conference')
				$type = 'proceedings_article';
			else if($type == 'incollection')
				$type = 'book_article';
			else if($type == 'manual')
				$type = 'report';
		}
/**
* 'article' could be journal, newspaper or magazine article
*/
		else if($type == 'article')
		{
			if(array_key_exists('month', $row) && array_key_exists('date', $this->styleMap->$type))
			{
				list($month, $day) = $parseDate->init($row['month']);
				if($day)
					$type = 'newspaper_article';
				else if($month)
					$type = 'magazine_article';
				$this->formatDate($day, $month);
			}
			else
				$type = 'journal_article';
		}
/**
* Is this a web article?
*/
		else if(($type == 'misc') && array_key_exists('howpublished', $row))
		{
			if(preg_match("#^\\\url{(.*://.*)}#", $row['howpublished'], $match))
			{
				$row['URL'] = $match[1];
				$type = 'web_article';
			}
		}
		$this->type = $type;
		if(array_key_exists('creator1', $row) && $row['creator1'] && 
			array_key_exists('creator1', $this->styleMap->$type))
		{
			$creators = $parseCreator->parse($row['creator1']);
			foreach($creators as $cArray)
			{
				$temp[] = array(
						'surname'	=>	trim($cArray[2]),
						'firstname'	=>	trim($cArray[0]),
						'initials'	=>	trim($cArray[1]),
						'prefix'	=>	trim($cArray[3]),
					);
			}
			$this->formatNames($temp, 'creator1');
			unset($temp);
		}
		if(array_key_exists('creator2', $row) && $row['creator2'] && 
			array_key_exists('creator2', $this->styleMap->$type))
		{
			$creators = $parseCreator->parse($row['creator2']);
			foreach($creators as $cArray)
			{
				$temp[] = array(
						'surname'	=>	trim($cArray[2]),
						'firstname'	=>	trim($cArray[0]),
						'initials'	=>	trim($cArray[1]),
						'prefix'	=>	trim($cArray[3]),
					);
			}
			$this->formatNames($temp, 'creator2');
		}
		if(array_key_exists('pages', $row) && array_key_exists('pages', $this->styleMap->$type))
		{
			list($start, $end) = $parsePages->init($row['pages']);
			$this->formatPages(trim($start), trim($end));
		}
		$this->formatTitle($row['title'], "{", "}");
		return array($type, $row);
	}
/**
* Find alternate fields dependent on previous or following fields and deal with accordingly.
* 
* @param array $typeArray 
* @author	Mark Grimshaw
* @version	1
*/
	private function findAlternates($typeArray)
	{
		foreach($typeArray as $field => $array)
		{
			if(is_array($array)&& array_key_exists($field, $this->item))
			{
				if(array_key_exists('alternatePreFirst', $array) || 
					array_key_exists('alternatePreSecond', $array))
				{
					if(!$this->item[$field])
					{
						if(array_key_exists($array['alternatePreFirst'], $this->item))
							unset($this->item[$array['alternatePreFirst']]);
					}
					else
					{
						if(array_key_exists($array['alternatePreSecond'], $this->item))
							unset($this->item[$array['alternatePreSecond']]);
					}
				}
				if(array_key_exists('alternatePostFirst', $array) || 
					array_key_exists('alternatePostSecond', $array))
				{
					if(!$this->item[$field])
					{
						if(array_key_exists($array['alternatePostFirst'], $this->item))
							unset($this->item[$array['alternatePostFirst']]);
					}
					else
					{
						if(array_key_exists($array['alternatePostSecond'], $this->item))
							unset($this->item[$array['alternatePostSecond']]);
					}
				}
			}
		}
	}
/**
* Check that the first field of $this->$type exists in $this->item and has a value.  If it does not and $this->$type has a partial template array, rearrange $this->$type
* 
* @author	Mark Grimshaw
* @version	1
*
* @param	string $type	Resource type
*/
	private function checkPartial($type)
	{
//print "TYPE: "; print_r($this->$type); print "<P>";
//print "ITEM: "; print_r($this->item); print "<P>";
		$this->partialDone = FALSE;
		if(!array_key_exists('partial', $this->$type) || !$this->{$type}['partial'])
			return; // nothing to do
		$typeKeys = array_keys($this->$type);
		if(array_key_exists($typeKeys[0], $this->item) && $this->item[$typeKeys[0]])
			return;	// item key exists -- nothing to do
		$this->{$type}['partial'] = $this->parsestyle->parseStringToArray($type, 
			$this->{$type}['partial'], $this->styleMap);
// Replace the whole template with the partial template?
		if($this->{$type}['partialReplace'])
		{
			$partial = $this->{$type}['partial'];
			$this->$type = $partial;
// If independent keys exists, because we have removed the first key from $this->$type, we must decrement the keys in independent
			if(array_key_exists('independent', $this->$type) && !empty($this->{$type}['independent']))
			{
				foreach($this->{$type}['independent'] as $key => $value)
					$tempInd[$key - 1] = $value;
				$this->{$type}['independent'] = $tempInd;
			}
			return;
		}
// At this point need to rearrange
		$oldType = $this->$type;
// Remove missing first item
		array_shift($oldType);
		foreach($this->{$type}['partial'] as $key => $value)
			unset($oldType[$key]);
		$this->$type = array_merge($this->{$type}['partial'], $oldType);
// If independent keys exists, because we have removed the first key from $this->$type, we must decrement the keys in independent
		if(array_key_exists('independent', $this->$type) && !empty($this->{$type}['independent']))
		{
			foreach($this->{$type}['independent'] as $key => $value)
				$tempInd[$key - 1] = $value;
			$this->{$type}['independent'] = $tempInd;
		}
		$this->partialDone = TRUE;
//print "TYPE: "; print_r($this->$type); print "<P>";
//print "ITEM: "; print_r($this->item); print "<P>";
	}
/**
* Map the $item array against the style array ($this->$type) for this resource type and produce a string ready to be 
* formatted for bold, italics etc.
* 
* @author	Mark Grimshaw
* @version	1
*
* @param	mixed $template	If called from CITEFORMAT, this is the array of template elements. Default is FALSE.
* @return	string For printing to the output medium.
*/
	public function map($template = FALSE)
	{
		$type = $pluralType = $this->type;
// Check for partial templates
		if(!$this->footnoteType)
			$this->checkPartial($type);
		else
		{
			$type = $this->footnoteType;
			$this->footnoteType = FALSE;
		}
		$this->findAlternates($this->$type);
		$ultimate = $preliminary = '';
		$index = 0;
		$previousFieldExists = $nextFieldExists = TRUE;
		if(array_key_exists('independent', $this->$type))
			$independent = $this->{$type}['independent'];
/**
* For dependency on next field, we must grab array keys of $this->$type, shift the first element then, in the loop, 
* check each element exists in $item.  If it doesn't, $nextFieldExists is set to FALSE
*/
		$checkPost = array_keys($this->$type);
		array_shift($checkPost);
		$lastFieldKey = FALSE;
// Add or replace pages field if this process is called from CITEFORMAT for footnotes where $this->footnotePages are the formatted citation pages.
		if($this->footnotePages)
			$this->item['pages'] = $this->footnotePages;
		foreach($this->$type as $key => $value)
		{
			if(($key == 'partial') || ($key == 'partialReplace'))
				continue;
			if($key == 'ultimate')
			{
				$ultimate = $value;
				continue;
			}
			if($key == 'preliminaryText')
			{
				$preliminary = $value;
				continue;
			}
			if(!array_key_exists($key, $this->item) || !$this->item[$key])
			{
				$keyNotExists[] = $index;
				$index++;
				array_shift($checkPost);
				$previousFieldExists = FALSE;
				continue;
			}
			$checkPostShift = array_shift($checkPost);
			if(!array_key_exists($checkPostShift, $this->item) || !$this->item[$checkPostShift])
				$nextFieldExists = FALSE;
			$pre = array_key_exists('pre', $value) ? $value['pre'] : '';
			$post = array_key_exists('post', $value) ? $value['post'] : '';
/**
* Deal with __DEPENDENT_ON_PREVIOUS_FIELD__ for characters dependent on previous field's existence and 
* __DEPENDENT_ON_NEXT_FIELD__ for characters dependent on the next field's existence
*/
			if($previousFieldExists && array_key_exists('dependentPre', $value))
				$pre = preg_replace("/__DEPENDENT_ON_PREVIOUS_FIELD__/", 
					$value['dependentPre'], $pre);
			else if(array_key_exists('dependentPreAlternative', $value))
				$pre = preg_replace("/__DEPENDENT_ON_PREVIOUS_FIELD__/", 
					$value['dependentPreAlternative'], $pre);
			else
				$pre = preg_replace("/__DEPENDENT_ON_PREVIOUS_FIELD__/", '', $pre);
			if($nextFieldExists && array_key_exists('dependentPost', $value))
				$post = str_replace("__DEPENDENT_ON_NEXT_FIELD__", 
					$value['dependentPost'], $post);
			else if(array_key_exists('dependentPostAlternative', $value))
				$post = preg_replace("/__DEPENDENT_ON_NEXT_FIELD__/", 
					$value['dependentPostAlternative'], $post);
			else
				$post = preg_replace("/__DEPENDENT_ON_NEXT_FIELD__/", '', $post);
/**
* Deal with __SINGULAR_PLURAL__ for creator lists and pages
*/			if($styleKey = array_search($key, $this->styleMap->$pluralType))
				$pluralKey = $styleKey . "_plural";
// For use with generic footnote templates which uses generic 'creator' field
			else
				$pluralKey = "creator_plural";
			if(isset($this->$pluralKey) && $this->$pluralKey) // plural alternative for this key
			{
				$pre = array_key_exists('plural', $value) ? 
					preg_replace("/__SINGULAR_PLURAL__/", $value['plural'], $pre) : $pre;
				$post = array_key_exists('plural', $value) ? 
					preg_replace("/__SINGULAR_PLURAL__/", $value['plural'], $post) : $post;
			}
			else if(isset($this->$pluralKey)) // singular alternative for this key
			{
				$pre = array_key_exists('singular', $value) ? 
					preg_replace("/__SINGULAR_PLURAL__/", $value['singular'], $pre) : $pre;
				$post = array_key_exists('singular', $value) ? 
					preg_replace("/__SINGULAR_PLURAL__/", $value['singular'], $post) : $post;
			}
// Deal with en dash characters in pages
			if($key == 'pages')
				$this->item[$key] = $this->export->format($this->item[$key]);
/**
* Strip backticks used in template
*/
			$pre = str_replace("`", '', $pre);
			$post = str_replace("`", '', $post);
/**
* Make sure we don't have multiple punctuation characters after a field
*/			$lastPre = substr($post, -1);
			$firstItem = substr($this->item[$key], 0, 1);
			if($firstItem === $lastPre)
				$this->item[$key] = substr($this->item[$key], 1);
// Match last character of this field with $post
			if($post && preg_match("/[.,;:?!]$/", $this->item[$key]) &&
				preg_match("/^(\[.*?[\]]+)*([.,;:?!])|^([.,;:?!])/", $post, $capture, PREG_OFFSET_CAPTURE))
// There is punctuation in post either immediately following BBCode formatting or at the start of the string.
// The offset for the punctuation character in $post is given at $capture[2][1]
				$post = substr_replace($post, '', $capture[2][1], 1);
// Match $itemArray[$lastFieldKey] with $pre
			if(($lastFieldKey !== FALSE) && $pre && preg_match("/^[.,;:?!]/", $pre) && 
				preg_match("/([.,;:?!])(\[.*?[\]]+)*$|([.,;:?!])$/", 
				$itemArray[$lastFieldKey], $capture, PREG_OFFSET_CAPTURE))
// There is punctuation in post either immediately following BBCode formatting or at the start of the string.
				$pre = substr_replace($pre, '', 0, 1);
			if($this->item[$key])
			{
				$itemArray[$index] = $pre . $this->item[$key] . $post;
				$lastFieldKey = $index;
			}
			$previousFieldExists = $nextFieldExists = TRUE;
			$index++;
		}
/**
* Check for independent characters.  These (should) come in pairs.
*/		if(isset($independent))
		{
			$independentKeys = array_keys($independent);
			while($independent)
			{
				$preAlternative = $postAlternative = FALSE;
				$startFound = $endFound = FALSE;
				$pre = array_shift($independent);
				$post = array_shift($independent);
				if(preg_match("/%(.*)%(.*)%|%(.*)%/U", $pre, $dependent))
				{
					if(sizeof($dependent) == 4)
						$pre = $dependent[3];
					else
					{
						$pre = $dependent[1];
						$preAlternative = $dependent[2];
					}
				}
				if(preg_match("/%(.*)%(.*)%|%(.*)%/U", $post, $dependent))
				{
					if(sizeof($dependent) == 4)
						$post = $dependent[3];
					else
					{
						$post = $dependent[1];
						$postAlternative = $dependent[2];
					}
				}
/**
* Strip backticks used in template
*/
				$preAlternative = str_replace("`", '', $preAlternative);
				$postAlternative = str_replace("`", '', $postAlternative);
				$firstKey = str_replace('independent_', '', array_shift($independentKeys));
				$secondKey = str_replace('independent_', '', array_shift($independentKeys));
				for($index = $firstKey; $index <= $secondKey; $index++)
				{
					if(array_key_exists($index, $itemArray))
					{
						$startFound = $index;
						break;
					}
				}
				for($index = $secondKey; $index >= $firstKey; $index--)
				{
					if(array_key_exists($index, $itemArray))
					{
						$endFound = $index;
						break;
					}
				}
				if(($startFound !== FALSE) && ($endFound !== FALSE)) // intervening fields found
				{
					$itemArray[$startFound] = $pre . $itemArray[$startFound];
					$itemArray[$endFound] = $itemArray[$endFound] . $post;
				}
				else // intervening fields not found - do we have an alternative?
				{
					if(array_key_exists($firstKey - 1, $itemArray) && $preAlternative)
						$itemArray[$firstKey - 1] .= $preAlternative;
					if(array_key_exists($secondKey + 1, $itemArray) && $postAlternative)
						$itemArray[$secondKey + 1] = $postAlternative . $itemArray[$secondKey + 1];
				}
			}
		}
		if(!isset($itemArray)) // Empty titles should not occur but, in case, this catches errors.
			$itemArray = array();
		$pString = join('', $itemArray);
/**
* if last character is punctuation (which it may be with missing fields etc.), and $ultimate is also 
* punctuation, set $ultimate to empty string.
*/		if(isset($ultimate) && $ultimate)
		{
			$pString = trim($pString);
/**
* Don't do ';' in case last element is URL with &gt; ...!
*/
			if(preg_match("/^[.,:?!]/", $ultimate) && 
				preg_match("/([.,:?!])(\[.*?[\]]+)*$|([.,:?!])$/", $pString))
				$ultimate = '';
		}
// If $this->editorSwitch or $this->dateMonthNoDay, we have altered $this->$bibformat->$type so need to reload styles
		if(!$this->preview && ($this->editorSwitch || $this->dateMonthNoDay || $this->partialDone))
		{
			$this->restoreTypes();
			$this->editorSwitch = $this->dateMonthNoDay = FALSE;
		}
		return $this->export->format($preliminary . trim($pString) . $ultimate);
	}
/**
* Format creator name lists (authors, editors, etc.)
* 
* @author	Mark Grimshaw
* @version	1
* 
* @param	array $creators	Multi-associative array of creator names e.g. this array might be of 
* the primary authors:
* <pre>
*	array([0] => array(['surname'] => 'Grimshaw', ['firstname'] => Mark, ['initials'] => 'N', ['prefix'] => ),
*	   [1] => array(['surname'] => 'Witt', ['firstname'] => Jan, ['initials'] => , ['prefix'] => 'de'))
* </pre>
* @param	string $nameType	'creator1', 'creator2' etc.
* @param	boolean $shortFootnote  If TRUE, this is being used for just the primary creator names in a footnote style citation using Ibid, Idem, op cit. etc. 
* Default is FALSE.
* @return	string Optional if $nameType == 'citation': formatted string of all creator names in the input array.
*/
	public function formatNames($creators, $nameType, $shortFootnote = FALSE)
	{
		$style = $this->citationFootnote ? $this->footnoteStyle : $this->style;
		$first = TRUE;
/**
* Citation creators
*/
		if($nameType == 'citation')
		{
			$limit = 'creatorListLimit';
			$moreThan = 'creatorListMore';
			$abbreviation = 'creatorListAbbreviation';
			$initialsStyle = 'creatorInitials';
			$firstNameInitial = 'creatorFirstName';
			$delimitTwo = 'twoCreatorsSep';
			$delimitFirstBetween = 'creatorSepFirstBetween';
			$delimitNextBetween = 'creatorSepNextBetween';
			$delimitLast = 'creatorSepNextLast';
			$uppercase = 'creatorUppercase';
			$italics = 'creatorListAbbreviationItalic';
		}
/**
* Primary creator
*/
		else if($nameType == 'creator1')
		{
			$limit = 'primaryCreatorListLimit';
			$moreThan = 'primaryCreatorListMore';
			$abbreviation = 'primaryCreatorListAbbreviation';
			$initialsStyle = 'primaryCreatorInitials';
			$firstNameInitial = 'primaryCreatorFirstName';
			$delimitTwo = 'primaryTwoCreatorsSep';
			$delimitFirstBetween = 'primaryCreatorSepFirstBetween';
			$delimitNextBetween = 'primaryCreatorSepNextBetween';
			$delimitLast = 'primaryCreatorSepNextLast';
			$uppercase = 'primaryCreatorUppercase';
			$italics = 'primaryCreatorListAbbreviationItalic';
		}
		else
		{
			$limit = 'otherCreatorListLimit';
			$moreThan = 'otherCreatorListMore';
			$abbreviation = 'otherCreatorListAbbreviation';
			$initialsStyle = 'otherCreatorInitials';
			$firstNameInitial = 'otherCreatorFirstName';
			$delimitTwo = 'otherTwoCreatorsSep';
			$delimitFirstBetween = 'otherCreatorSepFirstBetween';
			$delimitNextBetween = 'otherCreatorSepNextBetween';
			$delimitLast = 'otherCreatorSepNextLast';
			$uppercase = 'otherCreatorUppercase';
			$italics = 'otherCreatorListAbbreviationItalic';
		}
		$type = $this->type;
/**
* Set default plural behaviour for creator lists
*/
// For use with generic footnote templates which uses generic 'creator' field
		if($this->citationFootnote && ($nameType == 'creator1') && 
		($this->styleMap->{$type}[$nameType] != 'creator'))
			$pluralKey = "creator_plural";
		else
			$pluralKey = $nameType . "_plural";
		$this->$pluralKey = FALSE;
		$firstInList = TRUE;
		$rewriteCreatorBeforeDone = $rewriteCreatorFinal = FALSE;
		foreach($creators as $creator)
		{
			if(($nameType == 'citation') && $first)
				$nameStyle = 'creatorStyle';
			else if($nameType == 'citation')
				$nameStyle = 'creatorOtherStyle';
			else if(($nameType == 'creator1') && $first)
				$nameStyle = 'primaryCreatorFirstStyle';
			else if($nameType == 'creator1')
				$nameStyle = 'primaryCreatorOtherStyle';
			else if($first)
				$nameStyle = 'otherCreatorFirstStyle';
			else
				$nameStyle = 'otherCreatorOtherStyle';
			if(array_key_exists('creatorId', $creator))
				$creatorIds[] = $creator['creatorId'];
			$firstName = trim($this->checkInitials($creator, $style[$initialsStyle], 
				$style[$firstNameInitial]));
			$prefix = $creator['prefix'] ? trim($creator['prefix']) . ' ' : '';
			if($style[$nameStyle] == 0) // Joe Bloggs
			{
				$nameString = $firstName . ' ' . 
					$prefix . 
					$creator['surname'];
			}
			else if($style[$nameStyle] == 1) // Bloggs, Joe
			{
				$prefixDelimit = $firstName ? ', ' : '';
				$nameString = 
					$creator['prefix'] . ' ' . 
					$creator['surname'] . $prefixDelimit . 
					$firstName;
			}
			else if($style[$nameStyle] == 2) // Bloggs Joe
			{
				$nameString = 
					$creator['prefix'] . ' ' . 
					$creator['surname'] . ' ' . 
					$firstName;
			}
			else // Last name only
			{
				$nameString = 
					$creator['prefix'] . ' ' . 
					$creator['surname'];
			}
			if(isset($style[$uppercase]))
				$nameString = $this->utf8->utf8_strtoupper($nameString);
			$nameString = trim($nameString);
			if($firstInList)
			{
				$rewriteCreatorField = $nameType . "_firstString";
				$rewriteCreatorFieldBefore = $nameType . "_firstString_before";
			}
			else
			{
				$rewriteCreatorField = $nameType . "_remainderString";
				$rewriteCreatorFieldBefore = $nameType . "_remainderString_before";
				$rewriteCreatorFieldEach = $nameType . "_remainderString_each";
			}
			if(array_key_exists($rewriteCreatorField, $this->$type))
			{
				if($firstInList)
				{
					if(array_key_exists($rewriteCreatorFieldBefore, $this->$type))
						$nameString = $this->{$type}[$rewriteCreatorField] . $nameString;
					else
						$nameString .= $this->{$type}[$rewriteCreatorField];
					$firstInList = FALSE;
				}
				else if(array_key_exists($rewriteCreatorFieldEach, $this->$type))
				{
					if(array_key_exists($rewriteCreatorFieldBefore, $this->$type))
						$nameString = $this->{$type}[$rewriteCreatorField] . $nameString;
					else
						$nameString .= $this->{$type}[$rewriteCreatorField];
//print "$nameString<P>";
				}
				else
				{
					if(!$rewriteCreatorBeforeDone && array_key_exists($rewriteCreatorFieldBefore, $this->$type))
					{
						$nameString = $this->{$type}[$rewriteCreatorField] . $nameString;
						$rewriteCreatorBeforeDone = TRUE;
					}
					else if(!$rewriteCreatorBeforeDone && 
						!array_key_exists($rewriteCreatorFieldEach, $this->$type))
						$rewriteCreatorFinal = $this->{$type}[$rewriteCreatorField];
				}
			}
//print "$nameString<P>";
			$cArray[] = $nameString;
			$first = FALSE;
		}
/**
* Keep only some elements in array if we've exceeded $moreThan
*/
		$etAl = FALSE;
		if($style[$limit] && (sizeof($cArray) > $style[$moreThan]))
		{
			array_splice($cArray, $style[$limit]);
			if(isset($style[$italics]))
				$etAl = "[i]" . $style[$abbreviation] . "[/i]";
			else
				$etAl = $style[$abbreviation];
		}
/**
* add delimiters
*/
		if(sizeof($cArray) > 1)
		{
			if(sizeof($cArray) == 2)
				$cArray[0] .= $style[$delimitTwo];
			else
			{
				for($index = 0; $index < (sizeof($cArray) - 2); $index++)
				{
					if(!$index)
						$cArray[$index] .= $style[$delimitFirstBetween];
					else
						$cArray[$index] .= $style[$delimitNextBetween];
				}
				$cArray[sizeof($cArray) - 2] .= $style[$delimitLast];
			}
		}
/**
* If sizeof of $cArray > 1 or $etAl != FALSE, set this $nameType_plural to TRUE
*/
		if((sizeof($cArray) > 1) || $etAl)
		{
//			$pluralKey = $nameType . "_plural";
			$this->$pluralKey = TRUE;
		}
/**
* Finally flatten array
*/
		if($etAl)
			$pString = implode('', $cArray) . $etAl;
		else
			$pString = implode('', $cArray);
		if($rewriteCreatorFinal)
			$pString .= $rewriteCreatorFinal;
/**
* Check for repeating primary creator list in subsequent bibliographic item.
*/
		if($nameType == 'creator1')
		{
			$tempString = $pString;
			if(($style['primaryCreatorRepeat'] == 2) && ($this->previousCreator == $pString))
				$pString = $style['primaryCreatorRepeatString'];
			else if(($style['primaryCreatorRepeat'] == 1) && ($this->previousCreator == $pString))
				$pString = ''; // don't print creator list
			$this->previousCreator = $tempString;
		}
		if($shortFootnote)
			return array($pString, $creatorIds);
// For use with generic footnote templates, we must also place 'creator1' string (if not called 'creator') into the 'creator' slot
		if(($nameType == 'creator1') && ($this->styleMap->{$type}[$nameType] != 'creator'))
			$this->item['creator'] = $pString;
		$this->item[$this->styleMap->{$type}[$nameType]] = $pString;
	}
/**
* Handle initials.
* @see formatNames()
* 
* @author	Mark Grimshaw
* @version	1
* 
* @param	array $creator	Associative array of creator name e.g.
* <pre>
*	array(['surname'] => 'Grimshaw', ['firstname'] => Mark, ['initials'] => 'M N G', ['prefix'] => ))
* </pre>
* Initials must be space-delimited.
*
* @param	int $initialsStyle
* @param	boolean $firstNameInitial
* @return	string Formatted string of initials.
*/
	public function checkInitials(&$creator, $initialsStyle, $firstNameInitial)
	{
/**
* Format firstname
*/
//print_r($creator);
		if($creator['firstname'] && !$firstNameInitial) // Full name
			$firstName = $creator['firstname'];
		else if($creator['firstname']) // Initial only of first name.  'firstname' field may actually have several 'firstnames'
		{
			$fn = explode(" ", $creator['firstname']);
			$firstTime = TRUE;
			foreach($fn as $name)
			{
// May be the first name is a hyphenated name
// We separate each part of the name separated by a -
				$fn2 = explode("-", trim($name));
				if($firstTime)
				{
					if(sizeof($fn2) == 1) // no hyphen
					{
						$firstNameInitialMake = 
							$this->utf8->utf8_strtoupper($this->utf8->utf8_substr($fn2[0], 0, 1));
						$firstTime = FALSE;
						continue;
					}
					$firstNameInitialMake = "";
					foreach($fn2 as $nameparts)
					{
						if($firstNameInitialMake && ($initialsStyle <= 1)) // needs '.'
							$firstNameInitialMake .= ".-";
						else if($firstNameInitialMake)
							$firstNameInitialMake .= "-";
						$firstNameInitialMake .= 
							$this->utf8->utf8_strtoupper($this->utf8->utf8_substr($nameparts, 0, 1));
						$firstTime = FALSE;
					}
				}
				else
				{
					if(sizeof($fn2) == 1) // no hyphen
					{
						$initials[] = $this->utf8->utf8_strtoupper($this->utf8->utf8_substr($fn2[0], 0, 1));
						continue;
					}
					$fn3 = '';
					foreach($fn2 as $nameparts)
					{
						if($fn3)
						{
							$fn3 .= "-";
							$initials[] = $fn3 . $this->utf8->utf8_strtoupper(substr(trim($nameparts), 0, 1));
						}
						else
							$fn3 = $this->utf8->utf8_strtoupper(substr(trim($nameparts), 0, 1));
					}
				}
			}
			if(isset($initials))
			{
				if($creator['initials'])
					$creator['initials'] = join(" " , $initials) . ' ' . $creator['initials'];
				else
					$creator['initials'] = join(" " , $initials);
			}
		}
/**
* Initials are stored as space-delimited characters.
* If no initials, return just the firstname or its initial in the correct format.
*/
		if(!$creator['initials'])
		{
			if(isset($firstName))	// full first name only
				return $firstName;
			if(isset($firstNameInitialMake) && $initialsStyle > 1) // First name initial with no '.'
				return $firstNameInitialMake;
			if(isset($firstNameInitialMake)) // First name initial with  '.'
			{
// replace hyphen with '.-' so we get 'J.-F' for example
				str_replace('-', '.-', $firstNameInitialMake);
				return $firstNameInitialMake . '.';
			}
			return ''; // nothing here
		}
		$initialsArray = explode(' ', $creator['initials']);
		if($initialsStyle <= 1) // needs '.' before any hyphen
		{
			foreach($initialsArray as $key => $i)
				$initialsArray[$key] = str_replace('-', '.-', $i);
		}
/**
* If firstname is initial only, prepend to array
*/
		if(isset($firstNameInitialMake))
			array_unshift($initialsArray, $firstNameInitialMake);
		if($initialsStyle == 0) // 'T. U. '
			$initials = implode('. ', $initialsArray) . '.';
		else if($initialsStyle == 1) // 'T.U.'
			$initials = implode('.', $initialsArray) . '.';
		else if($initialsStyle == 2) // 'T U '
			$initials = implode(' ', $initialsArray);
		else // 'TU '
			$initials = implode('', $initialsArray);
/**
* If we have a full first name, prepend it to $initials.
*/
		if(isset($firstName))
			return ($firstName . ' ' . $initials);
		return $initials;
	}
/**
* Add an item to $this->item array
*
* @author	Mark Grimshaw
* @version	1
*
* @param	string $item		The item to be added.
* @param	string $fieldName	The database fieldName of the item to be added
*/
	public function addItem($item, $fieldName)
	{
		$type = $this->type;
		if($item === FALSE)
			return;
/**
* This item may already exist (e.g. edition field for WIKINDX)
*/
		if(isset($this->item) && array_key_exists($this->styleMap->{$type}[$fieldName], $this->item))
			return FALSE;
		$this->item[$this->styleMap->{$type}[$fieldName]] = $item;
	}
/**
* Add all remaining items to $this->item array
*
* @author	Mark Grimshaw
* @version	1
*
* @param	array $row		The items to be added.
*/
	public function addAllOtherItems($row)
	{
		$type = $this->type;
		foreach($row as $field => $value)
		{
			if(array_key_exists($field, $this->styleMap->$type) && 
				!array_key_exists($this->styleMap->{$type}[$field], $this->item))
			{
				$item = $row[$field];
				$this->addItem($item, $field);
			}
		}
	}
/**
* Format a title.  Anything enclosed in $delimitLeft...$delimitRight is to be left unchanged
*
* @author	Mark Grimshaw
* @version	1
*
* @param	string $pString	Raw title string.
* @param	string $delimitLeft Default is FALSE.
* @param	string $delimitRight Default is FALSE.
* @return	void
*/
	public function formatTitle($pString, $delimitLeft = FALSE, $delimitRight = FALSE)
	{
		$type = $this->type;
		if(($key = array_search('title', $this->styleMap->$type)) === FALSE)
		{
			$this->item[$this->styleMap->{$type}[$key]] = '';
			return;
		}
		if(array_key_exists($this->styleMap->{$type}[$key], $this->item)) // i.e. we're now dealing with subtitle
			$this->item[$this->styleMap->{$type}[$key]] .= 
				$this->titleCapitalization($pString, $delimitLeft, $delimitRight);
		else
			$this->item[$this->styleMap->{$type}[$key]] = 
				$this->titleCapitalization($pString, $delimitLeft, $delimitRight);
	}
/**
* Format a shortTitle.  Anything enclosed in $delimitLeft...$delimitRight is to be left unchanged
*
* @author	Mark Grimshaw
* @version	1
*
* @param	string $pString	Raw title string.
* @param	string $delimitLeft Default is FALSE.
* @param	string $delimitRight Default is FALSE.
* @return	void
*/
	public function formatShortTitle($pString, $delimitLeft = FALSE, $delimitRight = FALSE)
	{
		$type = $this->type;
		if(($key = array_search('shortTitle', $this->styleMap->$type)) === FALSE)
			return;
		$this->item[$this->styleMap->{$type}[$key]] = 
			$this->titleCapitalization($pString, $delimitLeft, $delimitRight);
	}
/**
* Format a title of the original work from which a translation has been made.  Anything enclosed in $delimitLeft...$delimitRight is to be left unchanged
*
* @author	Mark Grimshaw
* @version	1
*
* @param	string $pString	Raw title string.
* @param	string $delimitLeft Default is FALSE.
* @param	string $delimitRight Default is FALSE.
* @return	void
*/
	public function formatTransTitle($pString, $delimitLeft = FALSE, $delimitRight = FALSE)
	{
		$type = $this->type;
		if(($key = array_search('transTitle', $this->styleMap->$type) ) === FALSE)
			return;
		if(array_key_exists($this->styleMap->{$type}[$key], $this->item)) // i.e. we're now dealing with subtitle
			$this->item[$this->styleMap->{$type}[$key]] .= 
				$this->titleCapitalization($pString, $delimitLeft, $delimitRight);
		else
			$this->item[$this->styleMap->{$type}[$key]] = 
				$this->titleCapitalization($pString, $delimitLeft, $delimitRight);
	}
/**
* Do title capitalization
* 
* @author	Mark Grimshaw
* @version	1
*
* @param	string $pString	Raw title string.
* @param	string $delimitLeft
* @param	string $delimitRight
* @return	string Formatted title string.
*/
	private function titleCapitalization($pString, $delimitLeft, $delimitRight)
	{
/**
* '0' == 'Osbib Bibliographic Formatting'
* '1' == 'Osbib bibliographic formatting'
*/
		$oldString = ltrim($pString);
		$oldDelimitLeft = $delimitLeft;
		if($this->style['titleCapitalization'])
		{
			if(!$delimitLeft)
				$delimitLeft = '{';
			if(!$delimitRight)
				$delimitRight = '}';
			$delimitLeft = preg_quote($delimitLeft);
			$delimitRight = preg_quote($delimitRight);
			$match = "/" . $delimitLeft . "/";
// Something here (preg_split probably) interferes with UTF-8 encoding (data is stored in 
// the database as UTF-8 as long as web browser charset == UTF-8).  
// So first decode then encode back to UTF-8 at end.
// There is a 'u' UTF-8 parameter for preg_xxx but it doesn't work.
			$pString = $this->utf8->decodeUtf8($pString);
			$newString = '';
			while(preg_match($match, $pString))
			{
				$array = preg_split("/(.*)$delimitLeft(.*)$delimitRight(.*)/U", 
					$pString, 2, PREG_SPLIT_DELIM_CAPTURE);
/**
* in case user has input {..} incorrectly
*/
				if(sizeof($array) == 1)
					break;
				$newString .= $this->utf8->utf8_strtolower($this->utf8->encodeUtf8($array[1])) . $array[2];
				$pString = $array[4];
			}
			$newString .= $this->utf8->utf8_strtolower($this->utf8->encodeUtf8($pString));
		}
		$pString = isset($newString) ? $newString : $pString;
		if(strlen($oldString)>0) // perhaps requires first character maintained as is
			if($oldString{0} == $oldDelimitLeft) 
				return ltrim($pString);
		return $this->utf8->utf8_ucfirst(ltrim($pString));
	}
/**
* Format pages.
*
* $this->style['pageFormat']:
* 0 == 132-9
* 1 == 132-39
* 2 == 132-139
* 
* @author	Mark Grimshaw
* @version	1
*
* @param	string $start		Page start.
* @param	string $end		Page end. Default is FALSE.
* @param	mixed $citation	If called from CITEFORMAT, this is the array of citation stylings. Default is FALSE.
* @return	string Pages.
*/
	public function formatPages($start, $end = FALSE, $citation = FALSE)
	{
		$type = $this->type;
		$style = $citation ? $citation : $this->style;
/**
* Set default plural behaviour for pages
*/
		$this->pages_plural = FALSE;
/**
* If no page end, return just $start;
*/
		if(!$end)
		{
			$this->item[$this->styleMap->{$type}['pages']] = $start;
			return;
		}
/**
* Pages may be in roman numeral format etc.  Return unchanged
*/
		if(!is_numeric($start))
		{
			$this->item[$this->styleMap->{$type}['pages']] = $start . 'WIKINDX_NDASH' . $end;
			return;
		}
/**
* We have multiple pages...
*/
		$this->pages_plural = TRUE;
/**
* They've done something wrong so give them back exactly what they entered
*/
		if(($end <= $start) || (strlen($end) < strlen($start)))
		{
			$this->item[$this->styleMap->{$type}['pages']] = $start . 'WIKINDX_NDASH' . $end;
			return;
		}
		else if($style['pageFormat'] == 2)
		{
			$this->item[$this->styleMap->{$type}['pages']] = $start . 'WIKINDX_NDASH' . $end;
			return;
		}
		else
		{
/**
* We assume page numbers are not into the 10,000 range - if so, return the complete pages
*/
			if(strlen($start) <= 4)
			{
				$startArray = preg_split('//', $start);
				array_shift($startArray); // always an empty element at start?
				array_pop($startArray); // always an empty array element at end?
				if($style['pageFormat'] == 0)
				{
					array_pop($startArray);
					$endPage = substr($end, -1);
					$index = -2;
				}
				else
				{
					array_pop($startArray);
					array_pop($startArray);
					$endPage = substr($end, -2);
					$index = -3;
				}
				while(!empty($startArray))
				{
					$startPop = array_pop($startArray);
					$endSub = substr($end, $index--, 1);
					if($endSub == $startPop)
					{
						$this->item[$this->styleMap->{$type}['pages']] 
							= $start . '-' . $endPage;
						return;
					}
					if($endSub > $startPop)
						$endPage = $endSub . $endPage;
				}
			}
			else
			{
				$this->item[$this->styleMap->{$type}['pages']] = $start . 'WIKINDX_NDASH' . $end;
				return;
			}
		}
/**
* We should never reach here - in case we do, give back complete range so that something at least is printed
*/
		$this->item[$this->styleMap->{$type}['pages']] = $start . 'WIKINDX_NDASH' . $end;
	}
/**
* Format runningTime for film/broadcast
*
* @author	Mark Grimshaw
* @version	1
*
* @param	int $minutes
* @param	int $hours
*/
	public function formatRunningTime($minutes, $hours)
	{
		$type = $this->type;
		if($this->style['runningTimeFormat'] == 0) // 1'45"
		{
			if(isset($minutes) && $minutes)
			{
				if($minutes < 10)
					$minutes = '0' . $minutes;
				$runningTime = $hours . "'" . $minutes . "\"";
			}
			else
				$runningTime = $hours . "'00\"";
		}
		else if($this->style['runningTimeFormat'] == 1) // 1:45
		{
			if(isset($minutes) && $minutes)
			{
				if($minutes < 10)
					$minutes = '0' . $minutes;
				$runningTime = $hours . ":" . $minutes;
			}
			else
				$runningTime = $hours . ":00";
		}
		else if($this->style['runningTimeFormat'] == 2) // 1,45
		{
			if(isset($minutes) && $minutes)
			{
				if($minutes < 10)
					$minutes = '0' . $minutes;
				$runningTime = $hours . "," . $minutes;
			}
			else
				$runningTime = $hours . ",00";
		}
		else if($this->style['runningTimeFormat'] == 3) // 1 hours, 45 minutes
		{
			$hours = ($hours == 1) ? $hours . " hour" : $hours . " hours";
			if(isset($minutes) && $minutes)
			{
				$minutes = ($minutes == 1) ? $minutes . " minute" : $minutes . " minutes";
				$runningTime = $hours . ", " . $minutes;
			}
			else
				$runningTime = $hours;
		}
		else if($this->style['runningTimeFormat'] == 4) // 1 hours and 45 minutes
		{
			$hours = ($hours == 1) ? $hours . " hour" : $hours . " hours";
			if(isset($minutes) && $minutes)
			{
				$minutes = ($minutes == 1) ? $minutes . " minute" : $minutes . " minutes";
				$runningTime = $hours . " and " . $minutes;
			}
			else
				$runningTime = $hours;
		}
		else if($this->style['runningTimeFormat'] == 5) // 105 minutes
		{
			$time = 0;
			if(isset($hours))
				$time = $hours * 60;
			if(isset($minutes) && $minutes)
				$time += $minutes;
			$runningTime = ($time == 1) ? $time . " minute" : $time . " minutes";
		}
		else if($this->style['runningTimeFormat'] == 6) // 105 mins
		{
			$time = 0;
			if(isset($hours))
				$time = $hours * 60;
			if(isset($minutes) && $minutes)
				$time += $minutes;
			$runningTime = ($time == 1) ? $time . " min" : $time . " mins";
		}
		$this->item[$this->styleMap->{$type}['runningTime']] = $runningTime;
	}
/**
* Format date
*
* @author	Mark Grimshaw
* @version	2
*
* @param	int $startDay
* @param	int $startMonth
* @param	int $endDay
* @param	int $endMonth
* @param	boolean $webArticle Default is FALSE.
*
* for 'web_article', $startDay and $startMonth represent the publication date
*/
	public function formatDate($startDay, $startMonth, $endDay, $endMonth, $webArticle = FALSE)
	{
		$type = $this->type;
		$oldStartDay = $startDay;
		$oldEndDay = $endDay;
		if($this->dateMonthNoDay && !$startDay && !$endDay)
			$this->{$type}[$this->styleMap->{$type}['date']] = 
			$this->dateArray[$this->styleMap->{$type}['date']];
		if($startDay !== FALSE)
		{
			if($this->style['dayFormat'] == 1) // e.g. 10.
				$startDay .= '.';
			else if($this->style['dayFormat'] == 2) // e.g. 10th
				$startDay = $this->cardinalToOrdinal($startDay, 'dayMonth');
			if(array_key_exists('dayLeadingZero', $this->style) && $oldStartDay < 10)
				$startDay = '0' . $startDay;
		}
		if($endDay !== FALSE)
		{
			if($this->style['dayFormat'] == 1) // e.g. 10.
				$endDay .= '.';
			else if($this->style['dayFormat'] == 2) // e.g. 10th
				$endDay = $this->cardinalToOrdinal($endDay, 'dayMonth');
			if(array_key_exists('dayLeadingZero', $this->style) && $oldEndDay < 10)
				$endDay = '0' . $endDay;
		}
		if($this->style['monthFormat'] == 1) // Full month name
			$monthArray = $this->longMonth;
		else if($this->style['monthFormat'] == 2) // User-defined
		{
			for($i = 1; $i <= 12; $i++)
				$monthArray[$i] = $this->style["userMonth_$i"];
		}
		else // Short month name
			$monthArray = $this->shortMonth;
		if($startMonth !== FALSE)
			$startMonth = $monthArray[$startMonth];
		if($endMonth !== FALSE)
			$endMonth = $monthArray[$endMonth];
		if(!$endMonth)
		{
			if($this->style['dateFormat']) // Order == Month Day
			{
				$startDay = ($startDay === FALSE) ? '' : ' ' . $startDay;
				$date = $startMonth . $startDay;
			}
			else // Order == Day Month
			{
				$startDay = ($startDay === FALSE) ? '' : $startDay . ' ';
				$date = $startDay . $startMonth;
			}
		}
		else // date range
		{
			if(!$startDay)
				$delimit = $this->style['dateRangeDelimit2'];
			else
				$delimit = $this->style['dateRangeDelimit1'];
			if(($endMonth !== FALSE) && ($startMonth == $endMonth) && ($this->style['dateRangeSameMonth'] == 1))
			{
				$endMonth = FALSE;
				if(!$endDay)
					$delimit = FALSE;
			}
			if($this->style['dateFormat']) // Order == Month Day
			{
				$startDay = ($startDay === FALSE) ? '' : ' ' . $startDay;
				$startDate = $startMonth . $startDay;
				if($endMonth)
					$endDate = $endMonth . $endDay = ($endDay === FALSE) ? '' : ' ' . $endDay;
				else
					$endDate = $endDay;
			}
			else // Order == Day Month
			{
				if($endMonth)
				{
					$startDate = $startDay . ' ' . $startMonth;
					$endDate = $endDay = ($endDay === FALSE) ? '' : $endDay . ' ';
					$endDate .= $endMonth;
				}
				else
				{
					$startDate = $startDay;
					$endDate = ($endDay === FALSE) ? ' ' : $endDay . ' ';
					$endDate .= $startMonth;
				}
			}
			$date = $startDate . $delimit . $endDate;
		}
		if($webArticle)
		{
			if(array_key_exists('publicationDate', $this->styleMap->{$type}))
				$this->item[$this->styleMap->{$type}['publicationDate']] = $date;
		}
		else
			$this->item[$this->styleMap->{$type}['date']] = $date;
	}
/**
* Format edition
*
* @author	Mark Grimshaw
* @version	1
*
* @param	int $edition
*/
	public function formatEdition($edition)
	{
		$type = $this->type;
		if(!is_numeric($edition))
			$edition = $edition;
		else if($this->style['editionFormat'] == 1) // 10.
			$edition .= '.';
		else if($this->style['editionFormat'] == 2) // 10th
			$edition = $this->cardinalToOrdinal($edition, 'edition');
		$this->item[$this->styleMap->{$type}[array_search('edition', $this->styleMap->$type)]] = $edition;
	}
/**
* Create ordinal number from cardinal
*
* @author	Mark Grimshaw
* @version	1
*
* @param	string $cardinal Input number.
* @param	string $field Default is FALSE.
* @return	string
*/
	private function cardinalToOrdinal($cardinal, $field = FALSE)
	{
// WIKINDX-specific
		if($this->wikindx && method_exists($this->wikindxLanguageClass, "cardinalToOrdinal"))
			return $this->wikindxLanguageClass->cardinalToOrdinal($cardinal, $field);
		$modulo = $cardinal % 100;
		if(($modulo == 11) || ($modulo == 12) || ($modulo == 13))
			return $cardinal . 'th';
		$modulo = $cardinal % 10;
		if(($modulo >= 4) || !$modulo)
			return $cardinal . 'th';
		if($modulo == 1)
			return $cardinal . 'st';
		if($modulo == 2)
			return $cardinal . 'nd';
		if($modulo == 3)
			return $cardinal . 'rd';
	}
/**
* Set-up localisations etc.
*
* @author	Mark Grimshaw
* @version	1
*/
	private function loadArrays()
	{
// WIKINDX-specific
		if($this->wikindx)
		{
			$languageDir = $this->style['localisation'];
			if(!is_dir("languages/$languageDir"))
				$languageDir = "en";
			include_once("languages/$languageDir/CONSTANTS.php");
			$class = "CONSTANTS_" . $languageDir;
			$this->wikindxLanguageClass = new $class();
			if(method_exists($this->wikindxLanguageClass, "monthToLongName")
				&& method_exists($this->wikindxLanguageClass, "monthToShortName"))
			{
				$this->longMonth = $this->wikindxLanguageClass->monthToLongName();
				$this->shortMonth = $this->wikindxLanguageClass->monthToShortName();
				return;
			}
		}
// Defaults
		$this->longMonth = array(
				1	=>	'January',
				2	=>	'February',
				3	=>	'March',
				4	=>	'April',
				5	=>	'May',
				6	=>	'June',
				7	=>	'July',
				8	=>	'August',
				9	=>	'September',
				10	=>	'October',
				11	=>	'November',
				12	=>	'December',
			);
		$this->shortMonth = array(
				1	=>	'Jan',
				2	=>	'Feb',
				3	=>	'Mar',
				4	=>	'Apr',
				5	=>	'May',
				6	=>	'Jun',
				7	=>	'Jul',
				8	=>	'Aug',
				9	=>	'Sep',
				10	=>	'Oct',
				11	=>	'Nov',
				12	=>	'Dec',
			);
		$this->titleSubtitleSeparator = ": ";
	}
}
?>