<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/** 
* Common methods for collections
*
*	A collection is a journal, edited book, CD etc. - something which has 
*	multiple resources within it.
* 
* @version	1
*
*	@package wikindx4\core\collection
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class COLLECTION
{
/** object */
private $db;
/** object */
private $html;
/** object */
private $commonBib;
/** object */
private $config;
/** object */
private $stmt;
/** object */
private $messages;

/**
* COLLECTION
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->stmt = FACTORY_SQLSTATEMENTS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
	}
/**
* Get collection from WKX_collection.  Return associative array of id => collection
*
* @param string $type Default is FALSE
* @param boolean $userBib Default is FALSE
* @param mixed $typeArray Default is FALSE
* @param boolean $metadata Default is FALSE
* @return mixed
*/
	public function grabAll($type = FALSE, $userBib = FALSE, $typeArray = FALSE, $metadata = FALSE)
	{
		if(!$userBib && !$type && !is_array($typeArray))
		{
			if($metadata && is_array($collections = $this->db->readCache('cacheMetadataCollections')))
				return $collections;
			else if(!$metadata && is_array($collections = $this->db->readCache('cacheResourceCollections')))
				return $collections;
		}
		unset($collections);
		if($type)
			$this->db->formatConditions(array('collectionType' => $type));
		$thisType = $type;
		if(is_array($typeArray) && !empty($typeArray))
		{
			$this->db->leftJoin('resource_misc', 'resourcemiscId', 'resourceId');
			if($userBib)
				$this->commonBib->userBibCondition('resourcemiscId');
			foreach($typeArray as $type)
				$conditions[] = $type;
			$this->db->formatConditionsOneField($conditions, 'resourceType');
			$this->db->formatConditions(array('resourcemiscCollection' => ' IS NOT NULL'));
			$subQuery = $this->db->subQuery($this->db->selectNoExecute('resource', 'resourcemiscCollection', TRUE), 't', TRUE, TRUE);
			$this->db->formatConditions(array('collectionId' => ' IS NOT NULL'));
			$this->db->leftJoin('collection', 'collectionId', 'resourcemiscCollection');
			$userBib = FALSE;
		}
		if($metadata)
		{
			$ids = $conditionIds = array();
// quotes
			$this->db->leftJoin('resource_misc', 'resourcequoteResourceId', 'resourcemiscId');
			$this->db->formatConditions(array('resourcemiscCollection' => ' IS NOT NULL'));
			$recordset = $this->db->select('resource_quote', 'resourcemiscCollection', TRUE);
			while($row = $this->db->fetchRow($recordset))
			{
				foreach(explode(',', $row['resourcemiscCollection']) as $id)
				{
					if(array_search($id, $ids) === FALSE)
					{
						$ids[] = $id;
						$conditionIds[] = $this->db->formatFields('collectionId') . $this->db->equal . $this->db->tidyInput($id);
					}
				}
			}
// paraphrases
			$this->db->leftJoin('resource_misc', 'resourceparaphraseResourceId', 'resourcemiscId');
			$this->db->formatConditions(array('resourcemiscCollection' => ' IS NOT NULL'));
			$recordset = $this->db->select('resource_paraphrase', 'resourcemiscCollection', TRUE);
			while($row = $this->db->fetchRow($recordset))
			{
				foreach(explode(',', $row['resourcemiscCollection']) as $id)
				{
					if(array_search($id, $ids) === FALSE)
					{
						$ids[] = $id;
						$conditionIds[] = $this->db->formatFields('collectionId') . $this->db->equal . $this->db->tidyInput($id);
					}
				}
			}
// musing
			$this->db->leftJoin('resource_misc', 'resourcemusingResourceId', 'resourcemiscId');
			$this->db->formatConditions(array('resourcemiscCollection' => ' IS NOT NULL'));
			$recordset = $this->db->select('resource_musing', 'resourcemiscCollection', TRUE);
			while($row = $this->db->fetchRow($recordset))
			{
				foreach(explode(',', $row['resourcemiscCollection']) as $id)
				{
					if(array_search($id, $ids) === FALSE)
					{
						$ids[] = $id;
						$conditionIds[] = $this->db->formatFields('collectionId') . $this->db->equal . $this->db->tidyInput($id);
					}
				}
			}
			if(!empty($conditionIds))
				$this->db->formatConditions(join($this->db->or, $conditionIds));
			else
				return FALSE;
		}
		if($userBib)
		{
			$this->db->leftJoin('resource_misc', 'resourcemiscCollection', 'collectionId');
			$this->commonBib->userBibCondition('resourcemiscId');
		}
		$this->db->orderBy('collectionTitle');
		if(isset($subQuery))
			$recordset = $this->db->query($this->db->selectNoExecuteFromSubQuery(FALSE, 
				array('collectionId', 'collectionType', 'collectionTitle', 'collectionTitleShort'), $subQuery));
		else
		$recordset = $this->db->select('collection', 
			array('collectionId', 'collectionType', 'collectionTitle', 'collectionTitleShort'), TRUE);
		while($row = $this->db->fetchRow($recordset))
		{
			$short = $row['collectionTitleShort'] ? 
				" [" . $row['collectionTitleShort'] . ']' : FALSE;
			$title = $row['collectionTitle'] . $short;
			if(!$type)
				$thisType = $row['collectionType'];
			$collections[$row['collectionId']] = $this->html->dbToFormTidy($title, TRUE);
		}
		if(isset($collections))
		{
// (re)create cache
			if(!$userBib && !$type && !is_array($typeArray))
			{
				if($metadata)
					$this->db->writeCache('cacheMetadataCollections', $collections);
				else
					$this->db->writeCache('cacheResourceCollections', $collections);
			}
			return $collections;
		}
		return FALSE;
	}
/**
* Return array of menu items to browse collections from under the Resource|Browse menu.
*
* @return array Array is empty if no collections
*/
	public function browseMenuArray()
	{
		$this->db->groupBy('collectionType');
		$recordset = $this->db->select('collection', 'collectionType');
		if(!$this->db->numRows($recordset))
			return array();
// Add 'ALL' to array
		$array[$this->messages->text("menu", "browseSubCollection")] = FALSE;
		$array[$this->messages->text("collection", 'all')] = 'index.php?action=browse_BROWSECOLLECTION_CORE&method=display&CollectionType=0';
		while($row = $this->db->fetchRow($recordset))
		{
			if(!$row['collectionType'])
				continue;
			$array[$this->messages->text("collection", $row['collectionType'])] = 
				'index.php?action=browse_BROWSECOLLECTION_CORE&method=display&CollectionType=' . $row['collectionType'];
		}
		return $array;
	}
/**
* Return array of menu items to edit collections from under the Resource|BEdit menu.
*
* @return array Array is empty if no collections
*/
	public function editMenuArray()
	{
		$this->db->groupBy('collectionType');
		$recordset = $this->db->select('collection', 'collectionType');
		if(!$this->db->numRows($recordset))
			return array();
// Add 'ALL' to array
		$array[$this->messages->text("menu", "editSubCollection")] = FALSE;
		$array[$this->messages->text("collection", 'all')] = 
			'index.php?action=edit_EDITCOLLECTION_CORE&method=editChooseCollection&edit_CollectionType=0';
		while($row = $this->db->fetchRow($recordset))
		{
			if(!$row['collectionType'])
				continue;
			$array[$this->messages->text("collection", $row['collectionType'])] = 
				'index.php?action=edit_EDITCOLLECTION_CORE&method=editChooseCollection&edit_CollectionType=' . 
				$row['collectionType'];
		}
		return $array;
	}
/**
* check to see if collection title already exists
* @param int $id Collection ID. Default is FALSE
* @param string $title
* @param string $titleShort
* @param string $type
* @return mixed FALSE or ID
*/
	public function checkExists($id = FALSE, $title, $titleShort, $type)
	{
		if($id && !$type)
		{
			$this->db->formatConditions(array('collectionId' => $id));
			$recordset = $this->db->select('collection', 'collectionType');
			$type = $this->db->fetchOne($recordset);
		}
		$collection = str_replace(' ', '', $title . $titleShort . $type);
		$array = array($this->db->formatFields('collectionTitle'), $this->db->formatFields('collectionTitleShort'), 
			$this->db->formatFields('collectionType'));
// Concatenate (CONCAT_WS) fields and remove any spaces
		$this->db->formatConditions($this->db->replace($this->db->concat($array, ''), 
			' ', '', FALSE) . $this->db->equal . $this->db->tidyInput($collection));
		if($id)
			$this->db->formatConditions(array('collectionId' => $id));
		$resultset = $this->db->select('collection', 'collectionId');
		while($row = $this->db->fetchRow($resultset))
		{
			if($id && ($row['collectionId'] == $id)) // the collection being edited
				continue;
			return $row['collectionId'];
		}
		return FALSE;
	}
/**
* Remove resource-less collections from collection list
*/
	public function removeHanging()
	{
		$deleteIds = array();
		$this->db->formatConditions(array('resourcemiscCollection' => ' IS NOT NULL'));
		$subStmt = $this->db->subQuery($this->db->selectNoExecute('resource_misc', 'resourcemiscCollection'), FALSE, FALSE, TRUE);
		$this->db->formatConditions($this->db->formatFields('collectionId') . $this->db->inClause($subStmt, TRUE));
		$recordset = $this->db->select('collection', 'collectionId');
		while($row = $this->db->fetchRow($recordset))
			$deleteIds[] = $row['collectionId'];
		if(empty($deleteIds))
			return; // nothing to do
		if(!empty($deleteIds))
		{
			$this->db->formatConditionsOneField($deleteIds, 'collectionId');
			$this->db->delete("collection");
// remove cache files for collections
			$this->db->deleteCache('cacheResourceCollections');
			$this->db->deleteCache('cacheMetadataCollections');
			$this->db->deleteCache('cacheResourceCollectionTitles');
			$this->db->deleteCache('cacheResourceCollectionShorts');
		}
		$this->db->optimize('collection');
	}
}
?>
