<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* Common functions for creators
* 
* @version	1
*
*	@package wikindx4\core\creator
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class CREATOR
{
/** object */
private $db;
/** object */
private $html;
/** array */
private $names;
/** object */
private $commonBib;
/** object */
private $gatekeep; //!< there are some functions requiring write access
/** object */
private $errors;
/** object */
private $config;
/** object */
private $stmt;

/**
*	CREATOR class.
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->names = array("resourcecreatorCreator1", "resourcecreatorCreator2", 
			"resourcecreatorCreator3", "resourcecreatorCreator4", "resourcecreatorCreator5");
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->stmt = FACTORY_SQLSTATEMENTS::getInstance();
	}
/**
* Get creators from creators table.  
*
* @param int $userBib ID of user bibliography to use. Default is FALSE
* @param mixed $typeArray array of resource types within which to select creators. Default is FALSE
* @param mixed $metadata metadata type or metadata array ('quote', 'paraphrase' etc) -- gather only creators for resources which have metadata Default is FALSE
* @param boolean $group group database creators together into the same person if TRUE. Default is FALSE
* @return array of id => creator
*/
	public function grabAll($userBib = FALSE, $typeArray = FALSE, $metadata = FALSE, $group = FALSE)
	{
		if(!$userBib && !is_array($typeArray) && !$group)
		{
			if(!is_array($metadata) && $metadata && is_array($creators = $this->db->readCache('cacheMetadataCreators')))
				return $creators;
			else if(!is_array($metadata) && !$metadata && is_array($creators = $this->db->readCache('cacheResourceCreators')))
				return $creators;
		}
		unset($creators);
		$recordset = FALSE;
		if(is_array($typeArray) && !empty($typeArray))
		{
//			if($metadata)
//			{
//				$unions[] = $this->db->selectNoExecute('resource_quote', array(array('resourcequoteResourceId' => 'rId')), TRUE);
//				$unions[] = $this->db->selectNoExecute('resource_paraphrase', array(array('resourceparaphraseResourceId' => 'rId')), TRUE);
//				$unions[] = $this->db->selectNoExecute('resource_musing', array(array('resourcemusingResourceId' => 'rId')), TRUE);
//				$union = $this->db->union($unions);
//				$subSubQuery = $this->db->subQuery($union, 'u', TRUE, TRUE);
//				$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'rId');
//			}
			if(isset($subSubQuery))
			{
				$this->db->formatConditionsOneField($typeArray, 'resourceType');
				$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'rId');
				$this->db->leftJoin('resource', 'resourceId', 'rId');
				$this->db->formatConditions($this->db->formatFields('creatorId') . 
					$this->db->inClause($this->db->subQueryFields('resourcecreatorCreatorId', $subSubQuery, 't', TRUE, TRUE)));
			}
			else
			{
				$this->db->formatConditionsOneField($typeArray, 'resourceType');
				$this->db->formatConditions(array('creatorSameAs' => ' IS NULL'));
				$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourceId');
				$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
				if($userBib)
					$this->commonBib->userBibCondition('resourcecreatorResourceId');
				$u[] = $this->db->selectNoExecute('resource', 'resourcecreatorCreatorId', TRUE, TRUE, TRUE);
				$this->db->formatConditionsOneField($typeArray, 'resourceType');
				$this->db->formatConditions(array('creatorSameAs' => ' IS NOT NULL'));
				$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourceId');
				$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
				if($userBib)
					$this->commonBib->userBibCondition('resourcecreatorResourceId');
				$u[] = $this->db->selectNoExecute('resource', 'creatorSameAs', TRUE, TRUE, TRUE);
				$subQuery = $this->db->subQuery($this->db->union($u), 't');
				$this->db->formatConditions(array('creatorId' => ' IS NOT NULL'));
				$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
				$this->db->orderBy('creatorSurname');
				$recordset = $this->db->query($this->db->selectNoExecuteFromSubQuery(FALSE, 
					array('creatorId', "creatorSurname", "creatorInitials", "creatorFirstname", "creatorPrefix"), $subQuery, 
					FALSE, TRUE, TRUE));
			}
			$group = $userBib = FALSE;
		}
		else if(!is_array($metadata) && $metadata)
		{
			$unions[] = $this->db->selectNoExecute('resource_quote', array(array('resourcequoteResourceId' => 'rId')), TRUE);
			$unions[] = $this->db->selectNoExecute('resource_paraphrase', array(array('resourceparaphraseResourceId' => 'rId')), TRUE);
			$unions[] = $this->db->selectNoExecute('resource_musing', array(array('resourcemusingResourceId' => 'rId')), TRUE);
			$union = $this->db->union($unions);
			$subSubQuery = $this->db->subQuery($union, 'u', TRUE, TRUE);
			$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'rId');
			if($userBib)
				$this->commonBib->userBibCondition('resourcecreatorResourceId');
			$subQuery = $this->db->subQueryFields('resourcecreatorCreatorId', $subSubQuery, 't', TRUE, TRUE);
			$this->db->formatConditions($this->db->formatFields('creatorId') . $this->db->equal . 
				$this->db->formatFields('resourcecreatorCreatorId'));
			$this->db->orderBy('creatorSurname');
			$recordset = $this->db->query($this->db->selectNoExecuteFromSubQuery('creator', 
				array('creatorId', "creatorSurname", "creatorInitials", "creatorFirstname", "creatorPrefix"), $subQuery));
		}
		else if(is_array($metadata) && !empty($metadata))
		{
			foreach($metadata as $mType)
			{
				if(($mType == 'quote') || ($mType == 'quoteComment'))
					$unions[] = $this->db->selectNoExecute('resource_quote', array(array('resourcequoteResourceId' => 'rId')), TRUE);
				else if(($mType == 'paraphrase') || ($mType == 'paraphraseComment'))
					$unions[] = $this->db->selectNoExecute('resource_paraphrase', array(array('resourceparaphraseResourceId' => 'rId')), TRUE);
				else if($mType == 'musing')
					$unions[] = $this->db->selectNoExecute('resource_musing', array(array('resourcemusingResourceId' => 'rId')), TRUE);
			}
			$union = $this->db->union($unions);
			$subSubQuery = $this->db->subQuery($union, 'u', TRUE, TRUE);
			$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'rId');
			if($userBib)
				$this->commonBib->userBibCondition('resourcecreatorResourceId');
			$subQuery = $this->db->subQueryFields('resourcecreatorCreatorId', $subSubQuery, 't', TRUE, TRUE);
			$this->db->formatConditions($this->db->formatFields('creatorId') . $this->db->equal . 
				$this->db->formatFields('resourcecreatorCreatorId'));
			$this->db->orderBy('creatorSurname');
			$recordset = $this->db->query($this->db->selectNoExecuteFromSubQuery('creator', 
				array('creatorId', "creatorSurname", "creatorInitials", "creatorFirstname", "creatorPrefix"), $subQuery));
		}
		if(!$recordset)
		{
//$this->config = FACTORY_CONFIG::getInstance();
//$this->config->WIKINDX_PRINT_SQL = TRUE;
			if($group)
				$this->db->formatConditions(array('creatorSameAs' => ' IS NULL'));
			if($userBib)
			{
				$this->db->leftJoin('resource_creator', 'resourcecreatorCreatorId', 'creatorId');
				$this->commonBib->userBibCondition('resourcecreatorResourceId');
			}
			$this->db->orderBy('creatorSurname');
			if(isset($subQuery))
				$recordset = $this->db->query($this->db->selectNoExecuteFromSubQuery(FALSE, 
					array('creatorId', "creatorSurname", "creatorInitials", "creatorFirstname", "creatorPrefix"), $subQuery));
			else
				$recordset = $this->db->select('creator', 
					array('creatorId', "creatorSurname", "creatorInitials", "creatorFirstname", "creatorPrefix"));
//$this->config->WIKINDX_PRINT_SQL = FALSE;
		}
		$creators = array();
		while($row = $this->db->fetchRow($recordset))
			$this->formatNames($row, $creators);
		if(!empty($creators))
		{
// (re)create cache if getting entire list
			if(!$userBib && !is_array($typeArray) && !$group)
			{
				if($metadata)
					$this->db->writeCache('cacheMetadataCreators', $creators);
				else
					$this->db->writeCache('cacheResourceCreators', $creators);
			}
			return $creators;
		}
		return FALSE;
	}
/**
* Gather creators who are group masters
*
* @return mixed FALSE|array of creators
*/
	public function grabGroupMasters()
	{
		$this->db->formatConditions(array('creatorSameAs' => ' IS NOT NULL'));
		$subStmt = $this->db->subQuery($this->db->selectNoExecute('creator', 'creatorSameAs'), FALSE, FALSE, TRUE);
		$this->db->formatConditions($this->db->formatFields('creatorId') . $this->db->inClause($subStmt));
		$this->db->orderBy('creatorSurname');
		$recordset = $this->db->select('creator', 
			array('creatorId', "creatorSurname", "creatorInitials", "creatorFirstname", "creatorPrefix"));
		$creators = array();
		while($row = $this->db->fetchRow($recordset))
			$this->formatNames($row, $creators);
		if(!empty($creators))
			return $creators;
		return FALSE;
	}
/**
* Gather creators who are not group masters
*
* @param boolean $notExists
*
* @return mixed FALSE|array of creators
*/
	public function grabGroupAvailableMembers($notExists = FALSE)
	{
		$this->db->formatConditions($this->db->formatFields('creatorId') . $this->db->equal . $this->db->formatFields('t1.creatorId'));
		$this->db->formatConditions(array('creatorSameAs' => ' IS NOT NULL'));
//		$subStmt = $this->db->subQuery($this->db->selectNoExecute('creator', 'creatorSameAs'), FALSE, FALSE, TRUE);
//		$this->db->formatConditions($this->db->formatFields('creatorId') . $this->db->inClause($subStmt, TRUE));
		$subStmt = $this->db->selectNoExecute('creator', $this->db->tidyInput('1'), FALSE, FALSE, TRUE, 't1');
		if($notExists)
			$this->db->formatConditions($this->db->existsClause($subStmt, TRUE));
		else
			$this->db->formatConditions($this->db->existsClause($subStmt));
		$this->db->orderBy('creatorSurname');
		$recordset = $this->db->select('creator', 
			array('creatorId', "creatorSurname", "creatorInitials", "creatorFirstname", "creatorPrefix"));
		$creators = array();
		while($row = $this->db->fetchRow($recordset))
			$this->formatNames($row, $creators);
		if(!empty($creators))
			return $creators;
		return FALSE;
	}
/**
* Gather creators who are group members
*
* @param int $masterId
*
* @return mixed FALSE|array of creators
*/
	public function grabGroupMembers($masterId)
	{
		$this->db->formatConditions(array('creatorSameAs' => $masterId));
		$this->db->orderBy('creatorSurname');
		$recordset = $this->db->select('creator', 
			array('creatorId', "creatorSurname", "creatorInitials", "creatorFirstname", "creatorPrefix"));
		$creators = array();
		while($row = $this->db->fetchRow($recordset))
			$this->formatNames($row, $creators);
		if(!empty($creators))
			return $creators;
		return FALSE;
	}
/**
* Gather creators who are not group members
*
* @return mixed FALSE|array of creators
*/
	public function grabGroupAvailableMasters()
	{
		$this->db->formatConditions(array('creatorSameAs' => ' IS NULL'));
		$this->db->orderBy('creatorSurname');
		$recordset = $this->db->select('creator', 
			array('creatorId', "creatorSurname", "creatorInitials", "creatorFirstname", "creatorPrefix"));
		$creators = array();
		while($row = $this->db->fetchRow($recordset))
			$this->formatNames($row, $creators);
		if(!empty($creators))
			return $creators;
		return FALSE;
	}
/**
* Does this creator already exist in the database?
*
* @param string $surname
* @param string $firstname
* @param string $initials
* @param string $prefix
*
* @return int creator ID if exists, FALSE otherwise
*/
	public function checkExists($surname, $firstname, $initials, $prefix)
	{
		$name = str_replace(' ', '', $surname . $firstname . $initials . $prefix);
		$array = array($this->db->formatFields('creatorSurname'), $this->db->formatFields('creatorFirstname'), 
			$this->db->formatFields('creatorInitials'), $this->db->formatFields('creatorPrefix'));
// Concatenate (CONCAT_WS) fields and remove any spaces
		$this->db->formatConditions($this->db->replace($this->db->concat($array, ''), 
			' ', '', FALSE) . $this->db->equal . $this->db->tidyInput($name));
		$resultset = $this->db->select('creator', 'creatorId');
		if(!$this->db->numRows($resultset))
			return FALSE;
		else
			return $this->db->fetchOne($resultset);
	}
/**
* Remove resource-less and collection-less creators
*/
	public function removeHanging()
	{
		$deleteIds = array();
		$this->db->formatConditions($this->db->formatFields('resourcecreatorCreatorId') . ' IS NOT NULL');
		$subStmt = $this->db->subQuery($this->db->selectNoExecute('resource_creator', 'resourcecreatorCreatorId'), FALSE, FALSE, TRUE);
		$this->db->formatConditions($this->db->formatFields('creatorId') . $this->db->inClause($subStmt, TRUE));
		$recordset = $this->db->select('creator', 'creatorId');
		while($row = $this->db->fetchRow($recordset))
			$deleteIds[] = $row['creatorId'];
		if(empty($deleteIds))
			return; // nothing to do
		$this->db->formatConditionsOneField($deleteIds, 'creatorId');
		$this->db->delete('creator');
// remove references to this creator as group master
		$this->db->formatConditionsOneField($deleteIds, 'creatorSameAs');
		$this->db->updateNull('creator', 'creatorSameAs');
		$this->db->optimize('creator');
// remove cache files for creators
		$this->db->deleteCache('cacheResourceCreators');
		$this->db->deleteCache('cacheMetadataCreators');
	}
/**
* Inserts a new creator in the database.
*
* Requires write permissions.
*
* @param array $v Array of surname, firstname, initials, prefix
* @return int Inserted ID
*/
	public function insert($v)
	{
		$this->gatekeep->init();
		$surname = $v['surname'];
		$firstname = $initials = $prefix = NULL;
		if( array_key_exists('firstname',$v) )
			$firstname = $v['firstname'];
		if( array_key_exists('initials',$v) )
			$initials = $v['initials'];
		if( array_key_exists('prefix',$v) )
			$prefix = $v['prefix'];
		$fields[] = 'creatorSurname';
		$values[] = trim($surname);
		if(trim($firstname))
		{
			$fields[] = 'creatorFirstname';
			$values[] = trim($firstname);
		}
		if(trim($initials))
		{
			$fields[] = 'creatorInitials';
			$values[] = $this->formatInitials(trim($initials));
		}
		if(trim($prefix)) 
		{
			$fields[] = 'creatorPrefix';
			$values[] = $this->formatInitials(trim($prefix));
		}
		if($id = $this->checkExists($surname, $firstname, $initials, $prefix))
            return $id;
		$this->db->insert('creator', $fields, $values);
// remove cache files for creators -- whenever anything changes in the lsit of creators, delete the cache.  No cache is the 
// trigger to recreate the cache the next time the list of creators is requested.
		$this->db->deleteCache('cacheResourceCreators');
		$this->db->deleteCache('cacheMetadataCreators');
		return $this->db->lastAutoId();
	}
/**
* Format initials prior to storing in database
*
* Initials are stored unpunctuated and space-delimited.
* In the final result, they can only consist of A-Z (and UTF-8) and space characters
*
* @param string $raw
* @return string
*/
	public function formatInitials($raw)
	{
		$initials = preg_split("/\.\s*/", $raw);
// If there's an empty array element at the end, remove it.
		if(!$initials[sizeof($initials) - 1])
			array_pop($initials);
		return join(' ', $initials);
	}
/**
* Format names returned from database
*
* @param array $row
* @param array $creators Array reference to be written to
*/
	private function formatNames($row, &$creators)
	{
		if($row['creatorPrefix'])
			$name = $row['creatorPrefix'] . ' ' . $row['creatorSurname'];
		else
			$name = $row['creatorSurname'];
		if($row['creatorFirstname'])
		{
			$name .= ', ' . $row['creatorFirstname'] . ' ';
			if($row['creatorInitials'])
				$name .= ' ' . str_replace(' ', '.', $row['creatorInitials']) . '.';
		}
		else if($row['creatorInitials'])
			$name .= ', ' . str_replace(' ', '.', $row['creatorInitials']) . '.';
		if($row['creatorId'])
			$creators[$row['creatorId']] = $this->html->dbToFormTidy($name);
	}
}
?>
