<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/** 
* Front page of the system.
*
* @version	1
*
*	@package wikindx4\core\display
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class FRONT
{
/** object */
private $db;
/** object */
private $session;
/** object */
private $messages;
/** object */
private $html;
/** object */
private $template;
/** object */
private $stmt;
/** object */
private $listCommon;
/** object */
private $config;
/** string */
private $externalMessage;

/**
* FRONT
*
* @param string $message
*/
	public function __construct($message = FALSE)
	{
		$this->db = FACTORY_DB::getInstance();
		$this->externalMessage = $message;
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->stmt = FACTORY_SQLSTATEMENTS::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->listCommon = FACTORY_LISTCOMMON::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->listCommon->navigate = 'front';
		GLOBALS::setOutputHeading(''); // blank
		include_once("core/modules/help/HELPMESSAGES.php");
		$help = new HELPMESSAGES();
		GLOBALS::setOutputHelp($help->createLink('front'));
		$this->init();
	}
/**
* Display front page information.  If $noMenu, display WIKINDX submenu links
*/
	private function init()
	{
		$this->session->delVar('search_Highlight');
		$recordset = $this->db->select('config', array("configDescription", "configContactEmail"));
		$row = $this->db->fetchRow($recordset);
		$pString = $this->html->dbToHtmlTidy($row['configDescription']);
// Do we want the quick search form to be displayed?
		if(substr_count($pString, '$QUICKSEARCH$'))
		{
			include_once('core/modules/list/QUICKSEARCH.php');
			$qs = new QUICKSEARCH();
			$replace = $qs->init(FALSE, FALSE, TRUE);
			$pString = str_replace('$QUICKSEARCH$', $replace, $pString);
		}
		if($lastChanges = $this->session->getVar("setup_LastChanges"))
		{
			if($this->getChanges($lastChanges))
				$pString .= $this->html->h($this->messages->text("resources", "lastChanges"), 4);
		}
		$pString .= $this->externalMessage;
		GLOBALS::buildOutputString($pString);
		if($row['configContactEmail'])
		{
			$email = $this->html->dbToHtmlTidy($row['configContactEmail']);
			$this->template->tpl->assign('contactEmail', $email);
		}
	}
/**
* Get recently added/edited resources
*
* @param int $limit
* @return string
*/
	private function getChanges($limit)
	{
// If no resources, return FALSE
		$recordset = $this->db->select('resource', 'resourceId');
		if(!$this->db->numRows($recordset))
			return FALSE;
		$this->db->ascDesc = $this->db->desc; // descending order
		if($this->session->getVar("setup_LastChangesType") == 'D') // Display from last $limit days
		{
			if(($limitResources = $this->session->getVar("setup_LastChangesDayLimit")) < 0)
				$limitResources = FALSE;
			$sql = $this->stmt->frontSetDays($limit, $limitResources);
		}
		else // Display set number
			$sql = $this->stmt->frontSetNumber($limit);
		return $this->listCommon->display($sql, 'front');
	}
}
?>