<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* Common FILE routines
* 
* @version	1
*
*	@package wikindx4\core\file
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class FILE
{
/** object */
private $config;
/** object */
private $session;
/** object */
private $db;
/** string */
private $filesDir;

/**
* FILE
*/
	public function __construct()
	{
		$this->config = FACTORY_CONFIG::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->db = FACTORY_DB::getInstance();
		if(isset($this->config->WIKINDX_FILES_DIR))
			$this->filesDir = $this->config->WIKINDX_FILES_DIR;
		else
			$this->filesDir = "files"; // default
	}
/**
* Return filename, hash, type and size of an uploaded file
*
* @param string $filename
* @return array (filename, hash, type, size)
*/
	public function fileUpload($filename = FALSE)
	{
		if(isset($_FILES) && array_key_exists('file', $_FILES) && 
			$_FILES['file']['tmp_name'])
		{
			if(trim($filename))
				$fileName = trim($filename);
			else if(!get_magic_quotes_gpc())
				$fileName = addslashes($_FILES['file']['name']);
			else
				$fileName = $_FILES['file']['name']; // already slashed
			if(($fileName == '.') || ($fileName == '..'))
				return array(FALSE, FALSE, FALSE, FALSE);
			return array($fileName, sha1_file($_FILES['file']['tmp_name']), 
				$_FILES['file']['type'], $_FILES['file']['size']);
		}
		return array(FALSE, FALSE, FALSE, FALSE);
	}
/**
* Get file max upload size
*
* @return int
*/
	public function fileMaxSize()
	{
		$postMax = $this->return_bytes(ini_get('post_max_size'));
		$uploadMax = $this->return_bytes(ini_get('upload_max_filesize'));
		if($postMax < $uploadMax)
			return $postMax;
		else
			return $uploadMax;
	}
/**
* Convert some ini values to numerical values
*
* @param string $val
* @return int
*/
	private function return_bytes($val)
	{
		$val = trim($val);
		$last = strtolower($val{strlen($val)-1});
		switch($last) 
		{
// The 'g' modifier is available since PHP 5.1.0
			case 'g':
			   $val *= 1024;
			case 'm':
			   $val *= 1024;
			case 'k':
			   $val *= 1024;
		}
		return $val;
	}
/**
* Store uploaded file in given directory with given name
*
* @param string $dirName
* @param string $name
* @return boolean
*/
	public function fileStore($dirName, $name)
	{
		if(!move_uploaded_file($_FILES['file']['tmp_name'], $dirName . '/' . $name))
			return FALSE;
		return TRUE;
	}
/**
* list and HTML format all files for sessionID
*
* @return array (filesDir, fileDeleteSecs, fileArray)
*/
	public function listFiles()
	{
		$fileExports = $this->session->getVar('fileExports');
		if(!$dh = @opendir($this->filesDir))
			return array(FALSE, FALSE, TRUE);
		if(!$fileExports) // no files in directory
			return array(FALSE, FALSE, FALSE);
		$fileArray = array();
		if($fileExports)
		{
			$sessArray = unserialize($fileExports);
			while(FALSE !== ($file = readdir($dh)))
			{
				if($file == "." || $file == "..")
					continue;
				if(array_search($file, $sessArray) === FALSE)
					continue;
				if(($file == 'CVS') || ($file == 'FILES') || (strpos($file, '.') === 0))
					continue;
				$fileArray[$file] = filemtime($this->filesDir . '/' . $file);
			}
			asort($fileArray, SORT_NUMERIC);
			$fileArray = array_reverse($fileArray);
		}
		closedir($dh);
		$deleteSecs = $this->db->fetchOne($this->db->select('config', 'configFileDeleteSeconds'));
		return array($this->filesDir, $deleteSecs, $fileArray);
	}
/**
* Set download headers
* 
* @param string $type
* @param int $size
* @param string $filename
*/
	public function setHeaders($type, $size, $filename)
	{
		header("Content-type: $type");
		header( "Content-Disposition: inline; filename=\"$filename\"; size=\"$size\"" );
		header("Content-Length:  $size" );
		header("Expires:  0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Pragma: public");
	}
/**
* Download file to user
*
* From http://uk3.php.net/function.readfile
* @param string $file
*/
	public function readfile_chunked($file)
	{
		$chunksize = 1*(1024*1024); // how many bytes per chunk
		$buffer = '';
		$handle = fopen($file, 'rb');
		if ($handle === false)
			return false;
		while (!feof($handle))
		{
	       $buffer = fread($handle, $chunksize);
	       echo $buffer;
	       ob_flush();
	       flush();
		}
		fclose($handle);
	}
/**
* create a fileName for this file.  If directory based on session ID does not exist, create it.
*
* @param string $dirName
* @param string $string File contents
* @param string $extension File extension
*
* @return array (filename, full filepath)
*/
	public function createFileName($dirName, $string, $extension)
	{
		$fileName = sha1(session_id() . $string) . $extension;
		return array($fileName, $dirName . '/' . $fileName);
	}
/**
* tidy up the files directory by removing all files and folders older than 'configFileDeleteSeconds'
*/
	public function tidyFiles()
	{
// failure is an option;
		if(!$dh = @opendir($this->filesDir))
			return;
		$now = time();
		$maxTime = $this->db->fetchOne($this->db->select('config', 'configFileDeleteSeconds'));
		$fileDeleteArray = $fileKeepArray = array();
		while(($f = readdir($dh)) !== false)
		{
			if(($f == 'CVS') || ($f == 'FILES') || (strpos($f, '.') === 0))
				continue;
			$file = $this->filesDir . '/' . $f;
			if(($now - filemtime($file)) >= $maxTime)
			{
				@unlink($file);
				$fileDeleteArray[] = $f;
			}
			else
				$fileKeepArray[] = $f;
		}
// Remove reference to these files in session
		if(($sessVar = $this->session->getVar('fileExports')) && !empty($fileDeleteArray))
		{
			$sessArray = unserialize($sessVar);
			foreach($fileDeleteArray as $f)
				unset($sessArray[array_search($f, $sessArray)]);
			if(!empty($sessArray))
				$this->session->setVar('fileExports', serialize($sessArray));
			else
				$this->session->delVar('fileExports');
		}
		else if(!empty($fileKeepArray))
			$this->session->setVar('fileExports', serialize($fileKeepArray));
	}
}
?>