<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
*	RICHTEXTFORMAT extends TINYMCETEXTEXPORT
*/
include_once("core/importexport/TINYMCETEXTEXPORT.php");

/**
* RTF encoding
* 
* NB - this produces just the bare minimum of RTF code to work in Word and OO.org
* @version	1
*
*	@package wikindx4\core\importexport
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class RICHTEXTFORMAT extends TINYMCETEXTEXPORT
{
/** array */
private $vars;
/** array */
private $fontBlocks = array();
/** array */
private $colours = array();
/** string */
private $defaultFontBlock;
/** string */
public $fontBlock = FALSE;
/** string */
public $colourTable = FALSE;
/** 
* array 
*
* default black font and blue for hyperlinks
*/
public $colourArray = array('\\red0\\green0\\blue0', '\\red0\\green0\\blue255'); // default black font and blue for hyperlinks
/** string */
public $lineSpacing;
/** string */
private $listOverrideTable;
/** string */
public $listTable;
/** string */
private $listType;
/** int */
private $listId;
/** int */
private $listIndex = 2;
/** int */
private $listIndent = 720;
/** int */
private $listIndentExtra = 360;
/** object */
private $wpConfig;
/** string */
private $imConvertPath = FALSE;
/** int */
private $quoteNumWords;
/** boolean */
private $keepQuoteMarks;
/** string */
private $quoteFontSize;
/** string */
private $qme;
/** string */
private $qms;

/**
* RICHTEXTFORMAT
*/
	public function __construct()
	{
		parent::initClass('rtf');
		$this->vars = GLOBALS::getVars();
		include_once("plugins/wordProcessor/config.php");
		$this->wpConfig = new wordProcessor_CONFIG();
		if($this->wpConfig->imageMagickConvertUtilityPath)
		{
			$this->imConvertPath = $this->wpConfig->imageMagickConvertUtilityPath . 'convert';
			if(!file_exists($this->imConvertPath))
				die("Image Magick path defined in config.php but convert utility not found");
		}
		else if(file_exists("/usr/bin/convert"))
			$this->imConvertPath = '/usr/bin/convert';
		else if(file_exists("/usr/bin/local/convert"))
			$this->imConvertPath = '/usr/bin/local/convert';
		else if(file_exists("/usr/local/bin/convert"))
			$this->imConvertPath = '/usr/bin/local/convert';
// set up defaults and parameters
		$this->init();
	}
/**
* Initialize RTF values either as defaults or from user-requested settings
*/
	private function init()
	{
// Line spacing of main paper body
		if($this->session->getVar('wp_ExportPaperSpace') == 'oneHalfSpace')
			$this->lineSpacing = '\\sl360\\slmult1';
		else if($this->session->getVar('wp_ExportPaperSpace') == 'doubleSpace')
			$this->lineSpacing = '\\sl480\\slmult1';
		else
			$this->lineSpacing = '';
		$this->paperSize = $this->session->getVar('wp_ExportPaperSize');
// Line spacing of indented quotations
		if($this->session->getVar('wp_ExportSpaceIndentQ') == 'oneHalfSpace')
			$this->lineSpacingIndentQ = '\\sl360\\slmult1';
		else if($this->session->getVar('wp_ExportSpaceIndentQ') == 'doubleSpace')
			$this->lineSpacingIndentQ = '\\sl480\\slmult1';
		else
			$this->lineSpacingIndentQ = '\\sl240\\slmult1'; // singlepublic $paperSize = FALSE;
		$fontSizes = array(16, 20, 24, 28, 36, 48, 72);
		$temp = array();
		foreach($this->fontSizes as $value)
			$temp[$value] = array_shift($fontSizes);
		$this->fontSizes = $temp;
		$this->footnoteText = FALSE;
		$this->tableStyle = FALSE;
		$this->justify = array(
							"center"	=>	"\\qc",
							"left"		=>	"\\ql",
							"right"		=>	"\\qr",
							"justify"	=>	"\\qj",
						);
/**
* Lists:
* \\ls0 => a, b ('a')
* \\ls5 => A, B ('aa')
* \\ls2 => i, ii ('i')
* \\ls3 => I, II ('ii')
* \\ls4 => 1, 2
* \\ls1 => bullets -- currently, only this one is used from this array.  Arabic numbered lists are handled in the code.
*/
		$this->lists = array(
							'aa'	=>	"\\ls5\\fi-360",
							'a'		=>	"\\ls0\\fi-360",
							'ii'	=>	"\\ls3\\fi-360",
							'i'		=>	"\\ls2\\fi-360",
							'1'		=>	"\\ls4\\fi-360",
							'bullet'	=>	"\\ls1\\fi-360",
						);
		$this->fontIndex = 1;
		$this->listIndex = 2;
		$this->defaultFontBlock = "{\\f0\\fcharset0 Arial;}\n";
		$this->listOverrideTable = "{\\listoverride\\listid1\\ls1}\n";
// Default bullets - 'listid1' == bullets, 'listidN' where 'N' is greater than 1 == Arabic
// $this->listTable is closed in $this->closeListTable() below
		$this->listTable = "{\\*\\listtable\n
		{\\list
		{\\listlevel\\levelnfc23\\levelstartat1\\levelfollow0{\\leveltext \\'01\\u8226 ?;}{\\levelnumbers;}\\fi-360\\li720}
		\\listid1}\n";
	}
/**
* Set some defaults and create the RTF opening tag
*
* @return string
*/
	public function header()
	{
		$this->lineSpacingIndentQ = "\n" . $this->lineSpacingIndentQ . "\n";
		$text = "{\\rtf1\\ansi\\ansicpg1252\n\n";
// 'letter' is default size for RTF -- presumably, 'legal' has the default width of 'letter'
// The tables always go beyond the paper width so we set each table width to 96% of standard
		if($this->paperSize == 'letter')
			$this->tableWidth = floor(8748 * 0.96);
		else if($this->paperSize == 'A4')
		{
			$text .= "\\paperw11909\\paperh16834\n\n";
			$this->tableWidth = floor(8417 * 0.96);
		}
		else if($this->paperSize == 'A5')
		{
			$text .= "\\paperw8395\\paperh11909\n\n";
			$this->tableWidth = floor(4903 * 0.96);
		}
		else if($this->paperSize == 'legal')
		{
			$text .= "\\paperh20160\n\n";
			$this->tableWidth = floor(8748 * 0.96);
		}
		else if($this->paperSize == 'executive')
		{
			$text .= "\\paperw10440\\paperh15120\n\n";
			$this->tableWidth = floor(6948 * 0.96);
		}
		return $text . "\n\n";
	}
/**
* Create the RTF closing tag
*
* @return string
*/
	public function footer()
	{
		return "\n}\n";
	}
/**
* parse tinyMCE code to RTF tags
*
* @author Mark Grimshaw
* @version 1
* @param string input text
* @return string parsed text
*/
	public function parse($text)
	{
//print htmlentities($text); die;
///		include_once("core/cite/CITE.php");
///		$cite = new CITE($this->db, $this->vars);
		$text = $this->formatText($text);

// handle <font>...</font> MAY NOT BE NEEDED
//		$text = $this->parseFont($text, array($this, "styleCallback"));
// handle <span>...</span>
		$text = $this->parseSpan($text, array($this, "styleCallback"));
		$text = $this->parseLists($text, array($this, "callbackUnorderedList"), array($this, "callbackOrderedList"));
// Handle tables
		$text = $this->createTables($text);
// Handle images
		$text = preg_replace_callback("/<img.*[>]+/Usi", array($this, "imageCallback"), $text);
// URL + emails
		$text = $this->createEmail($text);
		$text = $this->createFancyUrl($text);
/*
// Handle <div>...</div>
		$text = $this->rtf->parseDiv($text);
//print "FINAL: " . htmlentities($text) . "<P>";
// Ensure there are no hanging HTML tags (some browsers produce untidy HTML or superfluous HTML)
		$patterns = array(
			"/<\/div>/iU",
			"/<\/span>/iU",
			"/<\/font>/iU",
			);
		$text = preg_replace($patterns, '', $text);
*/
// indent long quotations
		if(array_key_exists('exportIndentQuoteWords', $this->vars))
		{
			$quoteIndentWords = $this->vars['exportIndentQuoteWords'];
			if(is_numeric($quoteIndentWords))
			{
				settype($quoteIndentWords, "integer");
				if(array_key_exists('exportIndentQuoteFontSize', $this->vars))
					$indentQuoteFontSize = $this->vars['exportIndentQuoteFontSize'];
				else
					$indentQuoteFontSize = 8; // font size 16 in RTF
				if(($sizeKey = array_search($indentQuoteFontSize * 2, $this->fontSizes)) !== FALSE)
					$this->quoteFontSize = $this->fontSizes[$sizeKey];
				else
					$this->quoteFontSize = $this->fontSizes['large']; // font size 9/18
				if(array_key_exists('exportIndentQuoteMarks', $this->vars))
					$indentQuoteMarks = TRUE;
				else
					$indentQuoteMarks = FALSE;
				$text = $this->indentQuotations($text, $quoteIndentWords, $indentQuoteFontSize, $indentQuoteMarks);
// Perhaps footnote immediately follows (so should be part of indent) -- NB indentQ() method in WPRTF.php
//print "TEXT1: $text<P>";
				$text = preg_replace_callback("/(\[\/cite\][.,:;?!°ø])(\s*__WIKINDX__NEWLINEPAR__}__WIKINDX__NEWLINEPAR____WIKINDX__QUOTEINDENTREMOVESPACE__\}*)(\[footnote\].*\[\/footnote\])/Usi", 
					array($this, "callback_footnoteIndent"), $text);
//print "TEXT2: $text<P>";
//				$pattern = 	"__WIKINDX__QUOTEINDENTDONE__|__WIKINDX__NEWLINE__|__WIKINDX__QUOTEINDENTREMOVESPACE__(\s*)|__WIKINDX__QUOTEINDENTREMOVESPACE__\}*(\s*)";
				$text = preg_replace("/$pattern/", "\n", $text);
			}
		}
// Deal with citations
		$text = $this->cite->parseCitations($text, 'rtf', FALSE, FALSE, TRUE);
//print "TEXT3: $text<P>";
// handles section breaks
		$text = preg_replace("/(__WIKINDX__NEWLINEPAR__)*__WIKINDX__SECTION__(__WIKINDX__NEWLINEPAR__)*/", 
			"\n\n{\\sect}\n\\sectd\n\n", $text);
// Replace temporary newlines
		$text = str_replace("__WIKINDX__NEWLINEPAR__", "\\par\n", $text);
//		$text = str_replace("__WIKINDX__NEWLINE__", "\n", $text);
		$text = str_replace("&nbsp;", " ", $text);
//		$text = str_replace("__WIKINDX__QUOTEINDENTDONE__", "\n", $text);
		$pattern = "__WIKINDX__QUOTEINDENTDONE__|__WIKINDX__NEWLINE__|__WIKINDX__QUOTEINDENTREMOVESPACE__(\s*)";
		$text = preg_replace("/$pattern/", "\n", $text);
// when user is cut 'n' pasting, superfluous codes are sometimes inserted so remove them
		$text = preg_replace("/[ ]*<span.*[>]+(.*)<\/span[>]+[ ]*/Usi", "$1", $text);
// Deal with footnotes
		if(array_key_exists('exportFontSizeFt', $this->vars))
			$fontSizeFt = $this->vars['exportFontSizeFt'];
		else
			$fontSizeFt = 8; // font size 16 in RTF
		$text = $this->parseFootnotes($text, $footnoteOffsetIds, $fontSizeFt);
		if($this->footnoteText)
			$text .= $this->footnoteText;
		$text = str_replace('__WIKINDX__ENDNOTE__START__', '', $text);
		$text = str_replace('__WIKINDX__ENDNOTE__END__', '', $text);
		$text = str_replace(" } ", " }", $text);
// Sometimes (not sure why), single quote gets replaced as &#039; so replace it
		$text = str_replace("&#039;", "'", $text);
// sometimes, with footnote/endnote styles, __WIKINDX__ gets left behind.  Temp. FIX.
		$text = str_replace("__WIKINDX__\par\qj __", "", $text);
		if(!empty($this->fontBlocks))
		{
			$this->fontBlock = "{\\fonttbl\n";
			$this->fontBlock .= $this->defaultFontBlock;
			foreach($this->fontBlocks as $index => $font)
				$this->fontBlock .= "{\\f$index\\fcharset0 $font;}\n";
			$this->fontBlock .= "}\n\n";
		}
        $this->colourTable = "{\\colortbl;";
        foreach($this->colourArray as $colour)
        	$this->colourTable .= $colour . ';';
		$this->colourTable .= "}\n\n";
		$this->closeListTable();
		return $this->utf8_2_unicode($text);
	}
/**
* Format text for HTML characters
*
* @param string $text
* @return string
*/
	public function formatText($text)
	{
// Deal with potential RTF control characters first
		$text = str_replace("\\", "\\\\", $text);
		$text = str_replace("{", "\\{", $text);
		$text = str_replace("}", "\\}", $text);
// Replace CR and newline characters and combinations with a single space
		$text = preg_replace("/(\r\n)+|(\n\r)+|\n+|\r+/", ' ', $text);
// Simple substitutions - italics and bold sometimes come in as standard HTML tags if they were copy 'n' pasted from elsewhere.
		$pattern = array(
//							"/\s*<br>\s*<\/p>/si",
//							"/\s*(.*?)\s*<br>/si",
							"/<hr>/si",
							"/<p>\s*(.*?)\s*<\/p>\s*/si",
							"/<sup>(.*?)<\/sup>/si",
							"/<sub>(.*?)<\/sub>/si",
							"/&amp;/si",
							"/\s*<br.*?>\s*/si",
							"/<em>(.*?)<\/em>/si",
							"/<i>(.*?)<\/i>/si",
							"/<strong>(.*?)<\/strong>/si",
							"/<b>(.*?)<\/b>/si",
							"/&lt;/si",
							"/&gt;/si",
						);
		$change = array(
//							"</p>",
//							"\\s1\\cf1$1__WIKINDX__NEWLINEPAR__", // temporary replace
							"__WIKINDX__SECTION__",
							"{__WIKINDX__NEWLINEPAR____WIKINDX__NEWLINEPAR__$1}", // temporary replace
							"{\\super $1}",
							"{\\sub $1}",
							"&",
							"{__WIKINDX__NEWLINEPAR__}",
							"{\\i $1}",
							"{\\i $1}",
							"{\\b $1}",
							"{\\b $1}",
							"\\u60 ?",
							"\\u62 ?",
						);
		return preg_replace($pattern, $change, $text);
	}
/**
* Callback for adding a footnote to an indented quotation.
*
* @param array $matches
* @return string
*/
	private function callback_footnoteIndent($matches)
	{
		return $matches[1] . $matches[3] . $matches[2];
	}
/**
* Create list tables for Arabic numerals
*/
	private function createListTablesArabic()
	{
		$this->listTable .= "
{\\list
{\\listlevel\\levelnfc0\\levelstartat1\\levelfollow0{\\leveltext \\'02\\'00.;}{\\levelnumbers\\'01;}\\fi-360\\li720}\\listid" . $this->listIndex . "}\n
";
		$this->listId = "\\ls" . $this->listIndex;
		$this->listOverrideTable .= 
"{\\listoverride\\listid" . $this->listIndex . $this->listId . "}\n";
		++$this->listIndex;
	}
/**
* Close the list table and append the listOverrideTable string
*/
	public function closeListTable()
	{
		$this->listTable .= "}\n" . "{\\listoverridetable\n" . $this->listOverrideTable . "}\n\n";
	}
/**
* Create fancy url hyperlinks
*
* @param string $text
* @return string
*/
	public function createFancyUrl($text)
	{
		return preg_replace_callback("/<a\s*.*\s*href\s*=\s*\"http:\/\/(.*)\"\s*.*\s*>(.*)<\/a>/Usi",
			array($this, "setFancyUrl"), $text);
	}
/**
* Create email hyperlinks
*
* @param string $text
* @return string
*/
	private function createEmail($text)
	{
		return preg_replace_callback("/<a\s*href\s*=\s*\"mailto:(.*)\"\s*>(.*)<\/a>/Usi",
			array($this, "setEmail"), $text);
	}
/**
* Callback for createFancyUrl() above
*
* @param array $matchArray
* @return string
*/
	private function setFancyUrl($matchArray)
	{
		$url = trim($matchArray[1]);
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[2]);
		return "{\\field{\\fldinst { HYPERLINK \"http://$url\" }}{\\fldrslt {\\cs1\\ul\\cf2 $text}}}__WIKINDX__NEWLINE__";
	}
/**
* Callback for createEmail() above
*
* @param array $matchArray
* @return string
*/
	private function setEmail($matchArray)
	{
		$email = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[1]);
		return "{\\field{\\fldinst { HYPERLINK \"mailto:$email\" }}{\\fldrslt {\\cs1\\ul\\cf2 $email}}}__WIKINDX__NEWLINE__";
	}
/**
* Set font types and create top level font blocks
*
* @param string $font
* @return int Font index
*/
	private function createFontBlocks($font)
	{
		$fonts = explode(',', $font);
		$font = $fonts[0];
		$fontIndex = $this->setFontBlock($font);
		return $fontIndex;
	}
/**
* Set font blocks
*
* @param string $font
* @return int Font index
*/
	private function setFontBlock($font)
	{
		if(array_search(strtolower($font), $this->fonts) === FALSE)
			$font = 'arial';
		if(($index = array_search($font, $this->fontBlocks)) === FALSE)
		{
			$this->fontBlocks[$this->fontIndex] = $font;
			$fontIndex = $this->fontIndex;
			++$this->fontIndex;
		}
		else
			$fontIndex = $index;
		return $fontIndex;
	}
/**
* Find any tables
*
* @param string $text
* @return string
*/
	private function createTables($text)
	{
		return preg_replace_callback("/\s*<table(.*)>\s*(.*)\s*<\/table>\s*/Usi",
			array($this, "tableFormat"), $text);
	}
/**
* Format Tables
*
* We've no way of knowing the browser window size from PHP so we assume a width of 1000 -- if the resultant tableWidth is higher 
* than $this->tableWidth, we limit it to $this->tableWidth.
* Must count the number of cells in each row and divide them into $tableWidth to get cell width.
*
* @param array $matchArray
* @return string
*/
	private function tableFormat($matchArray)
	{
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[2]);
		preg_match("/width.*(\d+)px/Usi", $matchArray[1], $array);
		$tableWidth = array_key_exists(1, $array) ? $this->tableWidth * $array[1] / 1000 : $this->tableWidth;
		$tableWidth = $tableWidth > $this->tableWidth ? $this->tableWidth : floor($tableWidth);
		$row = "\n\\trowd\\trql\\trpaddft3\\trpaddt55\\trpaddfl3\\trpaddl55\\trpaddfb3\\trpaddb55\\trpaddfr3\\trpaddr55__WIKINDX__NEWLINE__";
		$cell = "\\clbrdrt\\brdrs\\brdrw1\\brdrcf1\\clbrdrl\\brdrs\\brdrw1\\brdrcf1\\clbrdrb\\brdrs\\brdrw1\\brdrcf1\\clbrdrr\\brdrs\\brdrw1\\brdrcf1";
		$finalCell = "\\clbrdrt\\brdrs\\brdrw1\\brdrcf1\\clbrdrl\\brdrs\\brdrw1\\brdrcf1\\clbrdrb\\brdrs\\brdrw1\\brdrcf1\\clbrdrr\\brdrs\\brdrw1\\brdrcf1\\cellx" . $tableWidth . "__WIKINDX__NEWLINE__";
		preg_match_all("/\s*<tr.*>\s*(.*)\s*\<\/tr>\s*/Usi", $text, $matches);
// Count no. table cells in each row
		$output = '';
		foreach($matches[1] as $data)
		{
			$cellString = '\\intbl\n ';
			$rowString = $row;
			$numCells = preg_match_all("/\s*<td(.*)>\s*(.*)\s*\<\/td>\s*/Usi", $data, $cells);
			if(empty($cells))
				return $output;
			$cellText = $cells[2];
			$cellStyle = $cells[1];
			$width = $baseWidth = floor($tableWidth / $numCells);
			$cellNumber = 1;
//print_r($cells); print "<P>";
			foreach($cellText as $cellData)
			{
				if($cellNumber != $numCells)
					$rowString .= $cell . "\\cellx$width" . "__WIKINDX__NEWLINE__";
				else
					$rowString .= $finalCell;
				$cellData = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $cellData);
				if(preg_match("/style\s*=\s*\"(.*?)\"/si", array_shift($cellStyle), $styleMatch))
				{
					$this->tableStyle = TRUE;
					$cellData = $this->style($styleMatch[1], $cellData);
				}
				else
				{
					$cellData .= "\\cell";
					$this->tableStyle = FALSE;
				}
				$cellString .= $cellData . "__WIKINDX__NEWLINE__\\s1\\cf1";
				++$cellNumber;
				$width += $baseWidth;
			}
			$output .= $rowString . $cellString . "\\row\\pard" . $this->lineSpacing . "\n\n";
		}
		$this->tableStyle = FALSE;
		return "{__WIKINDX__NEWLINEPAR____WIKINDX__NEWLINEPAR__}" . $output;
	}
/**
* Callback for createLists()
*
* Unordered lists.
* NB - IE sometimes closes the <li> tag sometimes doesn't - bravo!
* @param array $matchArray
* @return string
*/
	protected function callbackUnorderedList($matchArray)
	{
		$this->listType = $this->lists['bullet'] . '\\li' . ($this->listIndent + $this->nested * $this->listIndentExtra);
		$text = $matchArray[1];
		$text = preg_replace("/\s*<li>\s*(.*?)\s*<\/li>\s*/si", 
			$this->listType . ' {' . "$1}\\par__WIKINDX__NEWLINE__", $text);
		$text = preg_replace_callback("/\s*<li\s*style\s*=\s*\"(.*?)\">\s*(.*?)\s*<\/li>\s*/si", 
			array($this, "listStyleCallback"), $text);
		$text = preg_replace("/\s*<li>\s*(.*?)\s*/si", 
			$this->listType . ' {' . "$1}\\par__WIKINDX__NEWLINE__", $text);
		$text = preg_replace_callback("/\s*<li\s*style\s*=\s*\"(.*?)\">\s*(.*?)\s*/si", 
			array($this, "listStyleCallback"), $text);
		return "\\par\n$text" . "\\pard" . $this->lineSpacing . "\\par\n";
	}
/**
* Callback for createLists() 
*
* Ordered Lists.
* NB - IE sometimes closes the <li> tag sometimes doesn't - bravo!
* @param array $matchArray
* @return string
*/
	protected function callbackOrderedList($matchArray)
	{
		$this->createListTablesArabic();
		$this->listType = $this->listId . "\\fi-360\\li" . $this->listIndent;
		$text = $matchArray[1];
		$text = preg_replace("/\s*<li>\s*(.*?)\s*<\/li>\s*/si", 
			$this->listType . ' {' . "$1}\\par__WIKINDX__NEWLINE__", $text);
		$text = preg_replace_callback("/\s*<li\s*style\s*=\s*\"(.*?)\">\s*(.*?)\s*<\/li>\s*/si", 
			array($this, "listStyleCallback"), $text);
		$text = preg_replace("/\s*<li>\s*(.*?)\s*/si", 
			$this->listType . ' {' . "$1}\\par__WIKINDX__NEWLINE__", $text);
		$text = preg_replace_callback("/\s*<li\s*style\s*=\s*\"(.*?)\">\s*(.*?)\s*/si", 
			array($this, "listStyleCallback"), $text);
		return "\\par\n$text" . "\\pard" . $this->lineSpacing . "\\par\n";
	}
/**
* For the cases where a list element has style information
*
* @param array $match
* @return string
*/
	private function listStyleCallback($match)
	{
		$text = trim($match[2]);
		$text = $this->style($match[1], $text);
		return $this->listType . " {$text}\\par__WIKINDX__NEWLINE__";
	}
/**
* Read an image from either file or URL
*
* @param array $matchArray
* @return string
*/
	public function imageCallback($matchArray)
	{
		$editW = $editH = FALSE;
		if(preg_match("/src\s*=\s*\"(.*)\"/Usi", $matchArray[0], $array))
			$file = $array[1];
		else if(array_key_exists(1, $matchArray))
			$file = $matchArray[1];
		else
			return $matchArray[0]; // unable to read file so return link
		preg_match("/width.*(\d*)px/Usi", $matchArray[0], $array);
		if(array_key_exists(1, $array))
			$editW = $array[1];
		preg_match("/height.*(\d*)px/Usi", $matchArray[0], $array);
		if(array_key_exists(1, $array))
			$editH = $array[1];
// Currently just working for JPEG and PNG in both OO.org and Word. (NB - Word can open GIFs processed here.)
		$tempFile = FALSE;
		if($fp = @fopen($file, "rb")) // don't do anything on file read error
		{
			list($width, $height, $type, $atrr) = GetImageSize($file);
			if(!$editW || !$editH)
				$blipScale = "\\picscalex100\\picscaley100";
			else
			{
				$scaleX = floor(100 * ($editW/$width));
				$scaleY = floor(100 * ($editH/$height));
				$blipScale = "\\picscalex$scaleX\\picscaley$scaleY";
			}
			if($type == 2) // JPEG
				$blipType = "\\jpegblip";
			else if($type == 3) // PNG
				$blipType = "\\pngblip";
			else
			{
				$blipType = "\\pngblip";
				$tempFile = "files/" . session_id() . ".png";
// Try to run imageMagick convert utility if available (probably just on *NIX) to convert to a PNG
				if($this->imConvertPath)
					exec("$this->imConvertPath $file $tempFile");
				else
					return $matchArray[0]; // unable to read file so return link
			}
			$out = "";
			if($tempFile)
			{
				fclose($fp);
				$fp = fopen($tempFile, "rb");
				@unlink($tempFile);
			}
			while (!feof($fp))
			{
				$file = fread($fp, 64);
				$out .= bin2hex($file) . "__WIKINDX__NEWLINE__";
			}
			return "{\\pict__WIKINDX__NEWLINE__" . $blipScale . $blipType . "__WIKINDX__NEWLINE__" . $out . "}";
		}
		return $matchArray[0]; // unable to read file so return link
	}
/**
* callback for HTML style elements of font and span
*
* @author Mark Grimshaw
* @version 1
* @param array $match
* @return string
*/
	protected function styleCallback($match)
	{
/*
$temp = $match;
array_shift($temp);
print "MATCH: "; 
foreach($temp as $value)
	print htmlentities($value) . "<P>";
print "<BR>";
*/
		if($this->spanParse)
			$text = $match[2];
		else
			$text = trim($match[3]);
		$newText = $this->style($match[1], $text);
		return $newText;
	}
/**
* callback for HTML style elements of <P>
*
* @author Mark Grimshaw
* @version 1
* @param array $match
* @return string
*/
	function paraCallback($match)
	{
		$text = $match[2];
		return $this->paraStyle($match[1], $text) . "__WIKINDX__NEWLINEPAR____WIKINDX__NEWLINEPAR__";
	}
/**
* Convert hexadecimal colours to RTF colours
*
* @param string $colour
* @return string
*/
	private function convertColour($colour)
	{
		$colour = rtrim(ltrim($colour));
		$colorVal = hexdec($colour);
		$rgbArray['red'] = 0xFF & ($colorVal >> 0x10);
        $rgbArray['green'] = 0xFF & ($colorVal >> 0x8);
        $rgbArray['blue'] = 0xFF & $colorVal;
        $colour = '\\red' . $rgbArray['red'];
        $colour .= '\\green' . $rgbArray['green'];
        $colour .= '\\blue' . $rgbArray['blue'];
		if(($index = array_search($colour, $this->colourArray)) !== FALSE)
			$index++;
		else
		{
			$this->colourArray[] = $colour;
			$index = sizeof($this->colourArray);
		}
		return "\\s1\\cf$index";
	}
/**
* deal with style elements for DIV and SPAN
*
* @author Mark Grimshaw
* @version 1
* @param string $styleString
* @param string $text
* @return string
*/
	private function style($styleString, $text)
	{
//print "STYLESTRING: $styleString<P>";
		$text = str_replace(" ", "WIKINDX_SPACE", $text);
		if($this->isIE)
			$params = explode('&nbsp;', $styleString);
		else
			$params = explode(";", $styleString);
//print_r($params); print "<P>";
		$justify = $pixelsR = $pixelsL = $indentL = $indentR = FALSE;
		$slashCellAdded = FALSE;
//print_r($params); print ": $text<P>";
		foreach($params as $param)
		{
			if(!$param)
				continue;
			if($this->isIE)
				$splitParam = explode("=", $param);
			else
				$splitParam = explode(":", $param);
			if(!array_key_exists(1, $splitParam)) // not recognised - usually result of pasting from Office etc.
				continue;
//print "PARAM: $param<BR>";
//print "#" . $splitParam[0] . "# | #" . $splitParam[1] . "#<P>";
			$param0 = strtolower(trim($splitParam[0]));
			$param1 = trim(str_replace('"', '', $splitParam[1]));
//print html_entity_decode("param0: $param0; param1: $param1<P>");
//print "$param0<P>";
			if(($param0 == 'font-weight') && ($param1 == 'bold'))
				$text = "{\\b\n$text}";
			else if(($param0 == 'font-weight') && ($param1 == 'normal'))
				$text = "{$text}";
			else if(($param0 == 'text-decoration') && ($param1 == 'underline'))
				$text = "{\\ul $text}";
			else if(($param0 == 'font-style') && ($param1 == 'italic'))
				$text = "{\\i\n$text}";
			else if($param0 == 'color')
			{
				$colour = $this->convertColour($param1);
				$text = '__WIKINDX__NEWLINE__{' . "$colour" . "__WIKINDX__NEWLINE__$text}";
			}
			else if($param0 == 'font-family')
			{
				$fontIndex = $this->createFontBlocks($param1);
				$text = "{\\f$fontIndex $text}__WIKINDX__NEWLINE__";
			}
			else if($param0 == 'face')
			{
				$fontIndex = $this->createFontBlocks($param1);
				$text = "{\\f$fontIndex $text}__WIKINDX__NEWLINE__";
			}
			else if($param0 == 'font-size')
			{
				$fontSize = $this->fontSizes[$param1];
				$text = "{\\fs$fontSize $text}__WIKINDX__NEWLINE__";
			}
			else if($param0 == 'text-align')
				$justify = $param1;
			else if($param0 == 'margin-left')
			{
				if(preg_match("/(\d+)/", $param1, $array))
					$indentL = '\\li' . $array[1] * 18;
			}
			else if($param0 == 'margin-right')
			{
				if(preg_match("/(\d+)/", $param1, $array))
					$indentR = '\\ri' . $array[1] * 18;
			}
		}
// Although it works in OO.org, for Word, justification needs to be outside {...} and therefore not mixed up with bold, italics etc.  so deal with it last.
		if($justify)
		{
			if(!array_key_exists($justify, $this->justify))
				$justify = $this->justify['justify'];
			else
				$justify = $this->justify[$justify];
			if($this->tableStyle)
			{
				$text = "{" . "$justify\n$text \\cell}\n";
				$slashCellAdded = TRUE;
			}
			else
				$text = "$justify\n$text\n";
		}
		if($this->tableStyle && !$slashCellAdded)
			$text .= "\\cell";
		if($indentL || $indentR)
			$text = "\par$indentL$indentR\n" . '{' . "$text\par}\n\n\\li0\\ri0\n";
			$text = str_replace("WIKINDX_SPACE", " ", $text);
		return $text;
	}
/**
* deal with <P> parameters
*
* @author Mark Grimshaw
* @version 1
* @param string $param
* @param string $text
* @return string
*/
	function paraStyle($param, $text)
	{
		$text = str_replace(" ", "&nbsp;", $text);
		$justify = $pixelsR = $pixelsL = $indentL = $indentR = FALSE;
		$justify = trim($param);
// Although it works in OO.org, for Word, justification needs to be outside {...} and therefore not mixed up with bold, italics etc.  so deal with it last.
		if($justify)
		{
			if(!array_key_exists($justify, $this->justify))
				$justify = $this->justify['justify'];
			else
				$justify = $this->justify[$justify];
			$text = "$justify\n$text\n";
		}
		if($pixelsL)
		{
			if(!array_key_exists($pixelsL, $this->indentL))
				$indentL = $this->indentL[40];
			else
				$indentL = $this->indentL[$pixelsL];
		}
		if($pixelsR)
		{
			if(!array_key_exists($pixelsR, $this->indentR))
				$indentR = $this->indentR[40];
			else
				$indentR = $this->indentR[$pixelsR];
		}
		if($indentL || $indentR)
			$text = "$indentL$indentR\n$text\n\\par\n\\li0\\ri0\n";
		return $text;
	}
/**
* indent long quotations
*
* @author Mark Grimshaw
* @version 1
* @param string $text
* @param integer $numWords
* @param boolean
* @return string
*/
	private function indentQuotations($text, $numWords, $keepQuoteMarks)
	{
		$this->quoteNumWords = $numWords;
		$this->keepQuoteMarks = $keepQuoteMarks;
// Get quotation marks for this localization
		if((!$languageDir = $this->session->getVar("setup_Language")) || !is_dir("languages/$languageDir"))
			$languageDir = "en";
		include_once("languages/$languageDir/CONSTANTS.php");
		$class = "CONSTANTS_" . $languageDir;
		$wikindxLanguageClass = new $class();
		if(isset($wikindxLanguageClass->startQuotation) && $wikindxLanguageClass->startQuotation)
			$qms = preg_quote($wikindxLanguageClass->startQuotation);
		else
			$qms = "\"";
		$this->qms = $qms;
		if(isset($wikindxLanguageClass->endQuotation) && $wikindxLanguageClass->endQuotation)
			$qme = preg_quote($wikindxLanguageClass->endQuotation);
		else
			$qme = "\"";
		$this->qme = $qme;
		return preg_replace_callback("/($qms)(.*)($qme)(.*)(\[cite\].*\[\/cite\])(.*)(\[footnote\].*\[\/footnote\]){0,}([.,:;?!°ø])/Usi", array($this, "indentQ"), $text);
	}
/**
* callback for indent long quotations
*
* @author Mark Grimshaw
* @version 1
* @param array $matchArray
* @return string
*/
	private function indentQ($matchArray)
	{
// [1] => initial quote marker
// [2] => quoted text
// [3] => final quote marker
// [4] => intervening text (should be a single space)
// [5] => citation
// [6] => intervening text
// [7] => tags such as footnotes etc.
// [8] => punctuation immediately following citation

//print "HERE: "; print_r($matchArray); print "<P>";
//print $matchArray[4]; print "<P>";
// No indentation if this is part of a footnote...
		if(strpos($matchArray[4], '[/footnote]'))
			return $matchArray[0];
// are there quotes within the quote?
		if(strpos($matchArray[4], $this->qme))
		{
			$split = explode($this->qme, $matchArray[4]);
//print_r($split); print "<P>";
			$lastElement = array_pop($split);
			$matchArray[4] = $lastElement;
			$matchArray[2] .= $matchArray[3] . join($this->qme, $split);
//print $matchArray[2]; print "<P>";
//print $matchArray[4]; die;
		}
		if(str_word_count($matchArray[2]) >= $this->quoteNumWords)
		{
			if(trim($matchArray[6])) // intervening text
			{
				$trail = trim($matchArray[6]) . $matchArray[7] . $matchArray[8];
				$removeSpace = '';
			}
			else
			{
				$trail = $matchArray[6] . $matchArray[7] . trim($matchArray[8]);
				$removeSpace = '__WIKINDX__QUOTEINDENTREMOVESPACE__';
			}
			$fs = $this->quoteFontSize . $this->lineSpacingIndentQ;
			$start = "\n__WIKINDX__NEWLINEPAR__{\\fs$fs __WIKINDX__NEWLINEPAR__\\li720\\ri720";
			$codaA = $matchArray[4] . $matchArray[5] . $trail . 
				"__WIKINDX__NEWLINEPAR__}__WIKINDX__NEWLINEPAR__$removeSpace";
			$codaB = 
				"__WIKINDX__NEWLINEPAR__}__WIKINDX__NEWLINEPAR__" . $matchArray[4] . $matchArray[5] . 
				$trail . $removeSpace;
			$codaC = $matchArray[4] . $matchArray[5] . 
				"__WIKINDX__NEWLINEPAR__}__WIKINDX__NEWLINEPAR__" . $trail . $removeSpace;
			if($this->keepQuoteMarks)
			{
				if($matchArray[4] == ' ')
				{
					if(trim($matchArray[6])) // intervening text
						return $start . $this->qms . $matchArray[2] . $this->qme . $codaC;
					else
						return $start . $this->qms . $matchArray[2] . $this->qme . $codaA;
				}
				else // intervening text
					return $start . $this->qms . $matchArray[2] . $this->qme . $codaB;

			}
			else
			{
				if($matchArray[4] == ' ')
				{
					if(trim($matchArray[6])) // intervening text
						return $start . '__WIKINDX__QUOTEINDENTDONE__' . $matchArray[2] . 
						'__WIKINDX__QUOTEINDENTDONE__' . $codaC;
					else
						return $start . '__WIKINDX__QUOTEINDENTDONE__' . $matchArray[2] . 
						'__WIKINDX__QUOTEINDENTDONE__' . $codaA;
				}
				else // intervening text
					return $start . '__WIKINDX__QUOTEINDENTDONE__' . $matchArray[2] . 
					'__WIKINDX__QUOTEINDENTDONE__' . $codaB;

			}
		}
		return $matchArray[0];
//		unset($matchArray[0]);
//		return join('', $matchArray);
	}
/**
* parse [footnote]...[/footnote]
*
* @author Mark Grimshaw
* @version 1
* @param string $text
* @param array $footnoteOffsetIds
* @param int $fontSize
* @return string
*/
	private function parseFootnotes($text,$footnoteOffsetIds, $fontSize)
	{
//print_r($this->styleArray); die;
		if(($sizeKey = array_search($fontSize * 2, $this->fontSizes)) !== FALSE)
			$this->fontSizeFt = $this->fontSizes[$sizeKey];
		else
			$this->fontSizeFt = $this->fontSizes['large']; // font size 9/18
		if(!empty($footnoteOffsetIds)) // endnotes same IDs and there are existing cite tags being used
			$this->footnoteOffsetIds = $footnoteOffsetIds;
		return preg_replace_callback("/\[footnote\](.*)\[\/footnote\]([.,:;?!]*\s*)/Usi", 
			array($this, "footnotes"), $text);
	}
/**
* callback for parsing [footnote]...[/footnote]
*
* @author Mark Grimshaw
* @version 1
* @param array $matchArray
* @return string
*/
	private function footnotes($matchArray)
	{
		$text = $matchArray[1];
		$id = '\\chftn';
		if(!$this->styleArray['citationStyle']) // in-text citations so footnotes here really are footnotes
			$ft = "\\footnote";
		else // endnotes and footnotes
		{
			if(!$this->styleArray['endnoteStyle']) // endnotes incrementing
				$ft = "\\footnote\\ftnalt";
			else if($this->styleArray['endnoteStyle'] == 1) // endnotes same IDs
			{
				$ft = "\\footnote\\ftnalt";
				$id = array_shift($this->footnoteOffsetIds);
			}
			else	// '2' == footnotes incrementing
				$ft = "\\footnote";
		}
		$preInText = $this->styleArray['firstCharsEndnoteInText'];
		$postInText = $this->styleArray['lastCharsEndnoteInText'];
		$preId = $this->styleArray['firstCharsEndnoteID'];
		if($postInText && preg_match("/[.,:;?!]\s*/", $matchArray[2], $null))
			$matchArray[2] = '';
		$postId = $this->styleArray['lastCharsEndnoteID'];
		if($this->styleArray['formatEndnoteInText'] == 1) // superscript
		{
			$preInTextFormat = "{\\super";
			$postInTextFormat = '}';
		}
		else if($this->styleArray['formatEndnoteInText'] == 2) // subscript
		{
			$preInTextFormat = "{\\sub";
			$postInTextFormat = '}';
		}
		else
		{
			$preInTextFormat = '';
			$postInTextFormat = '';
		}
		if($this->styleArray['formatEndnoteID'] == 1) // superscript
		{
			$preIDFormat = "{\\super";
			$postIDFormat = '}';
		}
		else if($this->styleArray['formatEndnoteID'] == 2) // subscript
		{
			$preIDFormat = "{\\sub";
			$postIDFormat = '}';
		}
		else
		{
			$preIDFormat = '';
			$postIDFormat = '';
		}
		if($this->session->getVar('wp_ExportIndentFt') == 'indentAll')
			$ftf = "\\li720 ";
		else if($this->session->getVar('wp_ExportIndentFt') == 'indentFL')
			$ftf = "\\fi720 ";
		else if($this->session->getVar('wp_ExportIndentFt') == 'indentNotFL')
			$ftf = "\\li720\\fi-720 ";
		else
			$ftf = "\\li1\\fi1 ";
		$ftf .= "\\fs" . $this->fontSizeFt;
		if($this->session->getVar('wp_ExportSpaceFt') == 'oneHalfSpace')
			$ftf = "\\pard\\plain $ftf\\sl360\\slmult1 ";
		else if($this->session->getVar('wp_ExportSpaceFt') == 'doubleSpace')
			$ftf = "\\pard\\plain $ftf\\sl480\\slmult1 ";
		else
			$ftf = "\\pard\\plain$ftf";
		if(($this->styleArray['endnoteStyle'] == 1)
			 && array_key_exists('sameIdOrderBib', $this->styleArray)) // endnotes same IDs, bibliography order
		{
			$this->footnoteText .= "$ftf\\qj$preIDFormat$preId{\\cs2 $id}" . $postId . $postIDFormat . 
				$matchArray[1] . "\\par\n" . $postInTextFormat . $matchArray[2];
			return "\n$preInTextFormat{" . "$preInText$id$postInText\n}}";
		}
		return "\n$preInTextFormat$preInText{\\cs2 $id\n{" . 
			"$ft$ftf\\qj" . "$preIDFormat$preId{\\cs2 $id}" . $postId . "$postIDFormat{" . 
			$text . "}}}$postInText\n" . $postInTextFormat . $matchArray[2];
	}
/*
FUNCTIONS NOT USED
	function setFontSize($matchArray)
	{
		$size = $matchArray[1];
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[2]);
		$fontSize = $this->fontSizes[$size];
		return "{\\fs$fontSize $text}__WIKINDX__NEWLINE__";
	}
	function setFontColor($matchArray)
	{
		$colour = $matchArray[1];
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[2]);
		if(!array_key_exists($colour, $this->colours))
			$colour = $this->colours['black'];
		else
			$colour = $this->colours[$colour];
		return "__WIKINDX__NEWLINE__$colour" . "__WIKINDX__NEWLINE__$text";
	}
	function setFontFace($matchArray)
	{
		$face = $matchArray[1];
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[2]);
		$fontIndex = $this->createFontBlocks(trim($face));
		return "{\\f$fontIndex $text}__WIKINDX__NEWLINE__";
	}
	function createIndentFirstLine($text)
	{
		return preg_replace_callback("/\[indent\s*=\s*l_first_line\](.*)\[\/indent\]/Usi",
			array($this, "setIndentFirstLine"), $text);
	}
	function setIndentFirstLine($matchArray)
	{
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[1]);
		return "\fi720\n$text\n\\par\n\fi0\n";
	}
	function colourFormat($colours)
	{
		$colour = trim($colours[1]);
		if(!array_key_exists($colour, $this->colours))
			$colour = $this->colours['black'];
		else
			$colour = $this->colours[$colour];
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $colours[2]);
		return "__WIKINDX__NEWLINE__$colour" . "__WIKINDX__NEWLINE__$text";
	}
*/
}
?>