<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* LISTCOMMON common functions for listing, searching, selecting etc. resources
* 
* @version	1
*
*	@package wikindx4\core\lists
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class LISTCOMMON
{
/** object */
private $db;
/** array */
private $vars;
/** object */
private $icons;
/** object */
private $config;
/** object */
private $coins;
/** object */
private $html;
/** object */
private $session;
/** object */
private $messages;
/** object */
private $template;
/** object */
private $user;
/** object */
private $stats;
/** object */
public $pagingObject = FALSE;
/** object */
private $bibStyle;
/** object */
private $form;
/** boolean */
public $keepHighlight = FALSE;
/** string */
public $patterns = FALSE;
/** string */
public $navigate = 'list';
/** object */
private $commonBib;
/** object */
private $cite;
/** object */
private $resCommon;
/** boolean */
public $listQuarantined = FALSE;
/** string */
private $resourceLink = FALSE;
/** boolean */
public $metadata = FALSE;
/** string */
public $metadataKeyword = FALSE;
/** array */
public $metadataText = array();
/** array */
public $metadataTextCite = array();
/** array */
public $metadataTextJoin = array();
/** array */
public $metadataTextCond = array();
/** boolean */
public $quickSearch = FALSE;

/**
* LISTCOMMON
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->icons = FACTORY_LOADICONS::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->coins = FACTORY_EXPORTCOINS::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->user = FACTORY_USER::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->stats = FACTORY_STATISTICS::getInstance();
		$this->bibStyle = FACTORY_BIBSTYLE::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->cite = FACTORY_CITE::getInstance();
		$this->utf8 = FACTORY_UTF8::getInstance();
		$this->resCommon = FACTORY_RESOURCECOMMON::getInstance();
		$this->icons->setupIcons();
		$this->stats->list = TRUE;
	}
/**
* Check there are resources to display
*
* @return boolean
*/
	public function resourcesExist()
	{
		$recordset = $this->db->select('database_summary', 'databaseSummaryTotalResources');
		if(!$this->db->fetchOne($recordset))
		{
			GLOBALS::buildOutputString($this->messages->text('misc', 'noResources'));
			return FALSE;
		}
		if($useBib = $this->session->getVar("mywikindx_Bibliography_use"))
		{
			$this->db->formatConditions(array('userbibliographyresourceBibliographyId' => $useBib));
			$this->db->formatConditions($this->db->formatFields('userbibliographyresourceResourceId') . $this->db->equal . 
				$this->db->formatFields('resourceId'));
			$resultset = $this->db->select(array('resource', 'user_bibliography_resource'), 'resourceId');
		}
		else
			$resultset = $this->db->select('resource', 'resourceId');
		if(!$this->db->numRows($resultset))
		{
			GLOBALS::buildOutputString($this->messages->text('misc', 'noResourcesBib'));
			return FALSE;
		}
		return TRUE;
	}
/**
* Produce a list of resources
*
* @param string $sql
* @param string $listType Default is FALSE
*/
	public function display($sql, $listType = FALSE)
	{
		$this->session->setVar("list_On", TRUE);
		$form = FALSE;
		if(!$this->keepHighlight)
			$this->session->delVar('search_Highlight');
		$this->bibStyle->bibformat->patterns = $this->patterns;
		if($this->session->getVar('setup_Listlink') == 'Y')
			$this->bibStyle->linkUrl = FALSE;
		$resourceKey = 0;
		if($listType != 'cite')
			$this->session->setVar("bookmark_View", 'multi');
		if(($listType == 'front') || ($listType == 'cite'))
			$recordset = $this->db->query($sql); // Don't mess up Last Multi by saving querystring
		else
			$recordset = $this->db->query($sql, TRUE);
// Displaying only attachments?
		if($this->session->getVar($listType . '_DisplayAttachment'))
		{
			$this->listAttachments($recordset, $listType);
			$this->session->setVar('sql_DisplayAttachment', $listType . '_DisplayAttachment');
			return;
		}
		$this->session->delVar('sql_DisplayAttachment');
		if($this->pagingObject)
			$form = TRUE;
		$citeRadioButtonFirst = TRUE;
		while($row = $this->db->fetchRow($recordset))
		{
			if(isset($allIds) && (array_search($row['resourceId'], $allIds) !== FALSE))
				continue;
			if($listType == 'cite')
			{
				$resourceList[$resourceKey]['resource'] = $this->bibStyle->process($row);
				if($citeRadioButtonFirst)
				{
					$resourceList[$resourceKey]['links']['checkbox'] = $this->form->radioButton(FALSE, 'cite', $row['resourceId'], TRUE);
					$citeRadioButtonFirst = FALSE;
				}
				else
					$resourceList[$resourceKey]['links']['checkbox'] = $this->form->radioButton(FALSE, 'cite', $row['resourceId']);
				if($this->metadataKeyword)
				{
					$mArray = $this->listMetadata($row['resourceId']);
					if(!empty($mArray))
						$resourceList[$resourceKey]['metadata'] = $mArray;
				}
				else if(!empty($this->metadataText) || !empty($this->metadataTextCite))
				{
					$mArray = $this->listMetadataText($row['resourceId']);
					if(!empty($mArray))
						$resourceList[$resourceKey]['metadata'] = $mArray;
				}
				$resourceKey++;
				continue;
			}
			if(($row['resourcemiscQuarantine'] == 'Y') && ($this->session->getVar('setup_Quarantine') == 'Y'))
				$resourceList[$resourceKey]['quarantine'] = $this->icons->quarantine;
			$multiUser = $this->session->getVar('setup_MultiUser');
			if($multiUser == 'Y')
			{
				$resourceList[$resourceKey]['user'] = $this->user->displayUserAddEdit($row);
				$popularityIndex = $this->messages->text("misc", "popIndex", $this->stats->getPopularityIndex($row['resourceId']));
				$maturityIndex = $row['resourcemiscMaturityIndex'] ? 
					"&nbsp;" . $this->messages->text("misc", "matIndex") . "&nbsp;" . 
					$row['resourcemiscMaturityIndex'] . "/10" . $this->html->br()
					:
					FALSE;
				$resourceList[$resourceKey]['popIndex'] = $popularityIndex;
				$resourceList[$resourceKey]['maturity'] = $maturityIndex;
				$this->template->tpl->assign('multiUser', TRUE);
			}
			$resourceList[$resourceKey]['timestamp'] = $row['resourcetimestampTimestamp'];
			$resourceList[$resourceKey]['links'] = $this->createLinks($row);
			if(($form && $this->session->getVar('setup_Write')) || ($listType != 'front'))
				$resourceList[$resourceKey]['links']['checkbox'] = $this->form->checkBox(FALSE, "bib_" . $row['resourceId']);
			if($this->resourceLink)
				$resourceList[$resourceKey]['resource'] = $this->html->a('rLink', $this->bibStyle->process($row), $this->resourceLink) . 
					$this->coins->export($row, $this->bibStyle->coinsCreators);
			else
				$resourceList[$resourceKey]['resource'] = $this->bibStyle->process($row) . 
					$this->coins->export($row, $this->bibStyle->coinsCreators);
			if($this->metadataKeyword)
			{
				$mArray = $this->listMetadata($row['resourceId']);
				if(!empty($mArray))
					$resourceList[$resourceKey]['metadata'] = $mArray;
			}
			else if(!empty($this->metadataText))
			{
				$mArray = $this->listMetadataText($row['resourceId']);
				if(!empty($mArray))
					$resourceList[$resourceKey]['metadata'] = $mArray;
			}
			$allIds[] = $row['resourceId'];
			$resourceKey++;
		}
		if(isset($allIds))
			$this->session->setVar('list_AllIds', base64_encode(serialize($allIds)));
		else
			$this->session->delVar("list_AllIds");
		if(isset($resourceList))
		{
			if(!$this->listQuarantined && ($listType != 'cite'))
			{
				if($this->pagingObject)
					$this->displayListInfo($listType);
			}
			$this->template->tpl->assign('resourceList', $resourceList);
		}
		else if($this->pagingObject && ($listType != 'cite'))
			$this->displayListInfo($listType, FALSE);
		return TRUE;
	}
/**
* Get metadata for this resource when browsing a keyword
*
* @param int $resourceId
* @return array
*/
	private function listMetadata($resourceId)
	{
		$array = array();
// quotes
		$this->db->formatConditions(array('resourcekeywordKeywordId' => $this->metadataKeyword));
		$this->db->formatConditions(array('resourcekeywordQuoteId' => ' IS NOT NULL'));
		$this->db->formatConditions(array('resourcequoteResourceId' => $resourceId));
		$this->db->leftJoin('resource_quote', 'resourcequoteId', 'resourcekeywordQuoteId');
		$this->db->leftJoin('resource_quote_text', 'resourcequotetextId', 'resourcequoteId');
		$resultset = $this->db->select('resource_keyword', 'resourcequotetextText');
		while($row = $this->db->fetchRow($resultset))
			$array[] = $this->cite->parseCitations($this->html->dbToHtmlTidy($row['resourcequotetextText']), 'htmlNoBib', FALSE);
// paraphrases
		$this->db->formatConditions(array('resourcekeywordKeywordId' => $this->metadataKeyword));
		$this->db->formatConditions(array('resourcekeywordParaphraseId' => ' IS NOT NULL'));
		$this->db->formatConditions(array('resourceparaphraseResourceId' => $resourceId));
		$this->db->leftJoin('resource_paraphrase', 'resourceparaphraseId', 'resourcekeywordParaphraseId');
		$this->db->leftJoin('resource_paraphrase_text', 'resourceparaphrasetextId', 'resourceparaphraseId');
		$resultset = $this->db->select('resource_keyword', 'resourceparaphrasetextText');
		while($row = $this->db->fetchRow($resultset))
			$array[] = $this->cite->parseCitations($this->html->dbToHtmlTidy($row['resourceparaphrasetextText']), 'htmlNoBib', FALSE);
// musings
		$this->db->formatConditions(array('resourcekeywordKeywordId' => $this->metadataKeyword));
		$this->db->formatConditions(array('resourcekeywordMusingId' => ' IS NOT NULL'));
		$this->db->formatConditions(array('resourcemusingResourceId' => $resourceId));
		$this->db->leftJoin('resource_musing', 'resourcemusingId', 'resourcekeywordMusingId');
		$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
		$resultset = $this->db->select('resource_keyword', 'resourcemusingtextText');
		while($row = $this->db->fetchRow($resultset))
			$array[] = $this->cite->parseCitations($this->html->dbToHtmlTidy($row['resourcemusingtextText']), 'htmlNoBib', FALSE);
		return $array;
	}
/**
* Get metadata for this resource when selecting or searching metadata
*
* @param int $resourceId
* @return array
*/
	private function listMetadataText($resourceId)
	{
		$array = array();
		if(!empty($this->metadataText)) // i.e. not called from the word processor so no need for radio buttons on quotes, comments etc.
		{
			$cite = FALSE;
			$cycleArray = $this->metadataText;
		}
		else
		{
			$cite = TRUE;
			$cycleArray = $this->metadataTextCite;
		}
		foreach($cycleArray as $sql)
		{
			$sql = str_replace('RESID', $this->db->tidyInput($resourceId), $sql);
			$resultset = $this->db->query($sql);
			while($row = $this->db->fetchRow($resultset))
			{
				if($cite)
					$array[] = $this->form->radioButton(FALSE, 'cite', $resourceId . '_' . 
					base64_encode($this->utf8->smartUtf8_decode($this->html->dbToTinyMCE($row['text'])))) . 
					'&nbsp;' . 
					$this->resCommon->doHighlight($this->cite->parseCitations($this->html->dbToHtmlTidy($row['text']), 'htmlNoBib', FALSE));
				else
					$array[] = $this->resCommon->doHighlight($this->cite->parseCitations($this->html->dbToHtmlTidy($row['text']), 'htmlNoBib', FALSE));
			}
		}
		return $array;
	}
/**
* list only attachments
*
* @param object $recordset 
* @param string $listType
*/
	private function listAttachments($recordset, $listType)
	{
// Are only logged on users allowed to view this file and is this user logged on?
		if($this->session->getVar("setup_FileViewLoggedOnOnly") && !$this->session->getVar("setup_UserId"))
			$this->displayListInfo($listType, FALSE);
		include_once("core/miscellaneous/ATTACHMENT.php");
		$attachments = new ATTACHMENT();
		$files = array();
		while($row = $this->db->fetchRow($recordset))
		{
			$files[] = $attachments->makeLink($row, TRUE, FALSE);
			$ids[] = $this->html->a($this->icons->viewLink, $this->icons->view, "index.php?action=resource_RESOURCEVIEW_CORE" . 
			htmlentities("&id=" . $row['resourceattachmentsResourceId']));
		}
		if(empty($files))
		{
			$this->displayListInfo($listType, FALSE);
			return;
		}
		$this->displayListInfo($listType);
		$this->template->tpl->assign('fileList', $files);
		$this->template->tpl->assign('fileListIds', $ids);
	}
/**
* Create links for viewing, editing deleting etc. resources
*
* @param array $row Reference to $row
* @return array
*/
	private function createLinks(&$row)
	{
		$write = $this->session->getVar('setup_Write');
		$superAdmin = $this->session->getVar('setup_Superadmin');
		$userId = $this->session->getVar('setup_UserId');
		$links = array();
		$edit = $attachment = FALSE;
// Check if this resource has attachments and display view icon accordingly
		if($this->session->getVar("setup_UserId") || !$this->session->getVar("setup_FileViewLoggedOnOnly"))
		{
			$this->db->formatConditions($this->db->formatFields('resourceattachmentsResourceId') . $this->db->equal . 
				$this->db->tidyInput($row['resourceId']));
			$this->db->formatConditions(array('resourceattachmentsEmbargo' => 'N'));
			$resultSet = $this->db->select('resource_attachments', 'resourceattachmentsId');
			if($this->db->numRows($resultSet))
				$attachment = TRUE;
		}
		if($row['resourcesummaryQuotes'] || $row['resourcesummaryParaphrases'])
		{
			if($attachment)
				$view = $this->icons->viewmetaAttach;
			else
				$view = $this->icons->viewmeta;
		}
		else if($row['resourcesummaryMusings'])
		{
			$view = FALSE;
// Public musings -- display available meta
			if($row['resourcemusingtextPrivate'] == 'N')
			{
				if($attachment)
					$view = $this->icons->viewmetaAttach;
				else
					$view = $this->icons->viewmeta;
			}
// logged on user and own musing
			else if($userId == $row['resourcemusingtextAddUserIdMusing'])
			{
				if($attachment)
					$view = $this->icons->viewmetaAttach;
				else
					$view = $this->icons->viewmeta;
			}
			if(!$view)
			{
				if($attachment)
					$view = $this->icons->viewAttach;
				else
					$view = $this->icons->view; // default
			}
		}
		else
		{
			if($attachment)
				$view = $this->icons->viewAttach;
			else
				$view = $this->icons->view; // default
		}
		$links['view'] = $this->html->a($this->icons->viewLink, $view, "index.php?action=resource_RESOURCEVIEW_CORE" . 
			htmlentities("&id=" . $row['resourceId']));
		if($this->session->getVar('setup_Listlink') == 'Y')
			$this->resourceLink = "index.php?action=resource_RESOURCEVIEW_CORE" . htmlentities("&id=" . $row['resourceId']);
		if($write && !$this->config->WIKINDX_ORIGINATOR_EDITONLY)
		{
			$links['edit'] = $this->html->a($this->icons->editLink, $this->icons->edit, 
				"index.php?action=resource_RESOURCEFORM_CORE&amp;type=edit" . htmlentities("&id=" . $row['resourceId']));
			$edit = TRUE;
		}
		else if($write && ($row['resourcemiscAddUserIdResource'] == $userId))
		{
			$links['edit'] = $this->html->a($this->icons->editLink, $this->icons->edit, 
				"index.php?action=resource_RESOURCEFORM_CORE&amp;type=edit" . htmlentities("&id=" . $row['resourceId']));
			$edit = TRUE;
		}
		if($superAdmin)
		{
			if(!$edit)
				$links['edit'] = $this->html->a($this->icons->editLink, $this->icons->edit, 
				"index.php?action=resource_RESOURCEFORM_CORE&amp;type=edit" . htmlentities("&id=" . $row['resourceId']));
			$links['delete'] = $this->html->a($this->icons->deleteLink, $this->icons->delete, 
				"index.php?action=admin_DELETERESOURCE_CORE" . 
				htmlentities('&function=deleteResourceConfirm&navigate=' . 
				$this->navigate . '&resource_id=' . $row['resourceId']));
		}
// display CMS link if required
// link is actually a JavaScript call
		if($this->session->getVar('setup_DisplayCmsLink'))
			$links['cms'] = $this->html->a('cmsLink', "CMS:&nbsp;" . $row['resourceId'], 
				"javascript:openCmsWindow('index.php?action=cms_CMS_CORE&amp;method=display" . "&amp;id=" . $row['resourceId'] . "')");
// display bibtex link if required
// link is actually a JavaScript call
		if($this->session->getVar('setup_DisplayBibtexLink'))
			$links['bibtex'] = $this->html->a($this->icons->bibtexLink, $this->icons->bibtex, 
				"javascript:openBibtexWindow('index.php?action=resource_VIEWBIBTEX_CORE&amp;method=display" . 
				"&amp;id=" . $row['resourceId'] . "')");
		return $links;
	}
/**
* Print radio buttons ascending, descending for ordering
*
* @param string $type
* @return string
*/
	public function displayAscDesc($type)
	{
		if($ascDesc = $this->session->getVar($type . "_AscDesc"))
		{
			if($ascDesc == 'ASC')
				return $this->form->radioButton(FALSE, $type . "_AscDesc", 'ASC', TRUE) . 
					$this->messages->text("list", "ascending") . 
					$this->html->br() . $this->form->radioButton(FALSE, $type . "_AscDesc", 'DESC') . 
					$this->messages->text("list", "descending");
			else
				return $this->form->radioButton(FALSE, $type . "_AscDesc", 'ASC') . 
					$this->messages->text("list", "ascending") . 
					$this->html->br() . $this->form->radioButton(FALSE, $type . "_AscDesc", 'DESC', TRUE) . 
					$this->messages->text("list", "descending");
		}
		else
			return $this->form->radioButton(FALSE, $type . "_AscDesc", 'ASC', TRUE) . 
				$this->messages->text("list", "ascending") . 
				$this->html->br() .$this->form->radioButton(FALSE, $type . "_AscDesc", 'DESC') . 
				$this->messages->text("list", "descending");
	}
/**
* Set the paging object if paging is alphabetic or not
*
* @param string $sql
* @param string $listType
* @param string $order
* @param string $queryString
* @param string $alphaQuery Default is FALSE
*/
	public function pagingStyle($sql, $listType, $order, $queryString, $alphaQuery = FALSE)
	{
		if(($this->session->getVar('setup_PagingStyle') == 'A') && 
			(($order == 'title') || ($order == 'creator') || ($order == 'attachments')))
		{
			$this->pagingObject = FACTORY_PAGINGALPHA::getInstance();
			$this->pagingObject->sqlTotal = $sql;
			$this->pagingObject->listType = $listType;
			$this->pagingObject->order = $order;
			$this->pagingObject->countAlphaQuery = $alphaQuery;
		}
		else
		{
			$this->pagingObject = FACTORY_PAGING::getInstance();
			$this->pagingObject->sqlTotal = $sql;
		}
		$this->pagingObject->queryString = $queryString;
		$this->pagingObject->getPaging();
	}
/**
* Check for user bibliographies
*
* @return array (usingBib, bibUserId, bibs)
*/
	private function getUserBib()
	{
		$usingBib = $bibUserId = FALSE;
		$bibs = array();
		$uBibs = $this->commonBib->getUserBibs();
		$gBibs = $this->commonBib->getGroupBibs();
		$bibs = array_merge($uBibs, $gBibs);
		$useBib = $this->session->getVar('mywikindx_Bibliography_use');
		if($useBib)
		{
			$this->db->formatConditions(array('userbibliographyId' => $useBib));
			$recordset = $this->db->select('user_bibliography', array('userbibliographyTitle', 'userbibliographyUserId'));
			$row = $this->db->fetchRow($recordset);
			$usingBib = stripslashes($row['userbibliographyTitle']);
			$bibUserId = $row['userbibliographyUserId'];
			if(array_key_exists($useBib, $bibs))
				unset($bibs[$useBib]); // Remove the currently used one from the list
		}
		return array($usingBib, $bibUserId, $bibs);
	}
/**
* Display list information and userBib, category and keyword select box to add items to
*
* @param string $listType
* @param boolean $resourcesExist
*/
	private function displayListInfo($listType, $resourcesExist = TRUE)
	{
		list($usingBib, $bibUserId, $bibs) = $this->getUserBib();
		if($usingBib)
			$linksInfo['info'] = $this->pagingObject->linksInfo($usingBib);
		else
			$linksInfo['info'] = $this->pagingObject->linksInfo();
		$linksInfo['params'] = $this->listParams($listType);
		if(!$resourcesExist)
		{
			$this->template->tpl->assign('resourceListInfo', $linksInfo);
			return;
		}
		if(!$this->session->getVar($listType . '_DisplayAttachment'))
		{
			$linksInfo['selectformheader'] = $this->form->formHeaderName('list_LISTADDTO_CORE', 'formSortingAddingListInfo', FALSE);
			$linksInfo['selectformfooter'] = $this->form->formEnd();
			$linksInfo['select'] = $this->createAddToBox($bibUserId, $bibs, $listType);
			if($listType == 'list')
			{
				if($this->session->getVar('list_SomeResources'))
					$formHeader = 'list_LISTSOMERESOURCES_CORE';
				else
					$formHeader = 'list_LISTRESOURCES_CORE';
				$linksInfo['reorder'] =
					$this->form->hidden("method", "reorder") . $this->displayAscDesc($listType) . 
					$this->html->br() . $this->form->formSubmit("Proceed", "onclick=\"document.forms['formSortingAddingListInfo'].elements['action'].value='$formHeader'\"");
			}
			else if($listType == 'basket')
				$linksInfo['reorder'] =
					$this->form->hidden("method", "reorder") . $this->displayAscDesc('list') . 
					$this->html->br() . $this->form->formSubmit("Proceed", "onclick=\"document.forms['formSortingAddingListInfo'].elements['action'].value='basket_BASKET_CORE'\"");
			else if(($listType == 'select') && $this->metadata)
				$linksInfo['reorder'] =
					$this->form->hidden("method", "reprocess") . $this->displayOrder($listType, TRUE) . 
					$this->html->br() . $this->form->formSubmit("Proceed", "onclick=\"document.forms['formSortingAddingListInfo'].elements['action'].value='list_SELECTMETADATA_CORE'\"");
			else if($listType == 'select')
				$linksInfo['reorder'] =
					$this->form->hidden("method", "reprocess") . $this->displayOrder($listType, TRUE) . 
					$this->html->br() . $this->form->formSubmit("Proceed", "onclick=\"document.forms['formSortingAddingListInfo'].elements['action'].value='list_SELECTRESOURCES_CORE'\"");
			else if($listType == 'statistics')
				$linksInfo['reorder'] =
					$this->form->hidden("method", "reprocess") . $this->displayOrder('select', TRUE) . 
					$this->html->br() . $this->form->formSubmit("Proceed", "onclick=\"document.forms['formSortingAddingListInfo'].elements['action'].value='list_SELECTRESOURCES_CORE'\"");
			else if($listType == 'search')
			{
				if($this->quickSearch)
				
					$linksInfo['reorder'] =
						$this->form->hidden("method", "reprocess") . $this->displayOrder($listType, TRUE) . 
						$this->html->br() . $this->form->formSubmit("Proceed", "onclick=\"document.forms['formSortingAddingListInfo'].elements['action'].value='list_QUICKSEARCH_CORE'\"");
				else
					$linksInfo['reorder'] =
						$this->form->hidden("method", "reprocess") . $this->displayOrder($listType, TRUE) . 
						$this->html->br() . $this->form->formSubmit("Proceed", "onclick=\"document.forms['formSortingAddingListInfo'].elements['action'].value='list_SEARCHRESOURCES_CORE'\"");
			}
		}
// display CMS link if required
// link is actually a JavaScript call
		if($this->session->getVar('setup_DisplayCmsLink'))
			$linksInfo['cms'] = $this->html->a('cmsLink', "CMS", 
				"javascript:openCmsWindow('index.php?action=cms_CMS_CORE&amp;method=displayList" . "')");
		$this->template->tpl->assign('resourceListInfo', $linksInfo);
	}
/**
* Create select box allowing users to add to categories, keywords etc.
*
* @param int $bibUserId
* @param array $bibs
* @param string $listType
* @return string 
*/
	private function createAddToBox($bibUserId, $bibs, $listType)
	{
		$categories = $this->db->numRows($this->db->select('category', 'categoryId'));
		$subcategories = $this->db->numRows($this->db->select('subcategory', 'subcategoryId'));
		$keywords = $this->db->numRows($this->db->select('keyword', 'keywordId'));
		$this->db->formatConditions(array('usertagsUserId' => $this->session->getVar('setup_UserId')));
		$usertags = $this->db->numRows($this->db->select('user_tags', 'usertagsId'));
		$languages = $this->db->numRows($this->db->select('language', 'languageId'));
		if(!empty($bibs))
			$array[0] = $this->messages->text("resources", "addToBib");
		if($this->session->getVar('setup_Write'))
		{
			if($categories > 1)
				$array[1] = $this->messages->text("resources", "addToCategory");
			if($subcategories)
				$array[5] = $this->messages->text("resources", "addToSubcategory");
			if($keywords)
				$array[2] = $this->messages->text("resources", "addToKeyword");
			if($usertags)
				$array[6] = $this->messages->text("resources", "addToUserTag");
			if($languages)
				$array[9] = $this->messages->text("resources", "addToLanguage");
		}
		if($this->session->getVar('setup_UserId') && ($this->session->getVar('setup_UserId') == $bibUserId))
			$array[3] = $this->messages->text('resources', 'deleteFromBib');
		else if($this->session->getVar("resourceSelectedTo") == '3') // previously operation was 'deleteFromBib'
			$this->session->delVar("resourceSelectedTo");
		if($this->session->getVar('setup_Superadmin'))
			$array[4] = $this->messages->text('resources', 'deleteResource');
		if($listType == 'basket')
			$array[8] = $this->messages->text('resources', 'basketRemove');
		else
			$array[7] = $this->messages->text('resources', 'basketAdd');
		if(!isset($array))
			return FALSE;
		$sessVar = $this->session->getVar("resourceSelectedTo");
		$checkAll = $this->messages->text("resources", "selectAll") . ':&nbsp;&nbsp;' . $this->form->checkbox(FALSE, "selectAll", FALSE);
		
		if($sessVar !== FALSE)
			$select = $this->form->selectedBoxValue($this->messages->text("resources", "withChecked"), "resourceSelectedTo", $array, $sessVar, 1)
				. $this->html->br() . $checkAll 
				. "&nbsp;&nbsp;" . $this->form->formSubmit("Proceed", "onclick=\"document.forms['formSortingAddingListInfo'].elements['action'].value='list_LISTADDTO_CORE';document.forms['formSortingAddingListInfo'].elements['method'].value='init';\"");
		else
			$select = $this->form->selectFBoxValue($this->messages->text("resources", "withChecked"), "resourceSelectedTo", $array, 1)
				. $this->html->br() . $checkAll
				. "&nbsp;&nbsp;" . $this->form->formSubmit("Proceed", "onclick=\"document.forms['formSortingAddingListInfo'].elements['action'].value='list_LISTADDTO_CORE';document.forms['formSortingAddingListInfo'].elements['method'].value='init';\"");
		
		return $select;
	}
/**
* Ordering options for select and quicksearch
*
* @param string $type
* @param boolean $reorder Default is FALSE
* @return string
*/
	public function displayOrder($type, $reorder = FALSE)
	{
		$order = array(
					"creator" => $this->messages->text("list", "creator"),
					"title" => $this->messages->text("list", "title"),
					"publisher" => $this->messages->text("list", "publisher"),
					"year" => $this->messages->text("list", "year"),
					"timestamp" => $this->messages->text("list", "timestamp"),
				);
		if(!$reorder)
			$size = '5';
		else
			$size = '2';
		if($selected = $this->session->getVar($type . "_Order"))
			$pString = $this->form->selectedBoxValue($this->messages->text("list", "order"), 
			$type . "_Order", $order, $selected, 1);
		else
			$pString = $this->form->selectFBoxValue($this->messages->text("list", "order"), 
			$type . "_Order", $order, 1);
		if(!$reorder)
			$pString .= $this->html->p($this->displayAscDesc($type));
		else
			$pString .= $this->html->br() . $this->displayAscDesc($type);
		return $pString;
	}
/**
* Display some information about the search/select/list parameters
*
* @param string $listType
* @return string
*/
	private function listParams($listType)
	{
		$strings = array();
// Bookmarked multi view?
		if($this->session->getVar('bookmark_MultiView'))
		{
			$strings = unserialize(base64_decode($this->session->getVar('sql_ListParams')));
			if(empty($strings))
				return FALSE;
			$this->session->delVar('bookmark_MultiView');
			return $this->messages->text('listParams', 'listParams') . 
				$this->html->br() . join($this->html->br(), $strings);
		}
		if(array_key_exists('statistics', $this->vars) && ($this->vars['statistics'] == 'Type'))
			$strings[] = $this->messages->text('listParams', 'type') . ':&nbsp;&nbsp;' . $this->vars['id'];
		else if($id = $this->session->getVar($listType . "_Type"))
		{
			$ids = explode(',', $id);
			if(sizeof($ids) > 1)
				$strings[] = $this->messages->text('listParams', 'type') . ':&nbsp;&nbsp;' . 
					$this->html->i($this->messages->text('listParams', 'listParamMultiple'));
			else
				$strings[] = $this->messages->text('listParams', 'type') . ':&nbsp;&nbsp;' . $id;
		}
		if($listType == 'select')
		{
			if($id = $this->session->getVar($listType . '_Tag'))
			{
				$ids = explode(',', $id);
				if(sizeof($ids) > 1)
					$strings[] = $this->messages->text('listParams', 'tag') . ':&nbsp;&nbsp;' . 
						$this->html->i($this->messages->text('listParams', 'listParamMultiple'));
				else
				{
					$this->db->formatConditions(array('tagId' => $id));
					$recordset = $this->db->select('tag', 'tagTag');
					$strings[] = $this->messages->text('listParams', 'tag') . ':&nbsp;&nbsp;' . 
						$this->html->dbToHtmlTidy($this->db->fetchOne($recordset));
				}
			}
			if($id = $this->session->getVar($listType . "_attachment"))
				$strings[] = $this->messages->text('listParams', 'attachment');
		}
		if(($listType == 'listCategory') || ($id = $this->session->getVar($listType . '_Category')))
		{
			if($listType == 'listCategory')
				$id = array_key_exists("id", $this->vars) ? 
					$this->vars["id"] : $this->session->getVar("list_Ids");
			$cats = explode(',', $id);
			if(sizeof($cats) > 1)
				$strings[] = $this->messages->text('listParams', 'category') . ':&nbsp;&nbsp;' . 
					$this->html->i($this->messages->text('listParams', 'listParamMultiple'));
			else
			{
				$this->db->formatConditions(array('categoryId' => $id));
				$recordset = $this->db->select('category', 'categoryCategory');
				$strings[] = $this->messages->text('listParams', 'category') . ':&nbsp;&nbsp;' . 
					$this->html->dbToHtmlTidy($this->db->fetchOne($recordset));
			}
		}
		if(($listType == 'listSubcategory') || ($id = $this->session->getVar($listType . '_Subcategory')))
		{
			if($listType == 'listSubcategory')
				$id = array_key_exists("id", $this->vars) ? 
					$this->vars["id"] : $this->session->getVar("list_Ids");
			$cats = explode(',', $id);
			if(sizeof($cats) > 1)
				$strings[] = $this->messages->text('listParams', 'subcategory') . ':&nbsp;&nbsp;' . 
					$this->html->i($this->messages->text('listParams', 'listParamMultiple'));
			else
			{
				$this->db->formatConditions(array('subcategoryId' => $id));
				$recordset = $this->db->select('subcategory', 'subcategorySubcategory');
				$strings[] = $this->messages->text('listParams', 'subcategory') . ':&nbsp;&nbsp;' . 
					$this->html->dbToHtmlTidy($this->db->fetchOne($recordset));
			}
		}
		if(($listType == 'listUserTag') || ($id = $this->session->getVar($listType . '_UserTag')))
		{
			if($listType == 'listUserTag')
				$id = array_key_exists("id", $this->vars) ? 
					$this->vars["id"] : $this->session->getVar("list_Ids");
			$cats = explode(',', $id);
			if(sizeof($cats) > 1)
				$strings[] = $this->messages->text('listParams', 'userTag') . ':&nbsp;&nbsp;' . 
					$this->html->i($this->messages->text('listParams', 'listParamMultiple'));
			else
			{
				$this->db->formatConditions(array('usertagsId' => $id));
				$recordset = $this->db->select('user_tags', 'usertagsTag');
				$strings[] = $this->messages->text('listParams', 'userTag') . ':&nbsp;&nbsp;' . 
					$this->html->dbToHtmlTidy($this->db->fetchOne($recordset));
			}
		}
		if(($listType == 'listCollection') || ($id = $this->session->getVar($listType . '_Collection')))
		{
			if($listType == 'listCollection')
				$id = array_key_exists("id", $this->vars) ? 
					$this->vars["id"] : $this->session->getVar("list_Ids");
			$cats = explode(',', $id);
			if(sizeof($cats) > 1)
				$strings[] = $this->messages->text('listParams', 'collection') . ':&nbsp;&nbsp;' . 
					$this->html->i($this->messages->text('listParams', 'listParamMultiple'));
			else
			{
				$this->db->formatConditions(array('collectionId' => $id));
				$recordset = $this->db->select('collection', 'collectionTitle');
				$strings[] = $this->messages->text('listParams', 'collection') . ':&nbsp;&nbsp;' . 
					$this->html->dbToHtmlTidy($this->db->fetchOne($recordset));
			}
		}
		if(($listType == 'listPublisher') || ($id = $this->session->getVar($listType . '_Publisher')))
		{
			if($listType == 'listPublisher')
				$id = array_key_exists("id", $this->vars) ? 
					$this->vars["id"] : $this->session->getVar("list_Ids");
			$cats = explode(',', $id);
			if(sizeof($cats) > 1)
				$strings[] = $this->messages->text('listParams', 'publisher') . ':&nbsp;&nbsp;' . 
					$this->html->i($this->messages->text('listParams', 'listParamMultiple'));
			else
			{
				$this->db->formatConditions(array('publisherId' => $id));
				$recordset = $this->db->select('publisher', array('publisherName', 'publisherLocation'));
				$row = $this->db->fetchRow($recordset);
				if($row['publisherLocation'])
					$loc = ' (' . stripslashes($row['publisherLocation']) . ')';
				else
					$loc = FALSE;
				$publisher = stripslashes($row['publisherName']) . $loc;
				$strings[] = $this->messages->text('listParams', 'publisher') . ':&nbsp;&nbsp;' . 
					$this->html->dbToHtmlTidy($publisher);
			}
		}
		if(($listType == 'listKeyword') || ($id = $this->session->getVar($listType . '_Keyword')))
		{
			if($listType == 'listKeyword')
				$id = array_key_exists("id", $this->vars) ? 
					$this->vars["id"] : $this->session->getVar("list_Ids");
			$ids = explode(',', $id);
			if(sizeof($ids) > 1)
				$strings[] = $this->messages->text('listParams', 'keyword') . ':&nbsp;&nbsp;' . 
					$this->html->i($this->messages->text('listParams', 'listParamMultiple'));
			else
			{
				$this->db->formatConditions(array('keywordId' => $id));
				$recordset = $this->db->select('keyword', 'keywordKeyword');
				$strings[] = $this->messages->text('listParams', 'keyword') . ':&nbsp;&nbsp;' . 
					$this->html->dbToHtmlTidy($this->db->fetchOne($recordset));
			}
		}
		if($id = $this->session->getVar($listType . '_Language'))
		{
			$ids = explode(',', $id);
			if(sizeof($ids) > 1)
				$strings[] = $this->messages->text('listParams', 'language') . ':&nbsp;&nbsp;' . 
					$this->html->i($this->messages->text('listParams', 'listParamMultiple'));
			else
			{
				$this->db->formatConditions(array('languageId' => $id));
				$recordset = $this->db->select('language', 'languageLanguage');
				$strings[] = $this->messages->text('listParams', 'language') . ':&nbsp;&nbsp;' . 
					$this->html->dbToHtmlTidy($this->db->fetchOne($recordset));
			}
		}
		if(($listType == 'listCreator') || ($listType == 'select'))
		{
			if($id = $this->session->getVar($listType . '_BibId'))
			{
				$this->db->formatConditions(array('userbibliographyId' => $id));
				$recordset = $this->db->select('user_bibliography', 'userbibliographyTitle');
				$strings[] = $this->messages->text('listParams', 'notInUserBib') . ':&nbsp;&nbsp;' . 
					$this->html->dbToHtmlTidy($this->db->fetchOne($recordset));
			}
			if(($listType == 'listCreator') || ($id = $this->session->getVar($listType . '_Creator')))
			{
				if($listType == 'listCreator')
					$id = array_key_exists("id", $this->vars) ? 
					$this->vars["id"] : $this->session->getVar("list_Ids");
				$ids = explode(',', $id);
				if(sizeof($ids) > 1)
					$strings[] = $this->messages->text('listParams', 'creator') . ':&nbsp;&nbsp;' . 
						$this->html->i($this->messages->text('listParams', 'listParamMultiple'));
				else
				{
					$this->db->formatConditions(array('creatorId' => $id));
					$recordset = $this->db->select('creator', array('creatorPrefix', 'creatorSurname'));
					$row = $this->db->fetchRow($recordset);
					$name = $row['creatorPrefix'] ? $row['creatorPrefix'] . ' ' . 
						$row['creatorSurname'] : $row['creatorSurname'];
					$strings[] = $this->messages->text('listParams', 'creator') . ':&nbsp;&nbsp;' . 
						$this->html->dbToHtmlTidy($name);
				}
			}
		}
		else if($listType == 'search')
		{
			if($id = $this->session->getVar($listType . '_Field'))
			{
				$ids = explode(',', $id);
				if(sizeof($ids) > 1)
					$strings[] = $this->messages->text('listParams', 'field') . ':&nbsp;&nbsp;' . 
						$this->html->i($this->messages->text('listParams', 'listParamMultiple'));
				else
				{
					if(strpos($id, 'Custom_') === 0)
					{
						$customField = explode('_', $id);
						$this->db->formatConditions(array('customId' => $customField[2]));
						$id = $this->db->fetchOne($this->db->select('custom', 'customLabel'));
					}
					$strings[] = $this->messages->text('listParams', 'field') . ':&nbsp;&nbsp;' . $id;
				}
			}
			if($id = $this->session->getVar($listType . '_Word'))
			{
				if($this->session->getVar($listType . '_Partial') == 'on')
					$id .= "&nbsp;(" . $this->messages->text('listParams', 'partial') . ")";
				$strings[] = $this->messages->text('listParams', 'word') . ':&nbsp;&nbsp;' . $this->html->dbToHtmlTidy($id);
			}
		}
		if(empty($strings))
		{
			$this->session->delVar('sql_ListParams');
			return FALSE;
		}
		$this->session->setVar('sql_ListParams', base64_encode(serialize($strings)));
		return $this->messages->text('listParams', 'listParams') . $this->html->br() . join($this->html->br(), $strings);
	}
}
?>
