<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* PAGING Numeric paging of lists
* 
* @version	1
*
*	@package wikindx4\core\lists
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class PAGING
{
/** object */
private $db;
/** array */
private $vars;
/** object */
private $session;
/** object */
private $messages;
/** object */
private $html;
/** object */
private $template;
/** int */
public $total;
/** string */
public $sqlTotal;
/** int */
public $paging;
/** int */
public $start;
/** string */
public $queryString;
/** string */
public $whereStmt = FALSE;

/**
*	PAGING
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->session = FACTORY_SESSION::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
	}
/**
* grab paging from session
*/
	public function getPaging()
	{
		$this->getPagingStart();
		$this->total = $this->db->fetchOne($this->sqlTotal);
		$this->paging = $this->session->getVar('setup_Paging');
		$this->maxLinks = $this->session->getVar('setup_PagingMaxLinks');
		$this->maxLinksHalf = round($this->maxLinks/2);
// Has the paging limit been overriden in user preferences for this selection?
		if($this->paging == -1)
			return;
		$this->createLinks();
	}
/**
* get where to start display from
*/
	public function getPagingStart()
	{
		if(!array_key_exists('PagingStart', $this->vars))
		{
			if($start = $this->session->getVar('mywikindx_PagingStart'))
				$this->start = $start;
			else
				$this->start = 0;
		}
		else
		{
			$this->session->setVar('mywikindx_PagingStart', $this->vars['PagingStart']);
			$this->start = $this->vars['PagingStart'];
		}
	}
/**
* Links at on display screen to move to more resources.
*
* $this->total is total resources found for this operation.
* $this->start is where we currently are.
*/
	private function createLinks()
	{
		if(($this->paging <= 0) || ($this->total <= $this->paging))
			return;
		$end = $advanced = 0;
		$index = $maxLinks = 1;
		$advance = $this->start;
		if(strpos($this->queryString, '?') !== FALSE)
			$rootFile = FALSE;
		else
			$rootFile = 'index.php?';
		while($advance >= (($this->maxLinksHalf * $this->paging) - $this->paging))
		{
			$end += $this->paging;
			$index += $this->paging;
			$advance -= $this->paging;
			$advanced++;
		}
		if($advanced)
		{
			$links[] = $this->html->a("page", $this->messages->text("resources", "pagingStart"), 
				$rootFile . htmlentities($this->queryString . "&PagingStart=0"));
			$maxLinks++;
		}
		while($index <= $this->total)
		{
			if($maxLinks++ >= $this->maxLinks)
				break;
			$end += $this->paging;
			if($end > $this->total)
				$end = $this->total;
			$start = $index - 1;
			$link = htmlentities($this->queryString . "&PagingStart=$start");
			$name = $index . " - " . $end;
			if($this->start == $start)
				$links[] = $name;
			else
				$links[] = $this->html->a("page", $name, $rootFile . $link);
			$index += $this->paging;
		}
		if($end < $this->total)
		{
			if($this->start && sizeof($links) == 1)
			{
				$links = array($this->html->a("page", $this->messages->text("resources", "pagingStart"), 
					$rootFile . htmlentities($this->queryString . "&PagingStart=0")));
			}
			else if (sizeof($links) > 1)
			{
				$start = $this->total - ($this->total % $this->paging);
				if($start == $this->total)
					$start = $this->total - $this->paging;
				$links[] = $this->html->a("page", $this->messages->text("resources", "pagingEnd"), 
					$rootFile . htmlentities($this->queryString . "&PagingStart=$start"));
			}
		}
		$this->template->tpl->assign('pagingList', $links);
	}
/**
* Format display information string
*
* @param string $bibTitle Default is FALSE
* @return string 
*/
	public function linksInfo($bibTitle = FALSE)
	{
		if(!$this->total)
			return $this->messages->text("resources", "noResult");
		$displayEnd = $this->start + $this->paging;
		if(($this->paging <= 0) || ($displayEnd > $this->total))
			$displayEnd = $this->total;
		$displayStart = $this->start + 1;
		$bib = FALSE;
		if($bibTitle)
		{
			$bib = ' (' . $this->messages->text("user", "bibliography") . ': ' . 
			$this->html->dbToHtmlTidy($bibTitle) . ")";
		}
		else if($this->session->getVar('setup_MultiUser'))
			$bib = ' (' . $this->messages->text("user", "bibliography") . ': ' . 
				$this->messages->text("user", "masterBib") . ")";
		return $this->messages->text("hint", "pagingInfo", " $displayStart - $displayEnd&nbsp;") . 
			$this->messages->text("hint", "pagingInfoOf", $this->total . $bib);
	}
}
?>
