<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* PAGING Alphabetic paging of lists -- only used when ordering by creator or title or attachment file name
* 
* @version	1
*
*	@package wikindx4\core\lists
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class PAGINGALPHA
{
/** object */
private $db;
/** array */
private $vars;
/** object */
private $session;
/** object */
private $messages;
/** object */
private $html;
/** object */
private $template;
/** array */
public $pagingArray;
/** int */
public $sizePA;
/** string */
public $sqlTotal;
/** string */
public $countAlphaQuery;
/** int */
public $total;
/** int */
public $start;
/** string */
public $queryString;
/** string */
public $listType;   // set in LISTCOMMON::pagingStyle
/** string */
public $order;   // set in LISTCOMMON::pagingStyle
/** boolean */
public $nullFound = FALSE;

/**
*	PAGING
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->session = FACTORY_SESSION::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
// last element is for creators or titles beginning with a non-English alphabetic character 
		$this->pagingArray = range('A', 'Z');
		$this->sizeOfPA = sizeof($this->pagingArray);
		array_push($this->pagingArray, '??');
		$this->getPagingStart();
	}
/**
* grab paging either from default or from session
*/
	public function getPaging()
	{
		$this->grabTotal();
		if($this->nullFound && ($this->order == 'creator'))
			array_unshift($this->pagingArray, '--'); // NULL creators
		if(empty($this->pagingArray))
			return;
		if(!array_key_exists($this->start, $this->pagingArray))
			$this->start = array_shift(array_keys($this->pagingArray));
/*		if($this->order == 'attachments')//// NB NB NB NB  To be done!
		{
			if($this->start > $this->sizeOfPA)
				$this->start = 0;
			if($this->start == $this->sizeOfPA) // '--'  force this to 26th element and display all non-alpha attachments at the end
				$this->db->condition[] = $this->db->upper(
				$this->db->replace($this->db->replace($this->db->replace($this->db->replace('resourceattachmentsFileName', 
				'{', ''), '}', '', FALSE), '"', '', FALSE), '\\\'', '', FALSE), FALSE) . ' ' . 
				$this->db->regexp(FALSE, '^[^A-Z]', FALSE);
			else
				$this->db->condition[] = $this->db->upper($this->db->formatFields('resourceattachmentsFileName'), FALSE) . ' ' . 
					$this->db->like(FALSE, $this->pagingArray[$this->start], '%');
		}
*/		$this->createLinks();
	}
/**
* Remove elements of $this->pagingArray where there are no resources and grab total resources
*/
	private function grabTotal()
	{
		if($this->order == 'creator')
			$tempPagingArray = array_fill(0, $this->sizeOfPA + 1, ''); // 'creator' separately lists resources without a creator to non-alpha creator surnames
		else
			$tempPagingArray = array_fill(0, $this->sizeOfPA + 1, '');
		$this->total = $this->db->fetchOne($this->sqlTotal);
		$count = 0;
		while($row = $this->db->fetchRow($this->countAlphaQuery))
		{
			$count++;
			if($this->order == 'title')
				$char = strtoupper(substr($row['resourceTitleSort'], 0, 1));
			else if($this->order == 'creator')
			{
				if($row['creatorSurname'])
					$char = strtoupper(substr($row['creatorSurname'], 0, 1));
				else
				{
					$this->nullFound = TRUE;
					continue;
				}
			}
			else if($this->order == 'attachments')
				$char = strtoupper(substr($row['resourceattachmentsFileName'], 0, 1));
			if(array_search($char, $tempPagingArray) !== FALSE) // already noted
				continue;
			if(($index = array_search($char, $this->pagingArray)) !== FALSE)
			{
				$tempPagingArray[$index] = $this->pagingArray[$index];
				continue;
			}
// If we get here, we have a creator or title with a non-English alphabetic character initially
			$tempPagingArray[sizeof($tempPagingArray) - 1] = '??';
		}
		foreach($tempPagingArray as $index => $element)
		{
			if(!$element)
				unset($this->pagingArray[$index]);
		}
	}
/**
* Links on display screen to move to more resources.
*
* $this->total is total resources found for this operation.
* $this->start is where we currently are.
*/
	private function createLinks()
	{
		if(sizeof($this->pagingArray) <= 1)
			return FALSE;
		if(strpos($this->queryString, '?') !== FALSE)
			$rootFile = FALSE;
		else
			$rootFile = 'index.php?';
		foreach($this->pagingArray as $index => $char)
		{
			if($this->start == $index)
				$links[] = $char;
			else
			{
				$link = htmlentities($this->queryString . "&PagingStart=$index");
				$links[] = $this->html->a("page", "&nbsp;&nbsp;$char&nbsp;&nbsp;", $rootFile . $link);
			}
		}
		if($this->session->getVar($this->listType . '_AscDesc') == 'DESC')
			$links = array_reverse($links);
		$this->template->tpl->assign('pagingList', $links);
	}
/**
* Format display information string
*
* @param string $bibTitle Default is FALSE
* @return string 
*/
	public function linksInfo($bibTitle = FALSE)
	{
		if(!$this->total)
			return $this->messages->text("resources", "noResult");
		$bib = FALSE;
		if(sizeof($this->pagingArray) == 1)
			$num = $this->total;
		else
			$num = "'" . $this->pagingArray[$this->start] . "'";
		if($bibTitle)
		{
			$bib = " (" . $this->messages->text("user", "bibliography") . ": " . 
			$this->html->dbToHtmlTidy($bibTitle) . ")";
		}
		else if($this->session->getVar('setup_MultiUser'))
			$bib = " (" . $this->messages->text("user", "bibliography") . ": " . 
				$this->messages->text("user", "masterBib") . ")";
		return $this->messages->text("hint", "pagingInfo", " $num&nbsp;") . 
			$this->messages->text("hint", "pagingInfoOf", $this->total . $bib);
	}
/**
* get paging start
*/
	public function getPagingStart()
	{
		if(!array_key_exists('PagingStart', $this->vars))
		{
			if($start = $this->session->getVar('mywikindx_PagingStart'))
				$this->start = $start;
			else
				$this->start = 0;
		}
		else
		{
			$this->session->setVar('mywikindx_PagingStart', $this->vars['PagingStart']);
			$this->start = $this->vars['PagingStart'];
		}
	}
}
?>
