<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* PLUGINMESSAGES
*
* Handle plugin localizations
* 
* @version	1
*
*	@package wikindx4\core\messages
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class PLUGINMESSAGES
{
/** string */
private $pluginDir;
/** string */
private $pluginFile;
/** string */
private $language;
/** obect */
private $utf8;
/** object */
private $session;
/** array */
private $languageArray;
/** array */
private $messages = array();
/** array */
private $englishMessages = array();

/**
 * PLUGINMESSAGES
 *
 * @param string $pluginDir
 * @param string $pluginFile
 */
	public function __construct($pluginDir, $pluginFile)
	{
		$this->pluginFile = $pluginFile;
		$this->pluginDir = $pluginDir;
		$this->session = FACTORY_SESSION::getInstance();
		if(!$this->language = $this->session->getVar("setup_Language"))
			$this->language = "en";
		$this->utf8 = FACTORY_UTF8::getInstance();
		$this->loadLanguage();
		$this->loadEnglish();
	}
/**
* Grab the localized message
*
* @param string $indexName
* @param string $extra Optional string that replaces '###' in the array element value string. Default is FALSE
* @return string
*/
	public function text($indexName, $extra = FALSE)
	{
		if(!array_key_exists($indexName, $this->messages))
		{
			if(!array_key_exists($indexName, $this->englishMessages))
				die("Message <b>$indexName</b> not found in English language pack.<P>");
			else
				$message = $this->englishMessages["$indexName"];
		}
		else
			$message = $this->messages["$indexName"];
		$pString = $extra !== FALSE ? 
			preg_replace("/###/", $this->utf8->smartUtf8_decode(trim($extra)), $message) 
			: 
			preg_replace("/###/", '', $message);
		return $this->utf8->encodeUtf8(trim($pString));
	}
/**
* Load English message arrays in case of missing localized messages
*/
	private function loadEnglish()
	{
		$file = "plugins/" . $this->pluginDir . "/" . $this->pluginFile . '_en.php';
		if(!file_exists($file))
			return;
		include_once($file);
		$class = $this->pluginFile . "_en";
		$languageClass= new $class();
		$this->englishMessages = $languageClass->text;
	}
/**
Load localized language array
*/
	private function loadLanguage()
	{
		$file = "plugins/" . $this->pluginDir . "/" . $this->pluginFile . '_' . $this->language . '.php';
		if(!file_exists($file))
			return;
		include_once($file);
		$className = $this->pluginFile . "_" . $this->language;
		$languageClass = new $className;
		$this->messages = $languageClass->text;
	}
}