<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* ATTACHMENT
*
* Handle localizations
* 
* @version	1
*
*	@package wikindx4\core\miscellaneous
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class ATTACHMENT
{
/** object */
private $db;
/** array */
private $vars;
/** object */
private $html;
/** object */
private $form;
/** string */
public $primary = FALSE;

/**
* ATTACHMENT
*
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->html = FACTORY_HTML::getInstance();
		$this->form = FACTORY_FORM::getInstance();
	}
/**
* Show list of attachments as downloadable links.
*
* @param array $idArray
* @param boolean $zip If TRUE, return array of hash filenames ready for tarring and zipping. Default is FALSE
* @return array
*/
	public function attachedFileNames($idArray, $zip = FALSE)
	{
		$linkClass = "link";
		$zipFiles = $allFiles = array();
		foreach($idArray as $id)
		{
			$files = $this->listFiles($id);
			foreach($files as $hash => $fileName)
			{
				if($zip && !empty($files))
				{
					$zipFiles[$hash] = $fileName;
					continue;
				}
				$fileName = stripslashes($fileName);
				$allFiles[strtolower($fileName)] = $this->html->a($linkClass, $fileName, 
					"index.php?action=attachments_ATTACHMENTS_CORE" . 
					htmlentities("&method=downloadAttachment&resourceId=$id&hash=$hash&filename=$fileName"), "_new");
			}
		}
		if(!empty($zipFiles))
			return $zipFiles;
		if(empty($allFiles))
			return FALSE;
		return $allFiles;
	}
/**
* Make hyperlink of attachment
*
* @param array $row
* @param boolean $list
* @param boolean $reduce
* @param boolean $hyperlink
* @return string
*/
	public function makeLink($row, $list = FALSE, $reduce = TRUE, $hyperlink = TRUE)
	{
		$id = $row['resourceattachmentsId'];
		$icons = FACTORY_LOADICONS::getInstance();
		if(!$list)
		{
			if(!$hyperlink)
				return "index.php?action=attachments_ATTACHMENTS_CORE" . 
					htmlentities("&method=downloadAttachment&id=$id&filename=" . $row['resourceattachmentsHashFilename']);
			return $this->html->a($icons->fileLink, $icons->file, "index.php?action=attachments_ATTACHMENTS_CORE" . 
				htmlentities("&method=downloadAttachment&id=$id&filename=" . $row['resourceattachmentsHashFilename']), "_new");
		}
		else
		{
			if(!$row['resourceattachmentsFileName'])
			{
				$name = $icons->file;
				$label = FALSE;
			}
			else
			{
				if($reduce)
					$name = $this->form->reduceLongText($this->html->dbToHtmlTidy($row['resourceattachmentsFileName']));
				else
					$name = $this->html->dbToHtmlTidy($row['resourceattachmentsFileName']);
			}
			if(!$hyperlink)
				return "index.php?action=attachments_ATTACHMENTS_CORE" . 
					htmlentities("&method=downloadAttachment&id=$id&filename=" . $row['resourceattachmentsHashFilename']);
			return $this->html->a('link', $name, "index.php?action=attachments_ATTACHMENTS_CORE" . 
				htmlentities("&method=downloadAttachment&id=$id&filename=" . $row['resourceattachmentsHashFilename']), "_new");
		}
	}
/**
* List all attachments for this resource
*
* @param int $resourceId
* @return array
*/
	public function listFiles($resourceId)
	{
		$array = array();
		$this->db->formatConditions(array('resourceattachmentsResourceId' => $resourceId));
		$this->db->orderBy('resourceattachmentsFileName');
		$recordSet = $this->db->select('resource_attachments', array('resourceattachmentsHashFilename', 
			'resourceattachmentsFileName', 'resourceattachmentsPrimary'));
		while($row = $this->db->fetchRow($recordSet))
		{
			$array[$row['resourceattachmentsHashFilename']] = $this->html->dbToFormTidy($row['resourceattachmentsFileName']);
			if($row['resourceattachmentsPrimary'] == 'Y')
				$this->primary = $row['resourceattachmentsHashFilename'];
		}
		return $array;
	}
/**
* Increment the accesses and downloads counter for this resource
*
* @param int $id
*/
	public function incrementDownloadCounter($id)
	{
		$this->db->formatConditions(array('resourceattachmentsId' => $id));
		$this->db->updateSingle('resource_attachments', 
			$this->db->formatFields('resourceattachmentsDownloads') . "=" . 
			$this->db->formatFields('resourceattachmentsDownloads') . "+" . $this->db->tidyInput(1));
		$this->db->formatConditions(array('resourceattachmentsId' => $id));
		$this->db->updateSingle('resource_attachments', 
			$this->db->formatFields('resourceattachmentsDownloadsPeriod') . "=" . 
			$this->db->formatFields('resourceattachmentsDownloadsPeriod') . "+" . $this->db->tidyInput(1));
	}
}
?>
