<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
*	ADMINCATEGORIES class.
*
*	Administration of categories and subcategories
*****/
class ADMINCATEGORIES
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $config;
private $form;
private $keyword;
private $html;
private $gatekeep;
private $badInput;
private $categories;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->category = FACTORY_CATEGORY::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->gatekeep->requireSuper = TRUE;
		$this->gatekeep->init();
		$this->session->clearArray('edit');
		$this->categories = $this->category->grabAll();
// Administering 'General' category (key == 1) is not allowed.
		unset($this->categories[1]);
	}
// Add a category - display options.
	public function addInit($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "add", " (" . 
			$this->messages->text("resources", "categories") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message, "error", "center");
		$pString .= $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
		$pString .= $this->form->hidden("method", "add");
		$pString .= $this->form->textInput(FALSE, "categoryAdd", FALSE, 30, 255);
		$pString .= $this->form->formSubmit('Add');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Add a subcategory - display options.
	public function addSubInit($message = FALSE)
	{
		$table = FACTORY_TABLE::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "add", " (" . 
			$this->messages->text("resources", "subcategories") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message, "error", "center");
		$pString .= $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
		$pString .= $this->form->hidden("method", "addSub");
		$pString .= $table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $table->trStart();
		$pString .= $table->td($this->form->textInput(FALSE, "subcategoryAdd", FALSE, 30, 255));
		$categories = $this->category->grabAll();
		$pString .= $table->td($this->form->selectFBoxValue($this->messages->text('resources', 'subcategoryPart'), 
			'categoryId', $categories, 20));
		$pString .= $table->trEnd();
		$pString .= $table->tableEnd();
		$pString .= $this->form->formSubmit('Add');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Add a category
	public function add()
	{
		if(!$input = $this->validateInput('add'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'addInit');
// database match is case insensitive.
		$this->db->formatConditions(array('categoryCategory' => $input));
		$recordset = $this->db->select('category', 'categoryId');
		$pString = $this->success->text("category", " " . $this->messages->text("misc", "added") . " ");
// If category already exists quietly return without error.
		if($id = $this->db->fetchOne($recordset))
			return $this->addInit($pString);
		$this->db->insert('category', 'categoryCategory', $input);
		$this->categories = $this->category->grabAll();
		unset($this->categories[1]);
		return $this->addInit($pString);
	}
// Add a subcategory
	public function addSub()
	{
		if(!$input = $this->validateInput('addSub'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'addSubInit');
// database match is case insensitive.
		$this->db->formatConditions(array('subcategorySubcategory' => $input['subcategoryAdd']));
		$this->db->formatConditions(array('subcategoryCategoryId' => $input['categoryId']));
		$recordset = $this->db->select('subcategory', 'subcategoryId');
		$pString = $this->success->text("subcategory", " " . $this->messages->text("misc", "added") . " ");
// If subcategory already exists quietly return without error.
		if($id = $this->db->fetchOne($recordset))
			return $this->addSubInit($pString);
		$fields[] = 'subcategorySubcategory';
		$values[] = $input['subcategoryAdd'];
		$fields[] = 'subcategoryCategoryId';
		$values[] = $input['categoryId'];
		$this->db->insert('subcategory', $fields, $values);
		return $this->addSubInit($pString);
	}
// Delete categories display.
	public function deleteInit($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "delete2", " (" . 
			$this->messages->text("resources", "categories") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message, "error", "center");
		if(empty($this->categories))
		{
			GLOBALS::buildOutputString($pString . $this->messages->text("misc", "noCategories"));
			return;
		}
		$pString .= $this->html->p($this->messages->text("category", "deleteWarning"));
		$pString .= $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
		$pString .= $this->form->hidden("method", "deleteConfirm");
		foreach($this->categories as $key => $value)
			$pString .= $this->form->checkbox(FALSE, "categoryDelete_" . $key) . " $value" . $this->html->br();
		$pString .= $this->html->br() . $this->form->formSubmit('Delete');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Delete subcategories display.
	public function deleteSubInit($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "delete2", " (" . 
			$this->messages->text("resources", "subcategories") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message, "error", "center");
		$subcategories = $this->category->grabSubAll();
		if(empty($subcategories))
		{
			GLOBALS::buildOutputString($pString . $this->messages->text("misc", "noSubcategories"));
			return;
		}
		$subcategories = $this->category->grabSubAll(TRUE);
		$pString .= $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
		$pString .= $this->form->hidden("method", "deleteSubConfirm");
		foreach($subcategories as $key => $value)
			$pString .= $this->form->checkbox(FALSE, "subcategoryDelete_" . $key) . " $value" . $this->html->br();
		$pString .= $this->html->br() . $this->form->formSubmit('Delete');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Ask for confirmation of delete categories
	public function deleteConfirm()
	{
		if(!$input = $this->validateInput('delete'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'deleteInit');
		GLOBALS::setOutputHeading($this->messages->text("heading", "delete2", " (" . 
			$this->messages->text("resources", "categories") . ")"));
		$categories = "'" . implode("', '", array_keys(array_intersect(array_flip($this->categories), $input))) . "'";
		$categories = html_entity_decode($categories);
		$pString = $this->html->p($this->messages->text("category", "deleteConfirm", ": $categories"));
		$pString .= $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
		$pString .= $this->form->hidden("method", "delete");
		foreach($input as $id)
			$pString .= $this->form->hidden("categoryDelete_" . $id, $id);
		$pString .= $this->html->br() . $this->form->formSubmit('Confirm');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Ask for confirmation of delete subcategories
	public function deleteSubConfirm()
	{
		if(!$input = $this->validateInput('deleteSub'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'deleteSubInit');
		GLOBALS::setOutputHeading($this->messages->text("heading", "delete2", " (" . 
			$this->messages->text("resources", "subcategories") . ")"));
		$this->db->formatConditionsOneField($input, 'subcategoryId');
		$recordset = $this->db->select('subcategory', 'subcategorySubCategory');
		while($row = $this->db->fetchRow($recordset))
			$list[] = $row['subcategorySubCategory'];
		$subcategories = html_entity_decode(implode(', ', $list));
		$pString = $this->html->p($this->messages->text("category", "deleteConfirmSub", ": $subcategories"));
		$pString .= $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
		$pString .= $this->form->hidden("method", "deleteSub");
		foreach($input as $id)
			$pString .= $this->form->hidden("subcategoryDelete_" . $id, $id);
		$pString .= $this->html->br() . $this->form->formSubmit('Confirm');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Delete categories
	public function delete()
	{
		if(!$input = $this->validateInput('delete'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'deleteInit');
// ensure that category 1 'General' is never deleted
		if($key = array_search(1, $input))
			unset($input[$key]);
		if(!$this->deleteSql($input))
			$this->badInput($this->errors->text("inputError", "invalid"), 'deleteInit');
		$pString = $this->success->text("category", " " . $this->messages->text("misc", "deleted") . " ");
		$this->categories = $this->category->grabAll();
		unset($this->categories[1]);
		return $this->deleteInit($pString);
	}
// Delete subcategories
	public function deleteSub()
	{
		if(!$input = $this->validateInput('deleteSub'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'deleteSubInit');
		if(!$this->deleteSubSql($input))
			$this->badInput($this->errors->text("inputError", "invalid"), 'deleteSubInit');
		$pString = $this->success->text("category", " " . $this->messages->text("misc", "deleted") . " ");
		return $this->deleteSubInit($pString);
	}
// Delete category(s) from category table.  $input is array of ids in WKX_category.
// 'General' category always has id 1.
	private function deleteSql($input)
	{
// Delete resource_category rows having the category IDs
		$this->db->formatConditionsOneField($input, 'resourcecategoryCategoryId');
		$this->db->delete('resource_category');
// Find subcategories that are part of the categories and delete those rows from resource_category
		$this->db->formatConditionsOneField($input, 'subcategoryCategoryId');
		$recordset = $this->db->select('subcategory', 'subcategoryId');
		while($row = $this->db->fetchRow($recordset))
			$subcatIds[] = $row['subcategoryId'];
		if(isset($subcatIds))
		{
			$this->db->formatConditionsOneField($subcatIds, 'resourcecategorySubcategoryId');
			$this->db->delete('resource_category');
		}
// Ensure that all resources not listed in resource_category have at least the 'General' category
		$subStmt = $this->db->subQuery($this->db->selectNoExecute('resource_category', 'resourcecategoryResourceId'), FALSE, FALSE, TRUE);
		$this->db->formatConditions($this->db->formatFields('resourceId'). $this->db->inClause($subStmt, TRUE));
		$recordset = $this->db->select('resource', 'resourceId');
		while($row = $this->db->fetchRow($recordset))
			$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategoryCategoryId'), array($row['resourceId'], '1'));
// Finally delete categories and any subcategories and optimize tables
		foreach($input as $value)
		{
			$this->db->formatConditions(array('categoryId' => $value));
			$this->db->delete('category');
			$this->db->formatConditions(array('subcategoryCategoryId' => $value));
			$this->db->delete('subcategory');
		}
		$this->db->optimize('category');
		$this->db->optimize('subcategory');
		$this->db->optimize('resource_category');
		return TRUE;
	}
// Delete subcategory(s) from subcategory table and from subcategories field in WKX_resource_category.  
// $input is array of ids in WKX_subcategory.
	private function deleteSubSql($input)
	{
// Delete resource_category rows having the subcategory IDs
		$this->db->formatConditionsOneField($input, 'resourcecategorySubcategoryId');
		$this->db->delete('resource_category');
// Delete subcategories and optimize tables
		foreach($input as $value)
		{
			$this->db->formatConditions(array('subcategoryId' => $value));
			$this->db->delete('subcategory');
		}
		$this->db->optimize('resource_category');
		$this->db->optimize('subcategory');
		return TRUE;
	}
// display categories for editing
	public function editInit($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "edit", " (" . 
			$this->messages->text("resources", "categories") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message, "error", "center");
		if(empty($this->categories))
		{
			GLOBALS::buildOutputString($pString .= $this->messages->text("misc", "noCategories"));
			return;
		}
		$pString .= $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
		$pString .= $this->form->hidden("method", "edit");
		foreach($this->categories as $key => $value)
			$pString .= $this->form->textInput(FALSE, "categoryEdit_" . $key, $value, 30, 255) . $this->html->br();
		$pString .= $this->html->br() . $this->form->formSubmit('Edit');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// display sub categories for editing
	public function editSubInit($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "edit", " (" . 
			$this->messages->text("resources", "subcategories") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message, "error", "center");
		$subcategories = $this->category->grabSubAll(TRUE);
		if(empty($subcategories))
		{
			GLOBALS::buildOutputString($pString .= $this->messages->text("misc", "noSubcategories"));
			return;
		}
		$pString .= $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
		$pString .= $this->form->hidden("method", "editSubDisplay");
		$pString .= $this->form->selectFBoxValue(FALSE, 'subcategoryId', $subcategories, 20);
		$pString .= $this->html->br() . $this->form->formSubmit('Proceed');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Edit a subcategory - display single subcategory.
	function editSubDisplay($message = FALSE)
	{
		if(!$input = $this->validateInput('editSubInit'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'editSubInit');
		$this->db->formatConditions(array('subcategoryId' => $input));
		$recordset = $this->db->select('subcategory', array('subcategoryCategoryId', 'subcategorySubcategory'));
		$row = $this->db->fetchRow($recordset);
		$table = FACTORY_TABLE::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "edit", " (" . 
			$this->messages->text("resources", "subcategories") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message, "error", "center");
		$pString .= $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
		$pString .= $this->form->hidden("method", "editSub");
		$pString .= $this->form->hidden('subcategoryId', $input['subcategoryId']);
		$pString .= $table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $table->trStart();
		$pString .= $table->td($this->form->textInput(FALSE, "subcategoryEdit", $row['subcategorySubcategory'], 30, 255));
		$categories = $this->category->grabAll();
		$pString .= $table->td($this->form->selectedBoxValue($this->messages->text('resources', 'subcategoryPart'), 
			'categoryId', $categories, $row['subcategoryCategoryId'], 20) . $this->html->p() . 
			$this->messages->text('resources', 'subcategoryKeepCat') . '&nbsp;' . $this->form->checkbox(FALSE, 'subcategoryKeepCat'));
		$pString .= $table->trEnd();
		$pString .= $table->tableEnd();
		$pString .= $this->form->formSubmit('Edit');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Edit a subcategory
	public function editSub()
	{
		if(!$input = $this->validateInput('editSub'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'editSubDisplay');
		$this->db->formatConditions(array('subcategoryId' => $input['subcategoryId']));
		$oldCatId = $this->db->fetchOne($this->db->select('subcategory', 'subcategoryCategoryId'));
		if(!array_key_exists('subcategoryKeepCat', $this->vars)) // replace previous category of which subcategory a member
		{
// If a new category, add new category to all resources in this subcategory.  Select all those resources having both oldCatId and 
// subcategoryId
			if($oldCatId != $input['categoryId']) // the only condition that makes sense if we wish to replace category.  Else, ignore.
			{
				$this->db->formatConditions(array('t1.resourcecategoryCategoryId' => $oldCatId));
				$this->db->formatConditions(array('t2.resourcecategorySubcategoryId' => $input['subcategoryId']));
				$this->db->leftJoin(array(array('resource_category' => 't2')), 
					array('t1.resourcecategoryResourceId'), array('t2.resourcecategoryResourceId'));
				$resultset = $this->db->select(array(array('resource_category' => 't1')), 
					array(array('t1.resourcecategoryId' => 't1Id'), array('t2.resourcecategoryId' => 't2Id')));
				while($row = $this->db->fetchRow($resultset))
				{
					$this->db->formatConditions(array('resourcecategoryId' => $row['t1Id']));
					$this->db->update('resource_category', array('resourcecategoryCategoryId' => $input['categoryId']));
				}
			}
		}
		else
		{
// else, need to insert new rows to resource_category if the category has changed -- get resource Id from this subCategory
			if($oldCatId != $input['categoryId'])
			{
				$this->db->formatConditions(array('resourcecategorySubcategoryId' => $input['subcategoryId']));
				$resultset = $this->db->select('resource_category', array('resourcecategoryCategoryId', 'resourcecategoryResourceId'), TRUE);
				while($row = $this->db->fetchRow($resultset))
				{
					$this->db->formatConditions(array('resourcecategoryCategoryId' => $input['categoryId']));
					$this->db->formatConditions(array('resourcecategoryResourceId' => $row['resourcecategoryResourceId']));
					if(!$this->db->numRows($this->db->select('resource_category', '*')))
						$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategoryCategoryId'), 
							array($row['resourcecategoryResourceId'], $input['categoryId']));
				}
			}
		}
		$pString = $this->success->text("subcategory", " " . $this->messages->text("misc", "edited") . " ");
		$update['subcategorySubcategory'] = $input['subcategoryEdit'];
		$update['subcategoryCategoryId'] = $input['categoryId'];
		$this->db->formatConditions(array('subcategoryId' => $input['subcategoryId']));
		$this->db->update('subcategory', $update);
		return $this->editSubInit($pString);
	}
// Edit categories
	public function edit()
	{
		if(!$input = $this->validateInput('edit'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'editInit');
		$recordset = $this->db->select('category', array('categoryId', 'categoryCategory'));
		while($row = $this->db->fetchRow($recordset))
		{
			if(array_key_exists($row['categoryId'], $input))
			{
				$update['categoryCategory'] = $input[$row['categoryId']];
				$this->db->formatConditions(array('categoryId' => $row['categoryId']));
				$this->db->update('category', $update);
				unset($input[$row['categoryId']]);
			}
		}
		$pString = $this->success->text("category", " " . $this->messages->text("misc", "edited") . " ");
		$this->categories = $this->category->grabAll();
		unset($this->categories[1]);
		return $this->editInit($pString);
	}
// validate input
	private function validateInput($type)
	{
		if($type == 'add')
		{
			if(!$input = trim($this->vars['categoryAdd']))
				return FALSE;
		}
		else if($type == 'addSub')
		{
			if(!trim($this->vars['subcategoryAdd']) || !$this->vars['categoryId'])
				return FALSE;
			$input['subcategoryAdd'] = trim($this->vars['subcategoryAdd']);
			$input['categoryId'] = trim($this->vars['categoryId']);
		}
		else if($type == 'delete')
		{
			foreach($this->vars as $key => $value)
			{
				if(!preg_match("/categoryDelete_(.*)/", $key, $match))
					continue;
				$input[] = $match[1];
			}
			if(!isset($input))
				return FALSE;
		}
		else if($type == 'deleteSub')
		{
			foreach($this->vars as $key => $value)
			{
				if(!preg_match("/subcategoryDelete_(.*)/", $key, $match))
					continue;
				$input[] = $match[1];
			}
			if(!isset($input))
				return FALSE;
		}
		else if($type == 'edit')
		{
			foreach($this->vars as $key => $value)
			{
				if(!preg_match("/categoryEdit_(.*)/", $key, $match))
					continue;
				if(!$input[$match[1]] = trim($value))
					return FALSE;
			}
			if(!isset($input))
				return FALSE;
		}
		else if($type == 'editSubInit')
		{
			if(!$this->vars['subcategoryId'])
				return FALSE;
			$input = $this->vars['subcategoryId'];
		}
		else if($type == 'editSub')
		{
			if(!trim($this->vars['subcategoryEdit']) || !$this->vars['categoryId'] || !$this->vars['subcategoryId'])
				return FALSE;
			$input['subcategoryEdit'] = trim($this->vars['subcategoryEdit']);
			$input['categoryId'] = trim($this->vars['categoryId']);
			$input['subcategoryId'] = trim($this->vars['subcategoryId']);
		}
		return $input;
	}
}
?>