<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
*	ADMINCUSTOM class.
*
*	Administration of custom fields
*****/
class ADMINCUSTOM
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $form;
private $html;
private $gatekeep;
private $badInput;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->gatekeep->init();
		$this->session->clearArray('custom');
	}
// display options for adding fields
	public function addInit($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "adminCustom", 
			" (" . $this->messages->text("custom", "addLabel") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message);
		$pString .= $this->form->formHeader('admin_ADMINCUSTOM_CORE');
		$pString .= $this->form->hidden('method', 'add');
		$array = array("small" => $this->messages->text("custom", "small"), 
			"large" => $this->messages->text("custom", "large"));
		if(!$size = $this->session->getVar("custom_size"))
			$size = "small";
		$pString .= $this->form->selectedBoxValue($this->messages->text("custom", "size"), 
				"custom_size", $array, $size, 2) . " " . $this->html->span('*', 'required');
		$pString .= $this->html->p($this->form->textInput($this->messages->text("custom", "label"), 
			"custom_label", FALSE, 50, 255) . " " . $this->html->span('*', 'required'));
		$pString .= $this->form->formSubmit('Add');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Create the new field
	public function add()
	{
		if(!$this->validateInput('add'))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'addInit');
		$fields = array("customLabel", "customSize");
		$values[] = trim($this->label);
		if($this->size == 'large')
			$values[] = 'L';
		else
			$values[] = 'S';
		$this->db->insert('custom', $fields, $values);
		$pString = $this->success->text("field", " " . $this->messages->text("misc", "added") . " ");
		return $this->addInit($pString);
	}
// display fields for editing
	public function editInit($message = FALSE)
	{
		$pString = '';
		if($message)
			$pString .= $this->html->p($message);
		$pString .= $this->form->formHeader('admin_ADMINCUSTOM_CORE');
		$pString .= $this->form->hidden('method', 'editDisplay');
		$recordset = $this->db->select('custom', array('customId', 'customLabel'));
		while($row = $this->db->fetchRow($recordset))
			$fields[$row['customId']] = $this->html->dbToFormTidy($row['customLabel']);
		if(!isset($fields))
		{
			$this->addInit($message);
			FACTORY_CLOSE::getInstance();
		}
		GLOBALS::setOutputHeading($this->messages->text("heading", "adminCustom", 
			" (" . $this->messages->text("custom", "editLabel") . ")"));
		$pString .= $this->form->selectFBoxValue(FALSE, "editId", $fields, 5) . $this->html->br() . $this->form->formSubmit('Edit');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// display editing form
	public function editDisplay($message = FALSE)
	{
		if(!$this->validateInput('editDisplay'))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'editInit');
		GLOBALS::setOutputHeading($this->messages->text("heading", "adminCustom", 
			" (" . $this->messages->text("custom", "editLabel") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message);
		$this->db->formatConditions(array('customId' => $this->vars['editId']));
		$recordset = $this->db->select('custom', 'customLabel');
		$label = $this->html->dbToFormTidy($this->db->fetchOne($recordset));
		$pString .= $this->form->formHeader('admin_ADMINCUSTOM_CORE');
		$pString .= $this->form->hidden('method', 'edit');
		$pString .= $this->form->hidden("editId", $this->vars['editId']);
		$pString .= $this->html->p($this->form->textInput($this->messages->text("custom", "label"), 
			"custom_label", $label, 50, 255) . " " . $this->html->span('*', 'required'));
		$pString .= $this->form->formSubmit('Edit');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Edit field
	public function edit()
	{
		if(!$this->validateInput('edit'))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'editInit');
		$this->db->formatConditions(array('customId' => $this->vars['editId']));
		$this->db->update('custom', array("customLabel" => $this->label));
		$pString = $this->success->text("field", " " . $this->messages->text("misc", "edited") . " ");
		return $this->editInit($pString);
	}
// Delete fields display.
	public function deleteInit($message = FALSE)
	{
		$recordset = $this->db->select('custom', array('customId', 'customLabel'));
		while($row = $this->db->fetchRow($recordset))
			$fields[$row['customId']] = $this->html->dbToFormTidy($row['customLabel']);
		if(!isset($fields))
		{
			$this->addInit($message);
			FACTORY_CLOSE::getInstance();
		}
		GLOBALS::setOutputHeading($this->messages->text("heading", "adminCustom", 
			" (" . $this->messages->text("custom", "deleteLabel") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message);
		$pString .= $this->html->p($this->html->b($this->messages->text("custom", "warning")));
		$pString .= $this->form->formHeader('admin_ADMINCUSTOM_CORE');
		$pString .= $this->form->hidden('method', 'deleteConfirm');
		$pString .= $this->form->selectFBoxValueMultiple(FALSE, "delete", $fields, 5);
		$pString .= $this->html->br() . $this->messages->text("hint", "multiples") . $this->html->br();
		$pString .= $this->html->br() . $this->form->formSubmit('Delete');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Ask for confirmation of delete field(s)
	public function deleteConfirm()
	{
		if(!$this->validateInput('delete'))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'deleteInit');
		GLOBALS::setOutputHeading($this->messages->text("heading", "adminCustom", 
			" (" . $this->messages->text("custom", "deleteLabel") . ")"));
		$pString = $this->html->p($this->html->b($this->messages->text("custom", "warning")));
		$pString .= $this->form->formHeader('admin_ADMINCUSTOM_CORE');
		$pString .= $this->form->hidden('method', 'delete');
		foreach($this->vars['delete'] as $id)
		{
			$pString .= $this->form->hidden("delete_" . $id, $id);
			$ids[] = $this->db->tidyInput($id);
		}
		$this->db->formatConditions($this->db->formatFields('customId') . $this->db->equal . 
			join($this->db->or . $this->db->formatFields('customId') . $this->db->equal, $ids));
		$recordset = $this->db->select('custom', 'customLabel');
		while($row = $this->db->fetchRow($recordset))
			$fieldValues[] = "'" . $this->html->dbToHtmlTidy($row['customLabel']) . "'";
		$pString .= $this->html->p($this->messages->text("custom", "deleteConfirm", ": " . join(", ", $fieldValues)));
		$pString .= $this->html->br() . $this->form->formSubmit('Confirm');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Delete field(s)
	public function delete()
	{
		if(!$ids = $this->validateInput('deleteConfirm'))
			$this->badInput($this->errors->text("inputError", "invalid"), 'deleteInit');
// $ids is an array of field IDs
		$this->db->formatConditions($this->db->formatFields('customId') . $this->db->equal . 
			join($this->db->or . $this->db->formatFields('customId') . $this->db->equal, $ids));
		$this->db->delete('custom');
		$this->db->formatConditions($this->db->formatFields('resourcecustomId') . $this->db->equal . 
			join($this->db->or . $this->db->formatFields('resourcecustomId') . $this->db->equal, $ids));
		$this->db->delete('resource_custom');
		$this->db->optimize('custom');
		$this->db->optimize('resource_custom');
		$pString = $this->success->text("field", " " . $this->messages->text("misc", "deleted") . " ");
		return $this->deleteInit($pString);
	}
// validate input
	private function validateInput($type)
	{
		if($type == 'add')
		{
// Write to session
			$this->size = isset($this->vars['custom_size']) ? $this->vars['custom_size'] : FALSE;
			$this->label = isset($this->vars['custom_label']) ? trim($this->vars['custom_label']) : FALSE;
			$this->session->setVar("custom_size", $this->size);
			$this->session->setVar("custom_label", $this->label);
			if(!$this->size)
				return FALSE;
			if(!$this->label)
				return FALSE;
		}
		else if($type == 'editDisplay')
		{
			if(!array_key_exists('editId', $this->vars))
				return FALSE;
		}
		else if($type == 'edit')
		{
			$this->label = isset($this->vars['custom_label']) ? trim($this->vars['custom_label']) : FALSE;
			if(!array_key_exists('editId', $this->vars) || !$this->label)
				return FALSE;
		}
		else if($type == 'delete')
		{
			if(!array_key_exists('delete', $this->vars) || empty($this->vars['delete']))
				return FALSE;
		}
		else if($type == 'deleteConfirm')
		{
			$ids = array();
			foreach($this->vars as $key => $value)
			{
				if(!preg_match("/delete_/", $key))
					continue;
				$ids[] = $value;
			}
			if(empty($ids))
				return FALSE;
			return $ids;
		}
		return TRUE;
	}
}
?>
