<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* ADMINKEYWORD class
*
*/
class ADMINKEYWORD
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $config;
private $form;
private $keyword;
private $html;
private $table;
private $gatekeep;
private $badInput;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->keyword = FACTORY_KEYWORD::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->gatekeep->init();
		$this->session->clearArray('edit');
	}
	public function editInit($message = FALSE)
	{
// Edit operations use functions from core/modules/edit/EDITKEYWORD.php
		GLOBALS::setOutputHeading($this->messages->text("heading", "edit", " (" . 
			$this->messages->text("resources", "keyword") . ")"));
		$keywords = $this->keyword->grabAll();
		if(!$keywords)
		{
			GLOBALS::buildOutputString($this->messages->text('misc', 'noKeywords'));
			return;
		}
		$pString = $message;
		$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $this->table->trStart();
		$td = $this->form->formHeader('edit_EDITKEYWORD_CORE');
		$td .= $this->form->hidden("method", "editDisplayKeyword");
		$td .= $this->form->selectFBoxValue(FALSE, "edit_KeywordId", $keywords, 20);
		$td .= $this->html->p($this->form->formSubmit("Edit"));
		$td .= $this->form->formEnd();
		$pString .= $this->table->td($td);
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		GLOBALS::buildOutputString($pString);
	}
	public function mergeInit($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "adminKeywords"));
		$keywords = $this->keyword->grabAll();
		$pString = $this->html->p($this->messages->text("misc", "keywordMerge"));
		if($message)
			$pString .= $this->html->p($message);
		if(is_array($keywords) && !empty($keywords))
		{
			$pString .= $this->form->formHeader('admin_ADMINKEYWORD_CORE');
			$pString .= $this->form->hidden("method", "merge");
			$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left");
			$pString .= $this->table->trStart();
			$td = $this->form->selectFBoxValueMultiple(FALSE, "keywordIds", $keywords, 20) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint');
			$pString .= $this->table->td($td);
			$td = $this->form->textInput($this->messages->text("misc", "keywordMergeTarget"), 
				"keywordText", FALSE, 50);
			$td .= $this->html->br();
			$td .= $this->html->br();
			$td .= $this->form->formSubmit("Proceed");
			$pString .= $this->table->td($td);
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->form->formEnd();
		}
		else
			$pString .= $this->html->p($this->messages->text("misc", "noKeywords"));
		GLOBALS::buildOutputString($pString);
	}
// start merging process
	public function merge()
	{
		if(!array_key_exists("keywordIds", $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'mergeInit');
		if(!array_key_exists("keywordText", $this->vars) || !trim($this->vars['keywordText']))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'mergeInit');
		$keywordIds = $this->vars['keywordIds'];
		$newKeyword = trim($this->vars['keywordText']);
		$newKeywordId = $this->insertKeyword($newKeyword);
		if(($index = array_search($newKeywordId, $keywordIds)) !== FALSE)
			unset($keywordIds[$index]);
// Remove old keywords
		$this->db->formatConditionsOneField($keywordIds, 'keywordId');
		$this->db->delete('keyword');
// update references to keyword ID
		$this->db->formatConditionsOneField($keywordIds, 'resourcekeywordKeywordId');
		$this->db->update('resource_keyword', array('resourcekeywordKeywordId' => $newKeywordId));
// If we are merging, say, 2 keywords that a particular resource has, need to ensure we do not duplicate keyword entries in 
// resource_keyword for that resource
		$deleteIds = $rIds = $qIds = $pIds = $mIds = array();
		$resultset = $this->db->select('resource_keyword', array('resourcekeywordId', 'resourcekeywordResourceId', 
			'resourcekeywordQuoteId', 'resourcekeywordParaphraseId', 'resourcekeywordMusingId', 'resourcekeywordKeywordId'));
		while($row = $this->db->fetchRow($resultset))
		{
			if(!array_key_exists($row['resourcekeywordId'], $deleteIds) && 
				$row['resourcekeywordResourceId'] && array_key_exists($row['resourcekeywordResourceId'], $rIds) 
				&& ($rIds[$row['resourcekeywordResourceId']] == $row['resourcekeywordKeywordId']))
				$deleteIds[] = $row['resourcekeywordId'];
			else if($row['resourcekeywordResourceId'])
				$rIds[$row['resourcekeywordResourceId']] = $row['resourcekeywordKeywordId'];
			else if(!array_key_exists($row['resourcekeywordId'], $deleteIds) && 
				$row['resourcekeywordQuoteId'] && array_key_exists($row['resourcekeywordQuoteId'], $qIds) 
				&& ($qIds[$row['resourcekeywordQuoteId']] == $row['resourcekeywordKeywordId']))
				$deleteIds[] = $row['resourcekeywordId'];
			else if($row['resourcekeywordQuoteId'])
				$rIds[$row['resourcekeywordQuoteId']] = $row['resourcekeywordKeywordId'];
			else if(!array_key_exists($row['resourcekeywordId'], $deleteIds) && 
				$row['resourcekeywordParaphraseId'] && array_key_exists($row['resourcekeywordParaphraseId'], $qIds) 
				&& ($qIds[$row['resourcekeywordParaphraseId']] == $row['resourcekeywordKeywordId']))
				$deleteIds[] = $row['resourcekeywordId'];
			else if($row['resourcekeywordParaphraseId'])
				$rIds[$row['resourcekeywordParaphraseId']] = $row['resourcekeywordKeywordId'];
			else if(!array_key_exists($row['resourcekeywordId'], $deleteIds) && 
				$row['resourcekeywordMusingId'] && array_key_exists($row['resourcekeywordMusingId'], $qIds) 
				&& ($qIds[$row['resourcekeywordMusingId']] == $row['resourcekeywordKeywordId']))
				$deleteIds[] = $row['resourcekeywordId'];
			else if($row['resourcekeywordMusingId'])
				$rIds[$row['resourcekeywordMusingId']] = $row['resourcekeywordKeywordId'];
		}
		if(!empty($deleteIds))
		{
			$this->db->formatConditionsOneField($deleteIds, 'resourcekeywordId');
			$this->db->delete('resource_keyword');
			$this->db->optimize('resource_keyword');
		}
		$this->db->optimize('keyword');
// remove cache files for keywords
		$this->db->deleteCache('cacheResourceKeywords');
		$this->db->deleteCache('cacheMetadataKeywords');
		$this->db->deleteCache('cacheQuoteKeywords');
		$this->db->deleteCache('cacheParaphraseKeywords');
		$this->db->deleteCache('cacheMusingKeywords');
		$this->mergeInit($this->success->text("keywordMerge"));
	}
// Insert new keyword or return ID if already exists
	private function insertKeyword($keyword)
	{
		$this->keywordExists = TRUE;
		if($id = $this->keyword->checkExists($keyword))
			return $id;
		$this->keywordExists = FALSE;
		$fields[] = "keywordKeyword";
		$values[0] = $keyword;
// given keyword doesn't exist so now write to db
		$this->db->insert('keyword', $fields, $values);
		return $this->db->lastAutoId('keyword');
	}
	public function deleteInit($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "delete2", " (" . 
			$this->messages->text("resources", "keyword") . ")"));
		$keywords = $this->keyword->grabAll();
		if(!$keywords)
		{
			GLOBALS::buildOutputString($this->messages->text('misc', 'noKeywords'));
			return;
		}
		$pString = $message;
		$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $this->table->trStart();
		$td = $this->form->formHeader('admin_ADMINKEYWORD_CORE');
		$td .= $this->form->hidden("method", "deleteConfirm");
		$td .= $this->form->selectFBoxValueMultiple(FALSE, "delete_KeywordId", $keywords, 20) . 
			$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint');
		$td .= $this->html->p($this->form->formSubmit("Proceed"));
		$td .= $this->form->formEnd();
		$pString .= $this->table->td($td);
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		GLOBALS::buildOutputString($pString);
	}
/**
* Confirm deletes.
*/
	public function deleteConfirm()
	{
		$this->session->delVar('editLock');
		GLOBALS::setOutputHeading($this->messages->text("heading", "delete2", " (" . 
			$this->messages->text("resources", "keyword") . ")"));
		$input = array_values($this->vars['delete_KeywordId']);
		$keywords = $this->keyword->grabAll();
		$keywordInput = "'" . implode("', '", array_keys(array_intersect(array_flip($keywords), $input))) . "'";
		$keywordInput = html_entity_decode($keywordInput);
		$pString = $this->html->p($this->messages->text("resources", "deleteConfirmKeywords") . ":&nbsp;&nbsp;$keywordInput");
		$pString .= $this->form->formHeader("admin_ADMINKEYWORD_CORE");
		$pString .= $this->form->hidden("delete_KeywordId", base64_encode(serialize($this->vars['delete_KeywordId'])));
		$pString .= $this->form->hidden("method", 'delete');
		$pString .= $this->html->p($this->form->formSubmit("Proceed"));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// write to the database.
	public function delete()
	{
		if($this->session->getVar('editLock'))
			$this->badInput->close($this->errors->text("done", "keywordDelete"), $this, 'deleteInit');
		if(!array_key_exists('delete_KeywordId', $this->vars) || !$this->vars['delete_KeywordId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'deleteInit');
		foreach(unserialize(base64_decode($this->vars['delete_KeywordId'])) as $deleteId)
		{
// Delete old keyword
			$this->db->formatConditions(array('keywordId' => $deleteId));
			$this->db->delete('keyword');
// remove cache files for keywords
			$this->db->deleteCache('cacheResourceKeywords');
			$this->db->deleteCache('cacheMetadataKeywords');
			$this->db->deleteCache('cacheQuoteKeywords');
			$this->db->deleteCache('cacheParaphraseKeywords');
			$this->db->deleteCache('cacheMusingKeywords');
// Select all resources and metadata referencing this old keyword and remove keyword from list
			$this->db->formatConditions(array('resourcekeywordKeywordId' => $deleteId));
			$this->db->delete('resource_keyword');
		}
		$this->db->optimize('resource_keyword');
// lock reload
		$this->session->setVar('editLock', TRUE);
// Clear session
		$this->session->clearArray("edit");
// send back to deleteDisplay with success message
		$this->deleteInit($this->success->text("keywordDelete"));
	}
}
?>
