<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
*	ADMINLANGUAGES class.
*
*	ADMINLANGUAGES of resource languages
*****/
class ADMINLANGUAGES
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $config;
private $form;
private $keyword;
private $html;
private $gatekeep;
private $badInput;
private $languages;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->gatekeep->requireSuper = TRUE;
		$this->gatekeep->init();
		$this->session->clearArray('edit');
		$this->languages = $this->grabAll();
	}
// Add a language - display entry form
	public function addInit($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "add", " (" . 
			$this->messages->text("resources", "languages") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message, "error", "center");
		$pString .= $this->html->p($this->messages->text("resources", "languagesAdd"));
		$pString .= $this->form->formHeader("admin_ADMINLANGUAGES_CORE");
		$pString .= $this->form->hidden("method", "add");
		$pString .= $this->form->textInput(FALSE, "languageAdd", FALSE, 30, 255);
		$pString .= $this->html->p($this->form->formSubmit('Add'));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Add a language
	public function add()
	{
		if(!$input = $this->validateInput('add'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'addInit');
		$utf8 = FACTORY_UTF8::getInstance();
		$pString = $this->success->text("language", " " . $this->messages->text("misc", "added") . " ");
// If language already exists quietly return without error.
		foreach($this->languages as $language)
		{
			if($utf8->utf8_strtolower($input) == $utf8->utf8_strtolower($language))
				return $this->addInit($pString);
		}
		$this->db->insert('language', array('languageLanguage'), array($input));
		$this->languages = $this->grabAll();
		return $this->addInit($pString);
	}
// Delete languages display.
	public function deleteInit($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "delete2", " (" . 
			$this->messages->text("resources", "languages") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message, "error", "center");
		if(empty($this->languages))
		{
			GLOBALS::buildOutputString($pString . $this->messages->text("misc", "noLanguages"));
			return;
		}
		$pString .= $this->form->formHeader("admin_ADMINLANGUAGES_CORE");
		$pString .= $this->form->hidden("method", "deleteConfirm");
		foreach($this->languages as $key => $value)
			$pString .= $this->form->checkbox(FALSE, "languageDelete_" . $key) . " $value" . $this->html->br();
		$pString .= $this->html->br() . $this->form->formSubmit('Delete');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Ask for confirmation of delete languages
	public function deleteConfirm()
	{
		$input = $this->validateInput('delete');
		if(empty($input))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'deleteInit');
		GLOBALS::setOutputHeading($this->messages->text("heading", "delete2", " (" . 
			$this->messages->text("resources", "languages") . ")"));
		$languages = "'" . implode("', '", array_keys(array_intersect(array_flip($this->languages), $input))) . "'";
		$languages = html_entity_decode($languages);
		$pString = $this->html->p($this->messages->text("misc", "confirmDeleteLanguage") . '&nbsp;&nbsp;' . $languages);
		$pString .= $this->form->formHeader("admin_ADMINLANGUAGES_CORE");
		$pString .= $this->form->hidden("method", "delete");
		foreach($input as $id)
			$pString .= $this->form->hidden("languageDelete_" . $id, $id);
		$pString .= $this->html->p($this->form->formSubmit('Confirm'));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Delete languages
	public function delete()
	{
		$input = $this->validateInput('delete');
		if(empty($input))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'deleteInit');
		foreach($input as $id)
		{
			if(array_key_exists($id, $this->languages))
			{
				$this->db->formatConditions(array('resourcelanguageLanguageId' => $id));
				$this->db->delete('resource_language');
				$this->db->formatConditions(array('languageId' => $id));
				$this->db->delete('language');
			}
		}
		$pString = $this->success->text("language", " " . $this->messages->text("misc", "deleted") . " ");
		$this->languages = $this->grabAll();
		$pString = $this->success->text("language", " " . $this->messages->text("misc", "edited") . " ");
		return $this->deleteInit($pString);
	}
// display languages for editing
	public function editInit($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "edit", " (" . 
			$this->messages->text("resources", "languages") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message, "error", "center");
		if(empty($this->languages))
		{
			GLOBALS::buildOutputString($pString .= $this->messages->text("misc", "noLanguages"));
			return;
		}
		$pString .= $this->form->formHeader("admin_ADMINLANGUAGES_CORE");
		$pString .= $this->form->hidden("method", "edit");
		foreach($this->languages as $key => $value)
			$pString .= $this->form->textInput(FALSE, "languageEdit_" . $key, $value, 30, 255) . $this->html->br();
		$pString .= $this->html->p($this->form->formSubmit('Edit'));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Edit languages
	public function edit()
	{
		$input = $this->validateInput('edit');
		if(empty($input))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'editInit');
		$default = FALSE;
		foreach($this->languages as $key => $value)
		{
			if(array_key_exists($key, $input))
			{
				$this->db->formatConditions(array('languageId' => $key));
				$this->db->update('language', array('languageLanguage' => $input[$key]));
			}
		}
		$this->languages = $this->grabAll();
		$pString = $this->success->text("language", " " . $this->messages->text("misc", "edited") . " ");
		return $this->editInit($pString);
	}
// Grab any languages from config table
	private function grabAll()
	{
		$array = array();
		$this->db->orderBy('languageLanguage');
		$resultset = $this->db->select('language', array('languageId', 'languageLanguage'));
		while($row = $this->db->fetchRow($resultset))
			$array[$row['languageId']] = $row['languageLanguage'];
		return $array;
	}
// validate input
	private function validateInput($type)
	{
		if($type == 'add')
		{
			if(!$input = trim($this->vars['languageAdd']))
				return FALSE;
		}
		else if($type == 'delete')
		{
			$input = array();
			foreach($this->vars as $key => $value)
			{
				if(!preg_match("/languageDelete_(.*)/", $key, $match))
					continue;
				$input[] = $match[1];
			}
		}
		else if($type == 'edit')
		{
			$input = array();
			foreach($this->vars as $key => $value)
			{
				if(!preg_match("/languageEdit_(.*)/", $key, $match))
					continue;
				if(!$input[$match[1]] = trim($value))
					return array();
			}
		}
		return $input;
	}
}
?>