<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
*	ADMINUSER class.
*
*	Administration of users
*/
class ADMINUSER
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $form;
private $html;
private $table;
private $user;
private $template;
private $gatekeep;
private $badInput;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->user = FACTORY_USER::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->gatekeep->requireSuper = TRUE;
		$this->gatekeep->init();
	}
	public function init($method)
	{
		$this->$method();
	}
// Add a user - display options.
	public function addInit($error = FALSE)
	{
		$creator = FACTORY_CREATOR::getInstance();
		$creators = $creator->grabAll(FALSE, FALSE, FALSE, TRUE);
		if(is_array($creators))
		{
// add 0 => IGNORE to creators array
			$temp[0] = $this->messages->text("misc", "ignore");
			foreach($creators as $key => $value)
				$temp[$key] = $value;
			$creators = $temp;
			unset($temp);
		}
		GLOBALS::setOutputHeading($this->messages->text("heading", "userAdd"));
		$pString = $error ? $this->html->p($error, "error", "center") : '';
		$pString .= $this->form->formHeader('admin_ADMINUSER_CORE');
		$pString .= $this->form->hidden('method', 'add');
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->textInput($this->messages->text("user", "username"), 
			"username", FALSE, 20, 255) . " " . $this->html->span('*', 'required'));
		$pString .= $this->table->td($this->form->passwordInput($this->messages->text("user", "password"), 
			"password", FALSE, 15, 255) . " " . $this->html->span('*', 'required'));
		$pString .= $this->table->td($this->form->passwordInput($this->messages->text("user", "passwordConfirm"), 
			"passwordConfirm", FALSE, 15, 255) . " " . $this->html->span('*', 'required'));
		$pString .= $this->table->td($this->form->textInput($this->messages->text("user", "email"), 
			"email", FALSE, 30, 255) . " " . $this->html->span('*', 'required'));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->br() . "&nbsp;" . $this->html->br();
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->textInput($this->messages->text("user", "fullname"), "fullname", FALSE, 30, 255));
// Department
		$td = $this->form->textInput($this->messages->text("user", "department"), "department", FALSE, 30, 255);
		$resultset = $this->db->select('users', 'usersDepartment', TRUE);
		$initial = FALSE;
		while($row = $this->db->fetchRow($resultset))
		{
			$department = $this->html->dbToFormTidy($row['usersDepartment']);
			if(!$department)
				continue;
			$departments[$department] = $department;
		}
		if(isset($departments))
		{
// add 0 => IGNORE to departments array
			$temp[0] = $this->messages->text("misc", "ignore");
			foreach($departments as $key => $value)
				$temp[$key] = $value;
			$departments = $temp;
			unset($temp);
			$td .= $this->html->br() . $this->form->selectFBoxValue('&nbsp;', "departmentId", $departments, 1);
		}
		$pString .= $this->table->td($td);
// Institution
		$td = $this->form->textInput($this->messages->text("user", "institution"), "institution", FALSE, 30, 255);
		$resultset = $this->db->select('users', 'usersInstitution', TRUE);
		$initial = FALSE;
		while($row = $this->db->fetchRow($resultset))
		{
			$institution = $this->html->dbToFormTidy($row['usersInstitution']);
			if(!$institution)
				continue;
			$institutions[$institution] = $institution;
		}
		if(isset($institutions))
		{
// add 0 => IGNORE to insitutions array
			$temp[0] = $this->messages->text("misc", "ignore");
			foreach($institutions as $key => $value)
				$temp[$key] = $value;
			$institutions = $temp;
			unset($temp);
			$td .= $this->html->br() . $this->form->selectFBoxValue('&nbsp;', "institutionId", $institutions, 1);
		}
		$pString .= $this->table->td($td);
		if(is_array($creators))
			$pString .= $this->table->td($this->form->selectFBoxValue($this->messages->text("user", "isCreator"), 
				"creatorId", $creators, 1));
		$pString .= $this->table->td($this->form->checkbox($this->messages->text("user", "admin"), "admin", FALSE));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->p($this->form->formSubmit("Add"), FALSE, "left");
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Add a user
	public function add()
	{
		if(!$this->validateInput('adminAdd'))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, "addInit");
// NB - writeUser returns FALSE on success!
		if($this->user->writeUser(TRUE, 2))
			$this->badInput->close($this->errors->text("inputError", "userExists"), $this, "addInit");
		include_once("core/modules/email/EMAIL.php");
		$emailClass = new EMAIL();
		if(!$emailClass->registerUserAdd(TRUE)) // Display password in email to new user
			$this->badInput->close($this->errors->text("inputError", "mail"), $this, 'addInit');
		GLOBALS::buildOutputString($this->success->text("user", $this->messages->text("misc", "added")));
		return $this->init('addInit');
	}
// Delete users display.
	public function deleteInit($error = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "userDelete"));
		$pString = $error ? $this->html->p($error, "error", "center") : '';
		$this->grabUsers();
		if(empty($this->users))
			$this->badInput->close($this->messages->text("resources", "noUsers"), $this, "addInit");
		$pString .= $this->form->formHeader('admin_ADMINUSER_CORE');
		$pString .= $this->form->hidden('method', 'deleteConfirm');
		$pString .= $this->form->selectFBoxValueMultiple(FALSE, "userDelete", $this->users, 20);
		$pString .= $this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint')
			. $this->html->br();
		$pString .= $this->html->br() . $this->form->formSubmit('Delete');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Ask for confirmation of delete users
	public function deleteConfirm()
	{
		if(!$input = $this->validateInput('delete'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'deleteInit');
		GLOBALS::setOutputHeading($this->messages->text("heading", "userDelete"));
		$this->grabUsers();
		if(empty($this->users))
			$this->badInput->close($this->messages->text("resources", "noUsers"), $this, "addInit");
		$users = "'" . implode("', '", array_keys(array_intersect(array_flip($this->users), 
			$this->vars['userDelete']))) . "'";
		$pString = $this->html->p($this->messages->text("user", "deleteConfirm", $users));
		$pString .= $this->form->formHeader('admin_ADMINUSER_CORE');
		$pString .= $this->form->hidden('method', 'delete');
		foreach($this->vars['userDelete'] as $id)
			$pString .= $this->form->hidden("userDelete_" . $id, $id);
		$pString .= $this->html->br() . $this->form->formSubmit('Confirm');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Delete user(s)
	public function delete()
	{
		if(!$input = $this->validateInput('deleteConfirm'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'deleteInit');
// $input is an array of user IDs
		if(!$this->user->deleteSql($input))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'deleteInit');
		return $this->addInit($this->success->text("user", " " . $this->messages->text("misc", "deleted")));
	}
// display users for editing
	public function editInit($error = FALSE)
	{
		$this->session->clearArray('mywikindx');
		GLOBALS::setOutputHeading($this->messages->text("heading", "userEdit"));
		$pString = $error ? $this->html->p($error, "error", "center") : '';
		$pString .= $this->form->formHeader('admin_ADMINUSER_CORE');
		$pString .= $this->form->hidden('method', 'editDisplay');
		$this->grabUsers();
		if(empty($this->users))
			$this->badInput->close($this->messages->text("resources", "noUsers"), $this, "addInit");
		$pString .= $this->form->selectFBoxValue(FALSE, "editId", $this->users, 20) . $this->html->br() . 
			$this->form->formSubmit('Edit');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// display one user for editing
// We temporarily use the session array 'mywikindx_' for storage so we can use the USER class methods.
	public function editDisplay($error = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "userEdit"));
		$pString = $error ? $this->html->p($error, "error", "center") : '';
		$this->userId = FALSE;
		if(array_key_exists("editId", $this->vars))
			$this->userId = trim($this->vars["editId"]);
		else if(array_key_exists("userId", $this->vars))
			$this->userId = trim($this->vars["userId"]);
		if(!$this->validateInput('editId'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'editInit');
		$this->user->loadSession($this->userId);
		$pString .= $this->user->displayUserDetails('admin_ADMINUSER_CORE', 'edit', $this->userId);
		GLOBALS::buildOutputString($pString);
	}
// Edit user
	function edit()
	{
		if(!$this->validateInput('edit'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'editDisplay');
		GLOBALS::setOutputHeading($this->messages->text("heading", "userEdit"));
// writeUser() returns FALSE for success
		if($this->user->writeUser(FALSE, 2))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'editDisplay');
		$pString = $this->success->text("user", " " . $this->messages->text("misc", "edited") . " ");
		return $this->editInit($pString);
	}
// Show users requesting registration
	public function registrationInit($error = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "register"));
		$pString = $error ? $this->html->p($error, "error", "center") : FALSE;
		$this->db->formatConditions(array('userregisterConfirmed' => 'N'));
		$this->db->formatConditions(array('userregisterRequest' => ' IS NOT NULL'));
		$resultset = $this->db->select('user_register', array('userregisterId', 'userregisterRequest', 
			'userregisterEmail', 'userregisterTimestamp'));
		if(!$this->db->numRows($resultset))
			$pString .= $this->messages->text('user', 'noUsers');
		else
		{
			$pString .= $this->html->p($this->messages->text('user', 'pendingRegistration1'));
			$pString .= $this->html->p($this->messages->text('user', 'pendingRegistration2')) . $this->html->br();
			$pString .= $this->form->formHeader('admin_ADMINUSER_CORE');
			$pString .= $this->form->hidden('method', 'register');
			$pString .= $this->table->tableStart('generalTable', 1);
			while($row = $this->db->fetchRow($resultset))
			{
				$pString .= $this->table->trStart();
				$pString .= $this->table->td($row['userregisterEmail']);
				$pString .= $this->table->td($this->html->dbToHtmlTidy($row['userregisterRequest']));
				$pString .= $this->table->td($row['userregisterTimestamp']);
				$pString .= $this->table->td($this->messages->text('user', 'registrationAccept') . "&nbsp:&nbsp;" . 
					$this->form->radioButton(FALSE, 'registerUser_' . $row['userregisterId'], 'accept', TRUE) .  
					$this->html->br() . $this->messages->text('user', 'registrationDecline') . "&nbsp:&nbsp;" . 
					$this->form->radioButton(FALSE, 'registerUser_' . $row['userregisterId'], 'decline'));
				$pString .= $this->table->trEnd();
			}
			$pString .= $this->table->tableEnd();
			$pString .= $this->html->p($this->form->formSubmit("Add"), FALSE, "left");
			$pString .= $this->form->formEnd();
		}
		GLOBALS::buildOutputString($pString);
	}
// Mangage requests for registration
	public function register()
	{
		foreach($this->vars as $key => $value)
		{
			if(strpos($key, "registerUser_") !== FALSE)
			{
				$split = explode("_", $key);
				$registerIds[$split[1]] = $value;
			}
		}
		if(!isset($registerIds))
			return $this->badInput->close($this->errors->text("inputError", "missing"), $this, 'registrationInit');
		GLOBALS::setOutputHeading($this->messages->text("heading", "register"));
		include_once("core/modules/email/EMAIL.php");
		$emailClass = new EMAIL();
		if(!$emailClass->registerRequestManage($registerIds))
			$this->badInput->close($this->errors->text("inputError", "mail2"), $this, 'registrationInit');
// Delete those declined and confirm those accepted
		foreach($registerIds as $id => $value)
		{
			$this->db->formatConditions(array('userregisterId' => $id));
			if($value == 'accept')
				$this->db->update('user_register', array('userregisterConfirmed' => 'Y'));
			else
				$this->db->delete('user_register');
		}
		GLOBALS::buildOutputString($this->success->text('registerRequestManage'));
	}
// validate input
	private function validateInput($type)
	{
		if($type == 'adminAdd')
		{
			if(!trim($this->vars['username']) || !trim($this->vars['password']) || 
				!trim($this->vars['passwordConfirm']) || !trim($this->vars['email']))
				return FALSE;
			if(trim($this->vars['password']) != trim($this->vars['passwordConfirm']))
				return FALSE;
		}
		else if($type == 'userAdd')
		{
			if(!trim($this->vars['username']) || !trim($this->vars['password']) || 
				!trim($this->vars['passwordConfirm']) || !$this->vars['email'] || !$this->vars['hashKey'])
				return FALSE;
			if(trim($this->vars['password']) != trim($this->vars['passwordConfirm']))
				return FALSE;
		}
		else if($type == 'register')
		{
			if(!trim($this->vars['email']))
				return FALSE;
		}
		else if($type == 'delete')
		{
			if(!array_key_exists('userDelete', $this->vars) || empty($this->vars['userDelete']))
				return FALSE;
		}
		else if($type == 'deleteConfirm')
		{
			$input = array();
// Ensure we don't delete superadmin with id of 1
			foreach($this->vars as $key => $value)
			{
				if(!$value || ($value == 1))
					continue;
				if(!preg_match("/userDelete_/", $key, $matches))
					continue;
				$input[] = $value;
			}
			 if(empty($input))
				return FALSE;
		}
		else if($type == 'editId')
		{
			if(!$this->userId)
				return FALSE;
		}
		else if($type == 'edit')
		{
			if(!trim($this->vars['password']) || !trim($this->vars['email']) || !trim($this->vars['userId']))
				return FALSE;
			if(trim($this->vars['password']) != trim($this->vars['passwordConfirm']))
				return FALSE;
		}
		if(isset($input))
			return $input;
		return TRUE;
	}
// grab users from database
	private function grabUsers()
	{
		$this->users = $this->user->grabAll(TRUE);
// don't allow this user to operate on self or superadmin (id == 1)
		$this->selfId = $this->session->getVar('setup_UserId');
		if(array_key_exists($this->selfId, $this->users))
			unset($this->users[$this->selfId]);
		if(array_key_exists(1, $this->users))
			unset($this->users[1]);
	}
}
?>
