<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* DELETERESOURCE class
*
* Delete resources
* 
*/
class DELETERESOURCE
{
private $db;
private $vars;
private $messages;
private $errors;
private $success;
private $session;
private $form;
private $table;
private $html;
private $template;
private $badInput;
private $gatekeep;
private $deleteType = 'resource';
private $idsRaw;
private $quotes;
private $paraphrases;
private $musings;
private $checkPublishers = array();
private $checkConfPublishers = array();
private $checkCollections = array();
private $checkTags = array();
public $resourceIds = array();
public $navigate = FALSE;
public $nextResourceId;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
	}
// check we are allowed to delete and load appropriate method
	public function init($message = FALSE)
	{
		$this->gatekeep->requireSuper = TRUE; // only admins can delete resources
		$this->gatekeep->init();
		if(array_key_exists('function', $this->vars))
		{
			$function = $this->vars['function'];
			$this->{$function}();
		}
		else
			$this->display();
	}
// display select box of resources to delete
	private function display($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "delete"));
		if(!$this->resources = $this->grabAll())
		{
			GLOBALS::buildOutputString($this->messages->text('misc', 'noResources'));
			return;
		}
		include_once("core/miscellaneous/TAG.php");
		$tag = new TAG();
		$tags = $tag->grabAll();
		$pString = $message ? $message : FALSE;
		$pString .= $this->form->formHeader('admin_DELETERESOURCE_CORE');
		$pString .= $this->form->hidden('function', 'deleteResourceConfirm');
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->selectFBoxValueMultiple(FALSE, "resource_id", $this->resources, 20, 80) . 
			$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . $this->html->br() . 
			$this->html->br() . $this->form->formSubmit('Confirm'));
		if(is_array($tags))
		{
// add 0 => IGNORE to tags array
			$temp[0] = $this->messages->text("misc", "ignore");
			foreach($tags as $key => $value)
				$temp[$key] = $value;
			$tags = $temp;
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text("misc", "tag"), 'bibtex_tagId', $tags, 5) . 
			$this->html->br() . $this->messages->text('hint', 'multiples'));
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Ask for confirmation of delete resource
	public function deleteResourceConfirm()
	{
		if(!$this->validateInput())
		{
			$this->display($this->errors->text("inputError", "missing"));
			FACTORY_CLOSE::getInstance();
		}
		if($this->deleteType == 'tag')
			$this->vars['resource_id'] = $this->collectResourceFromTag();
		$res = FACTORY_RESOURCECOMMON::getInstance();
		if(is_array($this->vars['resource_id']))
			$this->db->formatConditionsOneField($this->vars['resource_id'], 'resourceId');
		else
			$this->db->formatConditions(array('resourceId' => $this->vars['resource_id']));
		$recordset = $res->getResource(FALSE, $this->db->formatFields('creatorSurname'));
		if(!$numDeletes = $this->db->numRows($recordset))
		{
			$this->display($this->errors->text('resources', 'noResult'));
			FACTORY_CLOSE::getInstance();
		}
		GLOBALS::setOutputHeading($this->messages->text("heading", "delete"));
// Rather than print 100s or 1000s of resources, we limit display to <= 50
		if($numDeletes <= 50)
		{
			$bibStyle = FACTORY_BIBSTYLE::getInstance();
			$bibStyle->output = 'html';
			while($row = $this->db->fetchRow($recordset))
				$resourceList[]['resource'] = $bibStyle->process($row);
			$resourceList['submit'] = $this->form->formSubmit('Delete') . $this->form->formEnd();
			$this->template->tpl->assign('resourceList', $resourceList);
			$pString = '';
		}
		else
			$pString = $this->messages->text("misc", "confirmDelete", " " . $numDeletes . " ");
		$pString .= $this->form->formHeader('admin_DELETERESOURCE_CORE');
		$pString .= $this->form->hidden('function', 'deleteResource');
		if($this->navigate)
			$pString .= $this->form->hidden('navigate', $this->navigate);
		if($this->nextResourceId)
			$pString .= $this->form->hidden('nextResourceId', $this->nextResourceId);
		if(is_array($this->vars['resource_id']))
			$pString .= $this->form->hidden("resource_id", implode(",", $this->vars['resource_id']));
		else
			$pString .= $this->form->hidden("resource_id", $this->vars['resource_id']);
		if(array_key_exists('nextDelete', $this->vars))
			$pString .= $this->form->hidden("nextDelete", $this->vars['nextDelete']);
		$pString .= $this->html->br() . "&nbsp;" . $this->html->br();
		if($numDeletes > 50)
			$pString .= $this->form->formSubmit('Delete') . $this->form->formEnd();
		$this->session->setVar("deleteResourceLock", FALSE);
		GLOBALS::buildOutputString($pString);
	}
	private function deleteResource()
	{
// redeleting an already deleted resource?
		if($this->session->getVar("deleteResourceLock"))
		{
			$this->display($this->errors->text('done', 'resource'));
			FACTORY_CLOSE::getInstance();
		}
		if(!$this->validateInput())
		{
			$this->display($this->errors->text("inputError", "missing"));
			FACTORY_CLOSE::getInstance();
		}
		$this->idsRaw = explode(',', $this->vars['resource_id']);
		$this->reallyDelete();
		$this->checkHanging();
		$this->decrementSummary();
		$tables = array('resource', 'keyword', 'creator', 'collection', 'publisher', 'statistics', 
			'resource_accessed', 'resource_category', 
			'resource_keyword', 'resource_misc', 'resource_musing', 'resource_musing_text', 
			'resource_text', 'resource_page', 'resource_paraphrase', 'resource_quote', 
			'resource_paraphrase_text', 'resource_quote_text', 'resource_summary', 
			'resource_timestamp', 'resource_year', 'import_raw', 'bibtex_string', 
			'resource_attachments', 'resource_creator');
		foreach($tables as $table)
			$this->db->optimize($table);
// If we have 0 resources left, remove 'sql_stmt' etc. from session so it doesn't cause problems with 
// exporting bibliographies etc.
		$recordset = $this->db->select('database_summary', 'databasesummaryTotalResources');
		if(!$this->db->fetchOne($recordset))
		{
			$this->session->delVar('sql_Stmt');
			$this->session->delVar('sql_LastMulti');
			$this->session->delVar('sql_LastSolo');
		}
		$pString = $this->success->text("resource", " " . $this->messages->text("misc", "deleted") . " ");
// Lock reload.
		$this->session->setVar("deleteResourceLock", TRUE);
// Which page do we return to?
		if($this->navigate == 'nextResource') // next single view
		{
			$navigate = FACTORY_NAVIGATE::getInstance();
			$navigate->resource($this->nextResourceId, $pString);
		}
		else if($this->navigate == 'list') // previous multi list
		{
			$navigate = FACTORY_NAVIGATE::getInstance();
			$navigate->listView($pString);
		}
		else if($this->navigate == 'front') // Return to home page
		{
			include_once("core/display/FRONT.php");
			$front = new FRONT($pString); // __construct() runs on autopilot
		}
		else
			$this->display($pString); // return to multiple resource delete page -- $this->navigate == FALSE
		FACTORY_CLOSE::getInstance();
	}
// delete resources and meta data
	private function reallyDelete()
	{
		$this->db->formatConditionsOneField($this->idsRaw, 'resourceId');
		$this->db->leftJoin('resource_misc', 'resourcemiscId', 'resourceId');
		$recordset = $this->db->select('resource', array('resourcemiscPublisher', 'resourcemiscCollection',   
			'resourcemiscTag', 'resourceId', 'resourceType', 'resourcemiscField1'));
		while($row = $this->db->fetchRow($recordset))
		{
			if($row['resourcemiscPublisher'])
				$this->checkPublishers[$row['resourcemiscPublisher']] = FALSE;
			if(($row['resourceType'] == 'proceedings_article') && $row['resourcemiscField1'])
				$this->checkConfPublishers[$row['resourcemiscField1']] = FALSE;
			if($row['resourcemiscCollection'])
				$this->checkCollections[$row['resourcemiscCollection']] = FALSE;
			if($row['resourcemiscTag'])
				$this->checkTags[$row['resourcemiscTag']] = FALSE;
		}
// now start delete
		$this->db->formatConditionsOneField($this->idsRaw, 'resourceId');
		$this->db->delete('resource');
		$this->db->formatConditionsOneField($this->idsRaw, 'resourcemiscId');
		$this->db->delete('resource_misc');
		$this->db->formatConditionsOneField($this->idsRaw, 'resourcetextId');
		$this->db->delete('resource_text');
		$this->db->formatConditionsOneField($this->idsRaw, 'resourcecreatorResourceId');
		$this->db->delete('resource_creator');
		$this->db->formatConditionsOneField($this->idsRaw, 'resourcekeywordResourceId');
		$this->db->delete('resource_keyword');
		$this->db->formatConditionsOneField($this->idsRaw, 'resourcecategoryResourceId');
		$this->db->delete('resource_category');
		$this->db->formatConditionsOneField($this->idsRaw, 'resourcepageId');
		$this->db->delete('resource_page');
		$this->db->formatConditionsOneField($this->idsRaw, 'resourcesummaryId');
		$this->db->delete('resource_summary');
		$this->db->formatConditionsOneField($this->idsRaw, 'resourcetimestampId');
		$this->db->delete('resource_timestamp');
		$this->db->formatConditionsOneField($this->idsRaw, 'resourceyearId');
		$this->db->delete('resource_year');
		$this->db->formatConditionsOneField($this->idsRaw, 'importrawId');
		$this->db->delete('import_raw');
		$this->db->formatConditionsOneField($this->idsRaw, 'resourcecustomResourceId');
		$this->db->delete('resource_custom');
		$this->db->formatConditionsOneField($this->idsRaw, 'statisticsResourceId');
		$this->db->delete('statistics');
		$this->db->formatConditionsOneField($this->idsRaw, 'resourcequoteResourceId');
		$this->deleteMetadata('resource_quote', 'resourcequoteResourceId');
		$this->db->formatConditionsOneField($this->idsRaw, 'resourceparaphraseResourceId');
		$this->deleteMetadata('resource_paraphrase', 'resourceparaphraseResourceId');
		$this->db->formatConditionsOneField($this->idsRaw, 'resourcemusingResourceId');
		$this->deleteMetadata('resource_musing', 'resourcemusingResourceId');
		$this->checkBibtexStringTable();
// delete these ids from any user bibliographies
		$this->db->formatConditionsOneField($this->idsRaw, 'userbibliographyresourceResourceId');
		$this->db->delete('user_bibliography_resource');
// check file attachments
		$this->db->formatConditionsOneField($this->idsRaw, 'resourceattachmentsId');
		$recordSet = $this->db->select('resource_attachments', 
			array('resourceattachmentsId', 'resourceattachmentsHashFilename'));
		while($row = $this->db->fetchRow($recordSet))
			$hashes[$row['resourceattachmentsId']] = $row['resourceattachmentsHashFilename'];
		if(isset($hashes))
		{
			foreach($hashes as $id => $hash)
			{
				$this->db->formatConditions(array('resourceattachmentsId' => $id));
				$this->db->delete('resource_attachments');
// Is file used by other resources?  If not, unlink it
				$this->db->formatConditions(array('resourceattachmentsHashFilename' => $hash));
				$recordSet = $this->db->select('resource_attachments', 'resourceattachmentsHashFilename');
				if(!$this->db->numRows($recordSet)) // Unlink it
					@unlink("attachments/" . $hash);
			}
		}
	}
// decrement summary table
	private function decrementSummary()
	{
		$countR = $this->db->fetchOne($this->db->selectCount('resource', 'resourceId'));
		$countQ = $this->db->fetchOne($this->db->selectCount('resource_quote', 'resourcequoteId'));
		$countP = $this->db->fetchOne($this->db->selectCount('resource_paraphrase', 'resourceparaphraseId'));
		$countM = $this->db->fetchOne($this->db->selectCount('resource_musing', 'resourcemusingId'));
		$this->db->update('database_summary', array('databasesummaryTotalResources' => $countR, 
			'databasesummaryTotalQuotes' => $countQ, 'databasesummaryTotalParaphrases' => $countP, 
			'databasesummaryTotalMusings' => $countM));
	}
// check that delete of resources hasn't left any resource-less creators, keywords, publisher, collections etc.
// If so, delete them.
	private function checkHanging()
	{
		$creator = FACTORY_CREATOR::getInstance();
		$keyword = FACTORY_KEYWORD::getInstance();
		$publisher = FACTORY_PUBLISHER::getInstance();
// Collections -- must be done first as they might contain publishers and creators
		if(!empty($this->checkCollections))
		{
			$removeCollections = $this->checkCollections;
			foreach($this->checkCollections as $collectionId => $null)
			{
				$this->db->formatConditions(array('resourcemiscCollection' => $collectionId));
				$recordset = $this->db->select('resource_misc', 'resourcemiscCollection');
				if($this->db->numRows($recordset))
					unset($removeCollections[$collectionId]);
			}
			if(!empty($removeCollections))
			{
				$this->db->formatConditionsOneField(array_keys($removeCollections), 'collectionId');
				$this->db->delete('collection');
				$this->db->deleteCache('cacheResourceCollections');
				$this->db->deleteCache('cacheResourceCollectionTitles');
				$this->db->deleteCache('cacheResourceCollectionShorts');
				$this->db->deleteCache('cacheMetadataCollections');
			}
		}
// Creators
		$creator->removeHanging();
// Keywords
		$keyword->removeHanging();
// Publishers
		$publisher->removeHanging();
// Tags
		foreach($this->checkTags as $tagId => $void)
		{
			$this->db->formatConditions(array('resourcemiscTag' => $tagId));
			if(!$this->db->fetchOne($this->db->select('resource_misc', 'resourcemiscTag')))
			{
				$this->db->formatConditions(array('tagId' => $tagId));
				$this->db->delete('tag');
			}
		}
	}
// check @strings still have resources in import_raw - else delete string entries
	private function checkBibtexStringTable()
	{
		$recordset = $this->db->select('import_raw', array('importrawId', 'importrawStringId'));
// Delete all from `bibtex_string`
		if(!$this->db->numRows($recordset))
		{
			$this->db->delete('bibtex_string');
			return;
		}
		$rawStringIds = array();
		while($row = $this->db->fetchRow($recordset))
		{
			if(!$row['importrawStringId'])
				continue;
			$rawStringIds[] = $row['importrawStringId'];
		}
		if(empty($rawStringIds))
			return;
		foreach(array_unique($rawStringIds) as $id)
			$deleteIds['bibtexstringId'] = $id;
		$this->db->formatConditions($deleteIds, TRUE); // not equal to...
		$this->db->delete('bibtex_string');
	}
// validate input
	private function validateInput()
	{
		if(array_key_exists('navigate', $this->vars))
			$this->navigate = $this->vars['navigate'];
		if(array_key_exists('nextResourceId', $this->vars))
			$this->nextResourceId = $this->vars['nextResourceId'];
		if(!empty($this->resourceIds))
			$this->vars = array_merge($this->vars, $this->resourceIds);
		if(array_key_exists('bibtex_tagId', $this->vars))
		{
			foreach($this->vars['bibtex_tagId'] as $tag)
			{
				if($tag)
				{
					$this->deleteType = 'tag';
					return TRUE;
				}
			}
		}
		if(!array_key_exists('resource_id', $this->vars))
			return FALSE;
		return TRUE;
	}
// run SQL delete statements on meta data
	private function deleteMetadata($table, $field)
	{
		$ids = array();
// First get meta data id
		$recordset = $this->db->select($table, $this->db->prependTableToField($table, 'Id'));
		while($row = $this->db->fetchRow($recordset))
			$ids[] = $row[$this->db->prependTableToField($table, 'Id')];
// Delete meta data parent row
		$this->db->formatConditionsOneField($this->idsRaw, $field);
		$this->db->delete($table);
		if(empty($ids))
			return;
// Delete meta data child rows
		$newTable = $table . "_text";
		$this->db->formatConditionsOneField($ids, $this->db->prependTableToField($newTable, 'Id'));
		$this->db->delete($newTable);
		if($table != 'resource_musing')
		{
			$newTable = $table . "_comment";
			$this->db->formatConditionsOneField($ids, $this->db->prependTableToField($newTable, 'Id'));
			$this->db->delete($newTable);
		}
// Delete metadata keywords
		$split = explode('_', $table);
		if($split[1] == 'quote')
		{
			$this->db->formatConditionsOneField($ids, 'resourcekeywordQuoteId');
			$this->db->delete('resource_keyword');
		}
		else if($split[1] == 'paraphrase')
		{
			$this->db->formatConditionsOneField($ids, 'resourcekeywordParaphraseId');
			$this->db->delete('resource_keyword');
		}
		else if($split[1] == 'musing')
		{
			$this->db->formatConditionsOneField($ids, 'resourcekeywordMusingId');
			$this->db->delete('resource_keyword');
		}
	}
// Grab ids and titles of resources
	private function grabAll()
	{
		$titles = array();
		$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourceId');
		$this->db->leftJoin('resource_year', 'resourceyearId', 'resourceId');
		$fields[] = $this->db->formatFields(array('resourceId', 'resourceType', 'resourceTitleSort', 
			array('resourcecreatorCreatorSurname' => 'creatorSurname')));
		$fields[] = $this->db->coalesce(array('resourceyearYear1', 'resourceyearYear2'), 'year');
		$this->db->orderBy('creatorSurname');
		$this->db->orderBy('resourceyearYear1');
		$this->db->orderBy('resourceTitleSort');
		$recordset = $this->db->select('resource', join(',', $fields), TRUE, FALSE);
		while($row = $this->db->fetchRow($recordset))
		{
			$final = array();
			if($row['creatorSurname'])
				$final[] = $row['creatorSurname'];
			if($row['year'])
				$final[] = '(' . $row['year'] . ')';
			$final[] = $row['resourceTitleSort'];
			$final[] = '[' . $row['resourceType'] . ']';
			$titles[$row['resourceId']] = $this->html->dbToFormTidy(join(', ', $final));
		}
		return $titles;
	}
// get array of resource ids belonging to tag ids
	private function collectResourceFromTag()
	{
		$this->db->formatConditionsOneField($this->vars['bibtex_tagId'], 'resourcemiscTag');
		$recordset = $this->db->select('resource_misc', 'resourcemiscId');
		if(!$this->db->numRows($recordset))
		{
			$this->display($this->errors->text('resources', 'noResult'));
			FACTORY_CLOSE::getInstance();
		}
		while($row = $this->db->fetchRow($recordset))
			$ids[] = $row['resourcemiscId'];
		return $ids;
	}
}
?>
