<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* BOOKMARK class
* Manage user's bookmarks.
*/
class BOOKMARK
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $template;
private $badInput;
private $form;
private $html;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "bookmark"));
	}
// Display form for adding a bookmark
	public function init($error = FALSE)
	{
		$pString = $error ? $this->html->p($error, "error", "center") : '';
		$pString .= $this->html->p($this->messages->text("misc", "bookmark"));
		$pString .= $this->form->formHeader("bookmarks_BOOKMARK_CORE");
		$pString .= $this->form->hidden("method", "add");
		$pString .= $this->html->p($this->form->textInput($this->messages->text("misc", "bookmarkName"), "name", FALSE, 16, 15));
		$bookmarks = $this->session->getArray("bookmark");
		if(is_array($bookmarks)) // no space left so display list for replacement
		{
			$max = TRUE;
			for($i = 1; $i <= 20; $i++)
			{
				if(!array_key_exists($i . "_name", $bookmarks))
				{
					$max = FALSE;
					break;
				}
			}
			if($max)
			{
				$pString .= $this->html->p($this->messages->text("misc", "bookmarkDelete"));
				$checked = TRUE;
				for($i = 1; $i <= 20; $i++)
				{
					if(array_key_exists($i . "_name", $bookmarks))
					{
						$pString .= $this->html->p($this->form->radioButton(FALSE, "bookmark_replace", $i, $checked) . 
						"&nbsp;&nbsp;" . stripslashes($bookmarks[$i . "_name"]));
						$checked = FALSE;
					}
				}
			}
		}
		$pString .= $this->form->formSubmit('Add');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Add a bookmark
	public function add()
	{
		if(!array_key_exists("name", $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$name = trim($this->vars['name']);
		if(!$name)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$bookmarks = $this->session->getArray("bookmark");
		$id = 1; // default
		if(array_key_exists('bookmark_replace', $this->vars))
		{
			$id = $this->vars['bookmark_replace'];
			$this->session->delVar("bookmark_" . $id . "_id");
			$this->session->delVar("bookmark_" . $id . "_multi");
		}
		else
		{
			if(is_array($bookmarks))
			{
				if($key = array_search($name, $bookmarks))
				{
					$split = explode('_', $key);
					$id = $split[0];
				}
				else
				{
					for($i = 1; $i <= 20; $i++)
					{
						if(!array_key_exists($i . "_name", $bookmarks))
						{
							$id = $i;
							break;
						}
					}
				}
			}
		}
		$this->session->setVar("bookmark_" . $id . "_name", $name);
		if($this->session->getVar('bookmark_View') == 'solo')
		{
			$this->session->setVar("bookmark_" . $id . "_id", $this->session->getVar('sql_LastSolo'));
			$this->session->saveState('bookmark');
// send back to view this resource with success message
			GLOBALS::buildOutputString($this->success->text("bookmark"));
			include_once("core/modules/resource/RESOURCEVIEW.php");
			$resource = new RESOURCEVIEW();
			$resource->init($this->session->getVar('sql_LastSolo'));
		}
		else // multi view
		{
			$bookmark['sql_ListParams'] = $this->session->getVar('sql_ListParams');
			$bookmark['sql_Stmt'] = $this->session->getVar('sql_Stmt');
			$bookmark['sql_LastMulti'] = $this->session->getVar('sql_LastMulti');
			$bookmark['sql_DisplayAttachment'] = $this->session->getVar('sql_DisplayAttachment');
			$bookmark['sql_CountStmt'] = $this->session->getVar('sql_CountStmt');
			$bookmark['sql_CountAlphaStmt'] = $this->session->getVar('sql_CountAlphaStmt');
			preg_match("/_(.*)_CORE/", $this->session->getVar('sql_LastMulti'), $match);
			if($match[1] == 'SELECTRESOURCES')
				$listType = 'select';
			else if($match[1] == 'SEARCHRESOURCES')
				$listType = 'search';
			else if($match[1] == 'QUICKSEARCH')
				$listType = 'search';
			else if(($match[1] == 'LISTRESOURCES') || ($match[1] == 'LISTSOMERESOURCES'))
				$listType = 'list';
			$bookmark['listType'] = $listType;
			$bookmark['listTypeArray'] = base64_encode(serialize($this->session->getArray($listType)));
			$this->session->setVar("bookmark_" . $id . "_multi", serialize($bookmark));
			$this->session->saveState('bookmark');
// send back to view list with success message
			$navigate = FACTORY_NAVIGATE::getInstance();
			$navigate->listView($this->success->text("bookmark"));
		}
	}
// display bookmarks for deletion
	public function deleteInit($message = FALSE)
	{
		$bookmarks = $this->session->getArray("bookmark");
		$bookmarkArray = array();
		$pString = $message ? $this->html->p($message, "error", "center") : '';
		for($i = 1; $i <= 20; $i++)
		{
			if(array_key_exists($i . "_name", $bookmarks) && 
				array_key_exists($i . "_id", $bookmarks))
				$bookmarkArray[$i] = stripslashes($bookmarks[$i . "_name"]);
			else if(array_key_exists($i . "_name", $bookmarks) && 
				array_key_exists($i . "_multi", $bookmarks))
				$bookmarkArray[$i] = stripslashes($bookmarks[$i . "_name"]);
		}
		if(empty($bookmarkArray)) // none left
		{
			GLOBALS::buildOutputString($pString);
			return;
		}
		$pString .= $this->form->formHeader("bookmarks_BOOKMARK_CORE");
		$pString .= $this->form->hidden("method", "delete");
		$pString .= $this->form->selectFBoxValueMultiple($this->messages->text("misc", "bookmarkDeleteInit"), 
			"bookmark_id", $bookmarkArray, 20) . $this->html->br() . 
			$this->html->span($this->messages->text('hint', 'multiples'), 'hint') . 
			$this->html->br() . $this->form->formSubmit('Delete');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Delete bookmark(s)
	public function delete()
	{
		if(!array_key_exists("bookmark_id", $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"), 'deleteInit');
		$bookmarks = $this->session->getArray("bookmark");
		for($i = 1; $i <= 20; $i++)
		{
			if(array_search($i, $this->vars['bookmark_id']) !== FALSE)
			{
				if(array_key_exists($i . "_id", $bookmarks))
					$this->session->delVar('bookmark_' . $i . '_id');
				else if(array_key_exists($i . "_multi", $bookmarks))
					$this->session->delVar('bookmark_' . $i . '_multi');
				$this->session->delVar('bookmark_' . $i . '_name');
			}
		}
		$this->session->saveState('bookmark');
		$this->deleteInit($this->success->text('bookmarkDelete'));
	}
// view a multi list bookmark
	public function multiView()
	{
		$bookmarks = $this->session->getArray("bookmark");
		$bookmark = unserialize($bookmarks[$this->vars['id'] . '_multi']);
// paging preferences may have changed...
		$stmt = base64_encode(preg_replace('/(LIMIT 0, )(\d+)/', '${1}' . $this->session->getVar('setup_Paging'), base64_decode($bookmark['sql_Stmt'])));
		$this->session->setVar('sql_Stmt', $stmt);
		$this->session->setVar('sql_LastMulti', $bookmark['sql_LastMulti']);
		$this->session->setVar('sql_CountStmt', $bookmark['sql_CountStmt']);
		$this->session->setVar('sql_CountAlphaStmt', $bookmark['sql_CountAlphaStmt']);
		$this->session->setVar('bookmark_MultiView', TRUE);
		$this->session->delVar('select_DisplayAttachment');
		$this->session->delVar('search_DisplayAttachment');
		$this->session->setVar($bookmark['sql_DisplayAttachment'], TRUE);
		$this->session->setVar('sql_ListParams', $bookmark['sql_ListParams']);
		$this->session->clearArray($bookmark['listType']);
		$this->session->writeArray(unserialize(base64_decode($bookmark['listTypeArray'])), $bookmark['listType']);
		$this->session->delVar('mywikindx_PagingStart');
		$this->session->delVar('mywikindx_PagingStartAlpha');
		$navigate = FACTORY_NAVIGATE::getInstance();
		$navigate->listView();
	}
}
?>
