<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net

**********************************************************************************/
/**
* BROWSEKEYWORD class
*
* Browse keywords as a 'tag cloud'
*
*/
class BROWSEKEYWORD
{
private $db;
private $vars;
private $common;
private $html;
private $template;
private $messages;
private $bib;
public $keyword;
public $sum;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		include_once("core/browse/BROWSECOMMON.php");
		$this->common = new BROWSECOMMON();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "browseKeyword"));
	}
// display keywords
	public function init()
	{
		$this->sum = $this->keyword = array();
		if(array_key_exists('metadata', $this->vars))
			$this->getMetaKeywords();
		else
			$this->getKeywords();
		if(empty($this->keyword))
		{
			if(array_key_exists('metadata', $this->vars))
				GLOBALS::buildOutputString($this->messages->text("misc", "noMetaKeywords"));
			else
				GLOBALS::buildOutputString($this->messages->text("misc", "noKeywords"));
			return;
		}
		natcasesort($this->sum);
		$this->keyword = $this->common->paging($this->keyword);
		$this->common->linksInfo();
		$pString = $this->html->pBrowse($this->process(), "center");
		if(array_key_exists('metadata', $this->vars))
			$this->common->pagingLinks('action=browse_BROWSEKEYWORD_CORE&metadata=1');
		else
			$this->common->pagingLinks('action=browse_BROWSEKEYWORD_CORE');
		GLOBALS::buildOutputString($pString);
	}
// Process and display results
	private function process()
	{
		$lowestSum = current($this->sum);
		$highestSum = end($this->sum);
		foreach($this->keyword as $id => $name)
		{
			$colour = $this->common->colourText($lowestSum, $highestSum, $this->sum[$id]);
			$size = $this->common->sizeText($lowestSum, $highestSum, $this->sum[$id]);
			if(array_key_exists('metadata', $this->vars))
				$links[] = $this->html->aBrowse($colour, $size, $name, 
					"index.php?action=list_LISTSOMERESOURCES_CORE&method=metaKeywordProcess" . 
					htmlentities("&id=" . $id)) . "&nbsp;[" . $this->sum[$id] . "]";
			else
				$links[] = $this->html->aBrowse($colour, $size, $name, 'index.php?' . 
				htmlentities('action=list_LISTSOMERESOURCES_CORE&method=keywordProcess&id=' . $id)) . 
				"&nbsp;[" . $this->sum[$id] . "]";
		}
		return join("&nbsp; ", $links);
	}
// Get resource keywords from db
	public function getKeywords()
	{
		$this->common->userBibCondition('resourcekeywordResourceId');
		$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
		$this->db->formatConditions($this->db->formatFields('resourcekeywordResourceId') . ' IS NOT NULL');
		$this->db->groupBy('resourcekeywordKeywordId');
		$this->db->orderBy('keywordKeyword');
		$recordset = $this->db->selectCounts('resource_keyword', 'resourcekeywordKeywordId', 'keywordKeyword');
		while($row = $this->db->fetchRow($recordset))
			$this->collate($row);
	}
// Get keywords for metadata from db
	private function getMetaKeywords()
	{
// quotes
		$this->db->formatConditions(array('resourcekeywordQuoteId' => ' IS NOT NULL'));
		$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
		$this->db->leftJoin('resource_quote', 'resourcequoteId', 'resourcekeywordQuoteId');
		$this->common->userBibCondition('resourcequoteResourceId', FALSE);
		$this->db->groupBy('resourcekeywordKeywordId');
		$this->db->orderBy('keywordKeyword');
		$recordset = $this->db->selectCounts('resource_keyword', 'resourcekeywordKeywordId', 'keywordKeyword');
		while($row = $this->db->fetchRow($recordset))
			$this->collate($row);
// paraphrases
		$this->db->formatConditions(array('resourcekeywordParaphraseId' => ' IS NOT NULL'));
		$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
		$this->db->leftJoin('resource_paraphrase', 'resourceparaphraseId', 'resourcekeywordParaphraseId');
		$this->common->userBibCondition('resourceparaphraseResourceId', FALSE);
		$this->db->groupBy('resourcekeywordKeywordId');
		$this->db->orderBy('keywordKeyword');
		$recordset = $this->db->selectCounts('resource_keyword', 'resourcekeywordKeywordId', 'keywordKeyword');
		while($row = $this->db->fetchRow($recordset))
			$this->collate($row);
// musings
		if($this->session->getVar('setup_ReadOnly'))
			$this->db->formatConditions(array('resourcemusingtextPrivate' => 'N'));
		else if($userId = $this->session->getVar('setup_UserId'))
		{
			$this->db->formatConditions(array('usergroupsusersUserId' => $userId));
			$this->db->formatConditions($this->db->formatFields('usergroupsusersGroupId') . $this->db->equal . 
				$this->db->formatFields('resourcemusingtextPrivate'));
			$subSql = $this->db->selectNoExecute('user_groups_users', 'usergroupsusersId', FALSE, TRUE, TRUE);
			$subject = $this->db->formatFields('resourcemusingtextPrivate') . $this->db->notEqual . $this->db->tidyInput('N') 
				. $this->db->and . 
				$this->db->formatFields('resourcemusingtextPrivate') . $this->db->notEqual . $this->db->tidyInput('Y');
			$case1 = $this->db->caseWhen($subject, FALSE, $subSql, FALSE, FALSE);
			$subject = $this->db->formatFields('resourcemusingtextPrivate') . $this->db->equal . $this->db->tidyInput('Y');
			$result = $this->db->formatFields('resourcemusingtextAddUserIdMusing') . $this->db->equal . $this->db->tidyInput($userId);
			$case2 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
			$subject = $this->db->formatFields('resourcemusingtextPrivate') . $this->db->equal . $this->db->tidyInput('N');
			$result = $this->db->tidyInput(1);
			$case3 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
			$this->db->formatConditions($case1 . $this->db->or . $case2 . $this->db->or . $case3);
		}
		$this->db->formatConditions(array('resourcekeywordMusingId' => ' IS NOT NULL'));
		$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
		$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcekeywordMusingId');
		$this->db->leftJoin('resource_musing', 'resourcemusingId', 'resourcekeywordMusingId');
		$this->common->userBibCondition('resourcemusingResourceId', FALSE);
		$this->db->groupBy('resourcekeywordKeywordId');
		$this->db->orderBy('keywordKeyword');
		$recordset = $this->db->selectCounts('resource_keyword', 'resourcekeywordKeywordId', 
			array('keywordKeyword', 'resourcemusingtextPrivate'));
		while($row = $this->db->fetchRow($recordset))
		{
			if(($userId = $this->session->getVar('setup_UserId')) && ($row['resourcemusingtextPrivate'] != 'N') && 
				($row['resourcemusingtextPrivate'] != 'Y')) // musing is part of user group
			{
				$this->db->formatConditions(array('usergroupsusersUserId' => $userId));
				$this->db->formatConditions(array('usergroupsusersGroupId' => $row['resourcemusingtextPrivate']));
				$resultset = $this->db->select('user_groups_users', 'usergroupsusersId');
				if($this->db->numRows($resultset))
					$this->collate($row);
			}
			else
				$this->collate($row);
		}
	}
// Add keywords to array and sum totals
	public function collate($row)
	{
		if(!array_key_exists($row['resourcekeywordKeywordId'], $this->keyword))
			$this->keyword[$row['resourcekeywordKeywordId']] = preg_replace("/{(.*)}/U", "$1", 
				$this->html->dbToHtmlTidy($row['keywordKeyword']));
		if(!array_key_exists($row['resourcekeywordKeywordId'], $this->sum))
			$this->sum[$row['resourcekeywordKeywordId']] = $row['count'];
		else
			$this->sum[$row['resourcekeywordKeywordId']] += $row['count'];
	}
}
?>
