<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net

**********************************************************************************/
/**
* BROWSESUBCATEGORY class
*
* Browse subcategories as a 'tag cloud'
*
*/
class BROWSESUBCATEGORY
{
private $db;
private $vars;
private $common;
private $html;
private $template;
private $messages;
public $subcategory;
public $categories;
public $sum;
public $bib;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		include_once("core/browse/BROWSECOMMON.php");
		$this->common = new BROWSECOMMON();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "browseSubcategory"));
	}
// display categories
	public function init()
	{
		$this->sum = $this->subcategory = array();
		$this->getSubcategories();
		if(empty($this->subcategory))
		{
			GLOBALS::buildOutputString($this->messages->text("misc", "noSubcategories"));
			return;
		}
		natcasesort($this->sum);
		$this->subcategory = $this->common->paging($this->subcategory);
		$this->common->linksInfo();
		$pString = $this->html->pBrowse($this->process(), "center");
		$this->common->pagingLinks('action=browse_BROWSESUBCATEGORY_CORE');
		GLOBALS::buildOutputString($pString);
	}
// Get subcategories from db
	public function getSubcategories()
	{
		$this->common->userBibCondition('resourcecategoryResourceId');
		$this->db->leftJoin('subcategory', 'subcategoryId', 'resourcecategorySubcategoryId');
		$this->db->leftJoin('category', 'categoryId', 'subcategoryCategoryId');
		$this->db->groupBy('resourcecategorySubcategoryId', TRUE, $this->db->formatFields('count') . 
			$this->db->greater . $this->db->tidyInput(0));
		$this->db->orderBy('subcategorySubcategory');
		$recordset = $this->db->selectCounts('resource_category', 'resourcecategorySubcategoryId', 
			array('subcategorySubcategory', 'subcategoryCategoryId', 'categoryId', 'categoryCategory'));
		while($row = $this->db->fetchRow($recordset))
		{
			if($row['resourcecategorySubcategoryId'])
				$this->categories[$row['resourcecategorySubcategoryId']] = $this->html->dbToHtmlTidy($row['categoryCategory']);
			$this->collate($row);
		}
	}
// Add categories to array and sum totals
	public function collate($row)
	{
		if(!$row['subcategorySubcategory'])
			return;
		$this->subcategory[$row['resourcecategorySubcategoryId']] = preg_replace("/{(.*)}/U", "$1", 
			$this->html->dbToHtmlTidy($row['subcategorySubcategory']));
		$this->sum[$row['resourcecategorySubcategoryId']] = $row['count'];
	}
// Process and display results
	private function process()
	{
		$lowestSum = current($this->sum);
		$highestSum = end($this->sum);
		foreach($this->subcategory as $id => $name)
		{
			$colour = $this->common->colourText($lowestSum, $highestSum, $this->sum[$id]);
			$size = $this->common->sizeText($lowestSum, $highestSum, $this->sum[$id]);
			$links[] = $this->html->aBrowse($colour, $size, $name, 'index.php?' . 
				htmlentities('action=list_LISTSOMERESOURCES_CORE&method=subcategoryProcess&id=' . $id)) . 
				'&nbsp;(' . $this->categories[$id] . ")&nbsp;[" . $this->sum[$id] . "]";
		}
		return join("&nbsp; ", $links);
	}
}
?>