<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net

**********************************************************************************/
/**
* BROWSEUSER class
*
* Browse registered users, departments and institutions as 'tag clouds' where each of those classes has users identified as creators in the wikindx
*
*/
class BROWSEUSER
{
private $db;
private $vars;
private $common;
private $html;
private $template;
private $messages;
private $bbCode;
private $user;
private $department;
private $institution;
public $sum;
private $lowestSum = 1;
private $highestSum = 0;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		include_once("core/browse/BROWSECOMMON.php");
		$this->common = new BROWSECOMMON();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->bbCode = FACTORY_BBCODE::getInstance();
	}
// display system users
	public function user()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "browseUser"));
		$this->sum = $this->user = $this->sameAs = array();
		$this->db->formatConditions($this->db->formatFields('usersIsCreator') . ' IS NOT NULL');
		$this->getUsers('usersFullname');
		if(empty($this->user))
		{
			GLOBALS::buildOutputString($this->messages->text("misc", "noUsers"));
			return;
		}
		natcasesort($this->sum);
		$this->user = $this->common->paging($this->user);
		$this->common->linksInfo();
		$pString = $this->html->pBrowse($this->process('id'), "center");
		$this->common->pagingLinks('action=browse_BROWSEUSER_CORE&method=user');
		GLOBALS::buildOutputString($pString);
	}
// display system departments
	public function department()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "browseDept"));
		$this->sum = $this->user = $this->sameAs = array();
		$this->db->formatConditions($this->db->formatFields('usersIsCreator') . ' IS NOT NULL');
		$this->db->formatConditions($this->db->formatFields('usersDepartment') . ' IS NOT NULL');
		$this->getUsers('usersDepartment');
		if(empty($this->user))
		{
			GLOBALS::buildOutputString($this->messages->text("misc", "noUsers"));
			return;
		}
		natcasesort($this->sum);
		$this->user = $this->common->paging($this->user);
		$this->common->linksInfo();
		$pString = $this->html->pBrowse($this->process('department'), "center");
		$this->common->pagingLinks('action=browse_BROWSEUSER_CORE&method=department');
		GLOBALS::buildOutputString($pString);
	}
// display system institutions
	public function institution()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "browseInst"));
		$this->sum = $this->user = $this->sameAs = array();
		$this->db->formatConditions($this->db->formatFields('usersIsCreator') . ' IS NOT NULL');
		$this->db->formatConditions($this->db->formatFields('usersInstitution') . ' IS NOT NULL');
		$this->getUsers('usersInstitution');
		if(empty($this->user))
		{
			GLOBALS::buildOutputString($this->messages->text("misc", "noUsers"));
			return;
		}
		natcasesort($this->sum);
		$this->user = $this->common->paging($this->user);
		$this->common->linksInfo();
		$pString = $this->html->pBrowse($this->process('institution'), "center");
		$this->common->pagingLinks('action=browse_BROWSEUSER_CORE&method=institution');
		GLOBALS::buildOutputString($pString);
	}
// Process and display users results
	private function process($type)
	{
		foreach($this->user as $id => $name)
		{
			$name = $this->bbCode->codeToHtml($name);
			$colour = $this->common->colourText($this->lowestSum, $this->highestSum, $this->sum[$id]);
			$size = $this->common->sizeText($this->lowestSum, $this->highestSum, $this->sum[$id]);
			$links[] = $this->html->aBrowse($colour, $size, $name, 'index.php?' . 
				htmlentities("action=list_LISTSOMERESOURCES_CORE&method=creatorProcess&$type=" . $id)) . "&nbsp;[" . $this->sum[$id] . "]";
		}
		return join("&nbsp; ", $links);
	}
// Get system users from db
	public function getUsers($field)
	{
		$bib = $this->common->userBibCondition('resourcecreatorResourceId');
		$this->db->leftJoin('resource_creator', 'resourcecreatorCreatorId', 'usersIsCreator');
		$subSql = $this->db->selectNoExecute('users', array($field, 'resourcecreatorResourceId', 'resourcecreatorCreatorId'), TRUE, TRUE, TRUE);
		$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
		$this->db->groupBy('resourcecreatorCreatorId', TRUE, $this->db->formatFields('count') . $this->db->greater . $this->db->tidyInput(0));
		$this->db->orderBy($field);
		$recordset = $this->db->selectCounts(FALSE, 'resourcecreatorCreatorId', array($field), $this->db->subQuery($subSql, 'rc', FALSE), FALSE);
		while($row = $this->db->fetchRow($recordset))
			$this->collate($row, $field);
		$this->sumSort = $this->sum;
		sort($this->sumSort, SORT_NUMERIC);
		$this->lowestSum = array_shift($this->sumSort);
		if(empty($this->sumSort)) // May be only one entry
			$this->highestSum = $this->lowestSum;
		else
			$this->highestSum = array_pop($this->sumSort);
	}
// Add users to array and sum totals
	public function collate($row, $field)
	{
		if($field == 'usersDepartment')
		{
			$this->user[$row['usersDepartment']] = preg_replace("/{(.*)}/U", "$1", 
				$this->html->dbToFormTidy($row[$field]));
			if(!array_key_exists($row['usersDepartment'], $this->sum))
				$this->sum[$row['usersDepartment']] = $row['count'];
			else
				$this->sum[$row['usersDepartment']] += $row['count'];
			$sumIndex = $row['usersDepartment'];
		}
		else if($field == 'usersInstitution')
		{
			$this->user[$row['usersInstitution']] = preg_replace("/{(.*)}/U", "$1", 
				$this->html->dbToFormTidy($row[$field]));
			if(!array_key_exists($row['usersInstitution'], $this->sum))
				$this->sum[$row['usersInstitution']] = $row['count'];
			else
				$this->sum[$row['usersInstitution']] += $row['count'];
			$sumIndex = $row['usersInstitution'];
		}
		else // creators
		{
			$this->user[$row['resourcecreatorCreatorId']] = preg_replace("/{(.*)}/U", "$1", 
				$this->html->dbToFormTidy($row[$field]));
			$this->sum[$row['resourcecreatorCreatorId']] = $row['count'];
			$sumIndex = $row['resourcecreatorCreatorId'];
		}
		$this->db->formatConditions($this->db->formatFields('creatorSameAs') . $this->db->equal . $this->db->tidyInput($row['resourcecreatorCreatorId']));
		$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
		$this->db->groupBy('resourcecreatorCreatorId', TRUE, $this->db->formatFields('count') . $this->db->greater . $this->db->tidyInput(0));
		$recordset = $this->db->selectCounts('resource_creator', 'resourcecreatorCreatorId', array('creatorId'), FALSE, FALSE);
		while($row2 = $this->db->fetchRow($recordset))
		{
			if($row2['count'])
				$this->sum[$sumIndex] += $row2['count'];
		}
	}
}
?>
