<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*
* CMS hooks
*
* @author Mark Grimshaw
* @version 1
* $Header $
*/

class CMS
{
private $db;
private $vars;
private $errors;
private $messages;
private $session;
private $template;
private $form;
private $html;
private $table;
private $badInput;

// Constructor
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->session = FACTORY_SESSION::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "cms"));
	}
// display pop-up for a single resource
	public function display()
	{
		if(!array_key_exists('id', $this->vars))
		{
			$this->badInput->closeType = 'closeNoMenu';
			$this->badInput->close($this->messages->text('inputError', 'missing'));
		}
		$id = $this->vars['id'];
		$pString = $this->html->p($this->messages->text('cms', 'introduction1'));
		$pString .= $this->html->p($this->getResourceQuery($id));
		$pString .= $this->displayTag();
		$pString .= $this->html->br() . $this->html->hr() . $this->html->br();
		$pString .= $this->html->h($this->messages->text('cms', 'displayIds'), 3);
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$links[] = $this->html->a('link', $this->messages->text('resources', 'categories'), 
			'index.php?action=cms_CMS_CORE&method=display&type=categories&id=' . $id);
		$links[] = $this->html->a('link', $this->messages->text('resources', 'subcategories'), 
			'index.php?action=cms_CMS_CORE&method=display&type=subcategories&id=' . $id);
		$links[] = $this->html->a('link', $this->messages->text('resources', 'keywords'), 
			'index.php?action=cms_CMS_CORE&method=display&type=keywords&id=' . $id);
		$links[] = $this->html->a('link', $this->messages->text('resources', 'creator'), 
			'index.php?action=cms_CMS_CORE&method=display&type=creators&id=' . $id);
		$links[] = $this->html->a('link', $this->messages->text('resources', 'publisher'), 
			'index.php?action=cms_CMS_CORE&method=display&type=publishers&id=' . $id);
		$links[] = $this->html->a('link', $this->messages->text('resources', 'collection'), 
			'index.php?action=cms_CMS_CORE&method=display&type=collections&id=' . $id);
		$pString .= $this->table->td(join("&nbsp;&nbsp;&nbsp;", $links), FALSE, FALSE, FALSE, FALSE, '50%');
		if(array_key_exists('type', $this->vars))
		{
			$ajax = FACTORY_AJAX::getInstance();
			$ajax->loadJavascript(array('core/modules/list/searchSelect.js'));
			if($this->vars['type'] == 'categories')
			{
				$category = FACTORY_CATEGORY::getInstance();
				$raw = $category->grabAll();
				if(is_array($raw) && !empty($raw))
				{
					$jScript = 'index.php?action=cms_CMS_CORE&method=qStringAjax&type=getCategory';
					$jsonArray = array();
					$jsonArray[] = array(
						'startFunction' => 'triggerFromMultiSelect', 
						'script' => "$jScript",
						'triggerField' => 'category',
						'targetDiv' => 'qString'
						);
					$ajax->jActionForm('onclick', $jsonArray);
					foreach($raw as $key => $value)
						$array[$key] = $value . ":&nbsp;&nbsp;" . $key;
					$pString .= $this->table->td($this->form->selectFBoxValue($this->messages->text('resources', 'categories'), 
						'category', $array, 1));
				}
			}
			else if($this->vars['type'] == 'subcategories')
			{
				$category = FACTORY_CATEGORY::getInstance();
				$raw = $category->grabSubAll();
				if(is_array($raw) && !empty($raw))
				{
					$jScript = 'index.php?action=cms_CMS_CORE&method=qStringAjax&type=getSubcategory';
					$jsonArray = array();
					$jsonArray[] = array(
						'startFunction' => 'triggerFromMultiSelect', 
						'script' => "$jScript",
						'triggerField' => 'subcategory',
						'targetDiv' => 'qString'
						);
					$ajax->jActionForm('onclick', $jsonArray);
					foreach($raw as $key => $value)
						$sarray[$key] = $value . ":&nbsp;&nbsp;" . $key;
					$pString .= $this->table->td($this->form->selectFBoxValue($this->messages->text('resources', 'subcategories'), 
						'subcategory', $sarray, 1));
				}
			}
			else if($this->vars['type'] == 'keywords')
			{
				$keyword = FACTORY_KEYWORD::getInstance();
				$raw = $keyword->grabAll();
				if(is_array($raw) && !empty($raw))
				{
					$jScript = 'index.php?action=cms_CMS_CORE&method=qStringAjax&type=getKeyword';
					$jsonArray = array();
					$jsonArray[] = array(
						'startFunction' => 'triggerFromMultiSelect', 
						'script' => "$jScript",
						'triggerField' => 'keyword',
						'targetDiv' => 'qString'
						);
					$ajax->jActionForm('onclick', $jsonArray);
					foreach($raw as $key => $value)
						$array[$key] = $value . ":&nbsp;&nbsp;" . $key;
					$pString .= $this->table->td($this->form->selectFBoxValue($this->messages->text('resources', 'keywords'), 
						'keyword', $array, 1));
				}
			}
			else if($this->vars['type'] == 'creators')
			{
				$creator = FACTORY_CREATOR::getInstance();
				$raw = $creator->grabAll();
				if(is_array($raw) && !empty($raw))
				{
					$jScript = 'index.php?action=cms_CMS_CORE&method=qStringAjax&type=getCreator';
					$jsonArray = array();
					$jsonArray[] = array(
						'startFunction' => 'triggerFromMultiSelect', 
						'script' => "$jScript",
						'triggerField' => 'creator',
						'targetDiv' => 'qString'
						);
					$ajax->jActionForm('onclick', $jsonArray);
					foreach($raw as $key => $value)
						$array[$key] = $value . ":&nbsp;&nbsp;" . $key;
					$pString .= $this->table->td($this->form->selectFBoxValue($this->messages->text('resources', 'creator'), 
						'creator', $array, 1));
				}
			}
			else if($this->vars['type'] == 'publishers')
			{
				$publisher = FACTORY_PUBLISHER::getInstance();
				$raw = $publisher->grabAll();
				if(is_array($raw) && !empty($raw))
				{
					$jScript = 'index.php?action=cms_CMS_CORE&method=qStringAjax&type=getPublisher';
					$jsonArray = array();
					$jsonArray[] = array(
						'startFunction' => 'triggerFromMultiSelect', 
						'script' => "$jScript",
						'triggerField' => 'publisher',
						'targetDiv' => 'qString'
						);
					$ajax->jActionForm('onclick', $jsonArray);
					foreach($raw as $key => $value)
						$array[$key] = $value . ":&nbsp;&nbsp;" . $key;
					$pString .= $this->table->td($this->form->selectFBoxValue($this->messages->text('resources', 'publisher'), 
						'publisher', $array, 1));
				}
			}
			else if($this->vars['type'] == 'collections')
			{
				$collection = FACTORY_COLLECTION::getInstance();
				$raw = $collection->grabAll();
				if(is_array($raw) && !empty($raw))
				{
					$jScript = 'index.php?action=cms_CMS_CORE&method=qStringAjax&type=getCollection';
					$jsonArray = array();
					$jsonArray[] = array(
						'startFunction' => 'triggerFromMultiSelect', 
						'script' => "$jScript",
						'triggerField' => 'collection',
						'targetDiv' => 'qString'
						);
					$ajax->jActionForm('onclick', $jsonArray);
					foreach($raw as $key => $value)
						$array[$key] = $value . ":&nbsp;&nbsp;" . $key;
					$pString .= $this->table->td($this->form->selectFBoxValue($this->messages->text('resources', 'collection'), 
						'collection', $array, 1));
				}
			}
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->table->tdEnd() . $this->table->trEnd() . $this->table->trStart() . $this->table->tdStart();
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->html->div('qString', '&nbsp;'));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		GLOBALS::buildOutputString($pString);
		FACTORY_CLOSENOMENU::getInstance();
	}
/**
* Display pop-up for a display list
*/
	public function displayList()
	{
		$pString = $this->html->p($this->messages->text('cms', 'introduction2'));
		$sql = base64_decode($this->session->getVar('sql_Stmt'));
		$sql = preg_replace('/(\s*)LIMIT(\s*)(\d+),(\s*)(\d+)/i', '', $sql);
		$pString .= $this->form->textareaReadonly(FALSE, FALSE, base64_encode($sql), 100, 40);
		GLOBALS::buildOutputString($pString);
		FACTORY_CLOSENOMENU::getInstance();
	}
/**
* AJAX-based DIV content creator for querystrings
*/
	public function qStringAjax()
	{
		$ajax = FACTORY_AJAX::getInstance();
// if no ajaxReturn, quietly exit
		$div = $this->table->td($this->html->div('qString', '&nbsp;')); // default
		if(array_key_exists('ajaxReturn', $this->vars))
		{
			$array = explode('/', $_SERVER['SCRIPT_NAME']);
			unset($array[sizeof($array) - 1]);
			$qString = 'http://' . $_SERVER['HTTP_HOST'] . join('/', $array) . 
				'/cmsprint.php?action=' . $this->vars['type'] . '&id=' . $this->vars['ajaxReturn'];
			$div = $this->table->td($this->html->div('qString', $qString));
		}
		GLOBALS::buildOutputString($ajax->encode_jArray(array('innerHTML' => $div)));
		FACTORY_CLOSERAW::getInstance();
	}
/**
** Generate a CMS querystring for a single resource ID
*/
	private function getResourceQuery($id)
	{
		$array = explode('/', $_SERVER['SCRIPT_NAME']);
		unset($array[sizeof($array) - 1]);
		return 'http://' . $_SERVER['HTTP_HOST'] . join('/', $array) . "/cmsprint.php?action=getResource&id=$id";
	}
// Function form for generating CMS tag
	private function displayTag()
	{
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'generateCmsTag'))
		{
			$this->checkInput();
			$tag = $this->generateTag();
			unset($this->vars['type']);
		}
		else
			$tag = FALSE;
		$pString = $this->html->h($this->messages->text('cms', 'generateCmsTag'), 3);
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->form->formHeader("cms_CMS_CORE");
		$pString .= $this->form->hidden('method', 'display');
		$pString .= $this->form->hidden('type', 'generateCmsTag');
		$pString .= $this->form->hidden('id', $this-> vars['id']);
		$session = $this->session->getVar('setup_cmsTagStart');
		$pString .= $this->table->td($this->form->textInput($this->messages->text('cms', 'cmsTagStart'), 
			'cmsTagStart', $session, 20));
		$session = array_key_exists('cmsPageStart', $this->vars) ? $this-> vars['cmsPageStart'] : FALSE;
		$pString .= $this->table->td($this->form->textInput($this->messages->text('cms', 'pageStart'), 
			'cmsPageStart', $session, 10));
		$session = array_key_exists('cmsPageEnd', $this->vars) ? $this-> vars['cmsPageEnd'] : FALSE;
		$pString .= $this->table->td($this->form->textInput($this->messages->text('cms', 'pageEnd'), 
			'cmsPageEnd', $session, 10), FALSE, 'left');
		$session = array_key_exists('cmsPreText', $this->vars) ? $this-> vars['cmsPreText'] : FALSE;
		$pString .= $this->table->td($this->form->textInput($this->messages->text('cms', 'preText'), 
			'cmsPreText', $session, 20));
		$session = array_key_exists('cmsPostText', $this->vars) ? $this-> vars['cmsPostText'] : FALSE;
		$pString .= $this->table->td($this->form->textInput($this->messages->text('cms', 'postText'), 
			'cmsPostText', $session, 20));
		$session = $this->session->getVar('setup_cmsTagEnd');
		$pString .= $this->table->td($this->form->textInput($this->messages->text('cms', 'cmsTagEnd'), 
			'cmsTagEnd', $session, 20));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
// Encapsulating TABLE element from template
		$pString .= $this->table->tdEnd() . $this->table->trEnd() . $this->table->trStart() . $this->table->tdStart();
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->html->br() . $this->form->formSubmit("Submit"));
		if($tag)
			$pString .= $this->table->td($this->html->br() . $this->form->textInput($this->messages->text('cms', 'tag'), 
			FALSE, $tag, 50));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->table->tdEnd() . $this->table->trEnd() . $this->table->trStart() . $this->table->tdStart();
		$pString .= $this->form->formEnd();
		return $pString;
	}
// Generate the CMS tag
	private function generateTag()
	{
		$tag = FALSE;
		if($this->vars['cmsTagStart'])
			$tag = $this->vars['cmsTagStart'];
		$tag .= ':' . $this->vars['id'];
		if($this->vars['cmsPageStart'])
		{
			$tag .= ':' . trim($this->vars['cmsPageStart']);
			if($this->vars['cmsPageEnd'])
				$tag .= '-' . trim($this->vars['cmsPageEnd']);
		}
		else if($this->vars['cmsPageEnd']) // Assume pageEnd is actually meant to be pageStart
		{
			$tag .= ':' . trim($this->vars['cmsPageEnd']);
			$this->vars['cmsPageStart'] = $this->vars['cmsPageEnd'];
			unset($this->vars['cmsPageEnd']);
		}
		if($this->vars['cmsPreText'])
		{
			$tag .= '|' . $this->vars['cmsPreText'];
			if($this->vars['cmsPostText'])
				$tag .= '`' . $this->vars['cmsPostText'];
		}
		else if($this->vars['cmsPostText'])
			$tag .= '|`' . $this->vars['cmsPostText'];
		if($this->vars['cmsTagEnd'])
			$tag .= $this->vars['cmsTagEnd'];
		return $tag;
	}
// Check input from generateTag and write database value if required.
	private function checkInput()
	{
		$cmsTagChanged = FALSE;
		if($this->session->getVar('setup_cmsTagStart') != $this->vars['cmsTagStart'])
		{
			$cmsTagChanged = TRUE;
			$this->session->setVar('setup_cmsTagStart', $this->vars['cmsTagStart']);
		}
		if($this->session->getVar('setup_cmsTagEnd') != $this->vars['cmsTagEnd'])
		{
			$cmsTagChanged = TRUE;
			$this->session->setVar('setup_cmsTagEnd', $this->vars['cmsTagEnd']);
		}
		if($cmsTagChanged && $this->session->getVar('setup_UserId'))
		{
			$string = base64_encode(serialize(array($this->vars['cmsTagStart'], $this->vars['cmsTagEnd'])));
			$this->db->formatConditions(array('usersId' => $this->session->getVar('setup_UserId')));
			$this->db->update('users', array('usersCmsTag' => $string));
		}
	}
/*********************
*
*	The following functions deal with external calls to http://..../wikindx3/cmsprint.php.  They are NOT called from within WIKINDX
*
*********************/
	
// Create SQL statement from querystring
	public function queryDb()
	{
		if(empty($_GET) || !array_key_exists('action', $_GET))
			die('Missing or incorrect queryString');
		if(($_GET['action'] != 'getRecent') && !array_key_exists('id', $_GET))
			die('Missing or incorrect queryString');
		$db = FACTORY_DB::getInstance();
		$session = FACTORY_SESSION::getInstance();
		if(array_key_exists('bibStyle', $_GET))
			$session->setVar('setup_Style', $_GET['bibStyle']);
		else
		{
			$config = FACTORY_CONFIG::getInstance();
			$session->setVar('setup_Style', $config->WIKINDX_CMS_BIBSTYLE);
		}
		$res = FACTORY_RESOURCECOMMON::getInstance();
		$bibStyle = FACTORY_BIBSTYLE::getInstance();
		if(array_key_exists('id', $_GET))
			$ids = explode(',', $_GET['id']);
// Remove WIKINDX-style hyperlink class.
		$pattern = preg_quote("<a class=\"rLink\"");
// $sqlMethod only used for 'getKeyword', 'getCategory' and 'getCreator'
		if(array_key_exists('sqlMethod', $_GET) && (strtolower($_GET['sqlMethod']) == 'and'))
		{
			$sqlMethod = ' AND ';
			$resourceFields = array('resourceId', 'resourceType', 'resourceTitle', 'resourceSubtitle', 'resourceShortTitle', 'resourceTitleSort', 'resourceTransTitle', 'resourceTransSubtitle', 'resourceTransShortTitle', 'resourceField1', 'resourceField2', 'resourceField3', 'resourceField4', 'resourceField5', 'resourceField6', 'resourceField7', 'resourceField8', 'resourceField9', 'resourceNoSort', 'resourceTransNoSort', 'resourceIsbn', 'resourceBibtexKey', 'resourceDoi');
		}
		else
			$sqlMethod = ' OR ';
		$order = 'creatorSurname';
		if(array_key_exists('order', $_GET))
		{
			if($_GET['order'] == 'timestamp')
				$order = 'resourcetimestampTimestamp';
			else if($_GET['order'] == 'year')
				$order = 'resourceyearYear1';
			else if($_GET['order'] == 'title')
				$order = 'resourceTitleSort';
		}
		if($_GET['action'] == 'getResource')
		{
			$resultset = $res->getResource($ids, $order);
			while($row = $db->fetchRow($resultset))
				$items[$row['resourceId']] = preg_replace("/$pattern/", '<a', $bibStyle->process($row));
		}
		else if($_GET['action'] == 'getCategory')
		{
			if($sqlMethod == ' AND ')
			{
				$index = 1;
				foreach($ids as $id)
				{
					$db->leftJoin(array(array('resource_category' => $index)), $index . '.resourcecategoryResourceId', 'resourceId');
					$db->formatConditions(array($index . '.resourcecategoryCategoryId' => $id));
					$index++;
				}
			}
			else
			{
				$db->formatConditionsOneField($ids, 'resourcecategoryCategoryId');
				$db->leftJoin('resource_category', 'resourcecategoryResourceId', 'resourceId');
			}
			$resultset = $res->getResource(FALSE, $order);
			while($row = $db->fetchRow($resultset))
				$items[$row['resourceId']] = preg_replace("/$pattern/", '<a', $bibStyle->process($row));
		}
		else if($_GET['action'] == 'getKeyword')
		{
			if(($sqlMethod == ' AND ') && (sizeof($ids) > 1))
			{
				$index = 1;
				foreach($ids as $id)
				{
					$db->leftJoin(array(array('resource_keyword' => $index)), $index . '.resourcekeywordResourceId', 'resourceId');
					$db->formatConditions(array($index . '.resourcekeywordKeywordId' => $id));
					$db->formatConditions(array($index . '.resourcekeywordResourceId' => ' IS NOT NULL'));
					$index++;
				}
			}
			else
			{
				$db->formatConditionsOneField($ids, 'resourcekeywordKeywordId');
				$db->leftJoin('resource_keyword', 'resourcekeywordResourceId', 'resourceId');
			}
			$resultset = $res->getResource(FALSE, $order);
			while($row = $db->fetchRow($resultset))
				$items[$row['resourceId']] = preg_replace("/$pattern/", '<a', $bibStyle->process($row));
		}
		else if($_GET['action'] == 'getCreator')
		{
			if(($sqlMethod == ' AND ') && (sizeof($ids) > 1))
			{
				$index = 1;
				foreach($ids as $id)
				{
					$db->leftJoin(array(array('resource_creator' => $index)), $index . '.resourcecreatorResourceId', 'resourceId');
					$db->formatConditions(array($index . '.resourcecreatorCreatorId' => $id));
					$index++;
				}
				$db->leftJoin(array(array('resource_creator' => 'c')), 'c.resourcecreatorResourceId', 'resourceId');
				$subQuery = $db->subQuery($db->selectNoExecute('resource', array_merge($resourceFields, 
					array(array('c.resourcecreatorCreatorSurname' => 'creatorSurname'), 'c.resourcecreatorResourceId')), 
					TRUE, TRUE, TRUE), 't1');
				$resultset = $res->getResource(FALSE, $order, FALSE, FALSE, $subQuery);
			}
			else
			{
				$db->formatConditionsOneField($ids, 'resourcecreatorCreatorId');
				$resultset = $res->getResource(FALSE, $order);
			}
			while($row = $db->fetchRow($resultset))
				$items[$row['resourceId']] = preg_replace("/$pattern/", '<a', $bibStyle->process($row));
		}
		else if($_GET['action'] == 'getPublisher')
		{
// 'proceedings_article', 'proceedings', 'book' and 'book_article' resource types have publisher in 'miscField1' (if publisher exists)
			foreach($ids as $id)
			{
				$conditionArray[] = $db->ifClause(
					$db->formatFields('resourceType') . $db->equal . $db->tidyInput('proceedings_article') . 
					$db->or . 
					$db->formatFields('resourceType') . $db->equal . $db->tidyInput('proceedings') . 
					$db->or . 
					$db->formatFields('resourceType') . $db->equal . $db->tidyInput('book') . 
					$db->or . 
					$db->formatFields('resourceType') . $db->equal . $db->tidyInput('book_article'), 
					FALSE, // test is in previous statement
					$db->formatFields('resourcemiscField1') . '=' . $db->tidyInput($id) . 
					$db->or . 
					$db->formatFields('resourcemiscPublisher') . '=' . $db->tidyInput($id),
					$db->formatFields('resourcemiscPublisher') . '=' . $db->tidyInput($id)
				);
			}
			$db->formatConditions(join($db->or, $conditionArray));
			$db->leftJoin('resource_misc', 'resourcemiscId', 'resourceId');
			$resultset = $res->getResource(FALSE, $order);
			while($row = $db->fetchRow($resultset))
				$items[$row['resourceId']] = preg_replace("/$pattern/", '<a', $bibStyle->process($row));
		}
		else if($_GET['action'] == 'getCollection')
		{
			$db->formatConditionsOneField($ids, 'resourcemiscCollection');
			$db->leftJoin('resource_misc', 'resourcemiscId', 'resourceId');
			$resultset = $res->getResource(FALSE, $order);
			while($row = $db->fetchRow($resultset))
				$items[$row['resourceId']] = preg_replace("/$pattern/", '<a', $bibStyle->process($row));
		}
		else if($_GET['action'] == 'getRecent')
		{
			$stmt = FACTORY_SQLSTATEMENTS::getInstance();
			$limit = array_key_exists('limit', $_GET) ? $_GET['limit'] : 10; // default limit of 10
			if(array_key_exists('days', $_GET))
			{
				$days = $_GET['days'];
				$resultset = $db->query($stmt->frontSetDays($_GET['days'], $limit));
			}
			else
				$resultset = $db->query($stmt->frontSetNumber($limit));
			while($row = $db->fetchRow($resultset))
				$items[$row['resourceId']] = preg_replace("/$pattern/", '<a', $bibStyle->process($row));
		}
		else
			die('Missing or incorrect queryString');
		if(!empty($items))
			return base64_encode(serialize($items));
		return FALSE; // no resources found
	}
// Parse and format a block of text with [cite]...[/cite]tags
	public function parseText()
	{
// Set bibliographic style
		$db = FACTORY_DB::getInstance();
		$session = FACTORY_SESSION::getInstance();
		if(array_key_exists('bibStyle', $_GET))
			$session->setVar('setup_Style', $_GET['bibStyle']);
		else
		{
			$config = FACTORY_CONFIG::getInstance();
			$session->setVar('setup_Style', $config->WIKINDX_CMS_BIBSTYLE);
		}
		$cite = FACTORY_CITE::getInstance();
		if(array_key_exists('wikindxLink', $_GET) && $_GET['wikindxLink'] == 1)
			$linkToWikindxResource = TRUE;
		else
			$linkToWikindxResource = FALSE;
		$address = urldecode($_GET['address']);
		$text = file_get_contents($address);
		return $cite->parseCitations($text, 'html', $linkToWikindxResource);
	}
// Parse and format a resource list from a SQL statement
	public function parseSql()
	{
// Remove WIKINDX-style hyperlink class.
		$pattern = preg_quote("<a class=\"rLink\"");
// Set bibliographic style
		if(array_key_exists('bibStyle', $_GET))
			$this->session->setVar('setup_Style', $_GET['bibStyle']);
		else
		{
			$config = FACTORY_CONFIG::getInstance();
			$this->session->setVar('setup_Style', $config->WIKINDX_CMS_BIBSTYLE);
		}
		$bibStyle = FACTORY_BIBSTYLE::getInstance();
		$address = urldecode($_GET['address']);
		$text = file_get_contents($address);
		$querystring = base64_decode($text);
// Open database for CMS user
		if($config->WIKINDX_DB_PERSISTENT === TRUE)
		{
			if(!$handle = mysql_pconnect($config->WIKINDX_DB_HOST, $config->WIKINDX_CMS_DB_USER, $config->WIKINDX_CMS_DB_PASSWORD))
				die($querystring . ' <P>--> (' . mysql_error() . ')');
		}
		else
		{
			if(!$handle = mysql_connect($config->WIKINDX_DB_HOST, $config->WIKINDX_CMS_DB_USER, $config->WIKINDX_CMS_DB_PASSWORD))
				die($querystring . ' <P>--> (' . mysql_error() . ')');
		}
		if(!mysql_select_db($config->WIKINDX_DB, $handle))
			die($querystring . ' <P>--> (' . mysql_error() . ')');
// v4.1 and up offer UTF8 support and v5.0 offers stored functions -- we require at least v4
		$mysqlVersion = mysql_get_server_info();
		$split = explode('\.', $mysqlVersion);
		if(($split[0] < 4) && ($split[1] < 1))
			die("In order to support UTF-8 character sets, WIKINDX requires MySQL 4.1.1 or greater.  
				Your MySQL version is {$mysqlVersion}. Please upgrade MySQL.");
// Set for UTF8
		mysql_query('SET NAMES utf8', $handle);
		mysql_query('SET CHARACTER SET utf8', $handle);	
		if(!$recordset = mysql_query($querystring, $handle))
			die($querystring . ' <P>--> (' . mysql_error() . ')');

			$row = mysql_fetch_assoc($recordset);
		if(!is_array($row))
			return $row;
		while($row = mysql_fetch_assoc($recordset))
		{
			$row = array_map(array($this, "sqlAddSlashes"), $row);
			$items[$row['resourceId']] = preg_replace("/$pattern/", '<a', $bibStyle->process($row));
		}
		if(!empty($items))
			return base64_encode(serialize($items));
		return FALSE; // Else, no resources found
	}
	private function sqlAddSlashes($element)
	{
		return addslashes($element);
	}
}
?>
