<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* EDITCREATOR class
*
*/
class EDITCREATOR
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $config;
private $form;
private $creator;
private $html;
private $table;
private $gatekeep;
private $badInput;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->creator = FACTORY_CREATOR::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "edit", " (" . 
			$this->messages->text("resources", "creator") . ")"));
	}
// check we are allowed to edit and load appropriate method
	public function init($message = FALSE)
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		$this->session->clearArray('edit');
		$creators = $this->creator->grabAll();
		if(!$creators)
		{
			GLOBALS::buildOutputString($this->messages->text('misc', 'noCreators'));
			return;
		}
		$pString = $message;
		$pString .= $this->form->formHeader('edit_EDITCREATOR_CORE');
		$pString .= $this->form->hidden("method", "editDisplayName");
		$pString .= $this->form->selectFBoxValue(FALSE, "edit_CreatorId", $creators, 20);
		$pString .= $this->html->p($this->form->formSubmit("Proceed"));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Display interface to edit chose name
	public function editDisplayName($message = FALSE)
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		$this->session->setVar('editLock', FALSE);
		if(!array_key_exists('edit_CreatorId', $this->vars) || !$this->vars['edit_CreatorId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$this->db->formatConditions(array('creatorId' => $this->vars['edit_CreatorId']));
		$recordset = $this->db->select('creator', 
			array("creatorSurname", "creatorFirstname", "creatorInitials", "creatorPrefix"));
		$row = $this->db->fetchRow($recordset);
		if($row['creatorInitials'])
			$this->session->setVar("edit_Initials", 
			str_replace(" ", ".", $this->html->dbToFormTidy($row['creatorInitials'])) . ".");
		if($row['creatorFirstname'])
			$this->session->setVar("edit_Firstname", $this->html->dbToFormTidy($row['creatorFirstname']));
		if($row['creatorPrefix'])
			$this->session->setVar("edit_Prefix", $this->html->dbToFormTidy($row['creatorPrefix']));
		$this->session->setVar("edit_Surname", $this->html->dbToFormTidy($row['creatorSurname']));
		$sessionVars = $this->session->getArray("edit");
		$pString = $this->form->formHeader('edit_EDITCREATOR_CORE');
		$pString .= $this->form->hidden("method", "edit");
		$pString .= $this->form->hidden("edit_CreatorId", $this->vars['edit_CreatorId']);
		$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $this->table->trStart();
		$text = array_key_exists("Firstname", $sessionVars) ? $sessionVars["Firstname"] : FALSE;
		$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "firstname"), 
			"edit_Firstname", $text, 30, 255));
		$text = array_key_exists("Initials", $sessionVars) ? $sessionVars["Initials"] : FALSE;
		$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "initials"), 
			"edit_Initials", $text, 6, 255) . $this->html->br() . 
			$this->html->span($this->messages->text("hint", "initials")), 'hint');
		$text = array_key_exists("Prefix", $sessionVars) ? $sessionVars["Prefix"] : FALSE;
		$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "prefix"), 
			"edit_Prefix", $text, 11, 10));
		$text = array_key_exists("Surname", $sessionVars) ? $sessionVars["Surname"] : FALSE;
		$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "surname"), 
			"edit_Surname", $text, 30, 255) . ' ' . $this->html->span('*', 'required'));
		$pString .= $this->table->td($this->form->formSubmit("Proceed") . $this->form->formEnd(), FALSE, "right", "bottom");
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		GLOBALS::buildOutputString($pString);
	}
// write to the database
	public function edit()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if($this->session->getVar('editLock'))
			$this->badInput->close($this->errors->text("done", "creator"), $this, 'init');
		if(!array_key_exists('edit_CreatorId', $this->vars) || !$this->vars['edit_CreatorId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$surname = array_key_exists('edit_Surname', $this->vars) ? trim($this->vars['edit_Surname']) : FALSE;
		if(!$surname)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$firstname = trim($this->vars['edit_Firstname']);
		$initials = $this->creator->formatInitials(trim($this->vars['edit_Initials']));
		$prefix = trim($this->vars['edit_Prefix']);
		if($creatorExistId = $this->creator->checkExists($surname, $firstname, $initials, $prefix))
		{
			if($creatorExistId != $this->vars['edit_CreatorId'])
				return $this->confirmDuplicate($creatorExistId, $surname);
		}
		$updateArray['creatorSurname'] = $surname;
		if($firstname)
			$updateArray['creatorFirstname'] = $firstname;
		else
			$nulls[] = 'creatorFirstname';
		if($initials)
			$updateArray['creatorInitials'] = $initials;
		else
			$nulls[] = 'creatorInitials';
		if(trim($this->vars['edit_Prefix']))
			$updateArray['creatorPrefix'] = $prefix;
		else
			$nulls[] = 'creatorPrefix';
		$this->db->formatConditions(array('creatorId' => $this->vars['edit_CreatorId']));
		$this->db->update('creator', $updateArray);
// set to NULL empty firstname, prefix and initials fields
		if(isset($nulls) && !empty($nulls))
		{
			$this->db->formatConditions(array('creatorId' => $this->vars['edit_CreatorId']));
			$this->db->updateNull('creator', $nulls);
		}
// lock reload
		$this->session->setVar('editLock', TRUE);
// Clear session
		$this->session->clearArray("edit");
// remove cache files for creators
		$this->db->deleteCache('cacheResourceCreators');
		$this->db->deleteCache('cacheMetadataCreators');
// send back to editDisplay with success message
		$this->init($this->success->text("creator"));
	}
// write to the database.
	function editConfirm()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if($this->session->getVar('editLock'))
			$this->badInput->close($this->errors->text("done", "creator"), $this, 'init');
		if(!array_key_exists('edit_CreatorId', $this->vars) || !$this->vars['edit_CreatorId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if(!array_key_exists('edit_CreatorExistId', $this->vars) || !$this->vars['edit_CreatorExistId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if(!array_key_exists('edit_CreatorSurname', $this->vars) || !$this->vars['edit_CreatorSurname'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$editId = $this->vars['edit_CreatorId'];
		$existId = $this->vars['edit_CreatorExistId'];
		$existSurname = $this->vars['edit_CreatorSurname'];
// Delete old creator
		$this->db->formatConditions(array('creatorId' => $editId));
		$this->db->delete('creator');
// remove cache files for creators
		$this->db->deleteCache('cacheResourceCreators');
		$this->db->deleteCache('cacheMetadataCreators');
// Select all resources referencing this old creator and replace reference with existing creator -- check if main or not.
// First update where old/new creator is not main creator, then update where old/new creator is main creator (and add newSurname).
		$this->db->formatConditions(array('resourcecreatorCreatorMain' => $editId), TRUE);
		$this->db->formatConditions(array('resourcecreatorCreatorId' => $editId));
		$this->db->update('resource_creator', array('resourcecreatorCreatorId' => $existId));
		$this->db->formatConditions(array('resourcecreatorCreatorMain' => $editId));
		$this->db->formatConditions(array('resourcecreatorCreatorId' => $editId));
		$updateArray['resourcecreatorCreatorId'] = $existId;
		$updateArray['resourcecreatorCreatorMain'] = $existId;
		$this->db->update('resource_creator', $updateArray);
// Update surname where resourcecreatorCreatorMain is $existId
		$this->db->formatConditions(array('resourcecreatorCreatorMain' => $existId));
		$this->db->update('resource_creator', array('resourcecreatorCreatorSurname' => $existSurname));
// lock reload
		$this->session->setVar('editLock', TRUE);
// Clear session
		$this->session->clearArray("edit");
// send back to editDisplay with success message
		$this->init($this->success->text("creator"));
	}
/**
* The new name equals one already in the database. Confirm that this edited one is to be removed and 
* all references to it replaced by the existing one.
*/
	private function confirmDuplicate($creatorExistId, $surname)
	{
		$pString = $this->errors->text("warning", "creatorExists");
		$pString .= $this->html->p($this->messages->text("creators", "creatorExists"));
		$pString .= $this->form->formHeader("edit_EDITCREATOR_CORE");
		$pString .= $this->form->hidden("edit_CreatorId", $this->vars['edit_CreatorId']);
		$pString .= $this->form->hidden("edit_CreatorExistId", $creatorExistId);
		$pString .= $this->form->hidden("edit_CreatorSurname", $surname);
		$pString .= $this->form->hidden("method", 'editConfirm');
		$pString .= $this->html->p($this->form->formSubmit("Proceed"), FALSE, "right");
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
}
?>