<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* EDITKEYWORD class
*
*/
class EDITKEYWORD
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $config;
private $form;
private $keyword;
private $html;
private $table;
private $gatekeep;
private $badInput;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->keyword = FACTORY_KEYWORD::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "edit", " (" . 
			$this->messages->text("resources", "keyword") . ")"));
	}
// check we are allowed to edit and load appropriate method
	public function init($message = FALSE)
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		$this->session->clearArray('edit');
		$keywords = $this->keyword->grabAll();
		if(!$keywords)
		{
			GLOBALS::buildOutputString($this->messages->text('misc', 'noKeywords'));
			return;
		}
		$pString = $message;
		$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $this->table->trStart();
		$td = $this->form->formHeader('edit_EDITKEYWORD_CORE');
		$td .= $this->form->hidden("method", "editDisplayKeyword");
		$td .= $this->form->selectFBoxValue(FALSE, "edit_KeywordId", $keywords, 20);
		$td .= $this->html->p($this->form->formSubmit("Proceed"));
		$td .= $this->form->formEnd();
		$pString .= $this->table->td($td);
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		GLOBALS::buildOutputString($pString);
	}
// Display interface to edit keyword
	public function editDisplayKeyword($message = FALSE)
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		$this->session->setVar('editLock', FALSE);
		if(!array_key_exists('edit_KeywordId', $this->vars) || !$this->vars['edit_KeywordId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$this->db->formatConditions(array('keywordId' => $this->vars['edit_KeywordId']));
		$recordset = $this->db->select('keyword', 'keywordKeyword');
		$keyword = $this->html->dbToFormTidy($this->db->fetchOne($recordset));
		$pString = $this->form->formHeader('edit_EDITKEYWORD_CORE');
		$pString .= $this->form->hidden("method", "edit");
		$pString .= $this->form->hidden("edit_KeywordId", $this->vars['edit_KeywordId']);
		$pString .= $this->html->p($this->form->textInput($this->messages->text('resources', 'keyword'), 
			'edit_Keyword', $keyword, 30, 255) . ' ' . $this->html->span('*', 'required') . 
			'&nbsp;&nbsp;' . $this->form->formSubmit("Edit"));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// write to the database
	public function edit()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if($this->session->getVar('editLock'))
		{
			if(array_key_exists('returnToAdminKeyword', $this->vars)) // called from Admin menu
			{
				include_once('core/modules/admin/ADMINKEYWORD.php');
				$admin = new ADMINKEYWORD();
				$this->badInput->close($this->errors->text("done", "keyword"), $admin, 'init');
			}
			else
				$this->badInput->close($this->errors->text("done", "keyword"), $this, 'init');
		}
		if(!array_key_exists('edit_KeywordId', $this->vars) || !$this->vars['edit_KeywordId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$keyword = array_key_exists('edit_Keyword', $this->vars) ? trim($this->vars['edit_Keyword']) : FALSE;
		if(!$keyword)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if($keywordExistId = $this->keyword->checkExists($keyword))
		{
			if($keywordExistId != $this->vars['edit_KeywordId'])
				return $this->confirmDuplicate($keywordExistId, $keyword);
		}
		$updateArray['keywordKeyword'] = $keyword;
		$this->db->formatConditions(array('keywordId' => $this->vars['edit_KeywordId']));
		$this->db->update('keyword', $updateArray);
// lock reload
		$this->session->setVar('editLock', TRUE);
// Clear session
		$this->session->clearArray("edit");
// remove cache files for keywords
		$this->db->deleteCache('cacheResourceKeywords');
		$this->db->deleteCache('cacheMetadataKeywords');
		$this->db->deleteCache('cacheQuoteKeywords');
		$this->db->deleteCache('cacheParaphraseKeywords');
		$this->db->deleteCache('cacheMusingKeywords');
// send back to editDisplay with success message
		$this->init($this->success->text("keyword"));
	}
// write to the database.
	function editConfirm()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if($this->session->getVar('editLock'))
			$this->badInput->close($this->errors->text("done", "keyword"), $this, 'init');
		if(!array_key_exists('edit_KeywordId', $this->vars) || !$this->vars['edit_KeywordId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if(!array_key_exists('edit_KeywordExistId', $this->vars) || !$this->vars['edit_KeywordExistId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$editId = $this->vars['edit_KeywordId'];
		$existId = $this->vars['edit_KeywordExistId'];
// Delete old keyword
		$this->db->formatConditions(array('keywordId' => $editId));
		$this->db->delete('keyword');
// Update references to keyword
		$this->db->formatConditions(array('resourcekeywordKeywordId' => $editId));
		$this->db->update('resource_keyword', array('resourcekeywordKeywordId' => $existId));
// remove cache files for keywords
		$this->db->deleteCache('cacheResourceKeywords');
		$this->db->deleteCache('cacheMetadataKeywords');
		$this->db->deleteCache('cacheQuoteKeywords');
		$this->db->deleteCache('cacheParaphraseKeywords');
		$this->db->deleteCache('cacheMusingKeywords');
// lock reload
		$this->session->setVar('editLock', TRUE);
// Clear session
		$this->session->clearArray("edit");
// send back to editDisplay with success message
		$this->init($this->success->text("keyword"));
	}
/**
* The new keyword equals one already in the database. Confirm that this edited one is to be removed and 
* all references to it replaced by the existing one.
*/
	private function confirmDuplicate($keywordExistId, $keyword)
	{
		$pString = $this->errors->text("warning", "keywordExists");
		$pString .= $this->html->p($this->messages->text("misc", "keywordExists"));
		$pString .= $this->form->formHeader("edit_EDITKEYWORD_CORE");
		$pString .= $this->form->hidden("edit_KeywordId", $this->vars['edit_KeywordId']);
		$pString .= $this->form->hidden("edit_KeywordExistId", $keywordExistId);
		$pString .= $this->form->hidden("method", 'editConfirm');
		$pString .= $this->html->p($this->form->formSubmit("Proceed"), FALSE, "right");
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
}
?>
