<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* EDITPUBLISHER class
*
*/
class EDITPUBLISHER
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $config;
private $form;
private $publisher;
private $html;
private $table;
private $gatekeep;
private $badInput;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->publisher = FACTORY_PUBLISHER::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "edit", " (" . 
			$this->messages->text("resources", "publisher") . ")"));
	}
// check we are allowed to edit and load appropriate method
	public function init($message = FALSE)
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		$this->session->clearArray('edit');
		$publisherType = $this->vars['PublisherType'];
		$publishers = $this->publisher->grabAll($publisherType);
		if(!$publishers)
		{
			GLOBALS::buildOutputString($this->messages->text('misc', 'noPublishers'));
			return;
		}
		$pString = $message;
		$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $this->table->trStart();
		$td = $this->form->formHeader('edit_EDITPUBLISHER_CORE');
		$td .= $this->form->hidden("method", "editDisplayPublisher");
		$td .= $this->form->selectFBoxValue(FALSE, "edit_PublisherId", $publishers, 20);
		$td .= $this->html->p($this->form->formSubmit("Proceed"));
		$td .= $this->form->formEnd();
		$pString .= $this->table->td($td);
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		GLOBALS::buildOutputString($pString);
	}
// Display interface to edit publisher
	public function editDisplayPublisher($message = FALSE)
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		$this->session->setVar('editLock', FALSE);
		if(!array_key_exists('edit_PublisherId', $this->vars) || !$this->vars['edit_PublisherId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$this->db->formatConditions(array('publisherId' => $this->vars['edit_PublisherId']));
		$recordset = $this->db->select('publisher', array('publisherName', 'publisherLocation'));
		$row = $this->db->fetchRow($recordset);
		$pString = $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $this->table->trStart();
		$pString .= $this->form->formHeader('edit_EDITPUBLISHER_CORE');
		$pString .= $this->form->hidden("method", "edit");
		$pString .= $this->form->hidden("edit_PublisherId", $this->vars['edit_PublisherId']);
		$pString .= $this->table->td($this->form->textInput($this->messages->text('resources', 'publisherName'),
			'edit_PublisherName', $this->html->dbToFormTidy($row['publisherName']), 30, 255) . ' ' . 
			$this->html->span('*', 'required'));
		$pString .= $this->table->td($this->form->textInput($this->messages->text('resources', 'publisherLocation'),
			'edit_PublisherLocation', $this->html->dbToFormTidy($row['publisherLocation']), 30, 255) . 
			'&nbsp;&nbsp;' . $this->form->formSubmit("Edit") . $this->form->formEnd());
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		GLOBALS::buildOutputString($pString);
	}
// write to the database
	public function edit()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if($this->session->getVar('editLock'))
			$this->badInput->close($this->errors->text("done", "publisher"), $this, 'init');
		if(!array_key_exists('edit_PublisherId', $this->vars) || !$this->vars['edit_PublisherId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$name = array_key_exists('edit_PublisherName', $this->vars) ? trim($this->vars['edit_PublisherName']) : FALSE;
		$location = array_key_exists('edit_PublisherLocation', $this->vars) ? 
			trim($this->vars['edit_PublisherLocation']) : FALSE;
		if(!$name & !$location)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if($publisherExistId = $this->publisher->checkExists($name, $location))
		{
			if($publisherExistId != $this->vars['edit_PublisherId'])
				return $this->confirmDuplicate($publisherExistId);
		}
		if($name)
			$updateArray['publisherName'] = $name;
		else
			$nulls[] = 'publisherName';
		if($location)
			$updateArray['publisherLocation'] = $location;
		else
			$nulls[] = 'publisherLocation';
		if(isset($updateArray))
		{
			$this->db->formatConditions(array('publisherId' => $this->vars['edit_PublisherId']));
			$this->db->update('publisher', $updateArray);
		}
		if(isset($nulls))
		{
			$this->db->formatConditions(array('publisherId' => $this->vars['edit_PublisherId']));
			$this->db->updateNull('publisher', $nulls);
		}
// lock reload
		$this->session->setVar('editLock', TRUE);
// Clear session
		$this->session->clearArray("edit");
// remove cache files for keywords
		$this->db->deleteCache('cacheResourcePublishers');
		$this->db->deleteCache('cacheMetadataPublishers');
// send back to editDisplay with success message
		$this->init($this->success->text("publisher"));
	}
// write to the database.
	function editConfirm()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if($this->session->getVar('editLock'))
			$this->badInput->close($this->errors->text("done", "publisher"), $this, 'init');
		if(!array_key_exists('edit_PublisherId', $this->vars) || !$this->vars['edit_PublisherId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if(!array_key_exists('edit_PublisherExistId', $this->vars) || !$this->vars['edit_PublisherExistId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$editId = $this->vars['edit_PublisherId'];
		$existId = $this->vars['edit_PublisherExistId'];
// Delete old publisher
		$this->db->formatConditions(array('publisherId' => $editId));
		$this->db->delete('publisher');
// remove cache files for publishers
		$this->db->deleteCache('cacheResourcePublishers');
		$this->db->deleteCache('cacheMetadataPublishers');
// Select all resources referencing this old publisher and replace reference with existing publisher
		$this->db->conditionSeparator = $this->db->or;
		$this->db->formatConditions(array('resourcemiscPublisher' => $editId, 
			'resourcemiscField1' => $editId));
		$this->db->leftJoin('resource', 'resourceId', 'resourcemiscId');
		$recordset = $this->db->select('resource_misc', 
			array('resourcemiscId', 'resourcemiscPublisher', 'resourcemiscField1', 'resourceType'));
		while($row = $this->db->fetchRow($recordset))
		{
			$updateArray = array();
			if((($row['resourceType'] == 'proceedings_article') || ($row['resourceType'] == 'proceedings_article'))
				&& $row['resourcemiscField1'])
				$updateArray['resourcemiscField1'] = $existId;
			else if(($row['resourceType'] != 'proceedings_article') && ($row['resourceType'] != 'proceedings_article')
				&& $row['resourcemiscPublisher'])
				$updateArray['resourcemiscPublisher'] = $existId;
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('resourcemiscId' => $row['resourcemiscId']));
				$this->db->update('resource_misc', $updateArray);
			}
		}
// lock reload
		$this->session->setVar('editLock', TRUE);
// Clear session
		$this->session->clearArray("edit");
// send back to editDisplay with success message
		$this->init($this->success->text("keyword"));
	}
/**
* The new publisher equals one already in the database. Confirm that this edited one is to be removed and 
* all references to it replaced by the existing one.
*/
	private function confirmDuplicate($publisherExistId)
	{
		$pString = $this->errors->text("warning", "publisherExists");
		$pString .= $this->html->p($this->messages->text("misc", "publisherExists"));
		$pString .= $this->form->formHeader("edit_EDITPUBLISHER_CORE");
		$pString .= $this->form->hidden("edit_PublisherId", $this->vars['edit_PublisherId']);
		$pString .= $this->form->hidden("edit_PublisherExistId", $publisherExistId);
		$pString .= $this->form->hidden("method", 'editConfirm');
		$pString .= $this->html->p($this->form->formSubmit("Proceed"), FALSE, "right");
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
}
?>