<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* LISTADDTO class
*
* Add selected resources in list to user bibliographies, categories, keywords etc.
* 
*/
class LISTADDTO
{
private $db;
private $vars;
private $messages;
private $errors;
private $session;
private $form;
private $table;
private $html;
private $template;
private $commonBib;
private $user;
private $badInput;
private $navigate;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->user = FACTORY_USER::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->navigate = FACTORY_NAVIGATE::getInstance();
	}
	public function init()
	{
		if(!array_key_exists('resourceSelectedTo', $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$this->session->setVar("resourceSelectedTo", $this->vars['resourceSelectedTo']);
		if($this->vars['resourceSelectedTo'] == 1)	// add to categories
			return $this->addToCategoryInit();
		else if($this->vars['resourceSelectedTo'] == 2)	// add to keywords
			return $this->addToKeywordInit();
		else if($this->vars['resourceSelectedTo'] == 3)	// remove resources from user bibliography
			return $this->deleteFromBib();
		else if($this->vars['resourceSelectedTo'] == 4)	// delete resources from WIKINDX
			return $this->deleteFromWikindx();
		else if($this->vars['resourceSelectedTo'] == 5)	// add to subcategories
			return $this->addToSubcategoryInit();
		else if($this->vars['resourceSelectedTo'] == 6)	// add to subcategories
			return $this->addToUserTagsInit();
		else if($this->vars['resourceSelectedTo'] == 7)	// add to basket
			return $this->addToBasket();
		else if($this->vars['resourceSelectedTo'] == 8)	// remove from basket
			return $this->RemoveFromBasket();
		if($this->vars['resourceSelectedTo'] == 9)	// add to languages
			return $this->addToLanguageInit();
		else	// 0 -> add to user bibs
			return $this->addToUserBibInit();
	}
// Check for proper resource ID input and produce hidden fields
	private function checkIdInput()
	{
		if(array_key_exists("selectAll", $this->vars))
			return array(TRUE, 'all');
		foreach($this->vars as $key => $value)
		{
			if(strstr($key, "bib_"))
				$string[] = substr($key, 4);
		}
		if(isset($string))
			return array(TRUE, base64_encode(serialize($string)));
		else
			return array(FALSE, FALSE);
	}
// Add selected resources to the basket
	private function addToBasket()
	{
		list($idFound, $string) = $this->checkIdInput();
		if(!$idFound)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		if($this->session->issetVar('basket_List'))
			$basket = unserialize($this->session->getVar('basket_List'));
		else
			$basket = array();
		if($string == 'all')
			$resourceIds = unserialize(base64_decode($this->session->getVar('list_AllIds')));
		else
			$resourceIds = unserialize(base64_decode($string));
		foreach($resourceIds as $resourceId)
		{
			if(array_search($resourceId, $basket) === FALSE)
				$basket[] = $resourceId;
		}
// Ensure array is unique
		array_unique($basket);
		$this->session->setVar('basket_List', serialize($basket));
		$this->session->saveState('basket');
		$this->session->setVar("addToKeywordCategory", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("basketAdd"));
		FACTORY_CLOSE::getInstance(); // die
	}
// Remove selected resources from the basket
	private function removeFromBasket()
	{
		list($idFound, $string) = $this->checkIdInput();
		if(!$idFound)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		if($this->session->issetVar('basket_List'))
			$basket = unserialize($this->session->getVar('basket_List'));
		else
			$basket = array();
		if($string == 'all')
			$resourceIds = unserialize(base64_decode($this->session->getVar('list_AllIds')));
		else
			$resourceIds = unserialize(base64_decode($string));
		foreach($resourceIds as $resourceId)
		{
			if(($key = array_search($resourceId, $basket)) !== FALSE)
				unset($basket[$key]);
		}
		if(empty($basket))
			$this->session->delVar('basket_List');
		else
			$this->session->setVar('basket_List', serialize($basket));
		$this->session->saveState('basket');
		$this->session->setVar("addToKeywordCategory", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("basketRemove"));
		FACTORY_CLOSE::getInstance(); // die
	}
// Add selected resources to categories - display form for this.
	private function addToCategoryInit()
	{
		$category = FACTORY_CATEGORY::getInstance();
		$categories = $category->grabAll($this->session->getVar("mywikindx_Bibliography_use"));
		$pString = $this->form->formHeader('list_LISTADDTO_CORE');
		$pString .= $this->form->hidden("method", "addResourceToCategory");
		list($idFound, $string) = $this->checkIdInput();
		if(!$idFound)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$pString .= $this->form->hidden("ids", $string);
		$checkBox = $this->messages->text("resources", "addToExistingCategory") . "&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, "replaceExisting");
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		if($string = $this->session->getVar("resourceAddToCategories"))
		{
			$selectedArray = explode(",", $string);
			$pString .= $this->table->td($this->form->selectedBoxValueMultiple(FALSE, "categories", $categories, 
				$selectedArray, 10) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
				$this->html->p($checkBox) . $this->html->p($this->form->formSubmit("Add")));
		}
		else
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple(FALSE, "categories", $categories, 10) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
				$this->html->p($checkBox) . $this->html->p($this->form->formSubmit("Add")));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->form->formEnd();
		GLOBALS::setOutputHeading($this->messages->text("heading", "addToCategory"));
		GLOBALS::buildOutputString($pString);
	}
// Add selected resources to languages - display form for this.
	private function addToLanguageInit()
	{
		$this->db->orderBy('languageLanguage');
		$resultset = $this->db->select('language', array('languageId', 'languageLanguage'));
		while($row = $this->db->fetchRow($resultset))
			$languages[$row['languageId']] = $row['languageLanguage'];
		$pString = $this->form->formHeader('list_LISTADDTO_CORE');
		$pString .= $this->form->hidden("method", "addResourceToLanguage");
		list($idFound, $string) = $this->checkIdInput();
		if(!$idFound)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$pString .= $this->form->hidden("ids", $string);
		$checkBox = $this->messages->text("resources", "addToExistingLanguage") . "&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, "replaceExisting");
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		if($string = $this->session->getVar("resourceAddToLanguages"))
		{
			$selectedArray = explode(",", $string);
			$pString .= $this->table->td($this->form->selectedBoxValueMultiple(FALSE, "languages", $languages, 
				$selectedArray, 10) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
				$this->html->p($checkBox) . $this->html->p($this->form->formSubmit("Add")));
		}
		else
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple(FALSE, "languages", $languages, 10) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
				$this->html->p($checkBox) . $this->html->p($this->form->formSubmit("Add")));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->form->formEnd();
		GLOBALS::setOutputHeading($this->messages->text("heading", "addToLanguage"));
		GLOBALS::buildOutputString($pString);
	}
// Add selected resources to user tags - display form for this.
	private function addToUserTagsInit()
	{
// Get this user's user tags
		$userTagsObject = FACTORY_USERTAGS::getInstance();
		$userTags = $userTagsObject->grabAll();
		$pString = $this->form->formHeader('list_LISTADDTO_CORE');
		$pString .= $this->form->hidden("method", "addResourceToUserTag");
		list($idFound, $string) = $this->checkIdInput();
		if(!$idFound)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$pString .= $this->form->hidden("ids", $string);
		$checkBox = $this->messages->text("resources", "addToExistingUserTag") . "&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, "replaceExisting");
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		if($string = $this->session->getVar("resourceAddToUserTags"))
		{
			$selectedArray = explode(",", $string);
			$pString .= $this->table->td($this->form->selectedBoxValueMultiple(FALSE, "userTags", $userTags, 
				$selectedArray, 10) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
				$this->html->p($checkBox) . $this->html->p($this->form->formSubmit("Add")));
		}
		else
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple(FALSE, "userTags", $userTags, 10) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
				$this->html->p($checkBox) . $this->html->p($this->form->formSubmit("Add")));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->form->formEnd();
		GLOBALS::setOutputHeading($this->messages->text("heading", "addToUserTag"));
		GLOBALS::buildOutputString($pString);
	}
// Add selected resources to subcategories - display form for this.
	private function addToSubcategoryInit()
	{
		$category = FACTORY_CATEGORY::getInstance();
		$categories = $category->grabAll($this->session->getVar("mywikindx_Bibliography_use"));
		$subcategories = $category->grabSubAll(TRUE, $this->session->getVar("mywikindx_Bibliography_use"));
		$pString = $this->form->formHeader('list_LISTADDTO_CORE');
		$pString .= $this->form->hidden("method", "addResourceToSubcategory");
		list($idFound, $string) = $this->checkIdInput();
		if(!$idFound)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$pString .= $this->form->hidden("ids", $string);
		$checkBox = $this->messages->text("resources", "addToExistingSubcategory") . "&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, "replaceExisting");
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		if($string = $this->session->getVar("resourceAddToSubcategories"))
		{
			$selectedArray = explode(",", $string);
			$pString .= $this->table->td($this->form->selectedBoxValueMultiple(FALSE, "subcategories", $subcategories, 
				$selectedArray, 10) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
				$this->html->p($checkBox) . $this->html->p($this->form->formSubmit("Add")));
		}
		else
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple(FALSE, "subcategories", $subcategories, 10) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
				$this->html->p($checkBox) . 
				$this->html->p($this->form->formSubmit("Add")));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->form->formEnd();
		GLOBALS::setOutputHeading($this->messages->text("heading", "addToSubcategory"));
		GLOBALS::buildOutputString($pString);
	}
// Add selected resources to keywords - display form for this.
	private function addToKeywordInit()
	{
		$keyword= FACTORY_KEYWORD::getInstance();
		$keywords = $keyword->grabAll($this->session->getVar("mywikindx_Bibliography_use"));
		$keywords = array(0 => $this->messages->text("misc", "ignore")) + $keywords;
		$pString = $this->form->formHeader('list_LISTADDTO_CORE');
		$pString .= $this->form->hidden("method", "addResourceToKeyword");
		list($idFound, $string) = $this->checkIdInput();
		if(!$idFound)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$pString .= $this->form->hidden("ids", $string);
		$checkBox = $this->messages->text("resources", "addToExistingKeyword") . "&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, "replaceExisting");
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->selectFBoxValueMultiple(FALSE, "keywords", $keywords, 10) . 
			$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint'));
		$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "keyword"), 
			"keywordText", FALSE, 50) . $this->html->br() . $this->messages->text('hint', 'keywordList'));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->p($checkBox) . $this->html->p($this->form->formSubmit("Add"));
		$pString .= $this->form->formEnd();
		GLOBALS::setOutputHeading($this->messages->text("heading", "addToKeyword"));
		GLOBALS::buildOutputString($pString);
	}
	private function addToUserBibInit()
	{
		$usingBib = $pString = FALSE;
		$bibs = array();
		$useBib = $this->session->getVar('mywikindx_Bibliography_use');
		if($useBib)
		{
			$this->db->formatConditions(array('userbibliographyId' => $useBib));
			$recordset = $this->db->select('user_bibliography', 'userbibliographyTitle');
			$row = $this->db->fetchRow($recordset);
			$usingBib = $row['userbibliographyTitle'];
		}
		$tempU = $this->commonBib->getUserBibs();
		$tempUG = $this->commonBib->getGroupBibs();
		if(!empty($tempU))
		{
			$bibs[0] = $this->messages->text('user', 'userBibs');
			foreach($tempU as $key => $value)
				$bibs[$key] = $value;
		}
		if(!empty($tempUG))
		{
			$bibs[-1] = $this->messages->text('user', 'userGroupBibs');
			foreach($tempUG as $key => $value)
				$bibs[$key] = $value;
		}
		if(!empty($bibs))
		{
			$pString = $this->form->formHeader('list_LISTADDTO_CORE');
				$pString .= $this->form->hidden("method", "addResourceToBib");
			list($idFound, $string) = $this->checkIdInput();
			if(!$idFound)
				$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
			$pString .= $this->form->hidden("ids", $string);
			$pString .= $this->table->tableStart();
			$pString .= $this->table->trStart();
			$sessVar = $this->session->getVar("mywikindx_Bibliography_add");
			if($sessVar)
				$pString .= $this->table->td($this->form->selectedBoxValue(FALSE, "bibId", $bibs, $sessVar, 10) . 
					$this->html->p($this->form->formSubmit("Add")));
			else
				$pString .= $this->table->td($this->form->selectFBoxValue(FALSE, "bibId", $bibs, 10) . 
					$this->html->p($this->form->formSubmit("Add")));
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->form->formEnd();
		}
//		$this->session->setVar("addToKeywordCategory", TRUE);
		GLOBALS::setOutputHeading($this->messages->text("heading", "addToBib"));
		GLOBALS::buildOutputString($pString);
	}
// Get resource ids ready for writing to database
	private function getHiddenIds()
	{
		if(!array_key_exists("ids", $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		if($this->vars['ids'] == 'all')
		{
			if(!$sql = base64_decode($this->session->getVar('sql_TotalSql')))
				$this->badInput->close($this->errors->text("file", "noSql"), $this->navigate, 'listView');
			$recordset = $this->db->query($sql);
			while($row = $this->db->fetchRow($recordset))
				$ids[] = $row['resourcemiscId'];
		}
		else
			$ids = unserialize(base64_decode($this->vars['ids']));
		return $ids;
	}
// Add resources to a bibliography
	public function addResourceToBib()
	{
		if(!array_key_exists('bibId', $this->vars) || ($this->vars['bibId'] < 1))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$ids = $this->getHiddenIds();
// valid user?
		$userBibs = $this->commonBib->getUserBibs();
		$groupBibs = $this->commonBib->getGroupBibs();
		if(!array_key_exists($this->vars['bibId'], $userBibs) && !array_key_exists($this->vars['bibId'], $groupBibs))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this->navigate, 'listView');
		foreach($ids as $id)
		{
			$this->db->formatConditions(array('userbibliographyresourceResourceId' => $id));
			$this->db->formatConditions(array('userbibliographyresourceBibliographyId' => $this->vars['bibId']));
			if(!$this->db->numRows($this->db->select('user_bibliography_resource', 'userbibliographyresourceId')))
				$this->db->insert('user_bibliography_resource', 
					array('userbibliographyresourceBibliographyId', 'userbibliographyresourceResourceId'), 
					array($this->vars['bibId'], $id));
		}
		$this->session->setVar('mywikindx_Bibliography_add', $this->vars['bibId']);
		$this->session->setVar("setup_Bibliographies", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("addBib"));
		FACTORY_CLOSE::getInstance(); // die
	}
// Add selected resources to languages(s)
	public function addResourceToLanguage()
	{
		$replaceExisting = FALSE;
		if(!array_key_exists("languages", $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$resourceIds = $this->getHiddenIds();
		$languageIds = $this->vars['languages'];
		$this->session->setVar("resourceAddToLanguages", join(",", $languageIds));
		if(array_key_exists("replaceExisting", $this->vars))
		{
// remove all old languages from resource
			$this->db->formatConditionsOneField($resourceIds, 'resourcelanguageResourceId');
			$this->db->delete('resource_language');
		}
// add new languages		
		foreach($languageIds as $lId)
		{
			foreach($resourceIds as $rId)
			{
				$this->db->formatConditions(array('resourcelanguageResourceId' => $rId));
				$this->db->formatConditions(array('resourcelanguageLanguageId' => $lId));
				$resultset = $this->db->select('resource_language', 'resourcelanguageId');
				if(!$this->db->fetchOne($resultset))
					$this->db->insert('resource_language', array('resourcelanguageResourceId', 'resourcelanguageLanguageId'), 
					array($rId, $lId));
			}
		}
		$this->session->setVar("addToKeywordCategory", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("addResourceToLanguage"));
		FACTORY_CLOSE::getInstance(); // die
	}
// Add selected resources to category(s)
	public function addResourceToCategory()
	{
		$replaceExisting = FALSE;
		if(!array_key_exists("categories", $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$resourceIds = $this->getHiddenIds();
		$categoryIds = $this->vars['categories'];
		$this->session->setVar("resourceAddToCategories", join(",", $categoryIds));
		if(array_key_exists("replaceExisting", $this->vars))
		{
// remove all old categories and subcategories from resource
			$this->db->formatConditionsOneField($resourceIds, 'resourcecategoryResourceId');
			$this->db->delete('resource_category');
		}
// add new categories		
		foreach($categoryIds as $cId)
		{
			foreach($resourceIds as $rId)
			{
				$this->db->formatConditions(array('resourcecategoryResourceId' => $rId));
				$this->db->formatConditions(array('resourcecategoryCategoryId' => $cId));
				$resultset = $this->db->select('resource_category', 'resourcecategoryId');
				if(!$this->db->fetchOne($resultset))
					$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategoryCategoryId'), 
					array($rId, $cId));
			}
		}
		$this->session->setVar("addToKeywordCategory", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("addResourceToCategory"));
		FACTORY_CLOSE::getInstance(); // die
	}
// Add selected resources to user tag(s)
	public function addResourceToUserTag()
	{
		$replaceExisting = FALSE;
		if(!array_key_exists("userTags", $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$ids = $this->getHiddenIds();
		$userTagIds = $this->vars['userTags'];
		$this->session->setVar("resourceAddToUserTags", join(",", $userTagIds));
		if(array_key_exists("replaceExisting", $this->vars))
		{
// remove old keywords from all selected resources
			foreach($userTagIds as $uId)
			{
				$this->db->formatConditions(array('resourceusertagsTagId' => $uId));
				$this->db->delete('resource_user_tags');
			}
		}
		foreach($userTagIds as $uId)
		{
			foreach($ids as $rId)
			{
				$this->db->formatConditions(array('resourceusertagsResourceId' => $rId));
				$this->db->formatConditions(array('resourceusertagsTagId' => $uId));
				$resultset = $this->db->select('resource_user_tags', 'resourceusertagsId');
				if(!$this->db->fetchOne($resultset))
					$this->db->insert('resource_user_tags', array('resourceusertagsResourceId', 'resourceusertagsTagId'), 
					array($rId, $uId));
			}
		}
		$this->session->setVar("addToKeywordCategory", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("addResourceToUserTag"));
		FACTORY_CLOSE::getInstance(); // die
	}
// Add selected resources to subcategory(s)
	public function addResourceToSubcategory()
	{
		$replaceExisting = FALSE;
		if(!array_key_exists("subcategories", $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$resourceIds = $this->getHiddenIds();
		$subcategoryIds = $this->vars['subcategories'];
// Find categories subcategories belong to in case category needs adding to resource_category.CategoryId
		foreach($subcategoryIds as $subcategoryId)
		{
			$this->db->formatConditions(array('subcategoryId' => $subcategoryId));
			$scIds[] = $this->db->fetchOne($this->db->select('subcategory', 'subcategoryCategoryId'));
			foreach($resourceIds as $rId)
				$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategorySubcategoryId'), 
				array($rId, $subcategoryId));
		}
// If category of which subcategory is a member is not in table row, add it
		if(isset($scIds))
		{
			foreach(array_unique($scIds) as $cId)
			{
//				$this->db->formatConditions(array('resourcecategoryResourceId' => $this->vars['id']));
				$this->db->formatConditions(array('resourcecategoryCategoryId' => $cId));
				$resultset = $this->db->select('resource_category', 'resourcecategoryId');
				if(!$this->db->fetchOne($resultset))
				{
					foreach($resourceIds as $rId)
						$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategoryCategoryId'), 
						array($rId, $cId));
				}
			}
		}		
		$this->session->setVar("addToKeywordCategory", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("addResourceToSubcategory"));
		FACTORY_CLOSE::getInstance(); // die
	}
// Create categories field when adding a new subcategory
	private function createCategoryField($scIds, $oldRow)
	{
		foreach($scIds as $cId)
		{
			if(!isset($existingCIds) && $oldRow['resourcecategoryCategories'])
				$existingCIds = explode(',', $oldRow['resourcecategoryCategories']);
			else if(!isset($existingCIds))
				$existingCIds = array();
			if(array_search($cId, $existingCIds) === FALSE) // need to add category
				$existingCIds[] = $cId;
		}
		return $existingCIds;
	}
// Add selected resources to keyword(s)
	public function addResourceToKeyword()
	{
		$keyword = FACTORY_KEYWORD::getInstance();
		$replaceExisting = FALSE;
		if(!array_key_exists("keywords", $this->vars) && !trim($this->vars['keywordText']))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$resourceIds = $this->getHiddenIds();
		$keywordIds = $this->vars['keywords'];
		if(($key = array_search(0, $keywordIds)) !== FALSE) // ignore
			unset($keywordIds[$key]);
		$keywordTexts = explode(',', trim($this->vars['keywordText']));
		if(empty($keywordIds) && empty($keywordTexts))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		foreach($keywordTexts as $text)
		{
			$keywordText = trim($text);
			if($keywordText)
			{
				if($id = $keyword->checkExists($keywordText))
				{
					if(array_search($id, $keywordIds) === FALSE)
						$keywordIds[] = $id;
				}
				else
				{
// given keyword doesn't exist so now write to db
					$fields[] = "keywordKeyword";
					$values[0] = $keywordText;
					$this->db->insert('keyword', $fields, $values);
					$keywordIds[] = $this->db->lastAutoId('keyword');
				}
			}
		}
		if(array_key_exists("replaceExisting", $this->vars))
		{
// remove old keywords from all selected resources
			$this->db->formatConditionsOneField($resourceIds, 'resourcekeywordResourceId');
			$this->db->delete('resource_keyword');
		}
// add new keywords		
		foreach($keywordIds as $kId)
		{
			foreach($resourceIds as $rId)
			{
				$this->db->formatConditions(array('resourcekeywordResourceId' => $rId));
				$this->db->formatConditions(array('resourcekeywordKeywordId' => $kId));
				$resultset = $this->db->select('resource_keyword', 'resourcekeywordId');
				if(!$this->db->fetchOne($resultset))
					$this->db->insert('resource_keyword', array('resourcekeywordResourceId', 'resourcekeywordKeywordId'), array($rId, $kId));
			}
		}
// remove keyword(s) if they no longer have resources attached to them
		$keyword->removeHanging();
		$this->session->setVar("addToKeywordCategory", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("addResourceToKeyword"));
		FACTORY_CLOSE::getInstance(); // die
	}
// Delete resources from a user bibliogaphy
	private function deleteFromBib()
	{
		if(array_key_exists("selectAll", $this->vars))
			$inputIds = unserialize(base64_decode($this->session->getVar('list_AllIds')));
		foreach($this->vars as $key => $value)
		{
			if(strstr($key, "bib_"))
				$inputIds[] = substr($key, 4);
		}
		if(!isset($inputIds))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$this->db->formatConditions(array('userbibliographyresourceBibliographyId' => $this->session->getVar("mywikindx_Bibliography_use")));
		$this->db->formatConditionsOneField($inputIds, 'userbibliographyresourceResourceId');
		$this->db->delete('user_bibliography_resource');
		$bibs = $this->commonBib->getUserBibs();
		if(empty($bibs))
			$bibs = $this->commonBib->getUserBibs();
		if(!empty($bibs))
			$this->session->setVar("setup_Bibliographies", TRUE);
		$this->session->setVar("addToKeywordCategory", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("deleteFromBib"));
		FACTORY_CLOSE::getInstance(); // die
	}
// Delete resources from WIKINDX
	public function deleteFromWikindx()
	{
		$gatekeep = FACTORY_GATEKEEP::getInstance();
		$gatekeep->requireSuper = TRUE; // only admins can delete resources
		$gatekeep->init();
		if(array_key_exists("selectAll", $this->vars))
		{
			if(!$sql = base64_decode($this->session->getVar('sql_Stmt')))
				$this->badInput->close($this->errors->text("file", "noSql"), $this->navigate, 'listView');
			$recordset = $this->db->query($sql);
			while($row = $this->db->fetchRow($recordset))
				$inputIds[] = $row['resourceId'];
		}
//			$inputIds = unserialize(base64_decode($this->session->getVar('list_AllIds')));
		else
		{
			foreach($this->vars as $key => $value)
			{
				if(strstr($key, "bib_"))
					$inputIds[] = substr($key, 4);
			}
		}
		if(!isset($inputIds))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$this->session->setVar("addToKeywordCategory", TRUE);
		include_once("core/modules/admin/DELETERESOURCE.php");
		$deleteResource = new DELETERESOURCE();
		$deleteResource->resourceIds['resource_id'] = $inputIds;
		$deleteResource->navigate = 'list';
		return $deleteResource->deleteResourceConfirm();
	}
}
?>