<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* LISTRESOURCES class
*
* List all database resources.
* 
*/
class LISTRESOURCES
{
private $db;
private $vars;
private $stmt;
private $messages;
private $common;
private $session;
private $form;
private $table;
private $html;
private $template;
private $commonBib;
private $user;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->stmt = FACTORY_SQLSTATEMENTS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->common = FACTORY_LISTCOMMON::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->user = FACTORY_USER::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "list"));
	}
// display form options
	public function init()
	{
///First check, do we have resources?
		if(!$this->common->resourcesExist())
			return;
		$this->session->delVar('mywikindx_PagingStart');
		$this->session->delVar('mywikindx_PagingStartAlpha');
		$linksInfo['info'] = $this->commonBib->displayBib();
		$this->template->tpl->assign('resourceListInfo', $linksInfo);
		$pString = $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->makeFormField());
		$pString .= $this->table->td($this->makeFormAdded());
		$pString .= $this->table->td($this->makeFormEdited());
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$this->session->clearArray('list');
		GLOBALS::buildOutputString($pString);
	}
// make a self-contained form with a select box for general resource fields
	private function makeFormField()
	{
		$methods = array(
					"creator" => $this->messages->text("list", "creator"),
					"title" => $this->messages->text("list", "title"),
					"publisher" => $this->messages->text("list", "publisher"),
					"year" => $this->messages->text("list", "year"),
					"timestamp" => $this->messages->text("list", "timestamp"),
				);
		if($this->session->getVar('setup_MultiUser') == 'Y')
		{
			$methods['viewsIndex'] = $this->messages->text("list", "views");
			$methods['downloadsIndex'] = $this->messages->text("list", "downloads");
			$methods['popularityIndex'] = $this->messages->text("list", "popularity");
			$methods['maturityIndex'] = $this->messages->text("list", "maturity");
		}
		$pString = $this->form->formHeader("list_LISTRESOURCES_CORE");
		$pString .= $this->form->hidden("method", "processGeneral");
		if($selected = $this->session->getVar("list_Method"))
		{
			$pString .= $this->form->selectedBoxValue($this->messages->text("list", "listBy"), 
				"list_Method", $methods, $selected, "7");
		}
		else
			$pString .= $this->form->selectFBoxValue(
				$this->messages->text("list", "listBy"), "list_Method", $methods, "7");
		$pString .= $this->html->p($this->common->displayAscDesc('list'));
		$pString .= $this->form->formSubmit("List") . $this->form->formEnd();
		return $pString;
	}
// make a self-contained form with a select box for resource added by
	private function makeFormAdded()
	{
		$users = $this->user->grabAll(TRUE, $this->session->getVar("mywikindx_Bibliography_use"), 'add');
		$pString = $this->form->formHeader("list_LISTRESOURCES_CORE");
		$pString .= $this->form->hidden("method", "processAdd");
		if($selected = $this->session->getVar("list_AddedBy"))
			$pString .= $this->form->selectedBoxValue($this->messages->text("list", "addUserIdResource"), 
				"list_AddedBy", $users, $selected, 7);
		else
			$pString .= $this->form->selectFBoxValue(
				$this->messages->text("list", "addUserIdResource"), "list_AddedBy", $users, 7);
		$pString .= $this->html->p($this->common->displayAscDesc('list'));
		$pString .= $this->form->formSubmit("List") . $this->form->formEnd();
		return $pString;
	}
// make a self-contained form with a select box for resource edited by
	private function makeFormEdited()
	{
		$users = $this->user->grabAll(TRUE, $this->session->getVar("mywikindx_Bibliography_use"), 'edit');
		if(!is_array($users))
			return;
		$pString = $this->form->formHeader("list_LISTRESOURCES_CORE");
		$pString .= $this->form->hidden("method", "processEdit");
		if($selected = $this->session->getVar("list_EditedBy"))
			$pString .= $this->form->selectedBoxValue($this->messages->text("list", "editUserIdResource"), 
				"list_EditedBy", $users, $selected, 7);
		else
			$pString .= $this->form->selectFBoxValue(
				$this->messages->text("list", "editUserIdResource"), "list_EditedBy", $users, 7);
		$pString .= $this->html->p($this->common->displayAscDesc('list'));
		$pString .= $this->form->formSubmit("List") . $this->form->formEnd();
		return $pString;
	}
// With a reorder list request, which type of process do we want?
	public function reorder()
	{
		if($this->session->getVar('list_AddedBy'))
			$this->processAdd();
		else if($this->session->getVar('list_EditedBy'))
			$this->processEdit();
		else
			$this->processGeneral();
	}
// Display titles....
	public function processGeneral()
	{
		$queryString = 'action=list_LISTRESOURCES_CORE&method=processGeneral';
		$this->writeSession();
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, $this->session->getVar('list_Method'), TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			$this->session->setVar("bookmark_DisplayAdd", TRUE);
			$this->session->saveState('list');
			$this->session->saveState('sql');
			$this->session->saveState('bookmark');
			return;
		}
		$sql = $this->stmt->listList(FALSE, FALSE, $this->session->getVar('list_Method'));
		$this->common->display($sql, 'list');
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
// Turn on the 'add bookmark' menu item
		$this->session->setVar("bookmark_DisplayAdd", TRUE);
		$this->session->saveState('list');
		$this->session->saveState('sql');
		$this->session->saveState('bookmark');
	}
	public function processAdd()
	{
		$queryString = 'action=list_LISTRESOURCES_CORE&method=processAdd';
		$this->writeSession();
		$this->stmt->createTotalResourceSubqueryList('resourcemiscAddUserIdResource', $this->session->getVar('list_AddedBy'), 
			FALSE, TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			$this->session->setVar("bookmark_DisplayAdd", TRUE);
			$this->session->saveState('list');
			$this->session->saveState('sql');
			$this->session->saveState('bookmark');
			return;
		}
		$sql = $this->stmt->listList('resourcemiscAddUserIdResource', $this->session->getVar('list_AddedBy'), 'creator');
		$this->common->display($sql, 'list');
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
// Turn on the 'add bookmark' menu item
		$this->session->setVar("bookmark_DisplayAdd", TRUE);
		$this->session->saveState('list');
		$this->session->saveState('sql');
		$this->session->saveState('bookmark');
	}
	public function processEdit()
	{
		$queryString = 'action=list_LISTRESOURCES_CORE&method=processEdit';
		$this->writeSession();
		$this->stmt->createTotalResourceSubqueryList('resourcemiscEditUserIdResource', $this->session->getVar('list_EditedBy'), 
			FALSE, TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			$this->session->setVar("bookmark_DisplayAdd", TRUE);
			$this->session->saveState('list');
			$this->session->saveState('sql');
			$this->session->saveState('bookmark');
			return;
		}
		$sql = $this->stmt->listList('resourcemiscEditUserIdResource', $this->session->getVar('list_EditedBy'), 'creator');
		$this->common->display($sql, 'list');
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
// Turn on the 'add bookmark' menu item
		$this->session->setVar("bookmark_DisplayAdd", TRUE);
		$this->session->saveState('list');
		$this->session->saveState('sql');
		$this->session->saveState('bookmark');
	}
// write input to session
	private function writeSession()
	{
// First, write all input with 'list_' prefix to session
		foreach($this->vars as $key => $value)
		{
			if(preg_match("/^list_/", $key))
				$temp[$key] = $value;
		}
		if(isset($temp))
			$this->session->writeArray($temp);
	}
}
?>
