<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* LISTSOMERESOURCES class
*
* Lists certain resources from keyword and citation links etc. clicked when viewing a solo resource.
* 
* @author Mark Grimshaw
* @version 1
*/
class LISTSOMERESOURCES
{
private $db;
private $vars;
private $stmt;
private $errors;
private $messages;
private $common;
private $template;
private $session;
private $badInput;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->stmt = FACTORY_SQLSTATEMENTS::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->common = FACTORY_LISTCOMMON::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "list"));
// Turn on the 'add bookmark' menu item
		$this->session->setVar("bookmark_DisplayAdd", TRUE);
	}
// With a reorder list request, which type of process do we want?
	public function reorder()
	{
		if(array_key_exists('list_AscDesc', $this->vars))
			$this->session->setVar('list_AscDesc', $this->vars['list_AscDesc']);
		if($this->session->getVar('list_SomeResources_id'))
			$this->vars['id'] = $this->session->getVar('list_SomeResources_id');
		else if($this->session->getVar('list_SomeResources_department'))
			$this->vars['department'] = $this->session->getVar('list_SomeResources_department');
		else if($this->session->getVar('list_SomeResources_institution'))
			$this->vars['institution'] = $this->session->getVar('list_SomeResources_institution');
		if($this->session->getVar('list_SomeResources') == 'category')
			$this->categoryProcess();
		else if($this->session->getVar('list_SomeResources') == 'subcategory')
			$this->subcategoryProcess();
		else if($this->session->getVar('list_SomeResources') == 'cite')
			$this->citeProcess();
		else if($this->session->getVar('list_SomeResources') == 'type')
			$this->typeProcess();
		else if($this->session->getVar('list_SomeResources') == 'language')
			$this->languageProcess();
		else if($this->session->getVar('list_SomeResources') == 'keyword')
			$this->keywordProcess();
		else if($this->session->getVar('list_SomeResources') == 'metaKeyword')
			$this->metaKeywordProcess();
		else if($this->session->getVar('list_SomeResources') == 'publisher')
			$this->publisherProcess();
		else if($this->session->getVar('list_SomeResources') == 'specialPublisher')
			$this->specialPublisherProcess();
		else if($this->session->getVar('list_SomeResources') == 'collection')
			$this->collectionProcess();
		else if($this->session->getVar('list_SomeResources') == 'creator')
			$this->creatorProcess();
		else if($this->session->getVar('list_SomeResources') == 'year')
			$this->yearProcess();
		else if($this->session->getVar('list_SomeResources') == 'userTag')
			$this->userTagProcess();
		$this->session->saveState('list');
		$this->session->saveState('sql');
		$this->session->saveState('bookmark');
	}
/**
* citeProcess - display resources citing this one
* 
* @author Mark Grimshaw
*/
	public function citeProcess($message = FALSE)
	{
		if(!array_key_exists("id", $this->vars) || !$this->vars["id"])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$common = FACTORY_RESOURCECOMMON::getInstance();
		$this->pagingReset();
		$queryString = 'action=list_LISTSOMERESOURCES_CORE&method=citeProcess&id=' . $this->vars["id"];
		$totalSql = $common->showCitations($this->vars["id"]);
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString, $totalSql);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		$this->session->setVar('list_SomeResources', 'cite');
		$this->session->setVar('list_SomeResources_id', $this->vars['id']);
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator');
		$this->common->display($sql, 'list');
	}
/**
* citeProcess - display resources citing creators
* 
* @author Mark Grimshaw
*/
	public function citeProcessCreator($message = FALSE)
	{
		if(!array_key_exists("id", $this->vars) || !$this->vars["id"])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$res = FACTORY_RESOURCECOMMON::getInstance();
		$this->pagingReset();
		$queryString = 'action=list_LISTSOMERESOURCES_CORE&method=citeProcessCreator&id=' . $this->vars["id"];
		$this->stmt->execCondOneField['resourceId'] = explode(',', $this->vars["id"]);
		$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
		$this->stmt->execJoin['resource']['intField'] = 'resourceId';
		$this->stmt->execJoin['resource']['extField'] = 'resourcemiscId';
		$this->stmt->subQueryFields = $res->resourceFields;
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		$this->session->setVar('list_SomeResources', 'cite');
		$this->session->setVar('list_SomeResources_id', $this->vars['id']);
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator');
		$this->common->display($sql, 'list');
	}
/**
* languageProcess - display resources with this language
* 
* @author Mark Grimshaw
*/
	public function languageProcess()
	{
		if(!array_key_exists("id", $this->vars) || !$this->vars["id"])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$this->pagingReset();
		$queryString = "action=list_LISTSOMERESOURCES_CORE&method=languageProcess&id=" . $this->vars["id"];
		$this->stmt->execCond[] = array('resourcelanguageLanguageId' => $this->vars['id']);
		$this->stmt->execJoin['resource_language']['intField'] = 'resourcelanguageResourceId';
		$this->stmt->execJoin['resource_language']['extField'] = 'resourcemiscId';
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		$this->session->setVar('list_SomeResources', 'language');
		$this->session->setVar('list_SomeResources_id', $this->vars['id']);
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator');
		$this->common->display($sql, 'list');
	}
/**
* keywordProcess - display resources with this keyword
* 
* @author Mark Grimshaw
*/
	public function keywordProcess()
	{
		if(!array_key_exists("id", $this->vars) || !$this->vars["id"])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$this->pagingReset();
		$queryString = "action=list_LISTSOMERESOURCES_CORE&method=keywordProcess&id=" . $this->vars["id"];
		$this->stmt->execCond[] = array('resourcekeywordKeywordId' => $this->vars['id']);
		$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
		if(array_key_exists('catId', $this->vars)) // From Category Tree browsing
		{
			$this->stmt->execCond[] = array('resourcecategoryCategoryId' => $this->vars["catId"]);
			$this->stmt->execJoin['resource_category']['intField'] = 'resourcecategoryResourceId';
			$this->stmt->execJoin['resource_category']['extField'] = 'resourcemiscId';
			$queryString = "action=list_LISTSOMERESOURCES_CORE&method=keywordProcess&id=" . $this->vars["id"] . 
				'&catId=' . $this->vars['catId'];
		}
		else
			$queryString = "action=list_LISTSOMERESOURCES_CORE&method=keywordProcess&id=" . $this->vars["id"];
		$this->stmt->execJoin['resource_keyword']['intField'] = 'resourcekeywordResourceId';
		$this->stmt->execJoin['resource_keyword']['extField'] = 'resourcemiscId';
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		$this->session->setVar('list_SomeResources', 'keyword');
		$this->session->setVar('list_SomeResources_id', $this->vars['id']);
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator');
		$this->common->display($sql, 'list');
	}
/**
* metaKeywordProcess - display resources with metadata having this keyword
* 
* @author Mark Grimshaw
*/
	public function metaKeywordProcess()
	{
		if(!array_key_exists("id", $this->vars) || !$this->vars["id"])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$this->pagingReset();
		$queryString = "action=list_LISTSOMERESOURCES_CORE&method=metaKeywordProcess&id=" . $this->vars["id"];
		$this->db->leftJoin('resource_keyword', 'resourcekeywordQuoteId', 'resourcequoteId');
		$this->db->formatConditions(array('resourcekeywordKeywordId' => $this->vars["id"]));
		$this->db->formatConditions(array('resourcekeywordQuoteId' => ' IS NOT NULL'));
		$unions[] = $this->db->selectNoExecute('resource_quote', array(array('resourcequoteResourceId' => 'rId')), 
			TRUE, TRUE, TRUE);
		$this->db->leftJoin('resource_keyword', 'resourcekeywordParaphraseId', 'resourceparaphraseId');
		$this->db->formatConditions(array('resourcekeywordKeywordId' => $this->vars["id"]));
		$this->db->formatConditions(array('resourcekeywordParaphraseId' => ' IS NOT NULL'));
		$unions[] = $this->db->selectNoExecute('resource_paraphrase', array(array('resourceparaphraseResourceId' => 'rId')), 
			TRUE, TRUE, TRUE);
		if($this->session->getVar('setup_ReadOnly'))
		{
			$this->db->formatConditions(array('resourcemusingtextPrivate' => 'N'));
			$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
		}
		$this->db->leftJoin('resource_keyword', 'resourcekeywordMusingId', 'resourcemusingId');
		$this->db->formatConditions(array('resourcekeywordKeywordId' => $this->vars["id"]));
		$this->db->formatConditions(array('resourcekeywordMusingId' => ' IS NOT NULL'));
		$unions[] = $this->db->selectNoExecute('resource_musing', array(array('resourcemusingResourceId' => 'rId')), 
			TRUE, TRUE, TRUE);
		$subQuery = $this->db->subQuery($this->db->union($unions), 'u');
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString, $subQuery);
		$this->common->metadataKeyword = $this->vars["id"];
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		$this->session->setVar('list_SomeResources', 'metaKeyword');
		$this->session->setVar('list_SomeResources_id', $this->vars['id']);
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator');
		$this->common->display($sql, 'list');
	}
/**
* yearProcess - display resources in this publication year
* 
* @author Mark Grimshaw
*/
	public function yearProcess()
	{
		if(!array_key_exists("id", $this->vars) || !$this->vars["id"])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$this->pagingReset();
		$queryString = "action=list_LISTSOMERESOURCES_CORE&method=yearProcess&id=" . $this->vars["id"];
		$this->stmt->execCond[] = array('resourceyearYear1' => $this->vars["id"]);
		$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
		$this->stmt->execJoin['resource_year']['intField'] = 'resourceyearId';
		$this->stmt->execJoin['resource_year']['extField'] = 'resourcemiscId';
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		$this->session->setVar('list_SomeResources', 'year');
		$this->session->setVar('list_SomeResources_id', $this->vars['id']);
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator', $queryString);
		$this->common->display($sql, 'list');
	}
/**
* typeProcess - display resources in this resource type
* 
* @author Mark Grimshaw
*/
	public function typeProcess()
	{
		if(!array_key_exists("id", $this->vars) || !$this->vars["id"])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$res = FACTORY_RESOURCECOMMON::getInstance();
		$this->pagingReset();
		$queryString = "action=list_LISTSOMERESOURCES_CORE&method=typeProcess&id=" . $this->vars["id"];
		$this->stmt->execCond[] = array('resourceType' => $this->vars["id"]);
		$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
		$this->stmt->execJoin['resource']['intField'] = 'resourceId';
		$this->stmt->execJoin['resource']['extField'] = 'resourcemiscId';
		$this->stmt->subQueryFields = $res->resourceFields;
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		$this->session->setVar('list_SomeResources', 'type');
		$this->session->setVar('list_SomeResources_id', $this->vars['id']);
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator', $queryString);
		$this->common->display($sql, 'list');
	}
/**
* categoryProcess - display resources in this category
* 
* @author Mark Grimshaw
*/
	public function categoryProcess()
	{
		if(!array_key_exists("id", $this->vars) || !$this->vars["id"])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$this->pagingReset();
		$queryString = "action=list_LISTSOMERESOURCES_CORE&method=categoryProcess&id=" . $this->vars["id"];
		$this->stmt->execCond[] = array('resourcecategoryCategoryId' => $this->vars["id"]);
		$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
		$this->stmt->execJoin['resource_category']['intField'] = 'resourcecategoryResourceId';
		$this->stmt->execJoin['resource_category']['extField'] = 'resourcemiscId';
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		$this->session->setVar('list_SomeResources', 'category');
		$this->session->setVar('list_SomeResources_id', $this->vars['id']);
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator', $queryString);
		$this->common->display($sql, 'list');
	}
/**
* userTagProcess - display resources in this user tag
* 
* @author Mark Grimshaw
*/
	public function userTagProcess()
	{
		if(!array_key_exists("id", $this->vars) || !$this->vars["id"])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$this->pagingReset();
		$queryString = "action=list_LISTSOMERESOURCES_CORE&method=userTagProcess&id=" . $this->vars["id"];
		$this->stmt->execCond[] = array('resourceusertagsTagId' => $this->vars["id"]);
		$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
		$this->stmt->execJoin['resource_user_tags']['intField'] = 'resourceusertagsResourceId';
		$this->stmt->execJoin['resource_user_tags']['extField'] = 'resourcemiscId';
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		$this->session->setVar('list_SomeResources', 'userTag');
		$this->session->setVar('list_SomeResources_id', $this->vars['id']);
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator', $queryString);
		$this->common->display($sql, 'list');
	}
/**
* subcategoryProcess - display resources in this category
* 
* @author Mark Grimshaw
*/
	public function subcategoryProcess()
	{
		if(!array_key_exists("id", $this->vars) || !$this->vars["id"])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$this->pagingReset();
		$queryString = "action=list_LISTSOMERESOURCES_CORE&method=subcategoryProcess&id=" . $this->vars["id"];
		$this->stmt->execCond[] = array('resourcecategorySubcategoryId' => $this->vars["id"]);
		$this->stmt->execCond[] = array('resourcecategorySubcategoryId' => $this->vars["id"]);
		$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
		$this->stmt->execJoin['resource_category']['intField'] = 'resourcecategoryResourceId';
		$this->stmt->execJoin['resource_category']['extField'] = 'resourcemiscId';
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		$this->session->setVar('list_SomeResources', 'subcategory');
		$this->session->setVar('list_SomeResources_id', $this->vars['id']);
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator', $queryString);
		$this->common->display($sql, 'list');
	}
/**
* creatorProcess - display resources by this creator
* 
* @author Mark Grimshaw
*/
	public function creatorProcess()
	{
		$found = FALSE;
		$this->session->getVar('setup_PagingStyle') == 'A' ? 
			$rcId = 'c.resourcecreatorCreatorId' : $rcId = 'resourcecreatorCreatorId';
		if(array_key_exists("id", $this->vars) && $this->vars["id"])
		{
			$found = TRUE;
			$type = 'id';
			$this->session->setVar('list_SomeResources_id', $this->vars['id']);
			$queryString = "action=list_LISTSOMERESOURCES_CORE&method=creatorProcess&id=" . $this->vars["id"];
// Set id to master creator if chosen creator is part of a group
			$this->db->formatConditions(array('creatorId' => $this->vars["id"]));
			$sameAsId = $this->db->fetchOne($this->db->select('creator', 'creatorSameAs'));
			if($sameAsId)
				$id = $sameAsId;
			else
				$id = $this->vars["id"];
			$this->stmt->execCond[] = $this->db->formatFields($rcId) . ' ' . 
				$this->db->equal . ' ' . $this->db->tidyInput($id) . 
				' ' . $this->db->or . ' ' . 
				$this->db->formatFields('creatorSameAs') . ' ' . 
				$this->db->equal . ' ' . $this->db->tidyInput($id);
		}
		else if(array_key_exists("department", $this->vars) && $this->vars["department"])
		{
			$found = TRUE;
			$type = 'department';
			$this->session->setVar('list_SomeResources_department', $this->vars['department']);
			$conds = array();
			$queryString = "action=list_LISTSOMERESOURCES_CORE&method=creatorProcess&department=" . $this->vars["department"];
			$this->db->formatConditions(array('usersDepartment' => $this->vars["department"]));
			$resultset = $this->db->select('users', 'usersIsCreator');
			while($row = $this->db->fetchRow($resultset))
				$conds[] = $this->db->formatFields($rcId) . ' ' . 
					$this->db->equal . ' ' . $this->db->tidyInput($row['usersIsCreator']) . 
					' ' . $this->db->or . ' ' . 
					$this->db->formatFields('creatorSameAs') . ' ' . 
					$this->db->equal . ' ' . $this->db->tidyInput($row['usersIsCreator']);
			if(!empty($conds))
				$this->stmt->execCond[] = join($this->db->or, $conds);
		}
		else if(array_key_exists("institution", $this->vars) && $this->vars["institution"])
		{
			$found = TRUE;
			$type = 'institution';
			$this->session->setVar('list_SomeResources_institution', $this->vars['institution']);
			$conds = array();
			$queryString = "action=list_LISTSOMERESOURCES_CORE&method=creatorProcess&institution=" . $this->vars["institution"];
			$this->db->formatConditions(array('usersInstitution' => $this->vars["institution"]));
			$resultset = $this->db->select('users', 'usersIsCreator');
			while($row = $this->db->fetchRow($resultset))
				$conds[] = $this->db->formatFields($rcId) . ' ' . 
					$this->db->equal . ' ' . $this->db->tidyInput($row['usersIsCreator']) . 
					' ' . $this->db->or . ' ' . 
					$this->db->formatFields('creatorSameAs') . ' ' . 
					$this->db->equal . ' ' . $this->db->tidyInput($row['usersIsCreator']);
			if(!empty($conds))
				$this->stmt->execCond[] = join($this->db->or, $conds);
		}
		if(!$found)
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$this->pagingReset();
		$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
		$this->stmt->execJoin['creator']['intField'] = 'creatorId';
		$this->stmt->execJoin['creator']['extField'] = 'resourcecreatorCreatorId';
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		$this->session->setVar('list_SomeResources', 'creator');
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator', $queryString);
		$this->common->display($sql, 'list');
	}
/**
* Publishers (conference, translated books) where publisher is stored in resourcemiscField1
*/
	public function specialPublisherProcess()
	{
		$this->publisherProcess(TRUE);
	}
/**
* publisherProcess - display resources by this publisher
* 
* @author Mark Grimshaw
*/
	public function publisherProcess($miscField1 = FALSE)
	{
		if(!array_key_exists("id", $this->vars) || !$this->vars["id"])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$this->pagingReset();
		if($miscField1)
			$queryString = "action=list_LISTSOMERESOURCES_CORE&method=specialPublisherProcess&id=" . $this->vars["id"];
		else
			$queryString = "action=list_LISTSOMERESOURCES_CORE&method=publisherProcess&id=" . $this->vars["id"];
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
			$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', $queryString);
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		if($miscField1) // conference and translated publisher
		{
			$this->session->setVar('list_SomeResources', 'specialPublisher');
			$this->stmt->execCond[] = array('resourcemiscField1' => $this->vars["id"]);
			$this->stmt->execCond[] = array('resourcemiscField1' => ' IS NOT NULL');
			$this->stmt->execCond[] = $this->db->formatFields('resourceType') . '=' . $this->db->tidyInput('proceedings') . 
				$this->db->or . $this->db->formatFields('resourceType') . '=' . $this->db->tidyInput('proceedings_article') . 
				$this->db->or . $this->db->formatFields('resourceType') . '=' . $this->db->tidyInput('book') . 
				$this->db->or . $this->db->formatFields('resourceType') . '=' . $this->db->tidyInput('book_article') . 
				$this->db->or . $this->db->formatFields('resourceType') . '=' . $this->db->tidyInput('book_chapter');
			$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
			$this->stmt->execJoin['resource']['intField'] = 'resourceId';
			$this->stmt->execJoin['resource']['extField'] = 'resourcemiscId';
			$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		}
		else
		{
			$this->session->setVar('list_SomeResources', 'publisher');
			$this->stmt->execCond[] = array('resourcemiscPublisher' => $this->vars["id"]);
			$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
			$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		}
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$this->session->setVar('list_SomeResources_id', $this->vars['id']);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator', $queryString);
		$this->common->display($sql, 'list');
	}
/**
*collection Process - display resources by this collection
* 
* @author Mark Grimshaw
*/
	public function collectionProcess()
	{
		if(!array_key_exists("id", $this->vars) || !$this->vars["id"])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$this->pagingReset();
		$queryString = "action=list_LISTSOMERESOURCES_CORE&method=collectionProcess&id=" . $this->vars["id"];
		$this->stmt->execCond[] = array('resourcemiscCollection' => $this->vars["id"]);
		$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		$this->session->setVar('list_SomeResources', 'collection');
		$this->session->setVar('list_SomeResources_id', $this->vars['id']);
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator', $queryString);
		$this->common->display($sql, 'list');
	}
/**
* bibliographyProcess - display resources in this user bibliography
* 
* @author Mark Grimshaw
*/
	public function bibliographyProcess()
	{
		$gatekeep = FACTORY_GATEKEEP::getInstance();
		$gatekeep->init(); // No Read-only access allowed
		if(!array_key_exists("id", $this->vars) || !$this->vars["id"])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$this->pagingReset();
		$queryString = "action=list_LISTSOMERESOURCES_CORE&method=bibliographyProcess&id=" . $this->vars["id"];
		if($this->session->getVar('mywikindx_Bibliography_use')) // If we are browsing a bibliography, turn of its condition in SQLSTATEMENTS
		{
			$commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
			$commonBib->bailOut = TRUE;
		}
		$this->stmt->execCond[] = array('userbibliographyresourceBibliographyId' => $this->vars["id"]);
		$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
		$this->stmt->execJoin['user_bibliography_resource']['intField'] = 'userbibliographyresourceResourceId';
		$this->stmt->execJoin['user_bibliography_resource']['extField'] = 'resourcemiscId';
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'list');
			return;
		}
		$this->session->setVar('list_SomeResources', 'bibliography');
		$this->session->setVar('list_SomeResources_id', $this->vars['id']);
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator', $queryString);
		$this->common->display($sql, 'list');
	}
// If this is a different resource listing to the previous one, reset the paging counter.
	private function pagingReset()
	{
		if(!array_key_exists('pagingStart', $this->vars))
		{
			$this->session->delVar('mywikindx_PagingStart');
			$this->session->delVar('mywikindx_PagingStartAlpha');
		}
	}
}
?>
