<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* SEARCHMETADATA class
*
* Search database metadata on a variety of parameters
*/
class SEARCHMETADATA
{
private $db;
private $vars;
private $utf8;
private $stmt;
private $errors;
private $messages;
private $common;
private $session;
private $form;
private $table;
private $html;
private $template;
private $type;
private $keyword;
private $types;
private $keywords;
private $radioButtons;
private $badInput;
private $commonBib;
private $parsePhrase;
private $words = '';
private $typeArray;
private $tAlias = 1;
private $quoteExists = FALSE;
private $quoteCommentExists = FALSE;
private $paraphraseExists = FALSE;
private $paraphraseCommentExists = FALSE;
private $musingExists = FALSE;
private $metadataTypeArray = array('quote', 'paraphrase', 'quoteComment', 'paraphraseComment', 'musing');
private $conditions = array();
private $conditionsOneField = array();
private $joins = array();

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->utf8 = FACTORY_UTF8::getInstance();
		$this->stmt = FACTORY_SQLSTATEMENTS::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->common = FACTORY_LISTCOMMON::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->type = FACTORY_TYPE::getInstance();
		$this->keyword = FACTORY_KEYWORD::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->parsePhrase = FACTORY_PARSEPHRASE::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "searchMeta"));
	}
// display form options
	public function init($error = FALSE)
	{
///First check, do we have resources?
		if(!$this->checkMetadataExists())
		{
			GLOBALS::buildOutputString($this->messages->text('misc', 'noMetadata'));
			return;
		}
		include_once("core/modules/help/HELPMESSAGES.php");
		$help = new HELPMESSAGES();
		GLOBALS::setOutputHeading($this->messages->text("heading", "searchMeta"));
		GLOBALS::setOutputHelp($help->createLink('search'));
		$this->session->delVar('mywikindx_PagingStart');
		$this->session->delVar('mywikindx_PagingStartAlpha');
		$linksInfo['info'] = $this->commonBib->displayBib();
		$this->template->tpl->assign('resourceListInfo', $linksInfo);
		$pString = $error ? $error : FALSE;
		$ajax = FACTORY_AJAX::getInstance();
		$ajax->loadJavascript(array('core/modules/list/searchSelect.js'));
		$pString .= $this->form->formHeader("list_SEARCHMETADATA_CORE");
		$pString .= $this->form->hidden("method", "process");
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$jsonArray = array();
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initTypes&type=search';
// Amend type list depending upon which metadata types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'search_Metadata',
			'targetDiv' => 'type',
			'restoreField' => 'search_Type',
			);
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initKeywords&type=search&metadata=true';
// Amend keyword list depending upon which metadata types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'search_Metadata',
			'targetDiv' => 'keyword',
			'restoreField' => 'search_Keyword',
			);
		$fields = $this->grabMetadataTypes();
		$ajax->jActionForm('onclick', $jsonArray);
		$pString .= $this->table->td($this->makeFormMultiple("metadata", $fields), FALSE, FALSE, FALSE, FALSE, '20%');
		$this->grabTypes();
		$pString .= $this->table->td($this->html->div('type', $this->makeFormMultiple("type", $this->types)), 
			FALSE, FALSE, FALSE, FALSE, '20%');
		$this->grabKeywords();
		if(is_array($this->keywords))
			$pString .= $this->table->td($this->html->div('keyword', $this->makeFormMultiple("keyword", $this->keywords)), 
			FALSE, FALSE, FALSE, FALSE, '20%');
		else
			$pString .= $this->table->td($this->html->div('keyword', "&nbsp;"), FALSE, FALSE, FALSE, FALSE, '20%');
		$this->radioButtons = FALSE;
		$word = $this->session->issetVar("search_Word") ? 
			htmlspecialchars(stripslashes($this->session->getVar("search_Word")), ENT_QUOTES) : FALSE;
		$checked = $this->session->issetVar("search_Partial") ? 'CHECKED' : FALSE;
		$pString .= $this->table->td($this->form->textInput($this->messages->text("search", "word"), "search_Word", 
			$word, 40) . " " . $this->html->span('*', 'required') . 
			$this->html->p($this->messages->text("search", "partial") . ":&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, "search_Partial", $checked)), FALSE, FALSE, FALSE, FALSE, '25%');
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->table->tableStart(FALSE, 0, 0, 0, 'left', '25%');
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->common->displayOrder('search'));
		$reset = $this->form->formHeader("list_SEARCHMETADATA_CORE") . $this->form->hidden("method", "reset") . 
			$this->form->formSubmit("Reset") . $this->form->formEnd();
// Always do a full reset because of AJAX controlling some select boxes
		$pString .= $this->table->td($this->form->formSubmit("Search") . $this->html->br() . $this->html->br() . 
			$this->form->formEnd() . $reset);
		$pString .= $this->table->tableEnd();
		GLOBALS::buildOutputString($pString);
	}
// make a self-contained form with a multiple select box
	private function makeFormMultiple($type, $array, $width = FALSE)
	{
// add 0 => IGNORE to $array
		$temp[0] = $this->messages->text("misc", "ignore");
		foreach($array as $key => $value)
			$temp[$key] = $value;
		if(($type == 'keyword') || ($type == 'type'))
			$label = 'member';
		else
			$label = 'search';
		if($selected = $this->session->getVar("search_" . ucfirst($type)))
		{
			$selectedArray = explode(",", $selected);
			$pString = $this->form->selectedBoxValueMultiple($this->messages->text("search", $label, " " . 
				$this->messages->text("search", $type)), "search_" . ucfirst($type), $temp, $selectedArray, "5", $width);
			if($type == 'type')
				$this->typeArray = $selectedArray;
		}
		else
		{
			$pString = $this->form->selectFBoxValueMultiple($this->messages->text("search", $label, " " . 
				$this->messages->text("search", $type)), "search_" . ucfirst($type), $temp, "5", $width);
		}
		$pString .= $this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
			$this->html->br();
		if($this->radioButtons)
			$pString .= $this->radioButtons . $this->html->br();
		return $pString;
	}
// get metadata types from database.
	private function grabMetadataTypes()
	{
		$fields = array("quote"	=>	$this->messages->text("metadata", "quotes"),
				"quoteComment"	=>	$this->messages->text("metadata", "quoteComments"),
				"paraphrase"	=>	$this->messages->text("metadata", "paraphrases"),
				"paraphraseComment"	=>	$this->messages->text("metadata", "paraphraseComments"),
				"musing"	=>	$this->messages->text("metadata", "musings"),
			);
		if(!$this->quoteExists)
			unset($fields['quote']);
		if(!$this->quoteCommentExists)
			unset($fields['quoteComment']);
		if(!$this->paraphraseExists)
			unset($fields['paraphrase']);
		if(!$this->paraphraseCommentExists)
			unset($fields['paraphraseComment']);
		if(!$this->musingExists)
			unset($fields['musing']);
		$this->radioButtons = FALSE;
		return $fields;
	}
// get types from database.
	private function grabTypes()
	{
		$this->types = $this->type->grabAll(TRUE, $this->metadataTypeArray);
		$this->radioButtons = FALSE;
	}
// get keywords from database.
	private function grabKeywords()
	{
		$this->keywords = $this->keyword->grabAll(TRUE, $this->metadataTypeArray);
		$this->makeRadioButtons('Keyword');
	}
// Create radio buttons for AND and OR
	private function makeRadioButtons($type)
	{
		$type = 'search_' . $type . 'Method';
		if($this->session->getVar($type) == 'AND')
		{
			$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR') . " OR", "small") . $this->html->br();
			$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND', TRUE) . " AND", "small");
		}
// Default
		else
		{
			$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR', TRUE) . " OR", "small") . $this->html->br();
			$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND') . " AND", "small");
		}
		$this->radioButtons = $pString;
	}
// parse the search word(s)
	private function parseWord()
	{
		$this->words = $this->parsePhrase->parse($this->input);
		if(!$this->words)
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'init');
	}
// Reset the form and clear the session
	public function reset()
	{
		$this->session->clearArray('search');
		$this->init();
	}
/**
* For re-ordering or paging
*/
	public function reprocess()
	{
		$this->input = $this->session->getArray("search");
		if(array_key_exists('search_Order', $this->vars) && $this->vars['search_Order'])
		{
			$this->input['Order'] = $this->vars['search_Order'];
			$this->session->setVar('search_Order', $this->input['Order']);
		}
		if(array_key_exists('search_AscDesc', $this->vars) && $this->vars['search_AscDesc'])
		{
			$this->input['AscDesc'] = $this->vars['search_AscDesc'];
			$this->session->setVar('search_AscDesc', $this->input['AscDesc']);
		}
		$this->process(TRUE);
	}
	public function process($reprocess = FALSE)
	{
		$this->stmt->listMethodAscDesc = 'search_AscDesc';
		$queryString = 'action=list_SEARCHMETADATA_CORE&method=reprocess';
		if(!$reprocess)
			$this->input = $this->checkInput();
		else
			$this->input = $this->session->getArray("search");
		$this->parseWord();
		$order = $this->input['Order'];
		$selectK = $selectT = FALSE;
		foreach($this->input as $type => $value)
		{
			if($type == 'Type')
			{
				$selectT = TRUE;
				$selectArray = explode(",", $value);
				$this->conditionsOneField['resourceType'] = $selectArray;
			}
			else if($type == 'Keyword')
			{
				$selectK = TRUE;
				$selectArray = explode(",", $this->input['Keyword']);
				if($this->input['KeywordMethod'] == 'OR')
					$this->conditionsOneField['resourcekeywordKeywordId'] = $selectArray;
				else
				{
					foreach($selectArray as $keyword)
					{
						$this->conditions[$this->tAlias . '.resourcekeywordKeywordId'] = $keyword;
						$selectKArray[] = $this->tAlias;
						++$this->tAlias;
					}
				}
			}
		}
		if(!array_key_exists('Metadata', $this->input)) // searching from all types of metadata
			$this->input['Metadata'] = join(',', $this->metadataTypeArray);
		$qDone = $pDone = $mDone = FALSE;
		if($userId = $this->session->getVar('setup_UserId'))
		{
// quote comments
			$this->db->formatConditions(array('usergroupsusersUserId' => $userId));
			$this->db->formatConditions($this->db->formatFields('usergroupsusersGroupId') . $this->db->equal . 
				$this->db->formatFields('resourcequotecommentPrivate'));
			$subSql = $this->db->selectNoExecute('user_groups_users', 'usergroupsusersId', FALSE, TRUE, TRUE);
			$subject = $this->db->formatFields('resourcequotecommentPrivate') . $this->db->notEqual . $this->db->tidyInput('N') 
				. $this->db->and . 
				$this->db->formatFields('resourcequotecommentPrivate') . $this->db->notEqual . $this->db->tidyInput('Y');
			$case1 = $this->db->caseWhen($subject, FALSE, $subSql, FALSE, FALSE);
			$subject = $this->db->formatFields('resourcequotecommentPrivate') . $this->db->equal . $this->db->tidyInput('Y');
			$result = $this->db->formatFields('resourcequotecommentAddUserIdQuote') . $this->db->equal . $this->db->tidyInput($userId);
			$case2 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
			$subject = $this->db->formatFields('resourcequotecommentPrivate') . $this->db->equal . $this->db->tidyInput('N');
			$result = $this->db->tidyInput(1);
			$case3 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
			$qCommentUserCheckCond = $case1 . $this->db->or . $case2 . $this->db->or . $case3;
// paraphrase comments
			$this->db->formatConditions(array('usergroupsusersUserId' => $userId));
			$this->db->formatConditions($this->db->formatFields('usergroupsusersGroupId') . $this->db->equal . 
				$this->db->formatFields('resourceparaphrasecommentPrivate'));
			$subSql = $this->db->selectNoExecute('user_groups_users', 'usergroupsusersId', FALSE, TRUE, TRUE);
			$subject = $this->db->formatFields('resourceparaphrasecommentPrivate') . $this->db->notEqual . $this->db->tidyInput('N') 
				. $this->db->and . 
				$this->db->formatFields('resourceparaphrasecommentPrivate') . $this->db->notEqual . $this->db->tidyInput('Y');
			$case1 = $this->db->caseWhen($subject, FALSE, $subSql, FALSE, FALSE);
			$subject = $this->db->formatFields('resourceparaphrasecommentPrivate') . $this->db->equal . $this->db->tidyInput('Y');
			$result = $this->db->formatFields('resourceparaphrasecommentAddUserIdParaphrase') . 
				$this->db->equal . $this->db->tidyInput($userId);
			$case2 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
			$subject = $this->db->formatFields('resourceparaphrasecommentPrivate') . $this->db->equal . $this->db->tidyInput('N');
			$result = $this->db->tidyInput(1);
			$case3 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
			$pCommentUserCheckCond = $case1 . $this->db->or . $case2 . $this->db->or . $case3;
// musings
			$this->db->formatConditions(array('usergroupsusersUserId' => $userId));
			$this->db->formatConditions($this->db->formatFields('usergroupsusersGroupId') . $this->db->equal . 
				$this->db->formatFields('resourcemusingtextPrivate'));
			$subSql = $this->db->selectNoExecute('user_groups_users', 'usergroupsusersId', FALSE, TRUE, TRUE);
			$subject = $this->db->formatFields('resourcemusingtextPrivate') . $this->db->notEqual . $this->db->tidyInput('N') 
				. $this->db->and . 
				$this->db->formatFields('resourcemusingtextPrivate') . $this->db->notEqual . $this->db->tidyInput('Y');
			$case1 = $this->db->caseWhen($subject, FALSE, $subSql, FALSE, FALSE);
			$subject = $this->db->formatFields('resourcemusingtextPrivate') . $this->db->equal . $this->db->tidyInput('Y');
			$result = $this->db->formatFields('resourcemusingtextAddUserIdMusing') . $this->db->equal . $this->db->tidyInput($userId);
			$case2 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
			$subject = $this->db->formatFields('resourcemusingtextPrivate') . $this->db->equal . $this->db->tidyInput('N');
			$result = $this->db->tidyInput(1);
			$case3 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
			$musingUserCheckCond = $case1 . $this->db->or . $case2 . $this->db->or . $case3;
		}
		foreach(explode(',', $this->input['Metadata']) as $mType)
		{
			if($mType == 'quote')
			{
				if($selectT)
					$this->joins['resource'] = array('resourceId', 'resourcequoteResourceId');
				if($selectK)
				{
					if(isset($selectKArray))
					{
						foreach($selectKArray as $alias)
							$this->joins[$alias . '.resource_keyword'] = array($alias . '.resourcekeywordQuoteId', 'resourcequoteId');
					}
					else
						$this->joins['resource_keyword'] = array('resourcekeywordQuoteId', 'resourcequoteId');
				}
				$this->db->leftJoin('resource_quote_text', 'resourcequotetextId', 'resourcequoteId');
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcequotetextText'), $this->words);
				$this->db->formatConditions("($wc)");
				$this->condJoin();
				$union = $this->db->selectNoExecute('resource_quote', array(array('resourcequoteResourceId' => 'rId')), 
					TRUE, TRUE, TRUE);
				$unions[] = $union;
				$union = str_replace('WHERE', 'WHERE (' . $this->db->formatFields('resourcequoteResourceId') . ' = RESID) AND ', $union);
				$this->common->metadataText[] = str_replace('DISTINCT', $this->db->formatFields('resourcequotetextText') . 
					' AS ' . $this->db->formatFields('text') . ', ', $union);
			}
			else if($mType == 'quoteComment')
			{
				if($selectT)
					$this->joins['resource'] = array('resourceId', 'resourcequoteResourceId');
				if($selectK)
				{
					if(isset($selectKArray))
					{
						foreach($selectKArray as $alias)
							$this->joins[$alias . '.resource_keyword'] = array($alias . '.resourcekeywordQuoteId', 'resourcequoteId');
					}
					else
						$this->joins['resource_keyword'] = array('resourcekeywordQuoteId', 'resourcequoteId');
				}
				$this->db->leftJoin('resource_quote_comment', 'resourcequotecommentQuoteId', 'resourcequoteId');
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcequotecommentComment'), $this->words);
				$this->db->formatConditions("($wc)");
				$this->condJoin();
				if($this->session->getVar('setup_ReadOnly'))
					$this->db->formatConditions(array('resourcequotecommentPrivate' => 'N'));
				else if($this->session->getVar('setup_UserId'))
					$this->db->formatConditions($qCommentUserCheckCond);
				$union = $this->db->selectNoExecute('resource_quote', array(array('resourcequoteResourceId' => 'rId')), 
					TRUE, TRUE, TRUE);
				$unions[] = $union;
				$union = str_replace('WHERE', 'WHERE (' . $this->db->formatFields('resourcequoteResourceId') . ' = RESID) AND ', $union);
				$this->common->metadataText[] = str_replace('DISTINCT', $this->db->formatFields('resourcequotecommentComment') . 
					' AS ' . $this->db->formatFields('text') . ', ', $union);
			}
			else if($mType == 'paraphrase')
			{
				if($selectT)
					$this->joins['resource'] = array('resourceId', 'resourceparaphraseResourceId');
				if($selectK)
				{
					if(isset($selectKArray))
					{
						foreach($selectKArray as $alias)
							$this->joins[$alias . '.resource_keyword'] = 
							array($alias . '.resourcekeywordParaphraseId', 'resourceparaphraseId');
					}
					else
						$this->joins['resource_keyword'] = array('resourcekeywordParaphraseId', 'resourceparaphraseId');
				}
				$this->db->leftJoin('resource_paraphrase_text', 'resourceparaphrasetextId', 'resourceparaphraseId');
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourceparaphrasetextText'), $this->words);
				$this->db->formatConditions("($wc)");
				$this->condJoin();
				$union = $this->db->selectNoExecute('resource_paraphrase', array(array('resourceparaphraseResourceId' => 'rId')), 
					TRUE, TRUE, TRUE);
				$unions[] = $union;
				$union = str_replace('WHERE', 'WHERE (' . $this->db->formatFields('resourceparaphraseResourceId') . ' = RESID) AND ', $union);
				$this->common->metadataText[] = str_replace('DISTINCT', $this->db->formatFields('resourceparaphrasetextText') . 
					' AS ' . $this->db->formatFields('text') . ', ', $union);
			}
			else if($mType == 'paraphraseComment')
			{
				if($selectT)
					$this->joins['resource'] = array('resourceId', 'resourceparaphraseResourceId');
				if($selectK)
				{
					if(isset($selectKArray))
					{
						foreach($selectKArray as $alias)
							$this->joins[$alias . '.resource_keyword'] = 
							array($alias . '.resourcekeywordParaphraseId', 'resourceparaphraseId');
					}
					else
						$this->joins['resource_keyword'] = array('resourcekeywordParaphraseId', 'resourceparaphraseId');
				}
				$this->db->leftJoin('resource_paraphrase_comment', 'resourceparaphrasecommentParaphraseId', 'resourceparaphraseId');
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourceparaphrasecommentComment'), $this->words);
				$this->db->formatConditions("($wc)");
				$this->condJoin();
				if($this->session->getVar('setup_ReadOnly'))
					$this->db->formatConditions(array('resourceparaphrasecommentPrivate' => 'N'));
				else if($this->session->getVar('setup_UserId'))
					$this->db->formatConditions($pCommentUserCheckCond);
				$union = $this->db->selectNoExecute('resource_paraphrase', array(array('resourceparaphraseResourceId' => 'rId')), 
					TRUE, TRUE, TRUE);
				$unions[] = $union;
				$union = str_replace('WHERE', 'WHERE (' . $this->db->formatFields('resourceparaphraseResourceId') . ' = RESID) AND ', $union);
				$this->common->metadataText[] = str_replace('DISTINCT', $this->db->formatFields('resourceparaphrasecommentComment') . 
					' AS ' . $this->db->formatFields('text') . ', ', $union);
			}
			else if($mType == 'musing')
			{
				if($selectT)
					$this->joins['resource'] = array('resourceId', 'resourcemusingResourceId');
				if($selectK)
				{
					if(isset($selectKArray))
					{
						foreach($selectKArray as $alias)
							$this->joins[$alias . '.resource_keyword'] = 
							array($alias . '.resourcekeywordMusingId', 'resourcemusingId');
					}
					else
						$this->joins['resource_keyword'] = array('resourcekeywordMusingId', 'resourcemusingId');
				}
				$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcemusingtextText'), $this->words);
				$this->db->formatConditions("($wc)");
				$this->condJoin();
				if($this->session->getVar('setup_ReadOnly'))
					$this->db->formatConditions(array('resourcemusingtextPrivate' => 'N'));
				else if($this->session->getVar('setup_UserId'))
					$this->db->formatConditions($musingUserCheckCond);
				$union = $this->db->selectNoExecute('resource_musing', array(array('resourcemusingResourceId' => 'rId')), 
					TRUE, TRUE, TRUE);
				$unions[] = $union;
				$union = str_replace('WHERE', 'WHERE (' . $this->db->formatFields('resourcemusingResourceId') . ' = RESID) AND ', $union);
				$this->common->metadataText[] = str_replace('DISTINCT', $this->db->formatFields('resourcemusingtextText') . 
					' AS ' . $this->db->formatFields('text') . ', ', $union);
			}
		}
		$subQuery = $this->db->subQuery($this->db->union($unions), 'u');
		$sqlTotal = $this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, $order, TRUE, $queryString, $subQuery);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'search');
			$this->session->saveState('search');
			$this->session->saveState('sql');
			$this->session->saveState('bookmark');
			return;
		}
		$searchTerms = explode(",", $this->session->getVar('search_Highlight'));
		foreach($searchTerms as $term)
//			$patterns[] = "/($term)/i";
			$patterns[] = "/($term)(?=[^>]*(<|$))/i";
		$this->common->patterns = $patterns;
		$this->common->keepHighlight = TRUE;
		$sql = $this->stmt->listList(FALSE, FALSE, $order);
		$this->common->metadata = TRUE;
		$this->common->display($sql, 'search');
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
// Turn on the 'add bookmark' menu item
		$this->session->setVar("bookmark_DisplayAdd", TRUE);
		$this->session->saveState('search');
		$this->session->saveState('sql');
		$this->session->saveState('bookmark');
	}
/**
* Create condition and join statements
*/
	private function condJoin()
	{
		foreach($this->conditions as $field => $cond)
		{
			if(!$field)
				$this->db->formatConditions($cond);
			else
				$this->db->formatConditions(array($field => $cond));
		}
		foreach($this->conditionsOneField as $field => $array)
			$this->db->formatConditionsOneField($array, $field);
		foreach($this->joins as $table => $array)
		{
			$split = explode('.', $table);
			if(sizeof($split) == 2)
				$this->db->leftJoin(array(array($split[1] => $split[0])), $array[0], $array[1]);
			else
				$this->db->leftJoin($table, $array[0], $array[1]);
		}
	}
// validate user input
// Input comes either from form input or, when paging, from the session.
	private function checkInput()
	{
		$typeInputFound = $inputFound = FALSE;
		$typesArray = array("Metadata", "Type", "Keyword");
		$temp = array();
		if((array_search('LastMulti', $this->vars) !== FALSE) || array_key_exists('PagingStart', $this->vars))
		{
			foreach($this->session->getArray("search") as $key => $value)
			{
				if(array_search($key, $typesArray) !== FALSE)
					$this->vars['search_' . $key] = explode(',', $value);
				else
					$this->vars['search_' . $key] = $value;
			}
		}
// Delete previous stored data
		$this->session->clearArray("search");
		foreach($typesArray as $type)
		{
			$inputKey = 'search_' . $type;
			$method = "search_" . $type . "Method";
			if(!array_key_exists($inputKey, $this->vars))
				continue;
			if((sizeof($this->vars[$inputKey]) == 1) && !$this->vars[$inputKey][0]) // IGNORE
			{
				if($type == 'Type')
					$inputFound = TRUE;
				continue;
			}
			$inputFound = TRUE;
			if(array_key_exists(0, $this->vars[$inputKey]) && !$this->vars[$inputKey][0])
				unset($this->vars[$inputKey][0]);
			$temp[$type] = implode(",", $this->vars[$inputKey]);
// If missing the following, set to default 'OR'.
			if(!array_key_exists($method, $this->vars))
				$this->vars[$method] = 'OR';
			$this->inputMethod[$type] = $this->vars[$method];
			$this->session->setVar($method, $this->vars[$method]);
		}
		if(array_key_exists('search_Order', $this->vars) && $this->vars['search_Order'])
			$temp['Order'] = $this->vars['search_Order'];
		if(array_key_exists('search_AscDesc', $this->vars) && $this->vars['search_AscDesc'])
			$temp['AscDesc'] = $this->vars['search_AscDesc'];
		if((array_key_exists("search_Word", $this->vars) && !trim($this->vars["search_Word"])))
			$inputFound = FALSE;
		else
			$temp['Word'] = trim($this->vars["search_Word"]);
		if(array_key_exists("search_Partial", $this->vars))
			$temp['Partial'] = trim($this->vars["search_Partial"]);
		$this->session->writeArray($temp, 'search');
		if(!$inputFound)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		return $this->session->getArray("search");
	}
/**
* Check we have metadata
*/
	private function checkMetadataExists()
	{
		$metadataExists = FALSE;
		$this->commonBib->userBibCondition('resourcequoteResourceId');
		$recordset = $this->db->select('resource_quote', 'resourcequoteId');
		if($this->db->numRows($recordset))
			$metadataExists = $this->quoteExists = TRUE;
		$this->commonBib->userBibCondition('resourceparaphraseResourceId');
		$recordset = $this->db->select('resource_paraphrase', 'resourceparaphraseId');
		if($this->db->numRows($recordset))
			$metadataExists = $this->paraphraseExists = TRUE;
		$this->commonBib->userBibCondition('resourcemusingResourceId');
		$recordset = $this->db->select('resource_musing', 'resourcemusingId');
		if($this->db->numRows($recordset))
			$metadataExists = $this->musingExists = TRUE;
		if(!$metadataExists)
			return FALSE;
		$this->db->leftJoin('resource_quote', 'resourcequoteId', 'resourcequotecommentQuoteId');
		$this->commonBib->userBibCondition('resourcequoteResourceId');
		$recordset = $this->db->select('resource_quote_comment', 'resourcequotecommentId');
		if($this->db->numRows($recordset))
			$this->quoteCommentExists = TRUE;
		$this->db->leftJoin('resource_paraphrase', 'resourceparaphraseId', 'resourceparaphrasecommentParaphraseId');
		$this->commonBib->userBibCondition('resourceparaphraseResourceId');
		$recordset = $this->db->select('resource_paraphrase_comment', 'resourceparaphrasecommentId');
		if($this->db->numRows($recordset))
			$this->paraphraseCommentExists = TRUE;
		return TRUE;
	}
}
?>
