<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* SELECTRESOURCES class
*
* Select database resources on a variety of parameters
*/
class SELECTRESOURCES
{
private $db;
private $vars;
private $stmt;
private $errors;
private $messages;
private $common;
private $session;
private $form;
private $table;
private $html;
private $template;
private $type;
private $category;
private $keyword;
private $creator;
private $collection;
private $publisher;
private $user;
private $types;
private $categories;
private $subcategories;
private $userTags;
private $keywords;
private $creators;
private $collections;
private $publishers;
private $tags;
private $languages = array();
private $radioButtons;
private $badInput;
private $commonBib;
private $typeArray;
private $tAlias = 1;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->stmt = FACTORY_SQLSTATEMENTS::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->common = FACTORY_LISTCOMMON::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->user = FACTORY_USER::getInstance();
		$this->type = FACTORY_TYPE::getInstance();
		$this->category = FACTORY_CATEGORY::getInstance();
		$this->keyword = FACTORY_KEYWORD::getInstance();
		$this->creator = FACTORY_CREATOR::getInstance();
		$this->publisher = FACTORY_PUBLISHER::getInstance();
		$this->collection = FACTORY_COLLECTION::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "select"));
	}
// display form options
	public function init($error = FALSE)
	{
///First check, do we have resources?
		if(!$this->common->resourcesExist())
			return;
		include_once("core/modules/help/HELPMESSAGES.php");
		$help = new HELPMESSAGES();
		GLOBALS::setOutputHelp($help->createLink('select'));
		GLOBALS::setOutputHeading($this->messages->text("heading", "select"));
		$this->session->delVar('mywikindx_PagingStart');
		$this->session->delVar('mywikindx_PagingStartAlpha');
		$linksInfo['info'] = $this->commonBib->displayBib();
		$this->template->tpl->assign('resourceListInfo', $linksInfo);
		$pString = $error ? $error : FALSE;
		$ajax = FACTORY_AJAX::getInstance();
		$ajax->loadJavascript(array('core/modules/list/searchSelect.js'));
		$pString .= $this->form->formHeader("list_SELECTRESOURCES_CORE");
		$pString .= $this->form->hidden("method", "process");
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$this->grabTypes();
		$jsonArray = array();
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initLanguages&type=select';
// Amend languages list depending upon which resource types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'select_Type',
			'targetDiv' => 'language',
			'restoreField' => 'select_Language'
			);
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initCategories&type=select';
// Amend category list depending upon which resource types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'select_Type',
			'targetDiv' => 'category',
			'restoreField' => 'select_Category'
			);
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initKeywords&type=select';
// Amend keyword list depending upon which resource types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'select_Type',
			'targetDiv' => 'keyword',
			'restoreField' => 'select_Keyword',
			);
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initCreators&type=select';
// Amend creators list depending upon which resource types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'select_Type',
			'targetDiv' => 'creator',
			'restoreField' => 'select_Creator',
			);
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initPublishers&type=select';
// Amend publishers list depending upon which resource types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'select_Type',
			'targetDiv' => 'publisher',
			'restoreField' => 'select_Publisher',
			);
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initCollections&type=select';
// Amend collections list depending upon which resource types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'select_Type',
			'targetDiv' => 'collection',
			'restoreField' => 'select_Collection',
			);
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initUsertags&type=select';
// Amend usertags list depending upon which resource types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'select_Type',
			'targetDiv' => 'usertag',
			'restoreField' => 'select_UserTag',
			);
		$ajax->jActionForm('onclick', $jsonArray);
		$pString .= $this->table->td($this->makeFormMultiple("type", $this->types));
		$this->grabLanguages();
		if(!empty($this->languages))
			$pString .= $this->table->td($this->html->div('language', $this->makeFormMultiple("language", $this->languages)));
		else
			$pString .= $this->table->td($this->html->div('language', "&nbsp;"));
// There may not yet be tags in the database.
		$this->grabTags();
		if(is_array($this->tags))
			$pString .= $this->table->td($this->makeFormMultiple("tag", $this->tags));
		else
			$pString .= $this->table->td("&nbsp;");
		if($this->session->getVar("mywikindx_Bibliography_use"))
		{
			$this->db->leftJoin('resource_misc', 'usersId', 'resourcemiscAddUserIdResource');
			$this->commonBib->userBibCondition('resourcemiscId');
		}
		$this->users = $this->user->grabAll(TRUE);
		if(is_array($this->users))
			$pString .= $this->table->td($this->makeFormMultiple("addedBy", $this->users));
		else
			$pString .= $this->table->td("&nbsp;");
		if($this->session->getVar("mywikindx_Bibliography_use"))
		{
			$this->db->leftJoin('resource_misc', 'usersId', 'resourcemiscEditUserIdResource');
			$this->commonBib->userBibCondition('resourcemiscId');
		}
		$this->users = $this->user->grabAll(TRUE);
		if(is_array($this->users))
			$pString .= $this->table->td($this->makeFormMultiple("editedBy", $this->users));
		else
			$pString .= $this->table->td("&nbsp;");
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->hr();
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
// There may not yet be subcategories in the database.
		$this->grabSubcategories();
		$tempRB = $this->radioButtons;
		$this->grabCategories();
		if(is_array($this->subcategories))
		{
			$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initSubcategories&type=select';
// Amend subcategory list depending upon which categories are chosen
			$jsonArray = array();
			$jsonArray[] = array(
				'startFunction' => 'triggerFromMultiSelect', 
				'script' => "$jScript",
				'triggerField' => 'select_Category',
				'targetDiv' => 'subcategory'
				);
			$ajax->jActionForm('onclick', $jsonArray);
		}
		$pString .= $this->table->td($this->html->div('category', $this->makeFormMultiple("category", $this->categories)),
			FALSE, FALSE, FALSE, FALSE, '20%');
		$this->radioButtons = $tempRB;
		if($selectedCategories = $this->session->getVar("select_Category"))
		{
			$this->grabSubcategories(explode(',', $selectedCategories));
			if(is_array($this->subcategories))
				$pString .= 
					$this->table->td($this->html->div('subcategory', $this->makeFormMultiple("subcategory", $this->subcategories)),
					FALSE, FALSE, FALSE, FALSE, '20%');
			else
				$pString .= $this->table->td($this->html->div('subcategory', "&nbsp;"),FALSE, FALSE, FALSE, FALSE, '20%');
		}
		else
			$pString .= $this->table->td($this->html->div('subcategory', "&nbsp;"));
// There may not yet be keywords in the database.
		$this->grabKeywords();
		if(is_array($this->keywords))
			$pString .= $this->table->td($this->html->div('keyword', $this->makeFormMultiple("keyword", $this->keywords)),
			FALSE, FALSE, FALSE, FALSE, '20%');
		else
			$pString .= $this->table->td($this->html->div('keyword', "&nbsp;"),FALSE, FALSE, FALSE, FALSE, '20%');
// There may not yet be user tags in the database.
		$this->grabUserTags();
		if(!empty($this->userTags))
			$pString .= $this->table->td($this->html->div('usertag', $this->makeFormMultiple("userTag", $this->userTags)),
			FALSE, FALSE, FALSE, FALSE, '20%');
		else
			$pString .= $this->table->td($this->html->div('usertag', "&nbsp;"),FALSE, FALSE, FALSE, FALSE, '20%');
		$pString .= $this->table->td($this->userBibs(),	FALSE, FALSE, FALSE, FALSE, '20%');
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->hr();
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$this->grabCreators();
		if(is_array($this->creators))
			$pString .= $this->table->td($this->html->div('creator', $this->makeFormMultiple("creator", $this->creators)),
			FALSE, FALSE, FALSE, FALSE, '33.3%');
		else
			$pString .= $this->html->div('creator', $this->table->td("&nbsp;", FALSE, FALSE, FALSE, FALSE, '33.3%'));
		$this->grabPublishers();
		if(is_array($this->publishers))
			$pString .= $this->table->td($this->html->div('publisher', $this->makeFormMultiple("publisher", $this->publishers)),
			FALSE, FALSE, FALSE, FALSE, '33.3%');
		else
			$pString .= $this->html->div('publisher', $this->table->td("&nbsp;", FALSE, FALSE, FALSE, FALSE, '33.3%'));
		$this->grabCollections();
		if(is_array($this->collections))
			$pString .= $this->table->td($this->html->div('collection', $this->makeFormMultiple("collection", $this->collections, 60)),
			FALSE, FALSE, FALSE, FALSE, '33.3%');
		else
			$pString .= $this->table->td($this->html->div('collection', "&nbsp;"), FALSE, FALSE, FALSE, FALSE, '33.3%');
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->hr();
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->common->displayOrder('select'));
		list($selectAttachmentString, $displayAttachmentString, $zipString) = $this->displayAttachments();
		if($this->session->getVar("mywikindx_Bibliography_use"))
			$this->commonBib->userBibCondition('resourcemiscId');
		$this->db->formatConditions(array('resourcemiscPeerReviewed' => 'Y'));
		if(!$this->db->fetchOne($this->db->selectCount('resource_misc', 'resourcemiscPeerReviewed')))
			$peerReviewedString = FALSE;
		else
		{
			$checked = $this->session->getVar('select_DisplayPeerReviewed') ? TRUE : FALSE;
			$peerReviewedString = $this->html->p($this->messages->text('select', 'displayPeerReviewed') .  
				":&nbsp;&nbsp;" . 
				$this->form->checkbox(FALSE, 'select_DisplayPeerReviewed', $checked), FALSE, FALSE, 'bottom');
		}
		
		if(!$selectAttachmentString && !$displayAttachmentString && !$zipString && !$peerReviewedString)
			$pString .= $this->table->td('&nbsp;');
		else
			$pString .= $this->table->td($selectAttachmentString . $displayAttachmentString . $zipString . $peerReviewedString);
		
		$pString .= $this->table->td($this->form->formSubmit("Select"));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->form->formEnd();
		
		// Always do a full reset because of AJAX controlling some select boxes
		$pString .= $this->form->formHeader("list_SELECTRESOURCES_CORE");
		$pString .= $this->html->p($this->form->hidden("method", "reset") . $this->form->formSubmit("Reset"), "right");
		$pString .= $this->form->formEnd();
		
		GLOBALS::buildOutputString($pString);
	}
// Return form elements to display attachments etc.
	private function displayAttachments()
	{
		if($this->session->getVar("setup_ReadOnly") && $this->session->getVar("setup_FileViewLoggedOnOnly"))
			return array(FALSE, FALSE, FALSE);
		if($this->session->getVar("mywikindx_Bibliography_use"))
			$this->commonBib->userBibCondition('resourceattachmentsId');
		if(!$this->db->fetchOne($this->db->selectCount('resource_attachments', 'resourceattachmentsId')))
			return array(FALSE, FALSE, FALSE);
		$attach = array(
				'ignore'	=>	$this->messages->text("misc", "ignore"),
				'no'		=>	$this->messages->text('select', 'noAttachment'),
				'yes'		=>	$this->messages->text('select', 'attachment')
			);
		if($selected = $this->session->getVar("select_Attachment"))
			$selectAttachmentString = $this->html->p($this->form->selectedBoxValue(FALSE, "select_Attachment", 
			$attach, $selected, 3));
		else
			$selectAttachmentString = $this->html->p($this->form->selectFBoxValue(FALSE, "select_Attachment", $attach, 3));
		$checked = $this->session->getVar('select_DisplayAttachment') ? TRUE : FALSE;
		$displayAttachmentString = $this->html->p("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" . 
			$this->messages->text('select', 'displayAttachment') . ":&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, 'select_DisplayAttachment', $checked));
		if(file_exists("/usr/bin/gzip") && file_exists("/bin/tar"))
		{
			$checked = $this->session->getVar('select_DisplayAttachmentZip') ? TRUE : FALSE;
			$zipString = $this->html->p("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" . 
				$this->messages->text('select', 'displayAttachmentZip') . ":&nbsp;&nbsp;" . 
				$this->form->checkbox(FALSE, 'select_DisplayAttachmentZip', $checked) . $zipString);
		}
		else
			$zipString = FALSE;
		return array($selectAttachmentString, $displayAttachmentString, $zipString);
	}
// Return select box of user bibliographies for this user if browsing the master bibliography
	private function userBibs()
	{
		$bibs = $this->commonBib->getUserBibs() + $this->commonBib->getGroupBibs();
		if($userBib = $this->session->getVar("mywikindx_Bibliography_use"))
			unset($bibs[$userBib]);
		if(empty($bibs))
			return "&nbsp;";
// add 0 => IGNORE to $array
		$temp[0] = $this->messages->text("misc", "ignore");
		foreach($bibs as $key => $value)
			$temp[$key] = $value;
		$selected = $this->session->getVar("select_BibId");
		if($selected && array_key_exists($selected, $temp))
			$pString = $this->form->selectedBoxValue($this->messages->text("select", "notInUserBib"),
			"select_BibId", $temp, $selected, 5);
		else
			$pString = $this->form->selectFBoxValue($this->messages->text("select", "notInUserBib"), "select_BibId", $temp, 5);
		return $pString;
	}
// make a self-contained form with a multiple select box
	private function makeFormMultiple($type, $array, $width = FALSE)
	{
// add 0 => IGNORE to $array
		$temp[0] = $this->messages->text("misc", "ignore");
		foreach($array as $key => $value)
			$temp[$key] = $value;
		if($selected = $this->session->getVar("select_" . ucfirst($type)))
		{
			$selectedArray = explode(",", $selected);
			$pString = $this->form->selectedBoxValueMultiple($this->messages->text("select", $type), 
				"select_" . ucfirst($type), $temp, $selectedArray, "5", $width);
			if($type == 'type')
				$this->typeArray = $selectedArray;
		}
		else
		{
			$pString = $this->form->selectFBoxValueMultiple($this->messages->text("select", $type), 
				"select_" . ucfirst($type), $temp, "5", $width);
		}
		$pString .= $this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
			$this->html->br();
		if($this->radioButtons)
			$pString .= $this->radioButtons . $this->html->br();
		return $pString;
	}
// get types from database.
	private function grabTypes()
	{
		$this->types = $this->type->grabAll();
		$this->radioButtons = FALSE;
	}
// get languages from database.
	private function grabLanguages()
	{
		$this->db->orderBy('languageLanguage');
		$this->db->leftJoin('language', 'languageId', 'resourcelanguageLanguageId');
		$resultset = $this->db->select('resource_language', array('resourcelanguageLanguageId', 'languageLanguage'), TRUE);
		while($row = $this->db->fetchRow($resultset))
			$this->languages[$row['resourcelanguageLanguageId']] = $row['languageLanguage'];
		if(empty($this->languages))
			return;
// add 0 => IGNORE to $array
		$temp[0] = $this->messages->text("misc", "ignore");
		foreach($this->languages as $key => $value)
			$temp[$key] = $value;
		$this->makeRadioButtons('Language');
		$this->languages = $temp;
	}
// get tags from database.
	private function grabTags()
	{
		include_once("core/miscellaneous/TAG.php");
		$tag = new TAG();
		$this->tags = $tag->grabAll();
		$this->radioButtons = FALSE;
	}
// get categories from database.
	private function grabCategories()
	{
		$this->categories = $this->category->grabAll($this->session->getVar("mywikindx_Bibliography_use"), TRUE, $this->typeArray);
		$this->makeRadioButtons('Category');
	}
// get subcategories from database.
	private function grabSubcategories($matchCategoryIds = FALSE)
	{
		$this->subcategories = 
			$this->category->grabSubAll(TRUE, $this->session->getVar("mywikindx_Bibliography_use"), $matchCategoryIds, TRUE);
		$this->makeRadioButtons('Subcategory');
	}
// get keywords from database.
	private function grabKeywords()
	{
		$this->keywords = $this->keyword->grabAll($this->session->getVar("mywikindx_Bibliography_use"), 
			'resource', $this->typeArray);
		$this->makeRadioButtons('Keyword');
	}
// get user tags from database.
	private function grabUserTags()
	{
		$userTagsObject = FACTORY_USERTAGS::getInstance();
		$this->userTags = $userTagsObject->grabAll($this->session->getVar("mywikindx_Bibliography_use"));
		$this->makeRadioButtons('UserTag');
	}
// get creators from database.
	private function grabCreators()
	{
		$this->creators = $this->creator->grabAll($this->session->getVar("mywikindx_Bibliography_use"), $this->typeArray);
		$this->makeRadioButtons('Creator');
	}
// get publishers from database.
	private function grabPublishers()
	{
		$this->publishers = $this->publisher->grabAll(FALSE, $this->session->getVar("mywikindx_Bibliography_use"), $this->typeArray);
		$this->radioButtons = FALSE;
	}
// get collections from database.
	private function grabCollections()
	{
		$this->collections = $this->collection->grabAll(FALSE, $this->session->getVar("mywikindx_Bibliography_use"), $this->typeArray);
		$this->radioButtons = FALSE;
	}
// Create radio buttons for AND and OR
	private function makeRadioButtons($type)
	{
		$type = 'select_' . $type . 'Method';
		if($this->session->getVar($type) == 'AND')
		{
			$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR') . " OR", "small") . $this->html->br();
			$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND', TRUE) . " AND", "small");
		}
// Default
		else
		{
			$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR', TRUE) . " OR", "small") . $this->html->br();
			$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND') . " AND", "small");
		}
		$this->radioButtons = $pString;
	}
// Reset the form and clear the session
	public function reset()
	{
		$this->session->clearArray('select');
		$this->init();
	}
/**
* For re-ordering or paging
*/
	public function reprocess()
	{
		$this->input = $this->session->getArray("select");
		if(array_key_exists('select_Order', $this->vars) && $this->vars['select_Order'])
		{
			$this->input['Order'] = $this->vars['select_Order'];
			$this->session->setVar('select_Order', $this->input['Order']);
		}
		if(array_key_exists('select_AscDesc', $this->vars) && $this->vars['select_AscDesc'])
		{
			$this->input['AscDesc'] = $this->vars['select_AscDesc'];
			$this->session->setVar('select_AscDesc', $this->input['AscDesc']);
		}
		$this->process(TRUE);
	}
	public function process($reprocess = FALSE)
	{
		$this->stmt->listMethodAscDesc = 'select_AscDesc';
		if(!array_key_exists('statistics', $this->vars))
			$queryString = 'action=list_SELECTRESOURCES_CORE&method=reprocess';
		else
			$queryString = 'action=list_SELECTRESOURCES_CORE' . 
				'&method=reprocess&statistics=' . $this->vars['statistics'] . 
				'&id=' . $this->vars['id'];
		if(!array_key_exists('statistics', $this->vars))
		{
			if(!$reprocess)
				$this->input = $this->checkInput();
			else
				$this->input = $this->session->getArray("select");
			if(array_key_exists('Creator', $this->input))
				$this->creatorGroupMembers();
		}
		else
		{
			$this->session->clearArray("select");
			$this->input['statistics'] = $this->vars['statistics'];
			$this->session->setVar('statistics_type', $this->vars['statistics']);
			$this->session->setVar('statistics_id', $this->vars['id']);
			if(!$reprocess || !array_key_exists('select_Order', $this->vars))
				$this->input['Order'] = 'creator';
			else if(array_key_exists('select_Order', $this->vars) && $this->vars['select_Order'])
			{
				$this->input['Order'] = $this->vars['select_Order'];
				$this->session->setVar('select_Order', $this->input['Order']);
			}
			if(array_key_exists('select_AscDesc', $this->vars) && $this->vars['select_AscDesc'])
			{
				$this->input['AscDesc'] = $this->vars['select_AscDesc'];
				$this->session->setVar('select_AscDesc', $this->input['AscDesc']);
			}
		}
		if($this->session->getVar('select_DisplayAttachment'))
		{
			$this->joinResourceId = 'resourceattachmentsResourceId';
			$this->stmt->execJoin['resource_misc']['intField'] = 'resourcemiscId';
			$this->stmt->execJoin['resource_misc']['extField'] = $this->joinResourceId;
			$order = 'attachments';
		}
		else
		{
			$order = $this->input['Order'];
			$this->joinResourceId = 'resourcemiscId';
		}
		$conditions = array();
		$listType = 'select';
		if(array_key_exists('BibId', $this->input)) // must do first as we need to execute the condition
		{
			$this->db->formatConditions(array('userbibliographyresourceBibliographyId' => $this->input['BibId']));
			$bibIdSubQ = $this->db->subQuery($this->db->selectNoExecute('user_bibliography_resource', 'userbibliographyresourceResourceId'), 
				FALSE, FALSE, TRUE);
		}
		foreach($this->input as $type => $value)
		{
			$conditionArray = array();
			if($type == 'Type')
			{
				$selectArray = explode(",", $value);
				$this->stmt->execCondOneField['resourceType'] = $selectArray;
				$this->stmt->execJoin['resource']['intField'] = 'resourceId';
				$this->stmt->execJoin['resource']['extField'] = $this->joinResourceId;
			}
			else if($type == 'Language')
			{
				$selectArray = explode(",", $value);
				if($this->input['LanguageMethod'] == 'OR')
				{
					$this->stmt->execCondOneField[$this->tAlias . '.resourcelanguageLanguageId'] = $selectArray;
					$this->stmt->execJoin[$this->tAlias . '.resource_language']['intField'] = $this->tAlias . '.resourcelanguageResourceId';
					$this->stmt->execJoin[$this->tAlias . '.resource_language']['extField'] = $this->joinResourceId;
					++$this->tAlias;
				}
				else
				{
					foreach($selectArray as $language)
					{
						$this->stmt->execCond[] = array($this->tAlias . '.resourcelanguageLanguageId' => $language);
						$this->stmt->execJoin[$this->tAlias . '.resource_language']['intField'] = $this->tAlias . '.resourcelanguageResourceId';
						$this->stmt->execJoin[$this->tAlias . '.resource_language']['extField'] = $this->joinResourceId;
						++$this->tAlias;
					}
				}
			}
			else if($type == 'Category')
			{
				$selectArray = explode(",", $this->input['Category']);
				if($this->input['CategoryMethod'] == 'OR')
				{
					$this->stmt->execCondOneField[$this->tAlias . '.resourcecategoryCategoryId'] = $selectArray;
					$this->stmt->execJoin[$this->tAlias . '.resource_category']['intField'] = $this->tAlias . '.resourcecategoryResourceId';
					$this->stmt->execJoin[$this->tAlias . '.resource_category']['extField'] = $this->joinResourceId;
					++$this->tAlias;
				}
				else
				{
					foreach($selectArray as $category)
					{
						$this->stmt->execCond[] = array($this->tAlias . '.resourcecategoryCategoryId' => $category);
						$this->stmt->execJoin[$this->tAlias . '.resource_category']['intField'] = $this->tAlias . '.resourcecategoryResourceId';
						$this->stmt->execJoin[$this->tAlias . '.resource_category']['extField'] = $this->joinResourceId;
						++$this->tAlias;
					}
				}
			}
			else if($type == 'Subcategory')
			{
				$selectArray = explode(",", $this->input['Subcategory']);
				if($this->input['SubcategoryMethod'] == 'OR')
				{
					$this->stmt->execCondOneField[$this->tAlias . '.resourcecategorySubcategoryId'] = $selectArray;
					$this->stmt->execJoin[$this->tAlias . '.resource_category']['intField'] = $this->tAlias . '.resourcecategoryResourceId';
					$this->stmt->execJoin[$this->tAlias . '.resource_category']['extField'] = $this->joinResourceId;
					++$this->tAlias;
				}
				else
				{
					foreach($selectArray as $subategory)
					{
						$this->stmt->execCond[] = array($this->tAlias . '.resourcecategorySubcategoryId' => $subategory);
						$this->stmt->execJoin[$this->tAlias . '.resource_category']['intField'] = $this->tAlias . '.resourcecategoryResourceId';
						$this->stmt->execJoin[$this->tAlias . '.resource_category']['extField'] = $this->joinResourceId;
						++$this->tAlias;
					}
				}
			}
			else if($type == 'Keyword')
			{
				$selectArray = explode(",", $this->input['Keyword']);
				if($this->input['KeywordMethod'] == 'OR')
				{
					$this->stmt->execCondOneField['resourcekeywordKeywordId'] = $selectArray;
					$this->stmt->execJoin['resource_keyword']['intField'] = 'resourcekeywordResourceId';
					$this->stmt->execJoin['resource_keyword']['extField'] = $this->joinResourceId;
				}
				else
				{
					foreach($selectArray as $keyword)
					{
						$this->stmt->execCond[] = array($this->tAlias . '.resourcekeywordKeywordId' => $keyword);
						$this->stmt->execJoin[$this->tAlias . '.resource_keyword']['intField'] = $this->tAlias . '.resourcekeywordResourceId';
						$this->stmt->execJoin[$this->tAlias . '.resource_keyword']['extField'] = $this->joinResourceId;
						++$this->tAlias;
					}
				}
			}
			else if($type == 'UserTag')
			{
				$selectArray = explode(",", $this->input['UserTag']);
				if($this->input['UserTagMethod'] == 'OR')
				{
					$this->stmt->execCondOneField['resourceusertagsTagId'] = $selectArray;
					$this->stmt->execJoin['resource_user_tags']['intField'] = 'resourceusertagsResourceId';
					$this->stmt->execJoin['resource_user_tags']['extField'] = $this->joinResourceId;
				}
				else
				{
					foreach($selectArray as $usertag)
					{
						$this->stmt->execCond[] = array($this->tAlias . '.resourceusertagsTagId' => $usertag);
						$this->stmt->execJoin[$this->tAlias . '.resource_user_tags']['intField'] = $this->tAlias . 			
							'.resourceusertagsResourceId';
						$this->stmt->execJoin[$this->tAlias . '.resource_user_tags']['extField'] = $this->joinResourceId;
						++$this->tAlias;
					}
				}
			}
			else if($type == 'Tag')
			{
				$selectArray = explode(",", $this->input['Tag']);
				$this->stmt->execCondOneField['resourcemiscTag'] = $selectArray;
			}
			else if($type == 'Creator')
			{
				$selectArray = explode(",", $this->input['Creator']);
				if($this->input['CreatorMethod'] == 'OR')
				{
					foreach($selectArray as $creator)
					{
						$c[] = $this->db->formatFields('resourcecreatorCreatorId') . ' ' . 
							$this->db->equal . ' ' . $this->db->tidyInput($creator);
						$c[] = $this->db->formatFields('creatorSameAs') . ' ' . 
							$this->db->equal . ' ' . $this->db->tidyInput($creator);
					}
					$c = join(' ' . $this->db->or . ' ' , $c);
					$this->stmt->execCond[] = $c;
					$this->stmt->execJoin['c.resource_creator']['intField'] = 'c.resourcecreatorResourceId';
					$this->stmt->execJoin['c.resource_creator']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['creator']['intField'] = 'creatorId';
					$this->stmt->execJoin['creator']['extField'] = 'resourcecreatorCreatorId';
				}
				else
				{
					foreach($selectArray as $creator)
					{
						$this->stmt->execCond[] = $this->db->formatFields($this->tAlias . '.resourcecreatorCreatorId') . ' ' . 
							$this->db->equal . ' ' . $this->db->tidyInput($creator) . 
							' ' . $this->db->or . ' ' . 
							$this->db->formatFields('creatorSameAs') . ' ' . 
							$this->db->equal . ' ' . $this->db->tidyInput($creator);
						$this->stmt->execJoin[$this->tAlias . '.resource_creator']['intField'] = $this->tAlias . 			
							'.resourcecreatorResourceId';
						$this->stmt->execJoin[$this->tAlias . '.resource_creator']['extField'] = $this->joinResourceId;
						$this->stmt->execJoin['creator']['intField'] = 'creatorId';
						$this->stmt->execJoin['creator']['extField'] = $this->tAlias . '.resourcecreatorCreatorId';
						++$this->tAlias;
					}
				}
			}
			else if($type == 'Publisher')
			{
				$selectArray = explode(",", $this->input['Publisher']);
				foreach($selectArray as $key => $in)
					$conditionArray[] = $this->db->ifClause(
						$this->db->formatFields('resourceType') . '=' . $this->db->tidyInput('proceedings_article') . 
						$this->db->or . $this->db->formatFields('resourceType') . '=' . $this->db->tidyInput('proceedings') . 
						$this->db->or . $this->db->formatFields('resourceType') . '=' . $this->db->tidyInput('book') . 
						$this->db->or . $this->db->formatFields('resourceType') . '=' . $this->db->tidyInput('book_article'),
						FALSE, // $test is in previous field
						$this->db->formatFields('resourcemiscField1') . '=' . $this->db->tidyInput($in) . 
						$this->db->or . $this->db->formatFields('resourcemiscPublisher') . '=' . $this->db->tidyInput($in),
						$this->db->formatFields('resourcemiscPublisher') . '=' . $this->db->tidyInput($in)
						);
				$this->stmt->execCond[] = join($this->db->or, $conditionArray);
				$this->stmt->execJoin['resource']['intField'] = 'resourceId';
				$this->stmt->execJoin['resource']['extField'] = $this->joinResourceId;
			}
			else if($type == 'Collection')
			{
				$selectArray = explode(",", $this->input['Collection']);
				$this->stmt->execCondOneField['resourcemiscCollection'] = $selectArray;
			}
			else if($type == 'BibId')
			{
// force no condition in SQLSTATEMENTS for bibliography being used
				$this->commonBib->bailOut = TRUE;
				$this->stmt->execCond[] = $this->db->formatFields('resourceId') . $this->db->inClause($bibIdSubQ, TRUE);
			}
			else if($type == 'Attachment')
			{
				$attach = $this->session->getVar('select_Attachment');
				if($attach == 'no')
					$this->stmt->execCond[] = array('resourceattachmentsId' => ' IS NULL');
				else if($attach == 'yes')
					$this->stmt->execCond[] = array('resourceattachmentsId' => ' IS NOT NULL');
				if(($attach != 'ignore') && ($order != 'attachments'))
				{
					$this->stmt->execJoin['resource_attachments']['intField'] = 'resourceattachmentsResourceId';
					$this->stmt->execJoin['resource_attachments']['extField'] = $this->joinResourceId;
				}
			}
			else if($type == 'AddedBy')
			{
				$selectArray = explode(',', $value);
				$this->stmt->execCondOneField['resourcemiscAddUserIdResource'] = $selectArray;
			}
			else if($type == 'EditedBy')
			{
				$selectArray = explode(',', $value);
				$this->stmt->execCondOneField['resourcemiscEditUserIdResource'] = $selectArray;
			}
			else if($type == 'DisplayPeerReviewed')
				$this->stmt->execCond[] = array('resourcemiscPeerReviewed' => 'Y');
			else if($type == 'statistics')
			{
				$listType = 'statistics';
				if($value == 'resourceUser')
					$this->stmt->execCond[] = array('resourcemiscaddUserIdResource' => $this->vars['id']);
				else if($value == 'quoteUser')
				{
					$this->stmt->execCond[] = array('resourcequotetextAddUserIdQuote' => $this->vars['id']);
					$this->stmt->execJoin['resource_quote']['intField'] = 'resourcequoteResourceId';
					$this->stmt->execJoin['resource_quote']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_quote_text']['intField'] = 'resourcequotetextId';
					$this->stmt->execJoin['resource_quote_text']['extField'] = 'resourcequoteId';
				}
				else if($value == 'paraphraseUser')
				{
					$this->stmt->execCond[] = array('resourceparaphrasetextAddUserIdParaphrase' => $this->vars['id']);
					$this->stmt->execJoin['resource_paraphrase']['intField'] = 'resourceparaphraseResourceId';
					$this->stmt->execJoin['resource_paraphrase']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_paraphrase_text']['intField'] = 'resourceparaphrasetextId';
					$this->stmt->execJoin['resource_paraphrase_text']['extField'] = 'resourceparaphraseId';
				}
				else if($value == 'musingUser')
				{
					$this->stmt->execCond[] = array('resourcemusingtextAddUserIdMusing' => $this->vars['id']);
					$this->stmt->execJoin['resource_musing']['intField'] = 'resourcemusingResourceId';
					$this->stmt->execJoin['resource_musing']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_musing_text']['intField'] = 'resourcemusingtextId';
					$this->stmt->execJoin['resource_musing_text']['extField'] = 'resourcemusingId';
				}
			}
		}
		$sqlTotal = $this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, $order, TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$sql = base64_decode($this->session->getVar('sql_Stmt'));
			$this->common->display($sql, 'select');
			$this->session->saveState('select');
			$this->session->saveState('sql');
			$this->session->saveState('bookmark');
			return;
		}
		if($this->session->getVar('select_DisplayAttachment'))
			$sql = $sqlTotal;
		else
			$sql = $this->stmt->listList(FALSE, FALSE, $order);
		$this->common->display($sql, 'select');
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
// Turn on the 'add bookmark' menu item
		$this->session->setVar("bookmark_DisplayAdd", TRUE);
		$this->session->saveState('select');
		$this->session->saveState('sql');
		$this->session->saveState('bookmark');
	}
/**
* Replace creators who are members of a group with the group master -- not written to session but used only to process the select
*/
	private function creatorGroupMembers()
	{
		$tempArray = $array = explode(",", $this->input['Creator']);
		foreach($array as $id)
		{
			$this->db->formatConditions(array('creatorId' => $id));
			$sameAsId = $this->db->fetchOne($this->db->select('creator', 'creatorSameAs'));
			if($sameAsId && (array_search($sameAsId, $tempArray) === FALSE))
			{
				unset($tempArray[array_search($id, $tempArray)]);
				$tempArray[] = $sameAsId;
			}
		}
		$this->input['Creator'] = join(',', $tempArray);
	}
// validate user input
// Input comes either from form input or, when paging, from the session.
	private function checkInput()
	{
		$typeInputFound = $inputFound = FALSE;
		$typesArray = array("Type", "Tag", "AddedBy", "EditedBy", "Category", "Subcategory", "Keyword", 
			"UserTag", "Creator", "Publisher", "Collection", "Language");
		$temp = array();
		if((array_search('LastMulti', $this->vars) !== FALSE) || array_key_exists('PagingStart', $this->vars))
		{
			foreach($this->session->getArray("select") as $key => $value)
			{
				if(array_search($key, $typesArray) !== FALSE)
					$this->vars['select_' . $key] = explode(',', $value);
				else
					$this->vars['select_' . $key] = $value;
			}
		}
		if(array_key_exists('select_DisplayAttachment', $this->vars))
		{
			if(array_key_exists('select_Attachment', $this->vars) && 
				(($this->vars['select_Attachment'] == 'ignore') || ($this->vars['select_Attachment'] == 'no')))
			{
				unset($this->vars['select_DisplayAttachment']);
				unset($this->vars['select_DisplayAttachmentZip']);
			}
		}
// Delete previous stored data
		$this->session->clearArray("select");
		foreach($typesArray as $type)
		{
			$inputKey = 'select_' . $type;
			$method = "select_" . $type . "Method";
			if(!array_key_exists($inputKey, $this->vars))
				continue;
			if((sizeof($this->vars[$inputKey]) == 1) && !$this->vars[$inputKey][0]) // IGNORE
			{
				if($type == 'Type')
					$inputFound = TRUE;
				continue;
			}
			$inputFound = TRUE;
			if(array_key_exists(0, $this->vars[$inputKey]) && !$this->vars[$inputKey][0])
				unset($this->vars[$inputKey][0]);
			$temp[$type] = implode(",", $this->vars[$inputKey]);
// If missing the following, set to default 'OR'.
			if(!array_key_exists($method, $this->vars))
				$this->vars[$method] = 'OR';
			$this->inputMethod[$type] = $this->vars[$method];
			$this->session->setVar($method, $this->vars[$method]);
		}
		if(array_key_exists('select_BibId', $this->vars) && $this->vars['select_BibId'])
		{
			$temp['BibId'] = $this->vars['select_BibId'];
			$inputFound = TRUE;
		}
		if(array_key_exists('select_Attachment', $this->vars))
		{
			$temp['Attachment'] = $this->vars['select_Attachment'];
			$inputFound = TRUE;
		}
		if(array_key_exists('select_DisplayAttachment', $this->vars) && 
			$this->vars['select_DisplayAttachment'])
		{
			$temp['DisplayAttachment'] = TRUE;
			$inputFound = TRUE;
		}
		if(array_key_exists('select_DisplayAttachmentZip', $this->vars) && 
			$this->vars['DisplayAttachmentZip'])
		{
			$temp['DisplayAttachmentZip'] = TRUE;
			$inputFound = TRUE;
		}
		if(array_key_exists('select_DisplayPeerReviewed', $this->vars) && 
			$this->vars['select_DisplayPeerReviewed'])
		{
			$temp['DisplayPeerReviewed'] = $this->vars['select_DisplayPeerReviewed'];
			$inputFound = TRUE;
		}
		if(array_key_exists('select_Order', $this->vars) && $this->vars['select_Order'])
			$temp['Order'] = $this->vars['select_Order'];
		if(array_key_exists('select_AscDesc', $this->vars) && $this->vars['select_AscDesc'])
			$temp['AscDesc'] = $this->vars['select_AscDesc'];
		if(!$inputFound)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$this->session->writeArray($temp, 'select');
		return $this->session->getArray("select");
	}
}
?>
