<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/

/**
* Display Random Metadata.
*/
class RANDOMMETADATA
{
private $db;
private $config;
private $icons;
private $badInput;
private $errors;
private $messages;
private $session;
private $common;
private $coins;
private $html;
private $template;
private $bibStyle;
private $user;
private $stats;
private $resourceLink = FALSE;
private $cite;
private $userId;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->icons = FACTORY_LOADICONS::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->common = FACTORY_RESOURCECOMMON::getInstance();
		$this->coins = FACTORY_EXPORTCOINS::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->user = FACTORY_USER::getInstance();
		$this->bibStyle = FACTORY_BIBSTYLE::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->stats = FACTORY_STATISTICS::getInstance();
		$this->cite = FACTORY_CITE::getInstance();
		$this->userId = $this->session->getVar("setup_UserId");
		GLOBALS::setOutputHeading($this->messages->text("heading", "randomMetadata"));
	}
// Select a random quote ID for viewing
	public function randomQuote()
	{
		$this->db->limit(1, 0);
		$this->db->orderByRandom();
		$resultset = $this->db->select('resource_quote', array('resourcequoteId', 'resourcequoteResourceId'));
		if(!$this->db->numRows($resultset))
			$this->badInput->close($this->messages->text("misc", "noQuotes"));
		$row = $this->db->fetchRow($resultset);
		$resourceId = $row['resourcequoteResourceId'];
		$this->db->formatConditions(array('resourcequotetextId' => $row['resourcequoteId']));
		$resultset = $this->db->select('resource_quote_text', 'resourcequotetextText');
		$row = $this->db->fetchRow($resultset);
		$mArray = $this->cite->parseCitations($this->html->dbToHtmlTidy($row['resourcequotetextText']), 'html');
		$resultset = $this->common->getResource($resourceId);
		$this->display($resultset, $mArray, 'randomQuote');
	}
// Select a random paraphrase ID for viewing
	public function randomParaphrase()
	{
		$this->db->limit(1, 0);
		$this->db->orderByRandom();
		$resultset = $this->db->select('resource_paraphrase', array('resourceparaphraseId', 'resourceparaphraseResourceId'));
		if(!$this->db->numRows($resultset))
			$this->badInput->close($this->messages->text("misc", "noParaphrases"));
		$row = $this->db->fetchRow($resultset);
		$resourceId = $row['resourceparaphraseResourceId'];
		$this->db->formatConditions(array('resourceparaphrasetextId' => $row['resourceparaphraseId']));
		$resultset = $this->db->select('resource_paraphrase_text', 'resourceparaphrasetextText');
		$row = $this->db->fetchRow($resultset);
		$mArray = $this->cite->parseCitations($this->html->dbToHtmlTidy($row['resourceparaphrasetextText']), 'html');
		$resultset = $this->common->getResource($resourceId);
		$this->display($resultset, $mArray, 'randomParaphrase');
	}
// Select a random musing ID for viewing
	public function randomMusing()
	{
		if($userId = $this->session->getVar('setup_UserId'))
		{
			$this->db->formatConditions(array('usergroupsusersUserId' => $userId));
			$this->db->formatConditions($this->db->formatFields('usergroupsusersGroupId') . $this->db->equal . 
				$this->db->formatFields('resourcemusingtextPrivate'));
			$subSql = $this->db->selectNoExecute('user_groups_users', 'usergroupsusersId', FALSE, TRUE, TRUE);
			$subject = $this->db->formatFields('resourcemusingtextPrivate') . $this->db->notEqual . $this->db->tidyInput('N') 
				. $this->db->and . 
				$this->db->formatFields('resourcemusingtextPrivate') . $this->db->notEqual . $this->db->tidyInput('Y');
			$case1 = $this->db->caseWhen($subject, FALSE, $subSql, FALSE, FALSE);
			$subject = $this->db->formatFields('resourcemusingtextPrivate') . $this->db->equal . $this->db->tidyInput('Y');
			$result = $this->db->formatFields('resourcemusingtextAddUserIdMusing') . $this->db->equal . $this->db->tidyInput($userId);
			$case2 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
			$subject = $this->db->formatFields('resourcemusingtextPrivate') . $this->db->equal . $this->db->tidyInput('N');
			$result = $this->db->tidyInput(1);
			$case3 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
			$this->db->formatConditions($case1 . $this->db->or . $case2 . $this->db->or . $case3);
		}
		else if($this->session->getVar('setup_ReadOnly'))
			$this->db->formatConditions(array('resourcemusingtextPrivate' => 'N'));
		$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
		$this->db->limit(1, 0);
		$this->db->orderByRandom();
		$resultset = $this->db->select('resource_musing', array('resourcemusingId', 'resourcemusingResourceId'));
		if(!$this->db->numRows($resultset))
			$this->badInput->close($this->messages->text("misc", "noMusings"));
		$row = $this->db->fetchRow($resultset);
		$resourceId = $row['resourcemusingResourceId'];
		$this->db->formatConditions(array('resourcemusingtextId' => $row['resourcemusingId']));
		$resultset = $this->db->select('resource_musing_text', 'resourcemusingtextText');
		$row = $this->db->fetchRow($resultset);
		$mArray = $this->cite->parseCitations($this->html->dbToHtmlTidy($row['resourcemusingtextText']), 'html');
		$resultset = $this->common->getResource($resourceId);
		$this->display($resultset, $mArray, 'randomMusing');
	}
/**
* Produce a list of resources
*/
	public function display($resultset, $mArray, $method)
	{
		$row = $this->db->fetchRow($resultset);
		if($this->session->getVar('setup_Listlink') == 'Y')
		{
			$this->resourceLink = "index.php?action=resource_RESOURCEVIEW_CORE" . htmlentities("&id=" . $row['resourceId']);
			$this->bibStyle->linkUrl = FALSE;
		}
		$this->icons->setupIcons();
		if(($row['resourcemiscQuarantine'] == 'Y') && ($this->session->getVar('setup_Quarantine') == 'Y'))
			$resourceList[0]['quarantine'] = $this->icons->quarantine;
		$multiUser = $this->session->getVar('setup_MultiUser');
		if($multiUser == 'Y')
		{
			$resourceList[0]['user'] = $this->user->displayUserAddEdit($row);
			$popularityIndex =$this->messages->text("misc", "popIndex", $this->stats->getPopularityIndex($row['resourceId']));
			$maturityIndex = $row['resourcemiscMaturityIndex'] ? 
				"&nbsp;" . $this->messages->text("misc", "matIndex") . "&nbsp;" . 
				$row['resourcemiscMaturityIndex'] . "/10" . $this->html->br()
				:
				FALSE;
			$resourceList[0]['popIndex'] = $popularityIndex;
			$resourceList[0]['maturity'] = $maturityIndex;
			$this->template->tpl->assign('multiUser', TRUE);
		}
		$resourceList[0]['timestamp'] = $row['resourcetimestampTimestamp'];
		$resourceList[0]['links'] = $this->createLinks($row);
		if($this->resourceLink)
			$resourceList[0]['resource'] = $this->html->a('rLink', $this->bibStyle->process($row), $this->resourceLink) . 
				$this->coins->export($row, $this->bibStyle->coinsCreators);
		else
			$resourceList[0]['resource'] = $this->bibStyle->process($row) . $this->coins->export($row, $this->bibStyle->coinsCreators);
		$resourceList[0]['metadata'][] = $mArray;
		$resourceList[0]['navigation'] = $this->nextRandomLink($row['resourceId'], $method);
		$this->template->tpl->assign('resourceList', $resourceList);
		return TRUE;
	}
/**
* Create links for viewing, editing deleting etc. resources
*/
	private function createLinks($row)
	{
		$write = $this->session->getVar('setup_Write');
		$links = array();
		$edit = FALSE;
		$view = $this->icons->viewmeta;
		$links['view'] = $this->html->a($this->icons->viewLink, $view, "index.php?action=resource_RESOURCEVIEW_CORE" . 
			htmlentities("&id=" . $row['resourceId']));
		if($write && (!$this->config->WIKINDX_ORIGINATOR_EDITONLY || ($row['resourcemiscAddUserIdResource'] == $this->userId)))
		{
			$links['edit'] = $this->html->a($this->icons->editLink, $this->icons->edit, 
				"index.php?action=resource_RESOURCEFORM_CORE&type=edit" . htmlentities("&id=" . $row['resourceId']));
			$edit = TRUE;
		}
		if($this->session->getVar('setup_Superadmin'))
		{
			if(!$edit)
				$links['edit'] = $this->html->a($this->icons->editLink, $this->icons->edit, 
				"index.php?action=resource_RESOURCEFORM_CORE&type=edit" . htmlentities("&id=" . $row['resourceId']));
			$links['delete'] = 
				"index.php?action=admin_DELETERESOURCE_CORE" . htmlentities('&function=deleteResourceConfirm');
			$links['delete'] .= htmlentities('&navigate=front&resource_id=' . $row['resourceId']);
			$links['delete'] = $this->html->a($this->icons->deleteLink, $this->icons->delete, $links['delete']);
		}
// display CMS link if required
// link is actually a JavaScript call
		if($this->session->getVar('setup_DisplayCmsLink'))
			$links['cms'] = $this->html->a('cmsLink', "CMS:&nbsp;" . $row['resourceId'], 
				"javascript:openCmsWindow('index.php?action=cmsDisplay" . "&amp;id=" . $row['resourceId'] . "')");
// display bibtex link if required
// link is actually a JavaScript call
		if($this->session->getVar('setup_DisplayBibtexLink'))
			$links['bibtex'] = $this->html->a($this->icons->bibtexLink, $this->icons->bibtex, 
				"javascript:openBibtexWindow('index.php?action=bibtexDisplay" . "&amp;id=" . $row['resourceId'] . "')");
		return $links;
	}
// Show random resource hyperlink
	private function nextRandomLink($thisId, $method)
	{
		$this->nextDelete = FALSE;
		if(($raw = $this->session->getVar("list_AllIds")) === FALSE)
			return FALSE;
		$allIds = unserialize(base64_decode($raw));
		$thisKey = array_search($thisId, $allIds);
		$array['forward'] = $this->html->a($this->icons->nextLink, $this->icons->next, 
			htmlentities("index.php?action=metadata_RANDOMMETADATA_CORE&method=$method"));
		if($this->session->getVar('setup_Superadmin'))
		{
			if(array_key_exists($thisKey + 1, $allIds))
				$this->nextDelete = $allIds[$thisKey + 1];
			else if(array_key_exists($thisKey - 1, $allIds))
				$this->nextDelete = $allIds[$thisKey - 1];
		}
		return $array;
	}
}
?>