<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
*	NEWS class.
*
*	News items
*****/
class NEWS
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $form;
private $html;
private $template;
private $gatekeep;
private $badInput;
private $newsTimstamp;

// Constructor
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
	}
// Add a news item - display options.
	public function initAdd($error = FALSE)
	{
		$this->gatekeep->requireSuper = TRUE;
		$this->gatekeep->init();
		$tinymce = FACTORY_LOADTINYMCE::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "newsAdd"));
		$pString = $tinymce->loadMinimalTextarea();
		$pString .= $error ? $this->html->p($error, "error", "center") : FALSE;
		$pString .= $this->form->formHeader('news_NEWS_CORE');
		$pString .= $this->form->hidden('method', 'add');
		$sessVar = $this->session->issetVar("news_Title") ? 
			$this->html->dbToHtmlTidy($this->session->getVar("news_Title")) : FALSE;
		$pString .= $this->form->textInput($this->messages->text("news", "title"), "title", $sessVar, 30, 255);
		$pString .= $this->html->br() . "&nbsp;" . $this->html->br();
		$sessVar = $this->session->issetVar("news_Body") ? 
			$this->html->dbToHtmlTidy($this->session->getVar("news_Body")) : FALSE;
		$pString .= $this->form->textAreaInput($this->messages->text("news", "body"), "body", $sessVar, 80, 10);
		$pString .= $this->html->br() . "&nbsp;" . $this->html->br();
		$pString .= $this->form->formSubmit('Add');
		$pString .= $this->form->formEnd();
		$this->session->delVar('news_Done');
		GLOBALS::buildOutputString($pString);
	}
// Add a news item
	public function add()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "newsAdd"));
		if($this->session->getVar('news_Done'))
			$this->badInput->close($this->errors->text("done", "news"));
		$this->gatekeep->requireSuper = TRUE;
		$this->gatekeep->init();
		if(!array_key_exists('title', $this->vars) || !trim($this->vars['title']) || 
			!array_key_exists('body', $this->vars) || !trim($this->vars['body']))
		{
			$this->session->setVar("news_Title", trim($this->vars['title']));
			$this->session->setVar("news_Body", trim($this->vars['body']));
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, "initAdd");
		}
		$title = trim($this->vars['title']);
		$news = trim($this->vars['body']);
		$this->db->insert("news", array('newsTitle', 'newsNews', 'newsTimestamp'), 
			array($title, $news, $this->db->formatTimestamp()));
		$this->session->delVar('news_Title');
		$this->session->delVar('news_Body');
		$this->session->setVar('setup_News', TRUE);
		$this->session->setVar('news_Done', TRUE);
		if($this->db->fetchOne($this->db->select('config', 'configEmailNews')) == 'Y')
		{
			include_once("core/modules/email/EMAIL.php");
			$emailClass = new EMAIL();
			if(!$emailClass->news($title, $news))
				$this->badInput->close($this->errors->text("inputError", "mail"));
		}
		GLOBALS::buildOutputString($this->success->text("news", " " . $this->messages->text("misc", "added") . " "));
	}
// Delete news display.
	public function initDelete($error = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "newsDelete"));
		$pString = $error ? $this->html->p($error, "error", "center") : FALSE;
		$news = $this->grabAll();
		if(empty($news))
			$pString .= $this->messages->text("news", "noNews");
		else
		{
			$pString .= $this->form->formHeader('news_NEWS_CORE');
			$pString .= $this->form->hidden('method', 'deleteConfirm');
			foreach($news as $key => $value)
				$pString .= $this->form->checkbox(FALSE, "newsDelete_" . $key) . " $value" . $this->html->br();
			$pString .= $this->html->br() . $this->form->formSubmit('Delete');
			$pString .= $this->form->formEnd();
		}
		$this->session->delVar('news_Done');
		GLOBALS::buildOutputString($pString);
	}
// Ask for confirmation of delete groups
	public function deleteConfirm()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "newsDelete"));
		foreach($this->vars as $key => $value)
		{
			if(!preg_match("/newsDelete_(.*)/", $key, $match))
				continue;
			$input[] = $match[1];
		}
		if(!isset($input))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, "initDelete");
		$news = $this->grabAll();
		$newsText = "'" . implode("', '", array_keys(array_intersect(array_flip($news), $input))) . "'";
		$news = html_entity_decode($newsText);
		$pString = $this->html->p($this->messages->text("news", "deleteConfirm", ": $news"));
		$pString .= $this->form->formHeader('news_NEWS_CORE');
		$pString .= $this->form->hidden('method', 'delete');
		foreach($input as $id)
			$pString .= $this->form->hidden("newsDelete_" . $id, $id);
		$pString .= $this->html->br() . $this->form->formSubmit('Confirm');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Delete group(s)
	public function delete()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "newsDelete"));
		if($this->session->getVar('news_Done'))
			$this->badInput->close($this->errors->text("done", "news"));
		foreach($this->vars as $key => $value)
		{
			if(!preg_match("/newsDelete_(.*)/", $key, $match))
				continue;
			$input[] = $match[1];
		}
		if(!isset($input))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, "initDelete");
		$this->db->formatConditionsOneField($input, 'newsId');
		$this->db->delete('news');
		$this->db->optimize('news');
		$news = $this->grabAll();
		if(empty($news))
			$this->session->delVar('setup_News');
		$this->session->setVar('news_Done', TRUE);
		GLOBALS::buildOutputString($this->success->text("news", " " . $this->messages->text("misc", "deleted") . " "));
	}
// display news items for editing
	public function initEdit($error = FALSE)
	{
		$this->gatekeep->requireSuper = TRUE;
		$this->gatekeep->init();
		GLOBALS::setOutputHeading($this->messages->text("heading", "newsEdit"));
		$news = $this->grabAll();
		$pString = $error ? $this->html->p($error, "error", "center") : FALSE;
		if(empty($news))
			$pString .= $this->messages->text("news", "noNews");
		else
		{
			$pString .= $this->form->formHeader('news_NEWS_CORE');
			$pString .= $this->form->hidden('method', 'editDisplay');
			$pString .= $this->form->selectFBoxValue(FALSE, "editId", $news, 20) . $this->html->br() . 
				$this->form->formSubmit('Edit');
			$pString .= $this->form->formEnd();
		}
		$this->session->delVar('news_Done');
		GLOBALS::buildOutputString($pString);
	}
// display news item for editing
	public function editDisplay($error = FALSE)
	{
		$this->gatekeep->requireSuper = TRUE;
		$this->gatekeep->init();
		GLOBALS::setOutputHeading($this->messages->text("heading", "newsEdit"));
		if(array_key_exists("editId", $this->vars))
			$editId = trim($this->vars["editId"]);
		else
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, "initEdit");
		$tinymce = FACTORY_LOADTINYMCE::getInstance();
		$pString = $tinymce->loadMinimalTextarea();
		$pString .= $error ? $this->html->p($error, "error", "center") : FALSE;
		$this->db->formatConditions(array('newsId' => $editId));
		$recordset = $this->db->select('news', array('newsId', 'newsTitle', 'newsNews'));
		$row = $this->db->fetchRow($recordset);
		$pString .= $this->form->formHeader('news_NEWS_CORE');
		$pString .= $this->form->hidden('method', 'edit');
		$pString .= $this->form->hidden('editId', $editId);
		$pString .= $this->form->textInput($this->messages->text("news", "title"), "title", 
			$this->html->dbToFormTidy($row['newsTitle']), 30, 255);
		$pString .= $this->html->br() . "&nbsp;" . $this->html->br();
		$pString .= $this->form->textAreaInput($this->messages->text("news", "body"), "body", 
			$this->html->dbToFormTidy($row['newsNews']), 80, 10);
		$pString .= $this->html->br() . "&nbsp;" . $this->html->br();
		$pString .= $this->form->formSubmit('Edit');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Edit news item
	public function edit()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "newsEdit"));
		if($this->session->getVar('news_Done'))
			$this->badInput->close($this->errors->text("done", "news"));
		$this->gatekeep->requireSuper = TRUE;
		$this->gatekeep->init();
		if(array_key_exists("editId", $this->vars))
			$editId = trim($this->vars["editId"]);
		else
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, "editDisplay");
		if(!array_key_exists('title', $this->vars) || !trim($this->vars['title']) || 
			!array_key_exists('body', $this->vars) || !trim($this->vars['body']))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, "editDisplay");
		$updateArray['newsTitle'] = trim($this->vars['title']);
		$updateArray['newsNews'] = trim($this->vars['body']);
		$updateArray['newsTimestamp'] = $this->db->formatTimestamp();
		$this->db->formatConditions(array('newsId' => $editId));
		$this->db->update('news', $updateArray);
		$pString = $this->success->text("news", " " . $this->messages->text("misc", "edited") . " ");
		$this->session->setVar('news_Done', TRUE);
		if($this->db->fetchOne($this->db->select('config', 'configEmailNews')) == 'Y')
		{
			include_once("core/modules/email/EMAIL.php");
			$emailClass = new EMAIL();
			if(!$emailClass->news($updateArray['newsTitle'], $updateArray['newsNews']))
				$this->badInput->close($this->errors->text("inputError", "mail"));
		}
		GLOBALS::buildOutputString($pString);
	}
// View all available news items
	public function viewNews()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "news"));
		$news = $this->grabAll();
		$pString = '';
		if(is_array($news))
		{
			foreach($news as $id => $title)
				$pString .= $this->html->p($this->html->a("link", $this->html->dbToHtmlTidy($title), 
					"index.php?action=news_NEWS_CORE&method=viewNewsItem&id=" . $id) . 
					'&nbsp;&nbsp;' . $this->html->i($this->newsTimestamp[$id]));
		}
		else
			$pString .= $this->messages->text("news", "noNews");
		GLOBALS::buildOutputString($pString);
	}
// View one news item
	public function viewNewsItem()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "news"));
		if(array_key_exists("id", $this->vars))
			$id = trim($this->vars["id"]);
		else
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, "viewNews");
		$this->db->formatConditions(array('newsId' => $id));
		$recordset = $this->db->select('news', array('newsTimestamp', 'newsTitle', 'newsNews'));
		$row = $this->db->fetchRow($recordset);
		$dateSplit = explode(' ', $row['newsTimestamp']);
		$dateSplit = explode('-', $dateSplit[0]);
		$date = date("d/M/Y", mktime(0, 0, 0, $dateSplit[1], $dateSplit[2], $dateSplit[0]));
		$pString = $this->html->p($this->html->b($this->html->dbToHtmlTidy($row['newsTitle'])) . $this->html->br() . $date);
		$pString .= $this->html->p($this->html->dbToHtmlTidy($row['newsNews']));
		GLOBALS::buildOutputString($pString);
	}
// Get news titles and id from news.  Return associative array of id => title.
	private function grabAll()
	{
		$this->db->ascDesc = $this->db->desc;
		$this->db->orderBy("newsTimestamp");
		$recordset = $this->db->select("news", array("newsId", "newsTitle", "newsTimestamp"));
		while($row = $this->db->fetchRow($recordset))
		{
			$news[$row['newsId']] = $this->html->dbToFormTidy($row['newsTitle']);
			$this->newsTimestamp[$row['newsId']] = $row['newsTimestamp'];
		}
		if(isset($news))
			return $news;
		return array();
	}
}
?>
