<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* RESOURCEMETA class
*
* Deal with resource's quotes, paraphrases and musings
*/
class RESOURCEMETA
{
private $db;
private $vars;
private $session;
private $html;
private $messages;
private $user;
private $icons;
private $common;
private $cite;
private $quote = array();
private $paraphrase = array();
private $musing = array();
private $userId;
private $config;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->session = FACTORY_SESSION::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->user = FACTORY_USER::getInstance();
		$this->icons = FACTORY_LOADICONS::getInstance();
		$this->common = FACTORY_RESOURCECOMMON::getInstance();
		$this->cite = FACTORY_CITE::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->userId = $this->session->getVar('setup_UserId');
	}
// Display resource's quotes
	public function viewQuotes($row)
	{
		$write = $this->session->getVar('setup_Write') ? TRUE : FALSE;
		$this->db->formatConditions(array('resourcequoteResourceId' => $row['resourceId']));
		$this->db->formatConditions($this->db->formatFields('resourcequotetextId') . $this->db->equal . 
			$this->db->formatFields('resourcequoteId'));
		$this->db->orderBy($this->db->tidyInputClause('resourcequotePageStart') . '+0', FALSE);
		$this->db->leftJoin('resource_quote_text', 'resourcequotetextId', 'resourcequoteId');
		$recordset = $this->db->select('resource_quote', 
			array('resourcequoteId', 'resourcequotePageStart', 'resourcequotePageEnd', 
			'resourcequoteParagraph', 'resourcequoteSection', 'resourcequoteChapter', 
			'resourcequotetextText', 'resourcequotetextAddUserIdQuote'));
		$numRows = $this->db->numRows($recordset);
		if(!$numRows && !$write)
			return array();
		if($write)
		{
			$this->quote['editLink'] = $this->html->a($this->icons->addLink, $this->icons->add, 
				"index.php?action=resource_RESOURCEQUOTE_CORE&method=quoteEdit" . htmlentities("&resourceId=" . $row['resourceId']));
		}
		$this->view($row['resourceId'], $recordset, 'quote');
		return $this->quote;
	}
// Display resource's paraphrases
	public function viewParaphrases($row)
	{
		$write = $this->session->getVar('setup_Write') ? TRUE : FALSE;
		$this->db->formatConditions(array('resourceparaphraseResourceId' => $row['resourceId']));
		$this->db->formatConditions($this->db->formatFields('resourceparaphrasetextId') . $this->db->equal . 
			$this->db->formatFields('resourceparaphraseId'));
		$this->db->orderBy($this->db->tidyInputClause('resourceparaphrasePageStart') . '+0', FALSE);
		$this->db->leftJoin('resource_paraphrase_text', 'resourceparaphrasetextId', 'resourceparaphraseId');
		$recordset = $this->db->select('resource_paraphrase', 
			array('resourceparaphraseId', 'resourceparaphrasePageStart', 'resourceparaphrasePageEnd', 
			'resourceparaphraseParagraph', 'resourceparaphraseSection', 'resourceparaphraseChapter', 
			'resourceparaphrasetextText', 'resourceparaphrasetextAddUserIdParaphrase'));
		$numRows = $this->db->numRows($recordset);
		if(!$numRows && !$write)
			return array();
		if($write)
			$this->paraphrase['editLink'] = $this->html->a($this->icons->addLink, $this->icons->add, 
				"index.php?action=resource_RESOURCEPARAPHRASE_CORE&method=paraphraseEdit" . htmlentities("&resourceId=" . $row['resourceId']));
		$this->view($row['resourceId'], $recordset, 'paraphrase');
		return $this->paraphrase;
	}
// Display resource's musings
	public function viewMusings($row)
	{
		$resourceId = $row['resourceId'];
		$write = $this->session->getVar('setup_Write') ? TRUE : FALSE;
		$this->db->formatConditions(array('resourcemusingResourceId' => $row['resourceId']));
		$this->db->formatConditions($this->db->formatFields('resourcemusingtextId') . $this->db->equal . 
			$this->db->formatFields('resourcemusingId'));
		$this->db->orderBy($this->db->tidyInputClause('resourcemusingPageStart') . '+0', FALSE);
		$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
		$recordset = $this->db->select('resource_musing', 
			array('resourcemusingId', 'resourcemusingPageStart', 'resourcemusingPageEnd', 
			'resourcemusingParagraph', 'resourcemusingSection', 'resourcemusingChapter', 'resourcemusingtextPrivate', 
			'resourcemusingtextText', 'resourcemusingtextAddUserIdMusing', 'resourcemusingtextTimestamp'));
		$numRows = $this->db->numRows($recordset);
		if(!$numRows && !$write)
			return array();
		if($write)
			$this->musing['editLink'] = $this->html->a($this->icons->addLink, $this->icons->add, 
				"index.php?action=resource_RESOURCEMUSING_CORE&method=musingEdit" . htmlentities("&resourceId=" . $row['resourceId']));
		$patterns = FALSE;		$write = $this->session->getVar('setup_Write') ? TRUE : FALSE;
		$index = 0;
		$thisUserId = $this->session->getVar('setup_UserId');
		if(array_key_exists("search", $this->vars) && $this->vars["search"] = 'highlight')
		{
			$searchTerms = explode(",", $this->session->getVar('search_Highlight'));
			foreach($searchTerms as $term)
				$patterns[] = "/($term)(?!\S*\" \S*>)/i";
		}
		$index = 0;
		while($row = $this->db->fetchRow($recordset))
		{
			if(($row['resourcemusingtextPrivate'] == 'Y') && ($this->userId != $row['resourcemusingtextAddUserIdMusing']))
				continue;
			$this->musing[$index]['details'] = $this->getDetails($row, 'musing');
			$text = $this->cite->parseCitations($this->html->dbToHtmlTidy($row['resourcemusingtextText']), 'html');
			$this->musing[$index]['musing'] = $this->common->doHighlight($text);
			$this->musing[$index]['timestamp'] = $row['resourcemusingtextTimestamp'];
			$this->db->formatConditions(array('resourcekeywordMusingId' => $row['resourcemusingId']));
			$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
			$recordset2 = $this->db->select('resource_keyword', array('keywordId', 'keywordKeyword'));
			while($row2 = $this->db->fetchRow($recordset2))
			{
				$this->musing[$index]['keywordTitle'] = $this->messages->text("resources", "keywords");
				$this->musing[$index]['keywords'][] = $this->html->a("link", 
					$this->html->dbToHtmlTidy($row2['keywordKeyword']), "index.php?action=list_LISTSOMERESOURCES_CORE" . 
					'&method=metaKeywordProcess' . htmlentities("&id=" . $row2['keywordId']));
			}
			$users = $this->user->displayUserAddEdit($row['resourcemusingtextAddUserIdMusing'], TRUE, 'musing');
			$this->musing[$index]['userAdd'] = $users[0];
			if($write && ($row['resourcemusingtextAddUserIdMusing'] == $this->userId))
				$this->musing[$index]['editLink'] = $this->html->a($this->icons->editLink, $this->icons->edit, 
				"index.php?action=resource_RESOURCEMUSING_CORE&method=musingEdit" . htmlentities("&resourceId=" . $resourceId . 
				'&resourcemusingId=' . $row['resourcemusingId']));
			$index++;
		}
		if(!empty($this->musing))
			$this->musing['title'] = $this->messages->text("viewResource", "musings");
		return $this->musing;
	}
// Display quotes, paraphrases etc.
	private function view($resourceId, $recordset, $type)
	{
		if($type == 'quote')
			$this->quote['title'] = $this->messages->text("viewResource", "quotes");
		else // 'paraphrase'
			$this->paraphrase['title'] = $this->messages->text("viewResource", "paraphrases");
		$patterns = FALSE;
		$addUserId = $type == 'quote' ? 'resourcequotetextAddUserIdQuote' : 'resourceparaphrasetextAddUserIdParaphrase';
		$addUserCommentId = $type == 'quote' ? 
			'resourcequotecommentAddUserIdQuote' : 'resourceparaphrasecommentAddUserIdParaphrase';
		$keywordField = $type == 'quote' ? 'resourcekeywordQuoteId' : 'resourcekeywordParaphraseId';
		$timestampField = $type == 'quote' ? 'resourcequotecommentTimestamp' : 'resourceparaphrasecommentTimestamp';
		$commentField = $type == 'quote' ? 'resourcequotecommentComment' : 'resourceparaphrasecommentComment';
		$textField = $type == 'quote' ? 'resourcequotetextText' : 'resourceparaphrasetextText';
		$textId = $type == 'quote' ? 'resourcequoteId' : 'resourceparaphraseId';
		$commentId = $type == 'quote' ? 'resourcequotecommentQuoteId' : 'resourceparaphrasecommentParaphraseId';
		$action = $type == 'quote' ? 'quoteEdit' : 'paraphraseEdit';
		$private = $type == 'quote' ? 'resourcequotecommentPrivate' : 'resourceparaphrasecommentPrivate';
		$commentTable = $type == 'quote' ? 'resource_quote_comment' : 'resource_paraphrase_comment';
		$phpFile = $type == 'quote' ? 'resource_RESOURCEQUOTE_CORE' : 'resource_RESOURCEPARAPHRASE_CORE';
		$write = $this->session->getVar('setup_Write') ? TRUE : FALSE;
		$index = 0;
		$thisUserId = $this->session->getVar('setup_UserId');
		if(array_key_exists("search", $this->vars) && $this->vars["search"] = 'highlight')
		{
			$searchTerms = explode(",", $this->session->getVar('search_Highlight'));
			foreach($searchTerms as $term)
				$patterns[] = "/($term)(?!\S*\" \S*>)/i";
		}
		$index = 0;
		while($row = $this->db->fetchRow($recordset))
		{
			$this->{$type}[$index]['details'] = $this->getDetails($row, $type);
			$this->{$type}[$index]['commentTitle'] = $this->messages->text("resources", "comment");
			$text = $this->cite->parseCitations($this->html->dbToHtmlTidy($row[$textField]), 'html');
			$this->{$type}[$index][$type] = $this->common->doHighlight($text);
			$this->db->formatConditions(array($keywordField => $row[$textId]));
			$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
			$recordset2 = $this->db->select('resource_keyword', array('keywordId', 'keywordKeyword'));
			while($row2 = $this->db->fetchRow($recordset2))
			{
				$this->{$type}[$index]['keywordTitle'] = $this->messages->text("resources", "keywords");
				$this->{$type}[$index]['keywords'][] = $this->html->a("link", 
					$this->html->dbToHtmlTidy($row2['keywordKeyword']), "index.php?action=list_LISTSOMERESOURCES_CORE" . 
					'&method=metaKeywordProcess' . htmlentities("&id=" . $row2['keywordId']));
			}
			$users = $this->user->displayUserAddEdit($row[$addUserId], TRUE, $type);
			$this->{$type}[$index]['userAdd'] = $users[0];
// check for comments
			$this->db->formatConditions(array($commentId => $row[$textId]));
			$this->db->orderBy($timestampField);
			$recordset2 = $this->db->select($commentTable, array($commentField, $timestampField, $addUserCommentId, $private));
			if($this->db->numRows($recordset2))
			{
				$index2 = 0;
				while($rowComment = $this->db->fetchRow($recordset2))
				{
// Read only access
					if($this->session->getVar('setup_ReadOnly') && 
						(($rowComment[$private] == 'Y') || ($rowComment[$private] == 'G')))
						continue;
					else if(($rowComment[$private] == 'Y') && 
						($thisUserId != $rowComment[$addUserCommentId]))
						continue;
// If 'G', this comment may be viewed by members of user groups of which $row[$addUserId] is a member
					else if(is_numeric($rowComment[$private]))
					{
						$this->db->formatConditions(array('usergroupsusersUserId' => $this->userId));
						$this->db->formatConditions(array('usergroupsusersGroupId' => $rowComment[$private]));
						$recordset3 = $this->db->select('user_groups_users', 'usergroupsusersId');
						if(!$this->db->numRows($recordset3))
							continue;
					}
// Else, comment is public
					$text = $this->cite->parseCitations($this->html->dbToHtmlTidy($rowComment[$commentField]), 'html');
					$users = $this->user->displayUserAddEdit($rowComment[$addUserCommentId], TRUE, 'comment');
					$this->{$type}[$index]['comments'][$index2]['userAdd'] = $users[0];
					$this->{$type}[$index]['comments'][$index2]['comment'] = $this->common->doHighlight($text);
					$this->{$type}[$index]['comments'][$index2]['timestamp'] = $rowComment[$timestampField];
					$index2++;
				}
			}
//			if($this->session->getVar("setup_Superadmin") || 
//				($write && (!$this->config->WIKINDX_ORIGINATOR_EDITONLY || ($row[$addUserId] == $this->userId))))
			if($write)
				$this->{$type}[$index]['editLink'] = $this->html->a($this->icons->editLink, $this->icons->edit, 
				"index.php?action=$phpFile&method=$action" . 
				htmlentities("&resourceId=" . $resourceId . "&$textId=" . $row[$textId]));
			$index++;
		}
	}
/** 
* Get metadata details such as pages, section etc.
*/
	private function getDetails($row, $type)
	{
		if($type == 'quote')
		{
			$pageStartField = 'resourcequotePageStart';
			$pageEndField = 'resourcequotePageEnd';
			$paragraphField = 'resourcequoteParagraph';
			$sectionField = 'resourcequoteSection';
			$chapterField = 'resourcequoteChapter';
		}
		else if ($type == 'paraphrase')
		{
			$pageStartField = 'resourceparaphrasePageStart';
			$pageEndField = 'resourceparaphrasePageEnd';
			$paragraphField = 'resourceparaphraseParagraph';
			$sectionField = 'resourceparaphraseSection';
			$chapterField = 'resourceparaphraseChapter';
		}
		else // musings
		{
			$pageStartField = 'resourcemusingPageStart';
			$pageEndField = 'resourcemusingPageEnd';
			$paragraphField = 'resourcemusingParagraph';
			$sectionField = 'resourcemusingSection';
			$chapterField = 'resourcemusingChapter';
		}
		$page_start = $row[$pageStartField] ? $row[$pageStartField] : FALSE;
		$page_end = $row[$pageEndField] ? "-" . $row[$pageEndField] : FALSE;
		if($page_start && $page_end)
			$page_start = 'pp.' . $page_start;
		else if($page_start)
			$page_start = 'p.' . $page_start;
		$page = $page_start? $page_start . $page_end : FALSE;
		if($page)
			$details[] = $page;
		$paragraph = $row[$paragraphField] ? $row[$paragraphField] : FALSE;
		if($paragraph)
			$details[] = $this->messages->text("resources", "paragraph") . "&nbsp;" . $paragraph;
		$section = $row[$sectionField] ? $row[$sectionField] : FALSE;
		if($section)
			$details[] = $this->messages->text("resources", "section") . "&nbsp;" . $section;
		$chapter = $row[$chapterField] ? $row[$chapterField] : FALSE;
		if($chapter)
			$details[] = $this->messages->text("resources", "chapter") . "&nbsp;" . $chapter;
		return isset($details) ? join(",&nbsp;", $details) : FALSE;
	}
}
?>
