<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
* RESOURCEMUSING class
*
* Deal with resource's musings
*****/
class RESOURCEMUSING
{
private $gatekeep;
private $db;
private $vars;
private $textqp;
private $session;
private $html;
private $messages;
private $errors;
private $success;
private $form;
private $table;
private $navigate;
private $badInput;
private $template;

// Constructor
	public function __construct()
	{
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->gatekeep->init();
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		include_once("core/modules/resource/TEXTQP.php");
		$this->textqp = new TEXTQP();
		$this->session = FACTORY_SESSION::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->navigate = FACTORY_NAVIGATE::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "musings"));
		if(!array_key_exists('resourceId', $this->vars) || !$this->vars['resourceId'] || 
			!array_key_exists('method', $this->vars) || !$this->vars['method'])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$function = $this->vars['method'];
		if(!method_exists($this, $function))
			$this->navigate->resource($this->vars['resourceId'], $this->errors->text("inputError", "invalid"));
	}
// display the editing form:
	public function musingEdit()
	{
		$tinymce = FACTORY_LOADTINYMCE::getInstance();
		$pString = $tinymce->loadMetadataTextarea(array('resourcemusingtextText'));
		$pString .= $this->form->formHeader('resource_RESOURCEMUSING_CORE');
		$hidden = $this->form->hidden("resourceId", $this->vars['resourceId']);
		$hidden .= $this->form->hidden('method', 'edit');
		$metadata['hidden'] =  $pString;
		$page_start = $page_end = $db_paragraph = $db_section = $db_chapter = $text = $rowKeywords = $private = FALSE;
// are we editing or adding?
		if(array_key_exists('resourcemusingId', $this->vars))
		{
			$hidden .= $this->form->hidden("resourcemusingId", $this->vars['resourcemusingId']);
			$this->db->formatConditions(array('resourcemusingId' => $this->vars['resourcemusingId']));
			$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
			$recordset = $this->db->select('resource_musing', 
				array('resourcemusingId', 'resourcemusingPageStart', 'resourcemusingPageEnd', 'resourcemusingtextText', 
				'resourcemusingParagraph', 'resourcemusingSection', 'resourcemusingChapter', 'resourcemusingtextPrivate'));
			$row = $this->db->fetchRow($recordset);
			$page_start = $this->html->dbToFormTidy($row['resourcemusingPageStart']);
			$page_end = $this->html->dbToFormTidy($row['resourcemusingPageEnd']);
			$db_paragraph = $this->html->dbToFormTidy($row['resourcemusingParagraph']);
			$db_section = $this->html->dbToFormTidy($row['resourcemusingSection']);
			$db_chapter = $this->html->dbToFormTidy($row['resourcemusingChapter']);
			$text = $this->html->dbToTinyMCE($row['resourcemusingtextText']);
			$private = $row['resourcemusingtextPrivate'];
		}
		else
			$private = 'Y';
		$metadata['keyword']= $this->textqp->displayKeywordForm('musing', 'resourcemusingId');
		$locations = $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$locations .= $this->table->trStart();
		$locations .= $this->table->td($hidden . $this->form->textInput($this->messages->text("resources", "page"), 
			'resourcemusingPageStart', $page_start, 6, 5) . "&nbsp;-&nbsp;" . $this->form->textInput(FALSE, 'resourcemusingPageEnd', $page_end, 6, 5));
		$locations .= $this->table->td($this->form->textInput($this->messages->text("resources", "paragraph"), 
			'resourcemusingParagraph', $db_paragraph, 11, 10));
		$locations .= $this->table->td($this->form->textInput($this->messages->text("resources", "section"), 
			'resourcemusingSection', $db_section, 20));
		$locations .= $this->table->td($this->form->textInput($this->messages->text("resources", "chapter"), 
			'resourcemusingChapter', $db_chapter, 20));
		$locations .= $this->table->trEnd();
		$locations .= $this->table->tableEnd();
		$metadata['locations'] = $locations;
// The second parameter ('musingText') to textareaInput is the textarea name
		$metadata['metadata'] = $this->form->textareaInput(FALSE, 'resourcemusingtextText', $text, 80, 10);
		$metadata['metadataTitle'] = $this->messages->text("resources", 'musing');
		$thisUserId = $write = $this->session->getVar('setup_UserId');
		$this->db->formatConditions(array('usergroupsusersUserId' => $thisUserId));
		$this->db->leftJoin('user_groups', 'usergroupsId', 'usergroupsusersGroupId');
		$recordset3 = $this->db->select('user_groups_users', array('usergroupsusersGroupId', 'usergroupsTitle'));
		if($this->db->numRows($recordset3))
		{
			$privateArray = array('Y' => $this->messages->text("resources", "private"), 
				'N' => $this->messages->text("resources", "public"));
			while($row = $this->db->fetchRow($recordset3))
				$privateArray[$row['usergroupsusersGroupId']] = 
					$this->messages->text("resources", "availableToGroups", $this->html->dbToFormTidy($row['usergroupsTitle']));
			$metadata['form']['private'] = $this->form->selectedBoxValue($this->messages->text("resources", "musingPrivate"), 
				"private", $privateArray, $private, 3);
		}
		else
		{
			$privateArray = array('Y' => $this->messages->text("resources", "private"), 
				'N' => $this->messages->text("resources", "public"));
			$metadata['form']['private'] = $this->form->selectedBoxValue($this->messages->text("resources", "musingPrivate"), 
				"private", $privateArray, $private, 2);
		}
		$metadata['form']['submit'] = $this->form->formSubmit("Proceed");
		$metadata['formfoot'] = $this->form->formEnd();
		$this->session->delVar('resourceMusingLock');
		$this->template->tpl->assign('metadata', $metadata);
	}
// write to the database.
// if there is no 'musingId' input, we are adding a new musing.  Otherwise, editing one.
	public function edit()
	{
		if($this->session->getVar('resourceMusingLock'))
			$this->badInput->close($this->errors->text("done", "musing"));
		$this->checkInput();
		$userId = $this->session->getVar('setup_UserId');
// insert
		if(!array_key_exists('resourcemusingId', $this->vars))
		{
			if(!array_key_exists('resourcemusingtextText', $this->vars) || 
				!trim($this->html->stripHtml($this->vars['resourcemusingtextText'])))
				$this->badInput->close($this->errors->text("inputError", "missing"));
			$addEdit = 'added';
			$fields[] = 'resourcemusingResourceId';
			$values[] = trim($this->vars['resourceId']);
			if(array_key_exists('resourcemusingPageStart', $this->vars) && $this->vars['resourcemusingPageStart'])
			{
				$fields[] = 'resourcemusingPageStart';
				$values[] = trim(strtolower($this->vars['resourcemusingPageStart']));
				if(array_key_exists('resourcemusingPageEnd', $this->vars) && $this->vars['resourcemusingPageEnd'])
				{
					$fields[] = 'resourcemusingPageEnd';
					$values[] = trim(strtolower($this->vars['resourcemusingPageEnd']));
				}
			}
			if(array_key_exists('resourcemusingParagraph', $this->vars) && $this->vars['resourcemusingParagraph'])
			{
				$fields[] = 'resourcemusingParagraph';
				$values[] = trim(strtolower($this->vars['resourcemusingParagraph']));
			}
			if(array_key_exists('resourcemusingSection', $this->vars) && $this->vars['resourcemusingSection'])
			{
				$fields[] = 'resourcemusingSection';
				$values[] = trim(strtolower($this->vars['resourcemusingSection']));
			}
			if(array_key_exists('resourcemusingChapter', $this->vars) && $this->vars['resourcemusingChapter'])
			{
				$fields[] = 'resourcemusingChapter';
				$values[] = trim(strtolower($this->vars['resourcemusingChapter']));
			}
			$this->db->insert('resource_musing', $fields, $values);
			$lastAutoId = $this->db->lastAutoId('resource_musing');
			$fields = $values = array();
			$fields[] = 'resourcemusingtextId';
			$values[] = $lastAutoId;
			$fields[] = 'resourcemusingtextText';
			$values[] = trim($this->vars['resourcemusingtextText']);
			$fields[] = 'resourcemusingtextPrivate';
			if(array_key_exists('private', $this->vars) && ($this->vars['private'] == 'N'))
				$values[] = 'N';
			else if(array_key_exists('private', $this->vars) && (is_numeric($this->vars['private'])))
				$values[] = $this->vars['private'];
			else
				$values[] = 'Y';
			$fields[] = 'resourcemusingtextTimestamp';
			$values[] = $this->db->formatTimestamp();
			if($userId)
			{
				$fields[] = "resourcemusingtextAddUserIdMusing";
				$values[] = $userId;
			}
			$this->db->insert('resource_musing_text', $fields, $values);
			$this->textqp->summary(1, 'resourcesummaryMusings');
			$this->textqp->writeKeywords($lastAutoId, 'resourcekeywordMusingId');
		}
// else edit/delete?
		else
		{
// if musingText is empty, delete the row
			if(!$this->vars['resourcemusingtextText'])
			{
				$addEdit = 'deleted';
				$this->db->formatConditions(array('resourcemusingId' => $this->vars['resourcemusingId']));
				$this->db->delete('resource_musing');
				$this->db->formatConditions(array('resourcemusingtextId' => $this->vars['resourcemusingId']));
				$this->db->delete('resource_musing_text');
				$this->db->formatConditions(array('resourcekeywordMusingId' => $this->vars['resourcemusingId']));
				$this->db->delete('resource_keyword');
				$this->db->optimize('resource_musing');
				$this->db->optimize('resource_musing_text');
				$this->db->optimize('resource_keyword');
				$this->textqp->summary(-1, 'resourcesummaryMusings');
			}
			else
			{
				$addEdit = 'edited';
				$updateArray = array();
				$updateArray['resourcemusingtextText'] = trim($this->vars['resourcemusingtextText']);
				if(array_key_exists('private', $this->vars) && ($this->vars['private'] == 'N'))
					$updateArray['resourcemusingtextPrivate'] = 'N';
				else if(array_key_exists('private', $this->vars) && (is_numeric($this->vars['private'])))
					$updateArray['resourcemusingtextPrivate'] = $this->vars['private'];
				else
					$updateArray['resourcemusingtextPrivate'] = 'Y';
				$updateArray['resourcemusingtextTimestamp'] = $this->db->formatTimestamp();
				$this->db->formatConditions(array('resourcemusingtextId' => $this->vars['resourcemusingId']));
				$this->db->update('resource_musing_text', $updateArray);
				$updateArray = $nulls = array();
// page number lowercased in case roman numerals input!
				if(array_key_exists('resourcemusingPageStart', $this->vars) && $this->vars['resourcemusingPageStart'])
				{
					$updateArray['resourcemusingPageStart'] = trim(strtolower($this->vars['resourcemusingPageStart']));
					if(array_key_exists('resourcemusingPageEnd', $this->vars) && $this->vars['resourcemusingPageEnd'])
						$updateArray['resourcemusingPageEnd'] = trim(strtolower($this->vars['resourcemusingPageEnd']));
					else
						$nulls[] = 'resourcemusingPageEnd';
				}
				else
				{
					$nulls[] = 'resourcemusingPageStart';
					$nulls[] = 'resourcemusingPageEnd';
				}
				if(array_key_exists('resourcemusingParagraph', $this->vars) && $this->vars['resourcemusingParagraph'])
					$updateArray['resourcemusingParagraph'] = trim($this->vars['resourcemusingParagraph']);
				else
					$nulls[] = 'resourcemusingParagraph';
				if(array_key_exists('resourcemusingSection', $this->vars) && $this->vars['resourcemusingSection'])
					$updateArray['resourcemusingSection'] = trim($this->vars['resourcemusingSection']);
				else
					$nulls[] = 'resourcemusingSection';
				if(array_key_exists('resourcemusingChapter', $this->vars) && $this->vars['resourcemusingChapter'])
					$updateArray['resourcemusingChapter'] = trim($this->vars['resourcemusingChapter']);
				else
					$nulls[] = 'resourcemusingChapter';
				if(!empty($updateArray))
				{
					$this->db->formatConditions(array('resourcemusingId' => $this->vars['resourcemusingId']));
					$this->db->update('resource_musing', $updateArray);
				}
				if(!empty($nulls))
				{
					$this->db->formatConditions(array('resourcemusingId' => $this->vars['resourcemusingId']));
					$this->db->updateNull('resource_musing', $nulls);
				}
				$this->textqp->writeKeywords($this->vars['resourcemusingId'], 'resourcekeywordMusingId');
			}
		}
// update resource_timestamp
		$this->db->formatConditions(array('resourcetimestampId' => $this->vars['resourceId']));
		$this->db->update('resource_timestamp', array('resourcetimestampTimestamp' => $this->db->formatTimestamp()));
		include_once("core/modules/email/EMAIL.php");
		$emailClass = new EMAIL();
		if(!$emailClass->notify($this->vars['resourceId']))
			$this->badInput->close($this->errors->text("inputError", "mail"));
// lock reload
		$this->session->setVar('resourceMusingLock', TRUE);
// send back to view this resource with success message
		$this->navigate->resource($this->vars['resourceId'], $this->success->text("musings", " " . 
			$this->messages->text("misc", $addEdit) . " "));
	}
// Check we have appropriate input.  Page and comment are optional
	private function checkInput()
	{
		if(!array_key_exists('resourceId', $this->vars) || !$this->vars['resourceId'])
			$this->badInput->close($this->errors->text("inputError", "missing"));
	}
}
?>
