<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
* TEXTQP class
*
* Common functions for quotes and paraphrases
*****/
class TEXTQP
{
private $db;
private $vars;
private $session;
private $html;
private $messages;
private $errors;
private $user;
private $form;
private $table;
private $keyword;
private $cite;
private $ajax;
private $badInput;

// Constructor
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->session = FACTORY_SESSION::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->user = FACTORY_USER::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->keyword = FACTORY_KEYWORD::getInstance();
		$this->cite = FACTORY_CITE::getInstance();
		$this->ajax = FACTORY_AJAX::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->ajax->loadJavascript('core/modules/resource/resourceCategoryEdit.js');
	}
// display the editing form:
// Only the original poster can edit the quote/paraphrase and only the original posters can edit their comment
// $type is either 'quote' or 'paraphrase'
	public function editDisplay($type)
	{
		$textId = $type == 'quote' ? 'resourcequoteId' : 'resourceparaphraseId';
		$action = $type == 'quote' ? 'quoteEdit' : 'paraphraseEdit';
		$addUserId = $type == 'quote' ? 'resourcequotetextAddUserIdQuote' : 'resourceparaphrasetextAddUserIdParaphrase';
		$pageStart = $type == 'quote' ? 'resourcequotePageStart' : 'resourceparaphrasePageStart';
		$pageEnd = $type == 'quote' ? 'resourcequotePageEnd' : 'resourceparaphrasePageEnd';
		$paragraph = $type == 'quote' ? 'resourcequoteParagraph' : 'resourceparaphraseParagraph';
		$section = $type == 'quote' ? 'resourcequoteSection' : 'resourceparaphraseSection';
		$chapter = $type == 'quote' ? 'resourcequoteChapter' : 'resourceparaphraseChapter';
		$typeText = $type == 'quote' ? 'resourcequotetextText' : 'resourceparaphrasetextText';
		$keywords = $type == 'quote' ? 'quote_keywords' : 'paraphrase_keywords';
		$typeComment = $type == 'quote' ? 'resourcequotecommentComment' : 'resourceparaphrasecommentComment';
		$privateComment = $type == 'quote' ? 'resourcequotecommentPrivate' : 'resourceparaphrasecommentPrivate';
		$table = $type == 'quote' ? 'resource_quote' : 'resource_paraphrase';
		$textTable = $type == 'quote' ? 'resource_quote_text' : 'resource_paraphrase_text';
		$textTableId = $type == 'quote' ? 'resourcequotetextId' : 'resourceparaphrasetextId';
		$commentTable = $type == 'quote' ? 'resource_quote_comment' : 'resource_paraphrase_comment';
		$commentAddUserId = $type == 'quote' ? 'resourcequotecommentAddUserIdQuote' : 'resourceparaphrasecommentAddUserIdParaphrase';
		$textCommentId = $type == 'quote' ? 'resourcequotecommentQuoteId' : 'resourceparaphrasecommentParaphraseId';
		$commentTimestamp = $type == 'quote' ? 'resourcequotecommentTimestamp' : 'resourceparaphrasecommentTimestamp';
		$userId = $this->session->getVar('setup_UserId');
		$hidden = $this->form->hidden("resourceId", $this->vars['resourceId']);
		$page_start = $page_end = $db_paragraph = $db_section = $db_chapter = $text = $comment = $td = FALSE;
		$private = 'Y';
// are we editing or adding?
		if(array_key_exists($textId, $this->vars))
		{
			$hidden .= $this->form->hidden($textId, $this->vars[$textId]);
			$this->db->formatConditions(array($textId => $this->vars[$textId]));
			$this->db->leftJoin($textTable, $textTableId, $textId);
			$row = $this->db->fetchRow($this->db->select($table, array($typeText, $pageStart, $pageEnd, $paragraph, $section, $chapter, 
				$addUserId)));
			$page_start = $this->html->dbToFormTidy($row[$pageStart]);
			$db_paragraph = $this->html->dbToFormTidy($row[$paragraph]);
			$db_section = $this->html->dbToFormTidy($row[$section]);
			$db_chapter = $this->html->dbToFormTidy($row[$chapter]);
			if($row[$pageEnd])
				$page_end = $this->html->dbToFormTidy($row[$pageEnd]);
			if($userId == $row[$addUserId])
				$text = $this->html->dbToTinyMCE($row[$typeText]);
			else
				$text = $this->html->dbToHtmlTidy($row[$typeText]);
			$this->db->formatConditions(array($commentAddUserId => $userId));
			$this->db->formatConditions(array($textCommentId => $this->vars[$textId]));
			$recordset = $this->db->select($commentTable, array($typeComment, $privateComment));
			if($this->db->numRows($recordset))
			{
				$rowComment = $this->db->fetchRow($recordset);
				$comment = $this->html->dbToTinyMCE($rowComment[$typeComment]);
				$private = $rowComment[$privateComment];
			}
		}
		if(isset($row) && $userId != $row[$addUserId])
			$hidden .= $this->form->hidden("commentOnly", TRUE);
		if(!isset($row) || ($text && ($userId == $row[$addUserId])))
			$metadata['keyword']= $this->displayKeywordForm($type, $textId);
		if(!$text || ($text && ($userId == $row[$addUserId])))
		{
			$locations = $this->table->tableStart(FALSE, 0, 0, 0, "left");
			$locations .= $this->table->trStart();
			$locations .= $this->table->td($hidden . $this->form->textInput($this->messages->text("resources", "page"), 
				$pageStart, $page_start, 6, 5) . '-&nbsp;' . $this->form->textInput(FALSE, $pageEnd, $page_end, 6, 5));
			$locations .= $this->table->td($this->form->textInput($this->messages->text("resources", "paragraph"), 
				$paragraph, $db_paragraph, 11, 10));
			$locations .= $this->table->td($this->form->textInput($this->messages->text("resources", "section"), 
				$section, $db_section, 20));
			$locations .= $this->table->td($this->form->textInput($this->messages->text("resources", "chapter"), 
				$chapter, $db_chapter, 20));
			$locations .= $this->table->trEnd();
			$locations .= $this->table->tableEnd();
			$metadata['locations'] = $locations;
		}
		else if($page_start && $page_end)
			$page = $this->html->b('pp.' . $page_start . '-&nbsp;' . $page_end) . "&nbsp;";
		else if($page_start)
			$page = $this->html->b('p.' . $page_start) . "&nbsp;";
		else
			$page = FALSE;
		$hint = ($type == 'quote') ? $this->html->span($this->messages->text("hint", $type), 'hint') : FALSE;
// The second parameter ($typeText) to textareaInput is the textarea name
		if(!$text || ($text && ($userId == $row[$addUserId])))
		{
			$metadata['metadata'] = $this->form->textareaInput(FALSE, $typeText, $text, 80, 10) . $hint;
			$metadata['metadataTitle'] = $this->messages->text("resources", $type);
		}
		else
			$metadata['original'] = $hidden . $page . $this->cite->parseCitations($text, 'html');
// The second parameter ($typeComment) to textareaInput is the textarea name
		$metadata['comment'] = $this->form->textareaInput(FALSE, $typeComment, $comment, 80, 10);
		$metadata['commentTitle'] = $this->messages->text("resources", "comment");
		$this->db->formatConditions(array('usergroupsusersUserId' => $userId));
		$this->db->leftJoin('user_groups', 'usergroupsId', 'usergroupsusersGroupId');
		$recordset3 = $this->db->select('user_groups_users', array('usergroupsusersGroupId', 'usergroupsTitle'));
		if($this->db->numRows($recordset3))
		{
			$privateArray = array('Y' => $this->messages->text("resources", "private"), 
				'N' => $this->messages->text("resources", "public"));
			while($row = $this->db->fetchRow($recordset3))
				$privateArray[$row['usergroupsusersGroupId']] = 
					$this->messages->text("resources", "availableToGroups", $this->html->dbToFormTidy($row['usergroupsTitle']));
			$metadata['form']['private'] = $this->form->selectedBoxValue($this->messages->text("resources", "commentPrivate"), 
				"private", $privateArray, $private, 3);
		}
		else
		{
			$privateArray = array('Y' => $this->messages->text("resources", "private"), 
				'N' => $this->messages->text("resources", "public"));
			$metadata['form']['private'] = $this->form->selectedBoxValue($this->messages->text("resources", "commentPrivate"), 
				"private", $privateArray, $private, 2);
		}
		$metadata['form']['submit'] = $this->form->formSubmit("Proceed");

// display other comments
		if($text)
		{
			$this->db->formatConditions(array($commentAddUserId => $userId), TRUE);
			$this->db->formatConditions(array($textCommentId => $this->vars[$textId]));
			$this->db->ascDesc = $this->db->desc;
			$this->db->orderBy($commentTimestamp);
			$recordset = $this->db->select($commentTable, array($typeComment, $privateComment, $commentAddUserId, $commentTimestamp));
			$index = 0;
			while($row = $this->db->fetchRow($recordset))
			{
				if(($row[$privateComment] == 'Y') && ($userId != $row[$commentAddUserId]))
					continue;
// If is_numeric(), this comment may be viewed by members of the user group's ID given by $privateComment
				else if(is_numeric($row[$privateComment]))
				{
					$this->db->formatConditions(array('usergroupsusersUserId' => $userId));
					$this->db->formatConditions(array('usergroupsusersGroupId' => $row[$privateComment]));
					$recordset3 = $this->db->select('user_groups_users', 'usergroupsusersId');
					if(!$this->db->numRows($recordset3))
						continue;
				}
// Comment can be viewed
				$username = $this->user->displayUserAddEdit($row[$commentAddUserId], TRUE, 'comment');
				$metadata['otherComments'][$index]['comment'] = $this->cite->parseCitations($this->html->dbToHtmlTidy($row[$typeComment]), 'html');
				$metadata['otherComments'][$index]['userAdd'] = $username[0];
				$metadata['otherComments'][$index]['timestamp'] = $row[$commentTimestamp];
				++$index;
			}
		}
		
		//$metadata['form']['footform'] = $this->form->formEnd();
		
		return $metadata;
	}
// display keyword form
	public function displayKeywordForm($type, $textId)
	{
		$keywords = $this->keyword->grabAll();
		$pString = $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $this->table->trStart();
		if(!empty($keywords))
		{
// If preferences reduce long keywords, we want to transfer the original rather than the condensed version. 
// Store the base64-encoded value for retrieval in the javascript.
			foreach($keywords as $key => $value)
			{
				$key = $key . '_' . base64_encode($value);
				$keywordList[$key] = $value;
			}
			if(array_key_exists($textId, $this->vars)) // editing
			{
				if($type == 'quote')
					$field = 'resourcekeywordQuoteId';
				else if($type == 'paraphrase')
					$field = 'resourcekeywordParaphraseId';
				else
					$field = 'resourcekeywordMusingId';
				$this->db->formatConditions(array($field => $this->vars[$textId]));
				$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
				$this->db->orderBy('keywordKeyword');
				$resultset = $this->db->select('resource_keyword', 'keywordKeyword');
				while($row = $this->db->fetchRow($resultset))
					$keywordArray[] = $row['keywordKeyword'];
			}
			$keywordString = isset($keywordArray) ? join(', ', $keywordArray) : FALSE;
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('resources', 'availableKeywords'), 
				'fromKeywords', $keywordList, 6) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$jsonArray = array();
			$jsonArray[] = array(
				'startFunction' => 'transferKeyword', 
				);
			$toLeftImage = $this->ajax->jActionIcon('toRight', 'onclick', $jsonArray);
			$pString .= $this->table->td($toLeftImage);
// Entry text area
			$pString .= $this->table->td($this->form->textareaInput($this->messages->text("resources", "keywords"), 
				"keywords", $this->html->dbToFormTidy($keywordString), 50, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "keywords"), 'hint') . $this->html->p('&nbsp;'));
		}
		else
		{
			$pString .= $this->table->td($this->form->textareaInput($this->messages->text("resources", "keywords"), 
				"keywords", FALSE, 50, 5) . $this->html->br() . $this->html->span($this->messages->text("hint", "keywordsAlt"), 
				'hint') . $this->html->p('&nbsp;'));
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		return $pString;
	}
// write to the database.
// if there is no 'quoteId'/'paraphraseId' input, we are adding a new quote/paraphrase.  Otherwise, editing one.
// $type is either 'quote' or 'paraphrase'
	public function edit($type)
	{
		$textId = $type == 'quote' ? 'resourcequoteId' : 'resourceparaphraseId';
		$resourceId = $type == 'quote' ? 'resourcequoteResourceId' : 'resourceparaphraseResourceId';
		$addUserId = $type == 'quote' ? 'resourcequotetextAddUserIdQuote' : 'resourceparaphrasetextAddUserIdParaphrase';
		$pageStart = $type == 'quote' ? 'resourcequotePageStart' : 'resourceparaphrasePageStart';
		$pageEnd = $type == 'quote' ? 'resourcequotePageEnd' : 'resourceparaphrasePageEnd';
		$paragraph = $type == 'quote' ? 'resourcequoteParagraph' : 'resourceparaphraseParagraph';
		$section = $type == 'quote' ? 'resourcequoteSection' : 'resourceparaphraseSection';
		$chapter = $type == 'quote' ? 'resourcequoteChapter' : 'resourceparaphraseChapter';
		$typeText = $type == 'quote' ? 'resourcequotetextText' : 'resourceparaphrasetextText';
		$typeComment = $type == 'quote' ? 'resourcequotecommentComment' : 'resourceparaphrasecommentComment';
		$privateComment = $type == 'quote' ? 'resourcequotecommentPrivate' : 'resourceparaphrasecommentPrivate';
		$typeTable = $type == 'quote' ? 'resource_quote' : 'resource_paraphrase';
		$textTable = $type == 'quote' ? 'resource_quote_text' : 'resource_paraphrase_text';
		$textTableId = $type == 'quote' ? 'resourcequotetextId' : 'resourceparaphrasetextId';
		$keywordTableId = $type == 'quote' ? 'resourcekeywordQuoteId' : 'resourcekeywordParaphraseId';
		$commentTable = $type == 'quote' ? 'resource_quote_comment' : 'resource_paraphrase_comment';
		$textCommentId = $type == 'quote' ? 'resourcequotecommentQuoteId' : 'resourceparaphrasecommentParaphraseId';
		$commentAddUserId = $type == 'quote' ? 'resourcequotecommentAddUserIdQuote' : 'resourceparaphrasecommentAddUserIdParaphrase';
		$summaryType = $type == 'quote' ? 'resourcesummaryQuotes' : 'resourcesummaryParaphrases';
		$commentTimestamp = $type == 'quote' ? 'resourcequotecommentTimestamp' : 'resourceparaphrasecommentTimestamp';
		$userId = $this->session->getVar('setup_UserId');
// insert
		if(!array_key_exists($textId, $this->vars))
		{
			if(!array_key_exists($typeText, $this->vars) || !trim($this->html->stripHtml($this->vars[$typeText])))
				return FALSE;
			$addEdit = 'added';
			$fields[] = $resourceId;
			$values[] = $this->vars['resourceId'];
			if(array_key_exists($pageStart, $this->vars) && $this->vars[$pageStart])
			{
				$fields[] = $pageStart;
				$values[] = trim(strtolower($this->vars[$pageStart]));
				if(array_key_exists($pageEnd, $this->vars) && $this->vars[$pageEnd])
				{
					$fields[] = $pageEnd;
					$values[] = trim(strtolower($this->vars[$pageEnd]));
				}
			}
			if(array_key_exists($paragraph, $this->vars) && $this->vars[$paragraph])
			{
				$fields[] = $paragraph;
				$values[] = trim(strtolower($this->vars[$paragraph]));
			}
			if(array_key_exists($section, $this->vars) && $this->vars[$section])
			{
				$fields[] = $section;
				$values[] = trim(strtolower($this->vars[$section]));
			}
			if(array_key_exists($chapter, $this->vars) && $this->vars[$chapter])
			{
				$fields[] = $chapter;
				$values[] = trim(strtolower($this->vars[$chapter]));
			}
			$this->db->insert($typeTable, $fields, $values);
			$lastAutoId = $this->db->lastAutoId($typeTable);
			$this->db->deleteCache('cacheMetadataCreators');
			$this->db->deleteCache('cacheMetadataCollections');
			$this->db->deleteCache('cacheMetadataPublishers');
			$fields = $values = array();
			$fields[] = $textTableId;
			$values[] = $lastAutoId;
			$fields[] = $typeText;
			$values[] = trim($this->vars[$typeText]);
			if($userId)
			{
				$fields[] = $addUserId;
				$values[] = $userId;
			}
			$this->db->insert($textTable, $fields, $values);
			$this->summary(1, $summaryType);
// Write comments table
			if(array_key_exists($typeComment, $this->vars) && $this->vars[$typeComment])
				$this->insertComment($lastAutoId, $userId, $textCommentId, $commentAddUserId, $typeComment, $privateComment, 
				$commentTimestamp, $commentTable);
			$this->writeKeywords($lastAutoId, $keywordTableId);
		}
// else edit/delete
		else
		{
// if quoteText is empty, delete the row
			if(!array_key_exists('commentOnly', $this->vars) && !trim($this->html->stripHtml($this->vars[$typeText])))
			{
				$addEdit = 'deleted';
				$this->db->formatConditions(array($textId => $this->vars[$textId]));
				$this->db->delete($typeTable);
				$this->db->formatConditions(array($textTableId => $this->vars[$textId]));
				$this->db->delete($textTable);
				$this->db->formatConditions(array($textCommentId => $this->vars[$textId]));
				$this->db->delete($commentTable);
				$this->db->formatConditions(array($keywordTableId => $this->vars[$textId]));
				$this->db->delete('resource_keyword');
				$this->db->optimize($typeTable);
				$this->db->optimize($textTable);
				$this->db->optimize($commentTable);
				$this->db->optimize('resource_keyword');
				$metadataExists = $this->summary(-1, $summaryType);
// remove cache files for keywords
				$this->db->deleteCache('cacheResourceKeywords');
				$this->db->deleteCache('cacheMetadataKeywords');
				$this->db->deleteCache('cacheQuoteKeywords');
				$this->db->deleteCache('cacheParaphraseKeywords');
				$this->db->deleteCache('cacheMusingKeywords');
// Remove these cache files if no metadata left in resource
				if(!$metadataExists)
				{
					$this->db->deleteCache('cacheMetadataCreators');
					$this->db->deleteCache('cacheMetadataCollections');
					$this->db->deleteCache('cacheMetadataPublishers');
				}
			}
			else
			{
				$addEdit = 'edited';
// Quote/paraphrase _text table
				$updateArray = array();
				if(!array_key_exists('commentOnly', $this->vars))
				{
					$this->db->formatConditions(array($textTableId => $this->vars[$textId]));
					$updateArray[$typeText] = trim($this->vars[$typeText]);
					$this->db->update($textTable, $updateArray);
					$this->writeKeywords($this->vars[$textId], $keywordTableId);
				}
// Comment
				$updateArray = array();
				if(array_key_exists($typeComment, $this->vars) && trim($this->vars[$typeComment]))
				{
// Is this a new comment or are we editing an old comment?
					$this->db->formatConditions(array($textCommentId => $this->vars[$textId]));
					$this->db->formatConditions(array($commentAddUserId => $userId));
					$recordset = $this->db->select($commentTable, $textCommentId);
					if($this->db->numRows($recordset))  // edit existing comment
					{
						$updateArray[$typeComment] = trim($this->vars[$typeComment]);
						if(array_key_exists('private', $this->vars) && ($this->vars['private'] == 'N'))
							$updateArray[$privateComment] = 'N';
						else if(array_key_exists('private', $this->vars) && (is_numeric($this->vars['private'])))
							$updateArray[$privateComment] = $this->vars['private'];
						else
							$updateArray[$privateComment] = 'Y';
						$updateArray[$commentTimestamp] = $this->db->formatTimestamp();
						$this->db->formatConditions(array($textCommentId => $this->vars[$textId]));
						$this->db->formatConditions(array($commentAddUserId => $userId));
						$this->db->update($commentTable, $updateArray);
					}
// new comment
					else
						$this->insertComment($this->vars[$textId], $userId, $textCommentId, $commentAddUserId, $typeComment, 
							$privateComment, $commentTimestamp, $commentTable);
				}
// remove comment row
				else
				{
					$this->db->formatConditions(array($textCommentId => $this->vars[$textId]));
					$this->db->delete($commentTable);
					$this->db->optimize($commentTable);
				}
				if(!array_key_exists('commentOnly', $this->vars))
				{
// Quote/paraphrase main type table
					$updateArray = $nulls = array();
// page number lowercased in case roman numerals input!
					if(array_key_exists($pageStart, $this->vars) && $this->vars[$pageStart])
					{
						$updateArray[$pageStart] = 
							trim(strtolower($this->vars[$pageStart]));
						if(array_key_exists($pageEnd, $this->vars) && $this->vars[$pageEnd])
						{
							$updateArray[$pageEnd] = 
								trim(strtolower($this->vars[$pageEnd]));
						}
						else
							$nulls[] = $pageEnd;
					}
					else
					{
						$nulls[] = $pageStart;
						$nulls[] = $pageEnd;
					}
					if(array_key_exists($paragraph, $this->vars) && $this->vars[$paragraph])
						$updateArray[$paragraph] = trim($this->vars[$paragraph]);
					else
						$nulls[] = $paragraph;
					if(array_key_exists($section, $this->vars) && $this->vars[$section])
						$updateArray[$section] = trim($this->vars[$section]);
					else
						$nulls[] = $section;
					if(array_key_exists($chapter, $this->vars) && $this->vars[$chapter])
						$updateArray[$chapter] = trim($this->vars[$chapter]);
					else
						$nulls[] = $chapter;
					if(!empty($updateArray))
					{
						$this->db->formatConditions(array($textId => $this->vars[$textId]));
						$this->db->update($typeTable, $updateArray);
					}
					if(!empty($nulls))
					{
						$this->db->formatConditions(array($textId => $this->vars[$textId]));
						$this->db->updateNull($typeTable, $nulls);
					}
				}
			}
		}
		include_once("core/modules/email/EMAIL.php");
		$emailClass = new EMAIL();
		if(!$emailClass->notify($this->vars['resourceId']))
			$this->badInput->close($this->errors->text("inputError", "mail"));
		return $addEdit;
	}
/**
* Insert a comment
*/
	private function insertComment($lastAutoId, $userId, $textCommentId, $commentAddUserId, $typeComment, $privateComment, 
		$commentTimestamp, $commentTable)
	{
		$fields = $values = array();
		$fields[] = $textCommentId;
		$values[] = $lastAutoId;
		if($userId)
		{
			$fields[] = $commentAddUserId;
			$values[] = $userId;
		}
		$fields[] = $typeComment;
		$values[] = trim($this->vars[$typeComment]);
		if(array_key_exists('private', $this->vars) && ($this->vars['private'] == 'N'))
		{
			$fields[] = $privateComment;
			$values[] = 'N';
		}
		else if(array_key_exists('private', $this->vars) && (is_numeric($this->vars['private'])))
		{
			$fields[] = $privateComment;
			$values[] = $this->vars['private'];
		}
		else
		{
			$fields[] = $privateComment;
			$values[] = 'Y';
		}
		$fields[] = $commentTimestamp;
		$values[] = $this->db->formatTimestamp();
		$this->db->insert($commentTable, $fields, $values);
	}
// update or insert resource_summary
	public function summary($incDec, $field)
	{
		$metadataExists = TRUE;
		if($field == 'resourcesummaryQuotes')
			$totalsField = 'databasesummaryTotalQuotes';
		else if($field == 'resourcesummaryParaphrases')
			$totalsField = 'databasesummaryTotalParaphrases';
		else
			$totalsField = 'databasesummaryTotalMusings';
		$this->db->formatConditions(array('resourcesummaryId' => $this->vars['resourceId']));
		if($this->db->numRows($this->db->select('resource_summary', 'resourcesummaryId'))) // update
			$this->db->updateSingle('resource_summary', $this->db->formatFields($field) . "=" .  
			"COALESCE(" . $this->db->formatFields($field) . "+" . 
			$this->db->tidyInput($incDec) . ", " . $this->db->tidyInput($incDec) . ")");
		else  // insert
			$this->db->insert('resource_summary', array('resourcesummaryId', $field), array($this->vars['resourceId'], 1));
// if a decrement leaves nothing, delete row.
		$this->db->formatConditions(array('resourcesummaryId' => $this->vars['resourceId']));
		$recordset = $this->db->select(array('resource_summary'), 
			array('resourcesummaryMusings', 'resourcesummaryParaphrases', 'resourcesummaryQuotes'));
		$row = $this->db->fetchRow($recordset);
		if(!$row['resourcesummaryMusings'] && !$row['resourcesummaryParaphrases'] && !$row['resourcesummaryQuotes'])
		{
			$this->db->formatConditions(array('resourcesummaryId' => $this->vars['resourceId']));
			$this->db->delete('resource_summary');
			$this->db->optimize('resource_summary');
			$metadataExists = FALSE;
		}
// update master summary table
		$this->db->updateSingle('database_summary', $this->db->formatFields($totalsField) .  
			"=" . $this->db->formatFields($totalsField) . "+" . $this->db->tidyInput($incDec));
		return $metadataExists;
	}
/**
* Write new keywords to keyword table and gather keyword ids
*/
	public function writekeywords($rkId, $field)
	{
		$deleteCache = FALSE;
		if($this->vars['keywords'])
		{
			$kIds = $this->keyword->writeKeywordTable($this->vars);
			if(is_array($kIds))
			{
// first delete existing rows then insert new ones
				$this->db->formatConditions(array($field => $rkId));
				$this->db->delete('resource_keyword');
				foreach($kIds as $kId)
					$this->db->insert('resource_keyword', array($field, 'resourcekeywordKeywordId'), array($rkId, $kId));
				$deleteCache = TRUE;
			}
		}
		else
		{
			$this->db->formatConditions(array($field => $rkId));
			$resultset = $this->db->select('resource_keyword', $field);
			if($this->db->numRows($resultset))
			{
				$this->db->formatConditions(array($field => $rkId));
				$this->db->delete('resource_keyword');
				$deleteCache = TRUE;
			}
		}
		if($deleteCache)
		{
// remove cache files for keywords
			$this->db->deleteCache('cacheResourceKeywords');
			$this->db->deleteCache('cacheMetadataKeywords');
			$this->db->deleteCache('cacheQuoteKeywords');
			$this->db->deleteCache('cacheParaphraseKeywords');
			$this->db->deleteCache('cacheMusingKeywords');
			$this->db->deleteCache('cacheKeywords');
		}
		$this->keyword->removeHanging();
	}
}
?>
