<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*
* VIEWBIBTEX View single resource as BibTeX
*
* NB Some variables and methods are public for use with the importexportbib plugin -- DO NOT CHANGE!
*
* @author Mark Grimshaw
* @version 1
*/

class VIEWBIBTEX
{
private $db;
public $vars; // public for use with importexportbib plugin
private $errors;
private $messages;
private $session;
private $template;
private $form;
private $html;
private $table;
private $badInput;
private $res;
private $bibConfig;
private $map;
private $utf8;
private $user;
private $constants;
private $startField;
private $endField;
private $metadataFields = array();
private $customIds = array();
private $titleString = array();
private $rawString = array();
public $rawStringString = ''; // public for use with importexportbib plugin
private $stringIdArray = array();
private $strValues = array();
private $strKeys = array();
private $rawEntries = array();
private $userId;
private $rawCitation = FALSE;
private $storedBibtexKey;
private $spChPlain;
private $editionArray;
private $monthArray;
private $cite = array();
public $resourceId; // public for use with importexportbib plugin
private $convertTex = FALSE;
private $keywordSeparator = ',';
public $customFieldString = FALSE;

// Constructor
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->session = FACTORY_SESSION::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->res = FACTORY_RESOURCECOMMON::getInstance();
		$this->bibConfig = FACTORY_BIBTEXCONFIG::getInstance();
		$this->map = FACTORY_BIBTEXMAP::getInstance();
		$this->utf8 = FACTORY_UTF8::getInstance();
		$this->user = FACTORY_USER::getInstance();
// need to use English constants for BibTeX
		include_once("languages/en/CONSTANTS.php");
		$this->constants = new CONSTANTS_en();
		$this->startField = $this->endField = "\""; // default field enclosures
		$this->userId = $this->session->getVar("setup_UserId");
// Load bibConfig arrays
		$this->bibConfig->bibtex();
// load default arrays
		$this->loadArrays();
		GLOBALS::setOutputHeading($this->messages->text("heading", "exportBibtex"));
	}
// display pop-up for a single resource
	public function display()
	{
		if(!array_key_exists('id', $this->vars))
		{
			$this->badInput->closeType = 'closeNoMenu';
			$this->badInput->close($this->messages->text('inputError', 'missing'));
		}
// START set up environment
		if(!array_key_exists('resubmit', $this->vars) && ($this->session->getArray("export") !== FALSE))
		{
// custom field mapping
			foreach($this->session->getArray("export") as $key => $value)
			{
				if(strpos($key, 'Map_') == 0)
					$this->vars[$key] = $value;
			}
		}
		$metadataFields = array("Quotation", "QuotationComment", "Paraphrase", "ParaphraseComment", "Musing");
		foreach($metadataFields as $field)
		{
// first time so few values in $this->vars -- check session
			if(!array_key_exists('resubmit', $this->vars) && ($value = $this->session->getVar("export_$field")))
				$this->vars[$field] = $value;
			else if(!array_key_exists('resubmit', $this->vars))
				$this->vars[$field] = '';
		}
		if(!array_key_exists('resubmit', $this->vars))
		{
			if($this->session->getVar('export_MergeStored'))
				$this->vars['MergeStored'] = TRUE;
			if($this->session->getVar('export_UseOriginalCitation'))
				$this->vars['UseOriginalCitation'] = TRUE;
			if($this->session->getVar('export_ShortString'))
				$this->vars['ShortString'] = TRUE;
			if($this->session->getVar('export_EncloseField')) // braces
				$this->vars['EncloseField'] = 'B';
			else // double quotes
				$this->vars['EncloseField'] = 'Q';
			if($this->session->getVar('export_CharacterSet')) // Latin + TeX
				$this->vars['CharacterSet'] = 'T';
			else // UTF-8
				$this->vars['CharacterSet'] = 'U';
			if($this->session->getVar('export_KeywordSeparator')) // ;
				$this->vars['KeywordSeparator'] = ';';
			else // ,
				$this->vars['KeywordSeparator'] = ',';
		}
		$pString = $this->process();
		$pString = str_replace("\n", "<br>", $pString);
		$pString .= $this->html->hr();
// These form elements kept here so the importexportbib plugin can use functions in this class such as options()
		$pString .= $this->form->formHeader("resource_VIEWBIBTEX_CORE");
		$pString .= $this->form->hidden('method', 'display');
		$pString .= $this->form->hidden("id", $this->vars['id']);
		$pString .= $this->options();
		GLOBALS::buildOutputString($pString);
		FACTORY_CLOSENOMENU::getInstance();
	}
/**
* Display options
*/
	public function options()
	{
		$pString = $this->form->hidden("resubmit", TRUE);
		$checked = $this->session->getVar("export_MergeStored") ? TRUE : FALSE;
		$pString .= $this->html->p($this->messages->text("misc", "mergeStored") . "&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, "MergeStored", $checked));
		$checked = $this->session->getVar("export_UseOriginalCitation") ? TRUE : FALSE;
		$pString .= $this->html->p($this->messages->text("misc", "useOriginalCitation") . "&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, "UseOriginalCitation", $checked));
		$checked = $this->session->getVar("export_ShortString") ? TRUE : FALSE;
		$pString .= $this->html->p($this->messages->text("misc", "shortString") . "&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, "ShortString", $checked));
		$pString .= $this->table->tableStart('generalTable', 1);
		$pString .= $this->table->trStart(FALSE);
		if($this->session->getVar("export_EncloseField")) // TRUE == {...}
		{
			$string = $this->form->radioButton(FALSE, "EncloseField", "Q", FALSE) . "&nbsp;&nbsp;" . 
				$this->messages->text("misc", "bibExportQuotes");
			$string .= $this->html->br() . $this->form->radioButton(FALSE, "EncloseField", "B", TRUE) . "&nbsp;&nbsp;" . 
				$this->messages->text("misc", "bibExportBraces");
			$pString .= $this->table->td($string);
		}
		else // FALSE == "..."
		{
			$string = $this->form->radioButton(FALSE, "EncloseField", "Q", TRUE) . "&nbsp;&nbsp;" . 
				$this->messages->text("misc", "bibExportQuotes");
			$string .= $this->html->br() . $this->form->radioButton(FALSE, "EncloseField", "B", FALSE) . "&nbsp;&nbsp;" . 
				$this->messages->text("misc", "bibExportBraces");
			$pString .= $this->table->td($string);
		}
		if($this->session->getVar("export_CharacterSet")) // TRUE == TeX
		{
			$string = $this->form->radioButton(FALSE, "CharacterSet", "U", FALSE) . "&nbsp;&nbsp;" . 
				$this->messages->text("misc", "bibExportCharacterSetUTF");
			$string .= $this->html->br() . $this->form->radioButton(FALSE, "CharacterSet", "T", TRUE) . "&nbsp;&nbsp;" . 
				$this->messages->text("misc", "bibExportCharacterSetTex");
			$pString .= $this->table->td($string);
		}
		else // FALSE == UTF-8
		{
			$string = $this->form->radioButton(FALSE, "CharacterSet", "U", TRUE) . "&nbsp;&nbsp;" . 
				$this->messages->text("misc", "bibExportCharacterSetUTF");
			$string .= $this->html->br() . $this->form->radioButton(FALSE, "CharacterSet", "T", FALSE) . "&nbsp;&nbsp;" . 
				$this->messages->text("misc", "bibExportCharacterSetTex");
			$pString .= $this->table->td($string);
		}
		if($this->session->getVar("export_KeywordSeparator")) // TRUE == ';'
		{
			$string = $this->form->radioButton(FALSE, "KeywordSeparator", ",", FALSE) . "&nbsp;&nbsp;" . 
				$this->messages->text("misc", "bibExportKeywordSeparatorComma");
			$string .= $this->html->br() . $this->form->radioButton(FALSE, "KeywordSeparator", ";", TRUE) . "&nbsp;&nbsp;" . 
				$this->messages->text("misc", "bibExportKeywordSeparatorSemicolon");
			$pString .= $this->table->td($string);
		}
		else // FALSE == ';'
		{
			$string = $this->form->radioButton(FALSE, "KeywordSeparator", ",", TRUE) . "&nbsp;&nbsp;" . 
				$this->messages->text("misc", "bibExportKeywordSeparatorComma");
			$string .= $this->html->br() . $this->form->radioButton(FALSE, "KeywordSeparator", ";", FALSE) . "&nbsp;&nbsp;" . 
				$this->messages->text("misc", "bibExportKeywordSeparatorSemicolon");
			$pString .= $this->table->td($string);
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		if(!$this->customFieldString) // cf importexportbib plugin
			$pString .= $this->getCustomFields();
		else if(($this->customFieldString != -1))
			$pString .= $this->customFieldString;
// Export metadata fields?
		$pString .= $this->html->p($this->html->b($this->messages->text("misc", "exportMetadata1")) . "&nbsp;&nbsp;" . 
			$this->messages->text("misc", "exportMetadata2"));
		$pString .= $this->table->tableStart('generalTable', 1) . $this->table->trStart();
		$label = $this->messages->text("metadata", "quotes");
		$text = $this->session->getVar("export_Quotation");
		$pString .= $this->table->td($this->form->textInput($label, "Quotation", $text, 10));
		$label = $this->messages->text("metadata", "quoteComments");
		$text = $this->session->getVar("export_QuotationComment");
		$pString .= $this->table->td($this->form->textInput($label, "QuotationComment", $text, 10));
		$label = $this->messages->text("metadata", "paraphrases");
		$text = $this->session->getVar("export_Paraphrase");
		$pString .= $this->table->td($this->form->textInput($label, "Paraphrase", $text, 10));
		$label = $this->messages->text("metadata", "paraphraseComments");
		$text = $this->session->getVar("export_ParaphraseComment");
		$pString .= $this->table->td($this->form->textInput($label, "ParaphraseComment", $text, 10));
		$label = $this->messages->text("metadata", "musings");
		$text = $this->session->getVar("export_Musing");
		$pString .= $this->table->td($this->form->textInput($label, "Musing", $text, 10));
		$pString .= $this->table->trEnd() . $this->table->tableEnd();
		$pString .= $this->html->br() . $this->form->formSubmit('Submit');
		$pString .= $this->form->formEnd();
		return $pString;
	}
// Check if there any custom fields in the SQL set and provide options to map these to bibtex fields
	private function getCustomFields()
	{
		if(!$this->db->numRows($this->db->select('custom', 'customId')))
			return FALSE;
		$this->db->leftJoin('resource_custom', 'resourcecustomCustomId', 'customId');
		$this->db->formatConditions(array('resourcecustomResourceId' => $this->resourceId));
		$recordset = $this->db->select('custom', array('resourcecustomId', 'customLabel', 'customSize', 'resourcecustomShort', 
			'resourcecustomLong', 'resourcecustomAddUserIdCustom', 'resourcecustomEditUserIdCustom'));
		while($row = $this->db->fetchRow($recordset))
		{
			if($row['resourcecustomId'])
				$customLabels[$row['resourcecustomId']] = stripslashes($row['customLabel']);
		}
		if(!isset($customLabels) || empty($customLabels))
			return FALSE;
		$customLabels = array_unique($customLabels);
		$pString = $this->html->p($this->html->b($this->messages->text("misc", "customFieldMap")) . 
			' ' . $this->messages->text("misc", "customFieldMap2"));
		foreach($customLabels as $id => $label)
		{
			$text = $this->session->getVar("export_Map_$id");
			$pString .= $this->html->p($this->form->textInput($label, "Map_$id", $text));
		}
		return $pString;
	}
/**
* Process and display
*/
	private function process()
	{
		$this->writeSession();
		$pString = $this->getData();
		if($this->formatRawString())
			return $this->rawStringString . $pString;
		return $pString;
	}
// write session
	public function writeSession()
	{
		if(isset($this->vars['MergeStored']))
			$this->session->setVar('export_MergeStored', $this->vars['MergeStored']);
		else
			$this->session->delVar('export_MergeStored');
		if(isset($this->vars['UseOriginalCitation']))
			$this->session->setVar('export_UseOriginalCitation', $this->vars['UseOriginalCitation']);
		else
			$this->session->delVar('export_UseOriginalCitation');
		if(isset($this->vars['ShortString']))
			$this->session->setVar('export_ShortString', $this->vars['ShortString']);
		else
			$this->session->delVar('export_ShortString');
		if(array_key_exists('EncloseField', $this->vars) && $this->vars['EncloseField'] == 'Q')
		{
			$this->session->setVar('export_EncloseField', FALSE);
			$this->startField = "\"";
			$this->endField = "\"";
		}
		else
		{
			$this->session->setVar('export_EncloseField', TRUE);
			$this->startField = "{";
			$this->endField = "}";
		}
		if(array_key_exists('CharacterSet', $this->vars) && $this->vars['CharacterSet'] == 'U')
			$this->session->setVar('export_CharacterSet', FALSE);
		else
		{
			$this->session->setVar('export_CharacterSet', TRUE);
			$this->convertTex = TRUE;
		}
		if(array_key_exists('KeywordSeparator', $this->vars) && $this->vars['KeywordSeparator'] == ',')
			$this->session->setVar('export_KeywordSeparator', FALSE);
		else
		{
			$this->session->setVar('export_KeywordSeparator', TRUE);
			$this->keywordSeparator = ';';
		}
		$checkDuplicates = array();
		$duplicateMapping = FALSE;
		foreach($this->vars as $key => $value)
		{
			$split = explode("Map_", $key); // custom fields mapping
			if(sizeof($split) == 2)
			{
				$value = trim($value);
				if($value)
				{
					$this->session->setVar("export_$key", $value);
					if(!$duplicateMapping && (array_search(strtolower($value), $checkDuplicates) !== FALSE))
						$duplicateMapping = TRUE;
					$this->customIds[$split[1]] = $value;
					$checkDuplicates[] = strtolower($value);
				}
				else
					$this->session->delVar("export_$key");
			}
		}
// Check for metadata input and write to session
		$metadataFields = array("Quotation", "QuotationComment", "Paraphrase", "ParaphraseComment", "Musing");
		foreach($metadataFields as $field)
		{
			$input = trim($this->vars[$field]);
			if($input)
			{
				if(($field == 'QuotationComment') || ($field == 'ParaphraseComment')) // may be duplicated
				{
					$this->session->setVar("export_$field", $input);
					$this->metadataFields[$field] = $input;
				}
				else if(array_search(strtolower($input), $checkDuplicates) === FALSE)
				{
					$this->session->setVar("export_$field", $input);
					$this->metadataFields[$field] = $input;
					$checkDuplicates[] = strtolower($input);
				}
				else
					$duplicateMapping = TRUE;
			}
			else
				$this->session->delVar("export_$field");
		}
// if duplicate custom mapping found, fail
		if($duplicateMapping)
			$this->badInput->close($this->errors->text("inputError", "duplicateFieldNames"));
	}
// format @STRING strings
	public function formatRawString()
	{
		$string = array();
		if(!empty($this->rawString))
		{
			foreach($this->titleString as $key => $value)
			{
				if(!array_key_exists($key, $this->rawString))
					$this->rawString[$key] = $value;
			}
			foreach($this->rawString as $key => $value)
			{
				$key = preg_replace("/[^a-zA-Z0-9]/", '', $key);
				if(is_numeric(substr($key, 0)))
					$key = "string$key";
				$string[] = "@STRING" . '{' . "$key = $value" . '}';
			}
			$this->rawStringString .= implode("\n", $string) . "\n\n";
			return TRUE;
		}
		else if(!empty($this->titleString))
		{
			foreach($this->titleString as $key => $value)
			{
				if($this->session->getVar("export_shortString"))
					$value = $key;
				$key = preg_replace("/[^a-zA-Z0-9]/", '', $key);
				if(is_numeric(substr($key, 0)))
					$key = "string$key";
				$string[] = "@STRING" . '{' . "$key = $value" . '}';
			}
			$this->rawStringString .= implode("\n", $string) . "\n\n";
			return TRUE;
		}
		return FALSE;
	}
// get data from database
	public function getData($resultset = FALSE)
	{
		$customSelect = $howPublishedDone = FALSE;
		$pluginExport = TRUE;
		if(!$resultset)
		{
			$this->resourceId = $this->vars['id'];
			$resultset = $this->res->getResource($this->resourceId);
			$pluginExport = FALSE;
		}
		if(!$this->db->numRows($resultset))
			$this->badInput->close($this->messages->text("resources", "noResult"));
		$resourceIds = array();
		$pString = '';
		while($row = $this->db->fetchRow($resultset))
		{
			if(array_search($row['resourceId'], $resourceIds) === FALSE)
				$resourceIds[] = $row['resourceId'];
			else
				continue;
			if($pluginExport)
				$this->resourceId = $row['resourceId'];
			$this->storedBibtexKey = $row['resourceBibtexKey'];
			$entryArray = array();
			$longtitle = $shorttitle = FALSE;
			$this->raw();
			$cite = $this->citeFormat($row);
			foreach($this->map->{$row['resourceType']}['resource_creator'] as $wkField => $bibField)
			{
				if(!$creators = $this->grabNames($wkField, $bibField))
					continue;
				else
					$entryArray[] = $creators;
			}
			$entryArray[] = "title = " . $this->convertCharacter($this->formatTitle($row));
	// For book_chapter, 'title' is bibtex 'chapter' and 'collectionTitle' is bibtex 'title'
			if($row['resourceType'] == 'book_chapter')
				$entryArray[] = "chapter = " . $this->startField . $row['resourceTitle'] . $this->endField . ",";
			if(($row['resourceType'] == 'web_article') || ($row['resourceType'] == 'database')) // 'misc' types
			{
				if($url = $this->grabUrl())
					$entryArray[] = $url;
				$hwoPublishedDone = TRUE;
			}
			list($publisher, $address) = $this->grabPublisher($row);
			if($publisher)
			{
				$entryArray[] = $publisher;
				if($address)
					$entryArray[] = $address;
			}
			foreach($this->map->{$row['resourceType']} as $table => $tableArray)
			{
				if(($table == 'resource_creator') || ($table == 'howpublished') || ($table == 'resource_publisher'))
					continue;
				foreach($tableArray as $wkField => $bibField)
				{
					if(isset($row[$wkField]) && $row[$wkField])
					{
						if(($wkField == 'collectionTitle') && isset($row['collectionTitleShort']) && $row['collectionTitleShort'])
						{
							$short = $row['collectionTitleShort'];
							$title = $row['collectionTitle'];
							$long = preg_quote($title);
	// preg_quote doesn't escape '/'
							$long = "/^" . str_replace('/', '\/', $long) . "$/u";
							if(!array_key_exists($short, $this->titleString))
								$this->titleString[$short] = '"' . $title . '"';
							$entryArray[] = "$bibField = " . $this->convertCharacter2($title, $short, $long);
						}
						else if($wkField == 'resourcetextUrls')
						{
							$urls = unserialize(base64_decode($row[$wkField]));
							$entryArray[] = "$bibField = " . array_shift($urls);
						}
						else
							$entryArray[] = "$bibField = " . $this->convertCharacter($row[$wkField]);
					}
				}
			}
			if($item = $this->pageFormat($row))
				$entryArray[] = "pages = ". $this->convertCharacter($item);
			if($item = $this->keywordFormat())
				$entryArray[] = "keywords = " . $this->convertCharacter($item);
			list($note, $abstract) = $this->grabNoteAbstract();
			if($note)
				$entryArray[] = "note = " . $this->convertCharacter($this->parseCitation($note));
			if($abstract)
				$entryArray[] = "abstract = " . $this->convertCharacter($this->parseCitation($abstract));
	// Deal with month/day
			if($row['resourcemiscField3'])
			{
				if($item = $this->formatDate($row))
	// No double quotes for month
					$entryArray[] = "month = $item,";
			}
	// 'howpublished'
			else if(!$howPublishedDone && isset($this->map->types[$row['resourceType']]) && 
				($this->map->types[$row['resourceType']] == 'misc') && 
				(array_key_exists('howpublished', $this->map->{$row['resourceType']})))
			{
				$field = $this->map->{$row['resourceType']}['howpublished'];
				if(!array_key_exists($field, $row))
					$entryArray[] = "howpublished = " . $this->convertCharacter(ucfirst($field));
				else
					$entryArray[] = "howpublished = " . $this->convertCharacter(ucfirst($row[$field]));
			}
			if(($row['resourceType'] == 'thesis') && 
				(($row['resourceField1'] == 'doctoral') || ($row['resourceField1'] == 'PhD') || ($row['resourceField1'] == 'EdD')))
				$pString .= "@" . 'phdthesis' . "{" . "$cite,\n\t";
			else if($row['resourceType'] == 'thesis') // masters + unknown
				$pString .= "@" . 'mastersthesis' . "{" . "$cite,\n\t";
			else
				$pString .= "@" . $this->map->types[$row['resourceType']] . "{" . "$cite,\n\t";
			if($this->session->getVar("export_MergeStored") && !empty($this->rawEntries))
			{
				foreach($this->rawEntries as $key => $value)
				{
					if(($key == 'month') && ($value{0} == '"'))
					{
						$value = substr($value, 1);
						$value = substr($value, 0, -1);
					}
					$entryArray[] = $key . " = " . $value . ",";
				}
			}
	// Deal with any custom field mappings
			if(!empty($this->customIds))
			{
				$this->db->formatConditionsOneField(array_keys($this->customIds), 'resourcecustomId');
				$this->db->formatConditions(array('resourcecustomResourceId' => $this->resourceId));
				$customSet = $this->db->select('resource_custom', 
					array('resourcecustomId', 'resourcecustomLong', 'resourcecustomShort'));
				while($customRow = $this->db->fetchRow($customSet))
				{
					if($customRow['resourcecustomShort'])
						$entryArray[] = $this->customIds[$customRow['resourcecustomId']] . " = " . 
						$this->convertCharacter($customRow['resourcecustomShort']);
					if($customRow['resourcecustomLong'])
						$entryArray[] = $this->customIds[$customRow['resourcecustomId']] . " = " . 
						$this->convertCharacter($this->html->stripHtml($customRow['resourcecustomLong']));
				}
			}
	// Deal with metadata
			if(array_key_exists('Quotation', $this->metadataFields))
				$this->getMetadataQ($entryArray);
			if(array_key_exists('Paraphrase', $this->metadataFields))
				$this->getMetadataP($entryArray);
			if(array_key_exists('Musing', $this->metadataFields))
				$this->getMetadataM($entryArray);
	// remove final ',' from last field
			$lastEntryIndex = sizeof($entryArray) - 1;
			$entryArray[$lastEntryIndex] = 
				substr($entryArray[$lastEntryIndex], 0, strlen($entryArray[$lastEntryIndex]) - 1);
			$pString .= join("\n\t", $entryArray) . "\n}\n\n";
		}
		return $pString;
	}
// grab any stored data and @strings for this resource from import_raw
	private function raw()
	{
		$this->db->formatConditions(array('importrawId' => $this->resourceId));
		$this->db->formatConditions(array('importrawImportType' => 'bibtex'));
		$recordset = $this->db->select('import_raw', array('importrawText', 'importrawStringId', 'importrawImportType'));
		if(!$this->db->numRows($recordset))
			return;
		$stringIdFound = FALSE;
		while($row = $this->db->fetchRow($recordset))
		{
			$rawEntries = unserialize(base64_decode($row['importrawText']));
			foreach($this->stringIdArray as $stringId)
			{
				if($row['importrawStringId'] == $stringId)
				{
					$stringIdFound = TRUE;
					break;
				}
			}
			if($stringIdFound)
				break;
			if(array_search($row['importrawStringId'], $this->stringIdArray) === FALSE)
			{
				$this->db->formatConditions(array('bibtexstringId' => $row['importrawStringId']));
				$recordset = $this->db->select('bibtex_string', 'bibtexstringText');
				$rawString = unserialize(base64_decode($this->db->fetchOne($recordset))) . "\n\n";
				if($rawString)
					preg_match_all("/@STRING{(.*)}/", $rawString, $strings);
				foreach($strings[1] as $string)
				{
					$split = explode("=", $string, 2);
					$key = trim($split[0]);
					if(array_search($key, $this->rawString) === FALSE)
						$this->rawString[$key] = trim($split[1]);
				}
				$this->orderArrayByKeyLength();
				foreach($this->rawString as $key => $value)
				{
					if(preg_match("/^[\"{](.*)[\"}]$/", trim($value), $matches))
						$value = $matches[1];
					$value = "/^" . preg_quote($value, "/") . "$/";
					if(!array_search($value, $this->strValues))
						$this->strValues[] = $value;
					if(!array_search($key, $this->strKeys))
						$this->strKeys[] = $key;
				}
				$this->stringIdArray[] = $row['importrawStringId'];
				break;
			}
		}
		$this->strValues = array_unique($this->strValues);
		$this->strKeys = array_unique($this->strKeys);
		if($rawEntries)
		{
			$rawEntries = explode("\n", $rawEntries);
			array_pop($rawEntries); // always an empty array at end so get rid of it.
			foreach($rawEntries as $entries)
			{
				$entry = explode("=", $entries, 2);
				if(!trim($entry[1]))
					continue;
				if(trim($entry[0]) == 'citation')
					$this->rawCitation = trim($entry[1]);
				else
				{
					$key = trim($entry[0]);
					$value = trim($entry[1]);
					if($this->startField == "{")
					{
						$match = "/^(\")(.*)(\")$/";
						if(preg_match($match, $value, $matches))
							$value = $this->startField . $matches[2] . $this->endField;
					}
					else if($this->startField == "\"")
					{
						$match = "/^(\{)(.*)(\})$/";
						if(preg_match($match, $value, $matches))
							$value = $this->startField . $matches[2] . $this->endField;
					}
					$this->rawEntries[$key] = $value;
				}
			}
		}
	}
// load default arrays
	private function loadArrays()
	{
		$temp = $this->bibConfig->bibtexSpCh;
// We need to remove first two elements of array as we don't want to convert '{' and '}'. Because the keys of the original 
// array are hexadecimal characters, this means array_shift will treat them as integer keys and will modify the array 
// keys - we don't want this!
		$index = 0;
		foreach($temp as $key => $value)
		{
			if($index >= 2)
				$this->spCh[$key] = $value;
			$index++;
		}
		$this->spChPlain = $this->bibConfig->bibtexSpChPlain;
		$this->constants->convertNumbers();
		$this->editionArray = $this->constants->cardinalToOrdinalWord();
		$this->monthArray = $this->constants->monthToLongName();
	}
// store surname of first author for BibTeX citation purposes.
// $this->cite[] holds existing citations which must not be repeated.
// Ensure $cite has no spaces, commas etc.
// If $id, we're requesting a citation ID for  cited resource in notes, abstracts or metadata
	private function citeFormat()
	{
		if(($this->session->getVar("export_UseOriginalCitation")) && 
			$this->rawCitation && !array_key_exists($this->rawCitation, $this->cite))
		{
			$this->cite[] = trim($this->rawCitation);
			return trim($this->rawCitation);
		}
		else if(!$this->session->getVar("setup_UseWikindxKey"))
			return $this->storedBibtexKey;
// Use wikindx=generated keys from this point on
		$cite = $this->wikindxCiteKey();
		return $cite . '.' . $this->resourceId;
	}
// Generate wikindx citation key
	private function wikindxCiteKey($id = FALSE)
	{
		if(!$id)
			$id = $this->resourceId;
		$this->db->formatConditions(array('resourcecreatorResourceId' => $id));
		$recordset = $this->db->select('resource_creator', 'resourcecreatorCreatorSurname', TRUE);
		if(!$this->db->numRows($recordset)) // shouldn't happen, but just in case
			$cite = 'anon';
		else
		{
			$name = $this->db->fetchOne($recordset);
			if(!$name)
				$cite = 'anon';
			else
			{
				$cite = $this->convertCharacter($name, 'plain');
				$cite = preg_replace("/\W/", '', $cite);
			}
		}
		return $cite;
	}
// convert string from raw database string fields.
	private function convertCharacter($string, $type = FALSE)
	{
		$string = html_entity_decode(stripslashes($string), ENT_QUOTES, 'UTF-8');
		if($this->convertTex)
			$string = $this->convertTex($string, $type);
// Do @string substitutions
		if(!empty($this->strValues))
		{
			$replace = preg_replace($this->strValues, $this->strKeys, $string);
// If a replacement has occurred, $replace != $c so return without quotes
			if($replace != $string)
			{
				$replace = preg_replace("/[^a-zA-Z0-9]/", '', $replace);
				if(is_numeric(substr($replace, 0)))
					$replace = "string$replace";
				return $replace . ",";
			}
		}
		return $this->startField . $string . $this->endField . ",";
	}
// convert string from WIKINDX stored fields
	private function convertCharacter2($string, $replace, $pattern, $type = FALSE)
	{
		if($this->convertTex)
			$string = $this->convertTex($string, $type);
// Do @string substitutions
		$replace = preg_replace($pattern, $replace, $string);
// If a replacement has occurred, $replace != $c so return without quotes
		if($replace != $string)
		{
			$replace = preg_replace("/[^a-zA-Z0-9]/", '', $replace);
			if(is_numeric(substr($replace, 0)))
				$replace = "string$replace";
			return $replace . ",";
	}
		return $this->startField . $string . $this->endField . ",";
	}
// convert special characters to TeX codes
	private function convertTex($string, $type = FALSE)
	{
// As web browser encoding is set to UTF-8, all input in the db is stored as UTF-8 - convert back to ISO-8859-1
		$c = utf8_decode($string);
		if($type == 'plain')
		{
			foreach($this->spChPlain as $key => $value)
			{
				$char = chr($key);
        			$c = preg_replace("/$char/", "$value", $c);
			}
		}
		else
		{
// '\' and '$' are special cases and must be treated separately.  Former MUST be treated first!
			$char = "\\" . chr(0x005C);	// '\'
			$rep = "\\textbackslash";
			$c = preg_replace("/$char/", $rep, $c);
			foreach($this->spCh as $key => $value)
			{
				$match[] = "/" . preg_quote(chr($key)) . "/";
				$replace[] = $value;
			}
        	$c = preg_replace($match, $replace, $c);
		}
// Convert some BBCode and any citeKeys to TeX and strip the rest
		$bbCode = array(
							"/\[b\](.*?)\[\/b\]/si",
							"/\[u\](.*?)\[\/u\]/si",
							"/\[i\](.*?)\[\/i\]/si",
							"/\[sup\](.*?)\[\/sup\]/si",
							"/\[sub\](.*?)\[\/sub\]/si",
							"/WIKINDXCITEKEYSTART(.*?)WIKINDXCITEKEYEND/si",
			);
		$tex = array(
							"\\textbf{\\1}",
							"\\underline{\\1}",
							"\\textit{\\1}",
							"^{\\1}",
							"_{\\1}",
							"\\cite{\\1}",
			);
		return preg_replace($bbCode, $tex, $c);
	}
// grab and format names.
	private function grabNames($type, $bibField)
	{
		$mapName = array();
		$this->db->formatConditions(array('resourcecreatorResourceId' => $this->resourceId));
		$this->db->formatConditions(array('resourcecreatorRole' => $type));
		$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
		$recordset = $this->db->select('resource_creator', 
			array('creatorSurname', 'creatorFirstname', 'creatorInitials', 'creatorPrefix', 'creatorId'));
		while($row = $this->db->fetchRow($recordset))
		{
			$name = $this->formatName($row);
			if($name)
				$mapName[$row['creatorId']] = $name;
		}
		if(!empty($mapName))
			return $bibField . " = " . $this->convertCharacter(join(' and ', $mapName));
		return FALSE;
	}
	private function orderArrayByKeyLength()
	{
		uksort($this->rawString, array($this, "cmp"));
	}
// user function for uksort above
	private function cmp($a, $b)
	{
		if(preg_match_all("/./", $a, $throwAway) > preg_match_all("/./", $b, $throwAway2))
			return 0;
		return preg_match_all("/./", $a, $throwAway) > preg_match_all("/./", $b, $throwAway2) ? -1 : 1;
	}
	private function formatName($row)
	{
		$surname = $firstname = $initials = '';
// WIKINDX stores Jr., IV etc. at end of surname...
		if($row['creatorSurname'])
		{
			if($row['creatorPrefix'])
				$surname = stripslashes($row['creatorPrefix']) . " " . 
				stripslashes($row['creatorSurname']);
			else
				$surname = stripslashes($row['creatorSurname']);
		}
		if($row['creatorFirstname'])
			$firstname = stripslashes($row['creatorFirstname']);
		if($row['creatorInitials'])
			$initials = implode('. ', explode(' ', stripslashes($row['creatorInitials']))) . ".";
		if(preg_match("/(.*)(Sr\.|jr\.)/i", $surname, $matches))
			$surname = trim($matches[1]) . ", " . trim($matches[2]);
		if(preg_match("/(.*)\s(I|II|III|IV|V|VI|VII|VIII|IX|X)$/", $surname, $matches))
			$surname = trim($matches[1]) . ", " . trim($matches[2]);
		if($firstname && $initials)
			return $surname . ", " . $firstname . ' ' . $initials;
		else if($firstname)
			return $surname . ", " . $firstname;
		else if($initials)
			return $surname . ", " . $initials;
		return $surname; // if all else fails
	}
	private function formatTitle($row)
	{
// For book_chapter, 'title' is bibtex 'chapter' and 'collectionTitle' is bibtex 'title'
		if($row['resourceType'] == 'book_chapter')
			return $row['collectionTitle'];
		$noSort = $row['resourceNoSort'] ? $row['resourceNoSort'] . ' ' : FALSE;
		if($row['resourceSubtitle'])
			$string = $noSort . $row['resourceTitle'] . ": " . $row['resourceSubtitle'];
		else
			$string = $noSort . $row['resourceTitle'];
		return $this->html->stripHtml($string);
	}
	private function grabUrl()
	{
		$this->db->formatConditions(array('resourcetextId' => $this->resourceId));
		$resultset = $this->db->select('resource_text', 'resourcetextUrls');
		if(!$this->db->numRows($resultset));
			return FALSE;
		$urls = $this->db->fetchOne($resultset);
		if(!$urls)
			return FALSE;
		$urls = unserialize(base64_decode($urls));
		return "howpublished = " . $this->startField . "\url{" . array_shift($urls) . "}" . $this->endField . ",";
	}
	private function grabPublisher($row)
	{
		if(empty($this->map->{$row['resourceType']}['resource_publisher']))
			return array(FALSE, FALSE);
		$publisher = $address = FALSE;
		if(array_key_exists('publisherName', $this->map->{$row['resourceType']}['resource_publisher']))
			$bibtexPubField = $this->map->{$row['resourceType']}['resource_publisher']['publisherName'];
		if(array_key_exists('publisherLocation', $this->map->{$row['resourceType']}['resource_publisher']))
			$bibtexAddField = $this->map->{$row['resourceType']}['resource_publisher']['publisherLocation'];
// For proceedings_article, resourcemiscField1 is the publisher
		if(($row['resourceType'] == 'proceedings_article') && $row['resourcemiscField1'])
			$this->db->formatConditions(array('publisherId' => $row['resourcemiscField1']));
		else 
			$this->db->formatConditions(array('publisherId' => $row['resourcemiscPublisher']));
		$recordset = $this->db->select('publisher', array('publisherName','publisherLocation'));
		if($this->db->numRows($recordset))
		{
			$pubArray = $this->db->fetchRow($recordset);
			if($pubArray['publisherName'])
			{
				$publisher = "$bibtexPubField = " . $this->convertCharacter($pubArray['publisherName']);
				if($pubArray['publisherLocation'])
					$address = "$bibtexAddField = " . $this->convertCharacter($pubArray['publisherLocation']);
			}
		}
		return array($publisher, $address);
	}
	private function pageFormat($row)
	{
		$page = FALSE;
		if($row['resourcepagePageStart'])
			$page = stripslashes($row['resourcepagePageStart']);
		if($row['resourcepagePageEnd'])
			$page .= '--' . stripslashes($row['resourcepagePageEnd']);
		return $page;
	}
	private function keywordFormat()
	{
		$this->db->formatConditions(array('resourcekeywordResourceId' => $this->resourceId));
		$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
		$recordset = $this->db->select('resource_keyword', 'keywordKeyword');
		if($this->db->numRows($recordset))
		{
			while($kw = $this->db->fetchRow($recordset))
				$k[] = $kw['keywordKeyword'];
			return join($this->keywordSeparator, $k);
		}
		return FALSE;
	}
	private function grabNoteAbstract()
	{
		$note = $abstract = FALSE;
		$this->db->formatConditions(array('resourcetextId' => $this->resourceId));
		$resultset = $this->db->select('resource_text', array('resourcetextNote', 'resourcetextAbstract'));
		if($this->db->numRows($resultset))
		{
			$text = $this->db->fetchRow($resultset);
// Ensure first letter is capitalized
			if($text['resourcetextNote'])
				$note = $this->utf8->utf8_ucfirst($this->html->stripHtml($text['resourcetextNote']));
			if($text['resourcetextAbstract'])
				$abstract = $this->utf8->utf8_ucfirst($this->html->stripHtml($text['resourcetextAbstract']));
		}
		return array($note, $abstract);
	}
/**
* For bibtex exports, return bibtex key instead of formatted citation for citations within the body of text
*/
	private function parseCitation($text)
	{
// If no citations, return doing nothing
		if(strpos(strtolower($text), "[cite]") === FALSE)
			return $text;
// Capture any text after last [cite]...[/cite] tag
		$explode = explode("]etic/[", strrev($text), 2);
		$tailText = strrev($explode[0]);
		$text = strrev("]etic/[" . $explode[1]);
		preg_match_all("/(.*)\s*\[cite\](.*)\[\/cite\]/Uis", $text, $match);
		return preg_replace_callback("/(\[cite\])(.*)(\[\/cite\])/Uis", array($this, "citeCallback"), $text . $tailText);
	}
/**
* Callback for citations
*/
	private function citeCallback($matches)
	{
		array_shift($matches);
		return $this->parseCiteTag($matches[1]);
	}
/**
* Parse the cite tag by extracting resource ID and any page numbers. Check ID is valid
* PreText and postText can also be encoded: e.g. (see Grimshaw 2003; Boulanger 2004 for example)
* [cite]23:34-35|see ` for example[/cite].  For multiple citations, only the first encountered preText 
* and postText will be used to enclose the citations.
*/
	private function parseCiteTag($cite)
	{
		$citeKey = FALSE;
		$rawCitation = explode("|", $cite);
		$idPart = explode(":", $rawCitation[0]);
		$id = $idPart[0];
		$this->db->formatConditions(array('resourceId' => $id));
		$resultset = $this->db->select('resource', array('resourceId'));
// Presumably an invalid citation ID
		if(!$this->db->numRows($resultset))
			return "INVALID CITATION";
		if($this->session->getVar("export_UseOriginalCitation"))
		{
			$this->db->formatConditions(array('importrawId' => $id));
			$recordset = $this->db->select('import_raw', 'importrawText');
			$rawEntries = unserialize(base64_decode($this->db->fetchOne($recordset)));
			if($rawEntries)
			{
				$rawEntries = explode("\n", $rawEntries);
				array_pop($rawEntries); // always an empty array at end so get rid of it.
				foreach($rawEntries as $entries)
				{
					$entry = explode("=", $entries, 2);
					if(!trim($entry[1]))
						continue;
					if(trim($entry[0]) == 'citation')
					{
						$citeKey = trim($entry[1]);
						break;
					}
				}
			}
		}
		else if(!$this->session->getVar("setup_UseWikindxKey"))
		{
			$this->db->formatConditions(array('resourceId' => $id));
			$citeKey = $this->db->fetchOne($this->db->select('resource', 'resourceBibtexKey'));
		}
		if(!$citeKey)
			$citeKey = $this->wikindxCiteKey($id); // Use wikindx=generated keys from this point on
		$citeKey = "WIKINDXCITEKEYSTART" . $citeKey . ".$id" . "WIKINDXCITEKEYEND";
		if(array_key_exists('1', $idPart))
		{
			$pages = explode("-", $idPart[1]);
			$pageStart = $pages[0];
			$pageEnd = array_key_exists('1', $pages) ? $pages[1] : FALSE;
			if($pageEnd)
				$pages = " pp.$pageStart-$pageEnd";
			else
				$pages = " p.$pageStart";
		}
		else
			$pages = FALSE;
		if(array_key_exists('1', $rawCitation))
		{
			$text = explode("`", $rawCitation[1]);
			$preText = $text[0];
			$postText = array_key_exists('1', $text) ? $text[1] : FALSE;
		}
		else
			$preText = $postText = FALSE;
		return "$preText$citeKey$pages$postText";
	}
// Deal with month/day
	private function formatDate($row)
	{
// NB 3-letter month abbreviation must not be quoted
		if(!$row['resourcemiscField3'])
			return FALSE;
		$startMonth = $this->monthArray[$row['resourcemiscField3']];
// if there's a day, append it
		if($row['resourcemiscField2'])
			$startDay = $this->startField . " " . $row['resourcemiscField2'] . $this->endField;
		if(isset($startDay) && $row['resourcemiscField5'] && $row['resourcemiscField6'] && 
			($row['resourcemiscField3'] == $row['resourcemiscField6']))
			$date = $this->startField . $startMonth . "~" . $this->endField . " # "
				. $this->startField . $row['resourcemiscField2'] 
				. "--" . $row['resourcemiscField5'] . $this->endField;
		else if(isset($startDay) && $row['resourcemiscField5'] && $row['resourcemiscField6'])
			$date = $this->startField . $startMonth . "~" . $this->endField . " # " 
			 . "~" . $this->startField . $row['resourcemiscField2'] . "--" . 
			$this->endField . " # " . $this->startField . $this->monthArray[$row['resourcemiscField6']] 
			. $this->endField . " # " . $this->startField . $row['resourcemiscField5'] . $this->endField;
		else if($row['resourcemiscField6'])
			$date = $this->startField . $startMonth . "~" . $this->endField . " # " 
			. $this->startField . "/" . $this->endField . 
			" # " . $this->startField . $this->monthArray[$row['resourcemiscField6']] . "~" . $this->endField;
		else if(isset($startDay))
			$date = $this->startField . $startMonth . "~" . $this->endField . " # " 
			. $this->startField . $startDay . $this->endField;
		else
			$date = $this->startField . $startMonth . $this->endField;
		return $date;
	}
/**
* Get any requested metadata (quotations)
*/
	private function getMetadataQ(&$entryArray)
	{
		$this->db->formatConditions(array('resourcequoteResourceId' => $this->resourceId));
		$this->db->formatConditions($this->db->formatFields('resourcequotetextId') . $this->db->equal . 
			$this->db->formatFields('resourcequoteId'));
		$this->db->orderBy($this->db->tidyInputClause('resourcequotePageStart') . '+0', FALSE);
		$this->db->leftJoin('resource_quote_text', 'resourcequotetextId', 'resourcequoteId');
		$recordset = $this->db->select('resource_quote', 
			array('resourcequoteId', 'resourcequotePageStart', 'resourcequotePageEnd',  
			'resourcequotetextText', 'resourcequotetextAddUserIdQuote'));
		if($this->db->numRows($recordset) > 1)	
			$index = 1;
		else
			$index = FALSE;
		while($row = $this->db->fetchRow($recordset))
		{
			if($row['resourcequotePageStart'])
			{
				$pages = $row['resourcequotePageStart'];
				if($row['resourcequotePageEnd'])
					$pages = '(pp.' . $pages . '-' . $row['resourcequotePageEnd'] . ') ';
				else
					$pages = "(p.$pages) ";
			}
			else
				$pages = FALSE;
			$text = $pages . $row['resourcequotetextText'];
			if($index)
			{
				$fieldNameQ = $this->metadataFields['Quotation'] . "_$index";
				$index++;
			}
			else
				$fieldNameQ = $this->metadataFields['Quotation'];
			$entryArray[] = $fieldNameQ . ' = ' . 
				$this->convertCharacter($this->parseCitation($this->html->stripHtml($text)));
// Grab any quote comments
			if(array_key_exists('QuotationComment', $this->metadataFields))
			{
				$this->db->conditionSeparator = $this->db->or;
				$this->db->formatConditions(array('resourcequotecommentAddUserIdQuote' => $this->userId, 
					'resourcequotecommentPrivate' => 'N'));
				$this->db->conditionSeparator = $this->db->and; // reset
				$this->db->formatConditions(array('resourcequotecommentQuoteId' => $row['resourcequoteId']));
				$recordset2 = $this->db->select('resource_quote_comment', 
					array('resourcequotecommentComment', 'resourcequotecommentAddUserIdQuote'));
				if($this->db->numRows($recordset2) > 1)	
					$indexC = 1;
				else
					$indexC = FALSE;
				while($rowC = $this->db->fetchRow($recordset2))
				{
					if($indexC)
					{
						$fieldName = $fieldNameQ . "_" . $this->metadataFields['QuotationComment'] . "_$indexC";
						$indexC++;
					}
					else
						$fieldName = $fieldNameQ . "_" . $this->metadataFields['QuotationComment'];
					$user = " (" . $this->user->displayUserAddEditPlain($rowC['resourcequotecommentAddUserIdQuote']) . ")";
					$text = $rowC['resourcequotecommentComment'];
					$entryArray[] = $fieldName . ' = ' . 
						$this->convertCharacter($this->parseCitation($this->html->stripHtml($text)) . $user);
				}
			}
		}
	}
/**
* Get any requested metadata (paraphrases)
*/
	private function getMetadataP(&$entryArray)
	{
		$this->db->formatConditions(array('resourceparaphraseResourceId' => $this->resourceId));
		$this->db->formatConditions($this->db->formatFields('resourceparaphrasetextId') . $this->db->equal . 
			$this->db->formatFields('resourceparaphraseId'));
		$this->db->orderBy($this->db->tidyInputClause('resourceparaphrasePageStart') . '+0', FALSE);
		$this->db->leftJoin('resource_paraphrase_text', 'resourceparaphrasetextId', 'resourceparaphraseId');
		$recordset = $this->db->select('resource_paraphrase', 
			array('resourceparaphraseId', 'resourceparaphrasePageStart', 'resourceparaphrasePageEnd', 
			'resourceparaphrasetextText', 'resourceparaphrasetextAddUserIdParaphrase'));
		if($this->db->numRows($recordset) > 1)	
			$index = 1;
		else
			$index = FALSE;
		while($row = $this->db->fetchRow($recordset))
		{
			if($row['resourceparaphrasePageStart'])
			{
				$pages = $row['resourceparaphrasePageStart'];
				if($row['resourceparaphrasePageEnd'])
					$pages = '(pp.' . $pages . '-' . $row['resourceparaphrasePageEnd'] . ') ';
				else
					$pages = "(p.$pages) ";
			}
			else
				$pages = FALSE;
			$text = $pages . $row['resourceparaphrasetextText'];
			if($index)
			{
				$fieldNameQ = $this->metadataFields['Paraphrase'] . "_$index";
				$index++;
			}
			else
				$fieldNameQ = $this->metadataFields['Paraphrase'];
			$entryArray[] = $fieldNameQ . ' = ' . 
				$this->convertCharacter($this->parseCitation($this->html->stripHtml($text)));
// Grab any paraphrase comments
			if(array_key_exists('ParaphraseComment', $this->metadataFields))
			{
				$this->db->conditionSeparator = $this->db->or;
				$this->db->formatConditions(array('resourceparaphrasecommentAddUserIdParaphrase' => $this->userId, 
					'resourceparaphrasecommentPrivate' => 'N'));
				$this->db->conditionSeparator = $this->db->and; // reset
				$this->db->formatConditions(array('resourceparaphrasecommentParaphraseId' => $row['resourceparaphraseId']));
				$recordset2 = $this->db->select('resource_paraphrase_comment', 
					array('resourceparaphrasecommentComment', 'resourceparaphrasecommentAddUserIdParaphrase'));
				if($this->db->numRows($recordset2) > 1)	
					$indexC = 1;
				else
					$indexC = FALSE;
				while($rowC = $this->db->fetchRow($recordset2))
				{
					if($indexC)
					{
						$fieldName = $fieldNameQ . "_" . $this->metadataFields['ParaphraseComment'] . "_$indexC";
						$indexC++;
					}
					else
						$fieldName = $fieldNameQ . "_" . $this->metadataFields['ParaphraseComment'];
					$user = " (" . $this->user->displayUserAddEditPlain($rowC['resourceparaphrasecommentAddUserIdParaphrase']) . ")";
					$text = $rowC['resourceparaphrasecommentComment'];
					$entryArray[] = $fieldName . ' = ' . 
						$this->convertCharacter($this->parseCitation($this->html->stripHtml($text)) . $user);
				}
			}
		}
	}
/**
* Get any requested metadata (musing)
*/
	private function getMetadataM(&$entryArray)
	{
		$this->db->formatConditions(array('resourcemusingResourceId' => $this->resourceId));
		$this->db->formatConditions($this->db->formatFields('resourcemusingtextId') . $this->db->equal . 
			$this->db->formatFields('resourcemusingId'));
		$this->db->orderBy($this->db->tidyInputClause('resourcemusingPageStart') . '+0', FALSE);
		$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
		$recordset = $this->db->select('resource_musing', 
			array('resourcemusingPageStart', 'resourcemusingPageEnd',  
			'resourcemusingtextText', 'resourcemusingtextAddUserIdMusing'));
		if($this->db->numRows($recordset) > 1)	
			$index = 1;
		else
			$index = FALSE;
		while($row = $this->db->fetchRow($recordset))
		{
			if($row['resourcemusingPageStart'])
			{
				$pages = $row['resourcemusingPageStart'];
				if($row['resourcemusingPageEnd'])
					$pages = '(pp.' . $pages . '-' . $row['resourcemusingPageEnd'] . ') ';
				else
					$pages = "(p.$pages) ";
			}
			else
				$pages = FALSE;
			$text = stripslashes($pages . $row['resourcemusingtextText']);
			if($index)
			{
				$fieldNameQ = $this->metadataFields['Musing'] . "_$index";
				$index++;
			}
			else
				$fieldNameQ = $this->metadataFields['Musing'];
			$user = " (" . $this->user->displayUserAddEditPlain($row['resourcemusingtextAddUserIdMusing']) . ")";
			$entryArray[] = $fieldNameQ . ' = ' . 
				$this->convertCharacter($this->parseCitation($this->html->stripHtml($text)) . $user);
		}
	}
}
?>