<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* STATISTICS class
*
*/
class STATS
{
private $db;
private $vars;
private $success;
private $messages;
private $errors;
private $session;
private $table;
private $html;
private $template;
private $resourceMap;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->resourceMap = FACTORY_RESOURCEMAP::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "statistics"));
	}
	public function init()
	{
		if(!$this->session->getVar("setup_Write"))
		{
			$authorize = FACTORY_AUTHORIZE::getInstance();
			return $authorize->initLogon();
		}
		$pString = $this->totals();
		$pString .= $this->resourceAccesses();
		$pString .= $this->resourceDates();
		$pString .= $this->userData();
		$pString .= $this->resourceTypes();
		GLOBALS::buildOutputString($pString);
	}
	private function totals()
	{
		$recordset = $this->db->select('database_summary', array('databasesummaryTotalResources', 
			'databasesummaryTotalQuotes', 'databasesummaryTotalParaphrases', 'databasesummaryTotalMusings'));
		$row = $this->db->fetchRow($recordset);
		$this->totalResources = $row['databasesummaryTotalResources'];
		$string = $this->html->br() . $this->messages->text("statistics", "totalResources") . 
			"&nbsp;&nbsp;" . $this->html->i($this->totalResources);
		if($this->session->getVar('setup_MetadataAllow') == 'Y')
		{
			$string .= $this->html->br() . $this->messages->text("statistics", "totalQuotes") . 
				"&nbsp;&nbsp;" . $this->html->i($row['databasesummaryTotalQuotes']);
			$string .= $this->html->br() . $this->messages->text("statistics", "totalParaphrases") . 
				"&nbsp;&nbsp;" . $this->html->i($row['databasesummaryTotalParaphrases']);
			$string .= $this->html->br() . $this->messages->text("statistics", "totalMusings") . 
				"&nbsp;&nbsp;" . $this->html->i($row['databasesummaryTotalMusings']);
		}
		return $this->html->p($string);
	}
	private function resourceAccesses()
	{
		$recordset = $this->db->selectMax('resource_misc', 'resourcemiscAccesses');
		$row = $this->db->fetchRow($recordset);
		$string = $this->messages->text("statistics", "maxAccesses") . 
			"&nbsp;&nbsp;" . $this->html->i($row['resourcemiscAccesses']);
		$recordset = $this->db->selectMin('resource_misc', 'resourcemiscAccesses');
		$row = $this->db->fetchRow($recordset);
		$string .= $this->html->br() . $this->messages->text("statistics", "minAccesses") . 
			"&nbsp;&nbsp;" . $this->html->i($row['resourcemiscAccesses']);
		return $this->html->p($string);
	}
	private function resourceDates()
	{
		$recordset = $this->db->selectMin('resource_timestamp', 'resourcetimestampTimestampAdd');
		$row = $this->db->fetchRow($recordset);
		$string = $this->messages->text("statistics", "firstAdded") . 
			"&nbsp;&nbsp;" . $this->html->i($row['resourcetimestampTimestampAdd']);
		$recordset = $this->db->selectMax('resource_timestamp', 'resourcetimestampTimestampAdd');
		$row = $this->db->fetchRow($recordset);
		$string .= $this->html->br() . $this->messages->text("statistics", "lastAdded") . 
			"&nbsp;&nbsp;" . $this->html->i($row['resourcetimestampTimestampAdd']);
		$recordset = $this->db->selectAverageDate('resource_timestamp', 'resourcetimestampTimestampAdd');
		$average = $this->db->fetchOne($recordset);
		$string .= $this->html->br() . $this->messages->text("statistics", "meanAddedResource") . 
			"&nbsp;&nbsp;" . $this->html->i($average);
		return $this->html->p($string);
	}
	private function userData()
	{
// Resources
		$this->db->leftJoin('users', 'usersId', 'resourcemiscAddUserIdResource');
		$recordset = $this->db->selectCountMax('resource_misc', 
			array('resourcemiscAddUserIdResource', 'usersUsername', 'usersFullname'));
		$row = $this->db->fetchRow($recordset);
		$user = $this->html->a('link', $this->getUsername($row), 
			"index.php?action=list_SELECTRESOURCES_CORE" . htmlentities("&method=process&statistics=resourceUser&id=" . 
			$row['resourcemiscAddUserIdResource']));
		$string = $this->messages->text("statistics", "userResourceTotal") . 
			"&nbsp;&nbsp;" . $this->html->i($row['count'] . "&nbsp;($user)");
		if($this->session->getVar('setup_MetadataAllow') == 'N')
			return $this->html->p($string);
// Quotes
		$this->db->leftJoin('users', 'usersId', 'resourcequotetextAddUserIdQuote');
		$recordset = $this->db->selectCountMax('resource_quote_text', 
			array('resourcequotetextAddUserIdQuote', 'usersUsername', 'usersFullname'));
		$row = $this->db->fetchRow($recordset);
		if($row['count'])
		{
			$user = $this->html->a('link', $this->getUsername($row), 
				"index.php?action=list_SELECTRESOURCES_CORE" . htmlentities("&method=process&statistics=quoteUser&id=" . 
				$row['resourcequotetextAddUserIdQuote']));
			$string .= $this->html->br() . $this->messages->text("statistics", "userQuoteTotal") . 
				"&nbsp;&nbsp;" . $this->html->i($row['count'] . "&nbsp;($user)");
		}
// Paraphrases
		$this->db->leftJoin('users', 'usersId', 'resourceparaphrasetextAddUserIdParaphrase');
		$recordset = $this->db->selectCountMax('resource_paraphrase_text', 
			array('resourceparaphrasetextAddUserIdParaphrase', 'usersUsername', 'usersFullname'));
		$row = $this->db->fetchRow($recordset);
		if($row['count'])
		{
			$user = $this->html->a('link', $this->getUsername($row), 
				"index.php?action=list_SELECTRESOURCES_CORE" . htmlentities("&method=process&statistics=paraphraseUser&id=" . 
				$row['resourceparaphrasetextAddUserIdParaphrase']));
			$string .= $this->html->br() . $this->messages->text("statistics", "userParaphraseTotal") . 
				"&nbsp;&nbsp;" . $this->html->i($row['count'] . "&nbsp;($user)");
		}
// Musings
		$this->db->leftJoin('users', 'usersId', 'resourcemusingtextAddUserIdMusing');
		$recordset = $this->db->selectCountMax('resource_musing_text', 
			array('resourcemusingtextAddUserIdMusing', 'usersUsername', 'usersFullname'));
		$row = $this->db->fetchRow($recordset);
		if($row['count'])
		{
			$user = $this->html->a('link', $this->getUsername($row), 
				"index.php?action=list_SELECTRESOURCES_CORE" . htmlentities("&method=process&statistics=musingUser&id=" . 
				$row['resourcemusingtextAddUserIdMusing']));
			$string .= $this->html->br() . $this->messages->text("statistics", "userMusingTotal") . 
				"&nbsp;&nbsp;" . $this->html->i($row['count'] . "&nbsp;($user)");
		}
		return $this->html->p($string);
	}
	private function getUsername(&$row)
	{
		if($row['usersFullname'])
			return $row['usersFullname'];
		else if($row['usersUsername'])
			return $row['usersUsername'];
		else
			return $this->messages->text("user", "unknown");
	}
	private function resourceTypes()
	{
		$pString = $this->html->b($this->messages->text("statistics", "resourceTypes"));
		$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$this->db->ascDesc = $this->db->desc;
		$this->db->orderBy('count');
		$recordset = $this->db->selectCounts('resource', 'resourceType');
		$maxNo = FALSE;
		while($row = $this->db->fetchRow($recordset))
		{
			if(!$maxNo)
				$maxNo = $row['count']; // first in row ordered DESC
			$pString .= $this->table->trStart();
			$pString .= $this->table->td($this->html->a('link', $this->messages->text("resourceType", $row['resourceType']), 
				'index.php?' . 
				htmlentities('action=list_LISTSOMERESOURCES_CORE&method=typeProcess&id=' . $row['resourceType'])));
// 100% width of bar == arbitrary 500 px
			$percentage = round((($row['count'] / $this->totalResources) * 100), 1);
			$width = round((($row['count'] / $maxNo) * 500), 0);
			$bar = $this->html->img("core/modules/statistics/green.gif", $width, 15, $row['resourceType']);
			$pString .= $this->table->td($bar . $this->html->i("&nbsp;&nbsp;" . $row['count'] . "&nbsp;($percentage%)"));
			$pString .= $this->table->trEnd();
		}
		$pString .= $this->table->tableEnd();
		return $pString;
	}
// Set the maturity index for a resource
	public function setMaturityIndex()
	{
		include_once("core/modules/resource/RESOURCEVIEW.php");
		$resource = new RESOURCEVIEW();
		$gatekeep = FACTORY_GATEKEEP::getInstance();
		$gatekeep->init();
		if(!array_key_exists('resourceId', $this->vars) || !array_key_exists('maturityIndex', $this->vars) || 
		!is_numeric($this->vars['maturityIndex']))
		{
			$resource->init($this->session->getVar('sql_LastSolo'));
			GLOBALS::buildOutputString($this->errors->text("inputError", "invalid"));
			return;
		}
		$mIndex = round(trim($this->vars['maturityIndex']), 1);
		if($mIndex > 10)
			$mIndex = 10;
		else if($mIndex < 0)
			$mIndex = 0;
		$this->db->formatConditions(array('resourcemiscId' => $this->vars['resourceId']));
		$this->db->updateSingle('resource_misc', 
			$this->db->formatFields('resourcemiscMaturityIndex') .  "=" . $this->db->tidyInput($mIndex));
		$resource->init($this->session->getVar('sql_LastSolo'));
		GLOBALS::buildOutputString($this->success->text("maturityIndex"));
	}
}
?>
