<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* SESSION
*
* Common session functions
* 
* @version	1
*
*	@package wikindx4\core\session
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class SESSION
{
/** array */
private $sessionVars = array();
/** string */
private $dbName;
/** object */
private $db;
/** array */
private $stateKeys;

/**
* SESSION
*/
	public function __construct()
	{
		if(isset($_SESSION))
			$this->sessionVars = &$_SESSION;
		$config = FACTORY_CONFIG::getInstance();
		$this->dbName = $config->WIKINDX_DB . "_";
		$this->stateKeys = array('Sql', 'Bookmark', 'List', 'Select', 'Search', 'SearchMeta', 'SelectMeta', 'Basket');
	}
/**
* Set a session variable
*
* @param string $inKey
* @param string $value
* @return boolean
*/
	public function setVar($inKey, $value)
	{
		if(!isset($inKey) || !isset($value)) return false;
		$key = $this->dbName . $inKey;
		$this->sessionVars[$key] = $value;
		if(!isset($this->sessionVars[$key]))
			return false;
		return true;
	}
/**
* Get a session variable
*
* @param string $key
* @return mixed FALSE|session value
*/
	public function getVar($key)
	{
		$key = $this->dbName . $key;
		if(isset($this->sessionVars[$key]))
			return $this->sessionVars[$key];
		return false;
	}
/**
* Delete a session variable
*
* @param string $key
* @param boolean $prepend Prepend database name to session key, default is TRUE
*/
	public function delVar($key, $prepend = TRUE)
	{
		if($prepend)
			$key = $this->dbName . $key;
		if(isset($this->sessionVars[$key]))
			unset($this->sessionVars[$key]);
	}
/**
* Is a session variable set?
*
* @param string $key
* @return boolean
*/
	public function issetVar($key)
	{
		$key = $this->dbName . $key;
		if(isset($this->sessionVars[$key])) return true;
		return false;
	}
/**
* Destroy the whole session
*/
	public function destroy()
	{
		$this->sessionVars = array();
	}
/**
* Return an associative array of all session variables starting with $prefix_.
*
* key in returned array is minus the prefix to aid in matching database table fields.
* @param string $prefix
* @return mixed FALSE|array
*/
	public function getArray($prefix)
	{
		$prefix = $this->dbName . $prefix . '_';
		foreach($this->sessionVars as $key => $value)
		{
			if(preg_match("/^$prefix(.*)/", $key, $matches))
				$array[$matches[1]] = $value;
		}
		if(isset($array))
			return $array;
		return FALSE;
	}
/**
* Write to session variables named with $prefix_ the given associative array.
*
* @param array $array
* @param string $prefix Default is FALSE
* @param boolean $addSlashes Default is FALSE
* @return boolean
*/
	public function writeArray($array, $prefix = FALSE, $addSlashes = FALSE)
	{
		foreach($array as $key => $value)
		{
			if(!$value)
				$value = FALSE;
			else if($addSlashes)
				$value = addslashes($value);
			if($prefix)
			{
				if(!$this->setVar($prefix . '_' . $key, $value))
					return FALSE;
			}
			else
			{
				if(!$this->setVar($key, $value))
					return FALSE;
			}
		}
		return TRUE;
	}
/**
* Clear session variables named with $prefix
*
* @param string $prefix
*/
	public function clearArray($prefix)
	{
		$prefix = $this->dbName . $prefix . '_';
		foreach($this->sessionVars as $key => $value)
		{
			if(preg_match("/^$prefix/", $key))
				$this->delVar($key, FALSE);
		}
	}
/**
* Clear all user sessions
*/
	public function clearSessionData()
	{
		$this->clearArray('preferences');
		$this->clearArray('config');
		$this->clearArray('mywikindx');
		$this->clearArray('sql');
		$this->clearArray('search');
		$this->clearArray('select');
		$this->clearArray('style');
		$this->clearArray('cite');
		$this->clearArray('resource');
		$this->clearArray('export');
		$this->clearArray('basket');
	}
/**
* Save the session state
*
* Save the session variables to the database for logged on users
* @param string $sessionKey
*/
	public function saveState($sessionKey)
	{
		if($userId = $this->getVar('setup_UserId'))
		{
			$this->db = FACTORY_DB::getInstance();
// Get existing state
			$this->db->formatConditions(array('usersId' => $userId));
			$state = $this->db->fetchOne($this->db->select('users', 'usersUserSession'));
			if($state)
				$sessionData = unserialize(base64_decode($state));
			$sessionData[$sessionKey] = base64_encode(serialize($this->getArray($sessionKey)));
			$this->db->formatConditions(array('usersId' => $userId));
			$this->db->update('users', array('usersUserSession' => base64_encode(serialize($sessionData))));
		}
	}
}
?>