<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* AUTHORIZE
*
* Logging on and system authorisation class.
* 
* @version	1
*
*	@package wikindx4\core\startup
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class AUTHORIZE
{
/** object */
private $db;
/** object */
private $session;
/** object */
private $config;
/** object */
private $loadConfig;
/** array */
private $vars;
/** object */
private $errors;
/** object */
private $messages;
/** object */
private $template;
/** object */
private $user;
/** object */
private $file;

/**
*	AUTHORIZE
*/
	public function __construct()
	{
		$this->config = FACTORY_CONFIG::getInstance();
		$this->config->WIKINDX_PRINT_SQL = FALSE; // default for initial opening of DB prior to authorisation which loads correct value
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->session = FACTORY_SESSION::getInstance();
		$this->loadConfig = FACTORY_LOADCONFIG::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->file = FACTORY_FILE::getInstance();
		$this->user = FACTORY_USER::getInstance();
		if(!$this->session->getVar("setup_Language"))
		{
// set the default language prior to displaying the login prompt
			$sql = $this->db->selectNoExecute('config', 'configLanguage');
			$recordset = $this->db->queryNoError($sql);
			if(!$recordset) // perhaps this is a first install or upgrade to v4 (v3 has config.language not config.configLanguage)
				$this->session->setVar('setup_Language', 'en');
			else
// populate session with default values from config
				$this->user->writeSessionPreferences(FALSE, 'config');
		}
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "logon"));
	}
/**
* gatekeeper to the system.
*
* Order is of checking is important!
* @return boolean
*/
	public function gatekeep()
	{
		if(array_key_exists("action", $this->vars))
		{
// logging out
			if($this->vars["action"] == 'logout')
			{
				$this->loadConfig->loadConfig();
				$this->logout();
			}
// User logging in from readOnly mode
			else if($this->vars["action"] == 'initLogon')
			{
				$this->loadConfig->loadConfig();
// First delete any pre-existing session in case this user has been logging on and off as different users -- 
// keep template and language etc.
				$language = $this->session->getVar('setup_Language');
				$template = $this->session->getVar('setup_Template');
				$userReg = $this->session->getVar('setup_UserRegistration');
				$multiUser = $this->session->getVar('setup_MultiUser');
				$this->session->setVar('setup_Language', $language);
				$this->session->setVar('setup_Template', $template);
				$this->session->setVar('setup_UserRegistration', $userReg);
				$this->session->setVar('setup_MultiUser', $multiUser);
				$this->initLogon(); // login prompt
				FACTORY_CLOSENOMENU::getInstance();
			}
			if(array_key_exists('method', $this->vars) && 
			(($this->vars["method"] == 'forgetInitStage1') ||
			($this->vars["method"] == 'forgetInitStage2') || 
			($this->vars["method"] == 'forgetProcess')))
			{
				$this->loadConfig->loadConfig();
				return TRUE;
			}
			else if($this->vars['action'] == 'upgradeDBLogon')
			{
				$this->logonCheckUpgradeDB($this->vars['username'], $this->vars['password']);
				return FALSE;
			}
// User supplying username and password to logon to WIKINDX.
// $auth->logonCheck() dies after printing logon screen if bad comparison.
			else if(($this->vars["action"] == 'logon') &&
				array_key_exists("password", $this->vars) && array_key_exists("username", $this->vars))
			{
				$this->loadConfig->loadConfig();
				$this->logonCheck($this->vars['username'], $this->vars['password']);
// tidy up old files
				$this->file->tidyFiles();
// FALSE means go to front of WIKINDX
				return FALSE;
			}
// superAdmin already logged in after upgrade so just set up the environment
			else if($this->vars["action"] == 'upgradeDB')
			{
				$this->session->clearSessionData();
				$this->session->setVar("setup_UserId", '1'); // superAdmin always id = '1'
				$this->session->setVar("setup_Write", TRUE);
				$this->session->delVar("setup_ReadOnly");
				$this->user->writeSessionPreferences(FALSE, 'config');
// restore some session variables if stored from last logout
				$this->restoreEnvironment();
				return FALSE;
			}
// User requesting readOnly access - clear previous sessions
			else if($this->vars["action"] == 'readOnly')
			{
				$this->loadConfig->loadConfig();
				if($this->config->WIKINDX_DENY_READONLY)
				{
					$this->initLogon(); // login prompt
					FACTORY_CLOSENOMENU::getInstance();
				}
// First delete any pre-existing session in case this user has been logging on and off as different users
// session array 'setup' is deleted at logout()
				$this->session->destroy();
				$this->session->setVar("setup_ReadOnly", TRUE);
// tidy up old files
				$this->file->tidyFiles();
// populate session with default values from config
				$this->user->writeSessionPreferences(FALSE, 'config');
				$this->clearEmbargoes();
				$this->checkNews();
// FALSE means go to front of WIKINDX
				return FALSE;
			}
// User registration
			else if($this->session->getVar("setup_UserRegistration") && $this->session->getVar("setup_MultiUser"))
			{
				$this->loadConfig->loadConfig();
				include_once("core/modules/usersgroups/REGISTER.php");
				$obj = new REGISTER();
				if($this->vars["action"] == 'initRegisterUser')
				{
					$obj->initRegister();
					if(!$this->session->getVar("setup_ReadOnly"))
						FACTORY_CLOSENOMENU::getInstance();
					else
						FACTORY_CLOSE::getInstance();
				}
				else if($this->vars["action"] == 'registerConfirm')
				{
					GLOBALS::buildOutputString($obj->registerConfirm());
					if(!$this->session->getVar("setup_ReadOnly"))
						FACTORY_CLOSENOMENU::getInstance();
					else
						FACTORY_CLOSE::getInstance();
				}
				else if(array_key_exists('method', $this->vars) && $this->vars["method"] == 'registerUser')
				{
					$obj->register();
					if(!$this->session->getVar("setup_ReadOnly"))
						FACTORY_CLOSENOMENU::getInstance();
					else
						FACTORY_CLOSE::getInstance();
				}
				else if(array_key_exists('method', $this->vars) && $this->vars["method"] == 'registerUserAdd')
				{
					GLOBALS::buildOutputString($obj->registerUserAdd());
					FACTORY_CLOSE::getInstance();
				}
				else if(array_key_exists('method', $this->vars) && $this->vars["method"] == 'registerRequest')
				{
					GLOBALS::buildOutputString($obj->registerRequest());
					FACTORY_CLOSE::getInstance();
				}
			}
		}
		if(isset($this->vars["method"]) && ($this->vars['method'] == 'RSS') && !$this->config->WIKINDX_DENY_READONLY)
		{
			$this->loadConfig->loadConfig();
			$this->session->setVar("setup_ReadOnly", TRUE);
			return TRUE;
		}
// access already granted
		if($this->session->getVar('setup_Write') || $this->session->getVar("setup_ReadOnly"))
		{
			if(!array_key_exists('action', $this->vars) || 
				(array_key_exists('action', $this->vars) && ($this->vars['action'] != 'upgradeDBLogon')))
				$this->loadConfig->loadConfig();
			return TRUE;
		}
		if((!array_key_exists('action', $this->vars) || $this->vars['action'] != 'upgradeDBLogon'))
		{
			$this->loadConfig->loadConfig();
			$cookie = FACTORY_COOKIE::getInstance();
// grabCookie() returns TRUE if valid cookie - otherwise, proceed to manual logon
			if($cookie->grabCookie())
			{
// Success - so restore some session variables if stored from last logout
				$this->restoreEnvironment();
				return TRUE;
			}
		}
		if(!$this->session->getVar("setup_Write") && !$this->session->getVar('setup_ReadOnly'))
		{
			$this->loadConfig->loadConfig();
// Default == read only access.
			if($this->config->WIKINDX_READONLYACCESS && !$this->config->WIKINDX_DENY_READONLY)
			{
				$this->session->setVar("setup_ReadOnly", TRUE);
				$this->checkNews();
				return TRUE;
			}
			$this->initLogon(); // login prompt
			FACTORY_CLOSENOMENU::getInstance();
		}
// FALSE indicates that index.php will print the front page of WIKINDX
		return FALSE;
	}
/**
* Display the empty form for logging on
*
* @param string $error Default is FALSE
*/
	public function initLogon($error = FALSE)
	{
		$this->session->delVar("setup_ReadOnly");
		$html = FACTORY_HTML::getInstance();
		$pString = $error;
		$this->template->tpl->assign('heading', $this->messages->text("heading", "logon"));
		$recordset = $this->db->select('config', array('configMultiUser', 'configUserRegistration'));
		$row = $this->db->fetchRow($recordset);
		if($row['configMultiUser'] == 'N')
			$pString .= $html->p($this->errors->text("warning", "superadminOnly"));
		$pString .= $html->p($this->messages->text("authorize", "writeLogon"));
		$link1 = "index.php?action=readOnly";
		$link2 = "index.php?action=usersgroups_FORGET_CORE&method=forgetInitStage1";
		$link3 = "index.php?action=initRegisterUser";
		$links = FALSE;
/**
* For a test user (see index.php)
*/
		if($this->config->WIKINDX_RESTRICT_USERID)
			$pString .= $html->p("For test drive purposes, " .
				$html->b($this->messages->text("user", "username") . ":&nbsp;&nbsp;") . "wikindx, " . 
				$html->b($this->messages->text("user", "password") . ":&nbsp;&nbsp;") . "wikindx");
		$forgot = $this->config->WIKINDX_MAIL_SERVER ? $html->a("link", $this->messages->text("user", "forget6"), $link2) : FALSE;
		$pString .= $this->printLogonTable();
// Give user the option to bypass logging in simply to read.
		if(!$this->config->WIKINDX_DENY_READONLY)
			$links = $html->a("link", $this->messages->text("authorize", "readOnly") . 
			$html->br() . $forgot, $link1);
		else
			$links = $forgot;
		if(($row['configUserRegistration'] == 'Y') && ($row['configMultiUser'] == 'Y'))
			$links .= $html->br() . $html->a("link", $this->messages->text("menu", "register"), $link3);
		$pString .= $html->p($links, FALSE, 'right');
		GLOBALS::buildOutputString($pString);
	}
/**
* print username/password text boxes
*
* @return string
*/
	private function printLogonTable()
	{
		$form = FACTORY_FORM::getInstance();
		$table = FACTORY_TABLE::getInstance();
		$pString = $form->formHeader("logon");
		$pString .= $table->tableStart(FALSE, 0, 0, 0, "left", "50%");
		$pString .= $table->trStart();
		$pString .= $table->td($this->messages->text("user", "username") . ":&nbsp;&nbsp;");
		$pString .= $table->td($form->textInput(FALSE, "username"));
		$pString .= $table->trEnd();
		$pString .= $table->trStart(FALSE);
		$pString .= $table->td($this->messages->text("user", "password") . ":&nbsp;&nbsp;");
		$pString .= $table->td($form->passwordInput(FALSE, "password"));
		$pString .= $table->trEnd();
		$pString .= $table->trStart();
		$pString .= $table->td("&nbsp;");
		$pString .= $table->td($form->formSubmit(), FALSE, "right");
		$pString .= $table->trEnd();
		$pString .= $table->tableEnd();
		$pString .= $form->formEnd();
		return $pString;
	}
/** 
* Initial logon to the system when upgrading the database.
*
* @param string $username
* @param string $password
* @return boolean TRUE if not able to authenticate
*/
	private function logonCheckUpgradeDB($username, $password)
	{
		if(array_search('dbVersion', $this->db->listFields('database_summary'))) // pre v4.0
		{
			$fields = array("id", "password", "admin", "cookie");
			$this->db->formatConditions(array('username' => $username));
			$passwordField = 'password';
			$idField = 'id';
		}
		else
		{
			$fields =  array("usersId", "usersPassword", "usersAdmin", "usersCookie");
			$this->db->formatConditions(array('usersUsername' => $username));
			$passwordField = 'usersPassword';
			$idField = 'usersId';
		}
		$recordset = $this->db->select('users', $fields);
		if(!$this->db->numRows($recordset))
			return TRUE; // NB -- unable to authenticate
		$row = $this->db->fetchRow($recordset);
		if(crypt($password, $row[$passwordField]) != $row[$passwordField])
			return TRUE;
		if($row[$idField] != '1') // superAdmin is id '1'
			return TRUE;
		$this->session->clearSessionData();
		$this->session->setVar("setup_Superadmin", TRUE);
		$this->session->setVar("setup_Write", TRUE);
		return FALSE;
	}
/**
* Initial logon to the system.
*
* Public for use with API.
* If cleared, user environment is set and embargoed resources checked for clearance, otherwise, user is presented with logon prompt again
* @param string $username
* @param string $password
*/
	public function logonCheck($username, $password)
	{
// If checkPassword is successful, it also sets up some session variables to allow access without reauthentication.
		if(!$this->user->checkPassword($username, $password))
			$this->failure();
// Success - so restore some session variables if stored from last logout
		$this->restoreEnvironment();
		$this->clearEmbargoes();
	}
/**
* Clear any expired embargoes
*/
	private function clearEmbargoes()
	{
		$this->db->formatConditions('resourceattachmentsEmbargoUntil' . $this->db->less . 'NOW()');
		$this->db->update('resource_attachments', array('resourceattachmentsEmbargo' => 'N'));
	}
/**
* Successful registered user logon so restore some session variables if they've been stored
*/
	private function restoreEnvironment()
	{
		$this->db->formatConditions(array('usersId' => $this->session->getVar('setup_UserId')));
		$recordset = $this->db->select('users', 'usersUserSession');
		$state = $this->db->fetchOne($recordset);
		if($state)
		{
			$sessionData = unserialize(base64_decode($state));
			foreach($sessionData as $key => $array)
			{
				$array = unserialize(base64_decode($array));
				if(!is_array($array))
					continue;
				foreach($array as $subKey => $value)
					$this->session->setVar($key . '_' . $subKey, $value);
			}
		}
		$this->checkNews();
	}
/**
* Check for any news items in the database
*/
	private function checkNews()
	{
		$resultset = $this->db->select('news', 'newsId');
		if($this->db->numRows($resultset))
			$this->session->setVar('setup_News', TRUE);
		else
			$this->session->delVar('setup_News');
	}
/**
* log out user
*
* Various bits of garbage disposal, session is destroyed, cookie is deleted and user is presented with logon prompt
*/
	private function logout()
	{
// Garbage disposal
// remove this session's files
		if(!isset($this->config->WIKINDX_FILES_DIR))
			$dir = "files";
		else
			$dir = $this->config->WIKINDX_FILES_DIR;
		if($sessVar = $this->session->getVar('FileExports'))
		{
			$sessArray = unserialize($sessVar);
			if($d = @opendir($dir))
			{
				while(FALSE !== ($f = readdir($d)))
				{
					if(($f == ".") || ($f == ".."))
						continue;
					if(array_search($f, $sessArray) === FALSE)
						continue;
					$file = $dir . "/" . $f;
					unlink($file);
				}
			}
//			$this->session->delVar('fileExports');
		}
		if($sessVar = $this->session->getVar('PaperExports'))
		{
			$sessArray = unserialize($sessVar);
			if($d = @opendir($dir))
			{
				while(FALSE !== ($f = readdir($d)))
				{
					if(($f == ".") || ($f == ".."))
						continue;
					if(!array_key_exists($f, $sessArray))
						continue;
					$file = $dir . "/" . $f;
					unlink($file);
				}
			}
//			$this->session->delVar('paperExports');
		}
		$this->session->destroy();
// remove any wikindx cookie that has been set
		$cookie = FACTORY_COOKIE::getInstance();
		$cookie->deleteCookie();
		$this->initLogon();
		FACTORY_CLOSENOMENU::getInstance();
	}
/**
* failure
*
* In case of failure, Exit back to logon prompt with optional error message
* @param string error Default is FALSE
*/
	private function failure($error = FALSE)
	{
// Exit back to logon prompt
		FACTORY_CLOSENOMENU::getInstance($this->initLogon($error));
	}
}
?>
