<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* ENVIRONMENT
*
* Set up the WIKINDX server environment
* 
* @version	1
*
*	@package wikindx4\core\startup
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class ENVIRONMENT
{
/** object */
private $config;
/** object */
private $errors;

/**
* ENVIRONMENT
*/
	public function __construct()
	{
		$this->config = FACTORY_CONFIG::getInstance();
		$this->setBasicEnvironment();
		$this->runConfig();
//var_dump($this->config);
		$this->getVars();
	}
/**
* start the SESSION
*/
	public function startSession()
	{
// Set session path if required
		if(isset($this->config->WIKINDX_SESSION_PATH) && $this->config->WIKINDX_SESSION_PATH && 
			is_dir($this->config->WIKINDX_SESSION_PATH))
		{
			session_save_path($this->config->WIKINDX_SESSION_PATH);
// If required, do some house-keeping
			if(isset($this->config->WIKINDX_SESSION_PATH_CLEAR) && $this->config->WIKINDX_SESSION_PATH_CLEAR)
			{
// failure is an option...
				if($dh = @opendir($this->config->WIKINDX_SESSION_PATH))
				{
					$now = time();
					$maxTime = $this->config->WIKINDX_SESSION_PATH_CLEAR * 24 * 60 * 60; // seconds
					while(false !== ($f = readdir($dh)))
					{
						if(($f == ".") || ($f == ".."))
							continue;
						$file = $this->config->WIKINDX_SESSION_PATH . '/' . $f;
						if(($now - filemtime($file)) >= $maxTime)
							@unlink($file);
					}
				}
			}
		}
// start session
		session_start();
	}
/**
* Make sure we get HTTP VARS in whatever format they come in
*
* Use $vars = GLOBALS::getVars(); to get querystrings and form elements
*/
	private function getVars()
	{
		if(!empty($_POST))
			$vars = $_POST;
		else if(!empty($_GET))
			$vars = $_GET;
		else
			return FALSE;

		if(!get_magic_quotes_gpc())
		{
			$dirtyVars = array_map(array($this, "magicSlashes"), $vars);
		}
		else
		{
			$dirtyVars = $vars;
		}

		$cleanVars = array_map(array($this, "stripHtmlTags"), $dirtyVars);
// Store globally
		GLOBALS::setVars($cleanVars, $dirtyVars);
	}
/**
* Set basic environment
*
* Check PHP version, set browser header, set set_magic_quotes_runtime(), buffer printing and attempt to set PHP memory.
*/
	private function setBasicEnvironment()
	{
// Check we have PHP 5.2.9 and above.
	if(version_compare(PHP_VERSION, '5.2.9', '<')) 
		die('WIKINDX requires PHP 5.2.9 or greater.  Your PHP version is ' . PHP_VERSION . 
			'. Please upgrade PHP.');
// Force HTTP to UTF-8 charset overriding any AddDefaultCharset set in Apache.
		@header('Content-type: text/html; charset=utf-8');
// buffer printing to browser
		ob_start();
// make sure that Session output is XHTML conform ('&amp;' instead of '&')
		ini_set('arg_separator.output', '&amp;');
		@set_magic_quotes_runtime(0);
// Attempt to set the memory the script uses
		if(isset($this->config->WIKINDX_MEMORY_LIMIT) && $this->config->WIKINDX_MEMORY_LIMIT && 
			preg_match("/\dM/", $this->config->WIKINDX_MEMORY_LIMIT, $null))
			ini_set("memory_limit", $this->config->WIKINDX_MEMORY_LIMIT);
	}
/**
* Check and load configuration from config.php
*/
	private function runConfig()
	{
		$configArray = array(
								'WIKINDX_DEACTIVATE_RESOURCE_TYPES',
								'WIKINDX_DB',
								'WIKINDX_DB_HOST',
								'WIKINDX_DB_PERSISTENT',
								'WIKINDX_DB_USER',
								'WIKINDX_DB_PASSWORD',
								'WIKINDX_DB_TABLEPREFIX',
								'WIKINDX_MAIL_SERVER',
								'WIKINDX_RESTRICT_USERID',
								'WIKINDX_TAG_LOW_COLOUR',
								'WIKINDX_TAG_HIGH_COLOUR',
								'WIKINDX_TAG_LOW_SIZE',
								'WIKINDX_TAG_HIGH_SIZE',
								'WIKINDX_CMS_ALLOW',
								'WIKINDX_GS_ALLOW',
								'WIKINDX_GS_ATTACHMENT'
				);
		foreach($configArray as $var)
		{
			if(!isset($this->config->{$var}))
				die("Missing configuration variable in config.php: $var.");
		}
		if($this->config->WIKINDX_MAIL_SERVER)
		{
			if(!isset($this->config->WIKINDX_MAIL_FROM))
				die("Missing configuration variable in config.php: WIKINDX_MAIL_FROM.");
			if(!isset($this->config->WIKINDX_MAIL_RETURN_PATH))
				die("Missing configuration variable in config.php: WIKINDX_MAIL_RETURN_PATH.");
			if(!isset($this->config->WIKINDX_MAIL_REPLYTO))
				die("Missing configuration variable in config.php: WIKINDX_MAIL_REPLYTO.");
			if($this->config->WIKINDX_PEARMAIL_USE)
			{
				if(!isset($this->config->WIKINDX_PEARMAIL_BACKEND))
					die("Missing configuration variable in config.php: WIKINDX_PEARMAIL_BACKEND.");
				if($this->config->WIKINDX_PEARMAIL_BACKEND == 'sendmail')
				{
					if(!isset($this->config->WIKINDX_PEARMAIL_SMPATH))
						die("Missing configuration variable in config.php: WIKINDX_PEARMAIL_SMPATH.");
					if(!isset($this->config->WIKINDX_PEARMAIL_ARGS))
						die("Missing configuration variable in config.php: WIKINDX_PEARMAIL_ARGS.");
				}
				else if($this->config->WIKINDX_PEARMAIL_BACKEND == 'smtp')
				{
					if(!isset($this->config->WIKINDX_PEARMAIL_SMTPSERVER))
						die("Missing configuration variable in config.php: WIKINDX_PEARMAIL_SMTPSERVER.");
					if(!isset($this->config->WIKINDX_PEARMAIL_SMTPPORT))
						die("Missing configuration variable in config.php: WIKINDX_PEARMAIL_SMTPPORT.");
					if(!isset($this->config->WIKINDX_PEARMAIL_SMTPAUTH))
						die("Missing configuration variable in config.php: WIKINDX_PEARMAIL_SMTPAUTH.");
					else if($this->config->WIKINDX_PEARMAIL_SMTPAUTH)
					{
						if(!isset($this->config->WIKINDX_PEARMAIL_SMTPUSERNAME))
							die("Missing configuration variable in config.php: WIKINDX_PEARMAIL_SMTPUSERNAME.");
						if(!isset($this->config->WIKINDX_PEARMAIL_SMTPPASSWORD))
							die("Missing configuration variable in config.php: WIKINDX_PEARMAIL_SMTPPASSWORD.");
					}
					if(!isset($this->config->WIKINDX_PEARMAIL_SMTPPERSIST))
						die("Missing configuration variable in config.php: WIKINDX_PEARMAIL_SMTPPERSIST.");
				}
			}
		}
		if($this->config->WIKINDX_CMS_ALLOW)
		{
			if(!isset($this->config->WIKINDX_CMS_BIBSTYLE))
				die("Missing configuration variable in config.php: WIKINDX_CMS_BIBSTYLE.");
			if(!isset($this->config->WIKINDX_CMS_SQL))
				die("Missing configuration variable in config.php: WIKINDX_CMS_BIBSTYLE.");
			if($this->config->WIKINDX_CMS_SQL)
			{
				if(!isset($this->config->WIKINDX_CMS_DB_USER) || !$this->config->WIKINDX_CMS_DB_USER)
					die("Missing configuration variable in config.php: WIKINDX_CMS_DB_USER.");
				if(!isset($this->config->WIKINDX_CMS_DB_PASSWORD) || !$this->config->WIKINDX_CMS_DB_PASSWORD)
					die("Missing configuration variable in config.php: WIKINDX_CMS_DB_PASSWORD.");
			}
		}
// Set default timezone
		if(!isset($this->config->WIKINDX_TIMEZONE) || !$this->config->WIKINDX_TIMEZONE)
			date_default_timezone_set(date_default_timezone_get()); // default is UTC
		else
			date_default_timezone_set(date_default_timezone_get($this->config->WIKINDX_TIMEZONE));
	}
/**
* Add slashes to all incoming GET/POST data.
*
* We now know what we're dealing with and can code accordingly.
* @param string $element
* @return string
*/
	private function magicSlashes($element)
	{
		if(is_array($element))
			return array_map(array($this, "magicSlashes"), $element);
		else
			return addslashes($element);
	}
/**
* Strip some HTML tags from string.
*
* Since adding tiny_mce, we only strip javascript and the enclosing <p> tag tinymce adds
* @param string $element
* @return string
*/
	private function stripHtmlTags($element)
	{
		if(is_array($element))
			return array_map(array($this, "stripHtmlTags"), $element);
		else
		{
/*			$search = array('@<script[^>]*?>.*?</script>@si',  // Strip out javascript
               '@<style[^>]*?>.*?</style>@siU',    // Strip style tags properly
               '@<[\/\!]*?[^<>]*?>@si',            // Strip out HTML tags
               '@<![\s\S]*?--[ \t\n\r]*>@'         // Strip multi-line comments including CDATA
			);
*/			$search = array(
				'@<script[^>]*?>.*?</script>@si',  // Strip out javascript
				'@<![\s\S]*?--[ \t\n\r]*>@'         // Strip multi-line comments including CDATA
			);
			$element = preg_replace($search, '', $element);
			if(strpos($element, '<p>') === 0)
				return preg_replace('@<p>(.*)</p>@', '$1', $element, 1);
			else
				return $element;
		}
	}
}
?>
