<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* RESOURCE TYPE
*
* Handle resource types
* 
* @version	1
*
*	@package wikindx4\core\type
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class TYPE
{
/** object */
private $db;
/** object */
private $commonBib;
/** object */
private $messages;

/**
*	RESOURCE TYPE
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
	}
/**
* Get resource types for which resources exist in database
*
* @param boolean $userBib FALSE (default) or TRUE (return only resource types within current user bibliography)
* @param mixed $metadata FALSE or array of metadata fields ('quote', 'quoteComment' etc.) which returned resource types must have. Default is array()
* @return mixed FALSE|array
*/
	function grabAll($userBib = FALSE, $metadata = array())
	{
		$subQuery = FALSE;
		if(is_array($metadata) && !empty($metadata))
		{
			$unions = array();
			foreach($metadata as $mType)
			{
				if($mType == 'quote')
					$unions[] = $this->db->selectNoExecute('resource_quote', array(array('resourcequoteResourceId' => 'id')), TRUE);
				else if($mType == 'quoteComment')
				{
					$this->db->leftJoin('resource_quote', 'resourcequoteId', 'resourcequotecommentQuoteId');
					$unions[] = $this->db->selectNoExecute('resource_quote_comment', array(array('resourcequoteResourceId' => 'id')), 
						TRUE, TRUE, TRUE);
				}
				else if($mType == 'paraphrase')
					$unions[] = $this->db->selectNoExecute('resource_paraphrase', array(array('resourceparaphraseResourceId' => 'id')), TRUE);
				else if($mType == 'paraphraseComment')
				{
					$this->db->leftJoin('resource_paraphrase', 'resourceparaphraseId', 'resourceparaphrasecommentParaphraseId');
					$unions[] = $this->db->selectNoExecute('resource_paraphrase_comment', array(array('resourceparaphraseResourceId' => 'id')), 
						TRUE, TRUE, TRUE);
				}
				if($mType == 'musing')
					$unions[] = $this->db->selectNoExecute('resource_musing', array(array('resourcemusingResourceId' => 'id')), TRUE);
			}
			if(!empty($unions))
				$subQuery = $this->db->subQuery($this->db->union($unions), 't');
		}
		if($userBib)
		{
			if($subQuery)
				$this->commonBib->userBibCondition('id');
			else
				$this->commonBib->userBibCondition('resourceId');
		}
		$this->db->groupBy('resourceType');
		$this->db->orderBy('resourceType');
		if($subQuery)
		{
			$this->db->leftJoin('resource', 'resourceId', 'id');
			$recordset = $this->db->selectFromSubQuery(FALSE, 'resourceType', $subQuery);
		}
		else
			$recordset = $this->db->select('resource', 'resourceType');
		while($row = $this->db->fetchRow($recordset))
			$types[$row['resourceType']] = $this->messages->text("resourceType", $row['resourceType']);
		if(isset($types))
			return $types;
		return FALSE;
	}
}
?>
