<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* RSS
*
* RSS feed
* 
* @version	1
*
*	@package wikindx4
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*+-------------------------------------+
*Based upon work by Laure Endrizzi October 2005
*+-------------------------------------+
*/

/***************** START CODE *****************/

include_once("core/startup/FACTORY.php");
include_once("core/startup/GLOBALS.php");
$db = FACTORY_DB::getInstance();
$config = FACTORY_CONFIG::getInstance();

// Check we have a sane environment or die
if(!isset($config->WIKINDX_RSS_LIMIT) || !isset($config->WIKINDX_RSS_HOST) || !isset($config->WIKINDX_RSS_BIBSTYLE) || 
	!isset($config->WIKINDX_RSS_DISPLAY) || !isset($config->WIKINDX_RSS_TITLE) || !isset($config->WIKINDX_RSS_DESCRIPTION) || 
	!isset($config->WIKINDX_RSS_LANGUAGE) || !isset($config->WIKINDX_RSS_ALLOW))
	die("Missing configuration");

	if(!$config->WIKINDX_RSS_ALLOW)
		die("Access forbidden");
$channel['title'] = $config->WIKINDX_RSS_TITLE;
$channel['description'] = $config->WIKINDX_RSS_DESCRIPTION;
$channel['language'] = $config->WIKINDX_RSS_LANGUAGE;
$channel['lastBuildDate'] = date("D, d M Y H:i:s T");


if($config->WIKINDX_RSS_DISPLAY) // display both added and edited resources
	$order = "resourcetimestampTimestamp";
else // display only added resources.
	$order = "resourcetimestampTimestampAdd";
list($numResults, $item) = queryDb($db, $order, $config->WIKINDX_RSS_LIMIT, $config->WIKINDX_RSS_BIBSTYLE);


/** declare text/xml as the header content-type */
header('Content-type: text/xml');
/** print the XML/RSS headers */
print "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n";
print "<rss version=\"2.0\">\n";

/** print channel data */
print "\t<channel>\n";
print "\t\t<title>". $channel['title'] ."</title>\n";
print "\t\t<description>". $channel['description'] ."</description>\n";
print "\t\t<language>". $channel['language'] ."</language>\n";
print "\t\t<lastBuildDate>". $channel['lastBuildDate'] ."</lastBuildDate>\n";

if ($numResults > 0)
{
    for ($i=0;$i<$numResults;$i++)
	{
        /**
         * loop thru the item array
         * print item data
        */
		$description = FALSE;
        print "\t\t<item>\n";
        if (strlen($item['description'][$i]) > 0) {
            $description .= $item['description'][$i] ." ";
		}
        if (strlen($item['editUser'][$i]) > 0)
		{
            $description .= " [Edited by: " . $item['editUser'][$i] . ']';
			$item['title'][$i] = 'Edited ' . $item['title'][$i];
        }
        else if (strlen($item['addUser'][$i]) > 0)
		{
            $description .= " [Added by: " . $item['addUser'][$i] . ']';
			$item['title'][$i] = 'Added ' . $item['title'][$i];
        }
        if (strlen($item['title'][$i]) > 0) {
            print "\t\t\t<title>". $item['title'][$i] ."</title>\n";
        }
        if (strlen($item['timestamp'][$i]) > 0) {
            print "\t\t\t<pubDate>". $item['timestamp'][$i] ."</pubDate>\n";
        }
        if (strlen($item['link'][$i]) > 0) {
            print "\t\t\t<link>" . $config->WIKINDX_RSS_HOST . "?method=RSS&amp;action=resource_RESOURCEVIEW_CORE&amp;id=" . 
			$item['link'][$i] ."</link>\n";
        }
        print "\t\t\t<description>". $description ."</description>\n";
        print "\t\t</item>\n";
    }
}

print "\t</channel>\n";
print "</rss>\n";

/**
* Function to query the database and return formatted entries
*
* @param object $db
* @param string $order
* @param int $WIKINDX_RSS_LIMIT
* @param string $WIKINDX_RSS_BIBSTYLE
* @return array ($numResults, $item)
*/
	function queryDb($db, $order, $WIKINDX_RSS_LIMIT, $WIKINDX_RSS_BIBSTYLE)
	{
		$session = FACTORY_SESSION::getInstance();
		$session->setVar('setup_Style', $WIKINDX_RSS_BIBSTYLE);
		$bibStyle = FACTORY_BIBSTYLE::getInstance();
		$rCommon = FACTORY_RESOURCECOMMON::getInstance();
		$db->limit($WIKINDX_RSS_LIMIT, 0);
		$db->ascDesc = $db->desc; // descending order
		$resultSet = $rCommon->getResource(FALSE, $order);
		$numResults = $db->numRows($resultSet);
		$x = 0;
		while($list_results = $db->fetchRow($resultSet))
		{
			/** construct a hierarchial array for the item node */
			if($list_results['resourceNoSort'])
				$item['title'][$x] = htmlspecialchars(stripslashes($list_results['resourceNoSort'])) . ' ' . 
				htmlspecialchars(stripslashes($list_results['resourceTitle']));
			else
				$item['title'][$x] = htmlspecialchars(stripslashes($list_results['resourceTitle']));
			$item['title'][$x] = htmlspecialchars(stripslashes($list_results['resourceType'])) . ': ' . 
				$item['title'][$x];
			$item['timestamp'][$x] = $list_results['resourcetimestampTimestamp'];
			list($item['addUser'][$x], $item['editUser'][$x]) = 
				getUser($db, $list_results['resourcemiscAddUserIdResource'], $list_results['resourcemiscEditUserIdResource']);
			$item['link'][$x] = $list_results['resourceId'];
			$item['description'][$x] = htmlspecialchars($bibStyle->process($list_results));
			$x++;
		}
		return array($numResults, $item);
	}

/**
* Function to grab add/edit full names or, if no full names, the user names.
*
* @param object $db
* @param int $addId
* @param int $editId
* @return array ($add, $edit)
*/
	function getUser($db, $addId, $editId)
	{
		$add = $edit = FALSE;
		if($addId)
		{
			$db->formatConditions(array('usersId' => $addId));
			$row = $db->fetchRow($db->select('users', array('usersUsername', 'usersFullname')));
			if($row['usersFullname'])
				$add = $row['usersFullname'];
			else if($row['usersUsername'])
				$add = $row['usersUsername'];
		}
		if($editId)
		{
			$db->formatConditions(array('usersId' => $editId));
			$row = $db->fetchRow($db->select('users', array('usersUsername', 'usersFullname')));
			if($row['usersFullname'])
				$edit = $row['usersFullname'];
			else if($row['usersUsername'])
				$edit = $row['usersUsername'];
		}
		return array($add, $edit);
	}
?>